@@ -24,15 +24,9 @@ function GetTechModifiedProperty(currentTechModifications, entityTemplateData, p
 
 	// TODO: will we ever need the full template?
 	// Get the classes which this entity template belongs to
-	var rawClasses;
+	var classes = [];
 	if (entityTemplateData && entityTemplateData.Identity)
-	{
-		rawClasses = entityTemplateData.Identity.Classes;
-		rawClasses = rawClasses && "_string" in rawClasses ?  rawClasses._string : "";
-		if (entityTemplateData.Identity.Rank)
-			rawClasses += " " + entityTemplateData.Identity.Rank;
-	} 
-	var classes = rawClasses && rawClasses.length ? rawClasses.split(/\s+/) : [];
+		classes = GetIdentityClasses(entityTemplateData.Identity);
 
 	var retValue = propertyValue;
 
@@ -63,17 +57,5 @@ function GetTechModifiedProperty(currentTechModifications, entityTemplateData, p
  */
 function DoesModificationApply(modification, classes)
 {
-	// See if any of the lists of classes matches this entity
-	for (var j in modification.affects)
-	{
-		var hasAllClasses = true;
-		// Check each class in affects is present for the entity
-		for (var k in modification.affects[j])
-			hasAllClasses = hasAllClasses && (classes.indexOf(modification.affects[j][k]) !== -1);
-
-		if (hasAllClasses)
-			return true;
-	}
-
-	return false;
+	return MatchesClassList(classes, modification.affects);
 }
