@@ -33,6 +33,7 @@ function GetVisibleIdentityClasses(template)
  * @param match Either a string in the form 
  *     "Class1 Class2+Class3"
  * where spaces are handled as OR and '+'-signs as AND,
+ * and ! is handled as NOT, thus Class1+!Class2 = Class1 AND NOT Class2
  * Or a list in the form
  *     [["Class1"], ["Class2", "Class3"]]
  * where the outer list is combined as OR, and the inner lists are AND-ed
@@ -56,7 +57,8 @@ function MatchesClassList(classes, match)
 		// if the elements are still strings, split them by space or by '+'
 		if (typeof sublist == "string")
 			sublist = sublist.split(/[+\s]+/);
-		if (sublist.every(function(c) { return classes.indexOf(c) != -1; }))
+		if (sublist.every(c => (c[0] == "!" && classes.indexOf(c.substr(1)) == -1)
+		                    || (c[0] != "!" && classes.indexOf(c) != -1)))
 			return true;
 	}
 
@@ -90,27 +92,43 @@ function GetTemplateDataHelper(template, player)
 
 	if (template.Attack)
 	{
-		ret.attack = {};
-		for (var type in template.Attack)
+		let getAttackStat = function(type, stat)
 		{
-			ret.attack[type] = {
-				"hack": func("Attack/"+type+"/Hack", +(template.Attack[type].Hack || 0), player, template),
-				"pierce": func("Attack/"+type+"/Pierce", +(template.Attack[type].Pierce || 0), player, template),
-				"crush": func("Attack/"+type+"/Crush", +(template.Attack[type].Crush || 0), player, template),
-				"minRange": func("Attack/"+type+"/MinRange", +(template.Attack[type].MinRange || 0), player, template),
-				"maxRange": func("Attack/"+type+"/MaxRange", +template.Attack[type].MaxRange, player, template),
-				"elevationBonus": func("Attack/"+type+"/ElevationBonus", +(template.Attack[type].ElevationBonus || 0), player, template),
-				"repeatTime": +(template.Attack[type].RepeatTime || 0),
-			};
+			return func("Attack/"+type+"/"+stat, +(template.Attack[type][stat] || 0), player, template);
+		};
+
+		ret.attack = {};
+		for (let type in template.Attack)
+		{
+			if (type == "Capture")
+				ret.attack.Capture = {
+					"value": getAttackStat(type,"Value"),
+				};
+			else
+				ret.attack[type] = {
+					"hack": getAttackStat(type, "Hack"),
+					"pierce": getAttackStat(type, "Pierce"),
+					"crush": getAttackStat(type, "Crush"),
+					"minRange": getAttackStat(type, "MinRange"),
+					"maxRange": getAttackStat(type, "MaxRange"),
+					"elevationBonus": getAttackStat(type, "ElevationBonus"),
+				};
+			ret.attack[type].repeatTime = +(template.Attack[type].RepeatTime || 0);
 		}
 	}
 
 	if (template.Auras)
 	{
 		ret.auras = {};
-		for each (var aura in template.Auras)
+		for (let auraID in template.Auras)
+		{
+			let aura = template.Auras[auraID];
 			if (aura.AuraName)
-				ret.auras[aura.AuraName] = aura.AuraDescription || null;
+				ret.auras[auraID] = {
+					"name": aura.AuraName,
+					"description": aura.AuraDescription || null
+				};
+		}
 	}
 
 	if (template.BuildRestrictions)
@@ -229,8 +247,12 @@ function GetTemplateDataHelper(template, player)
 	}
 
 	if (template.Trader)
-		ret.trader = template.Trader;
-
+	{
+		ret.trader = {
+			"GainMultiplier": func("Trader/GainMultiplier", +template.Trader.GainMultiplier, player, template)
+		};
+	}
+	    
 	if (template.WallSet)
 	{
 		ret.wallSet = {
