@@ -4,8 +4,6 @@ var API3 = (function() {
 
 var m = {};
 
-m.DebugEnabled = false;
-
 m.BaseAI = function(settings)
 {
 	if (!settings)
@@ -62,7 +60,7 @@ m.BaseAI.prototype.Init = function(state, playerID, sharedAI)
 	this.barterPrices = sharedAI.barterPrices;
 
 	this.CustomInit(this.gameState, this.sharedScript);
-}
+};
 
 m.BaseAI.prototype.CustomInit = function()
 {	// AIs override this function
@@ -91,14 +89,6 @@ m.BaseAI.prototype.chat = function(message)
 {
 	Engine.PostCommand(PlayerID,{"type": "aichat", "message": message});
 };
-m.BaseAI.prototype.chatTeam = function(message)
-{
-	Engine.PostCommand(PlayerID,{"type": "aichat", "message": "/team " +message});
-};
-m.BaseAI.prototype.chatEnemies = function(message)
-{
-	Engine.PostCommand(PlayerID,{"type": "aichat", "message": "/enemy " +message});
-};
 
 return m;
 
