@@ -16,22 +16,25 @@ m.Template = m.Class({
 	get: function(string)
 	{
 		var value = this._template;
-		if (this._auraTemplateModif && this._auraTemplateModif.has(string))		{
+		if (this._auraTemplateModif && this._auraTemplateModif.has(string))
 			return this._auraTemplateModif.get(string);
-		} else if (this._techModif && this._techModif.has(string)) {
+		else if (this._techModif && this._techModif.has(string))
 			return this._techModif.get(string);
-		} else {
+		else
+		{
 			if (!this._tpCache.has(string))
 			{
 				var args = string.split("/");
-				for (var i = 0; i < args.length; ++i)
-					if (value[args[i]])
-						value = value[args[i]];
+				for (let arg of args)
+				{
+					if (value[arg])
+						value = value[arg];
 					else
 					{
 						value = undefined;
 						break;
 					}
+				}
 				this._tpCache.set(string, value);
 			}
 			return this._tpCache.get(string);
@@ -183,13 +186,11 @@ m.Template = m.Class({
 
 	isRepairable: function()
 	{
-		if (this.get("Health") !== undefined)
-			return this.get("Health/Repairable") === "true";
-		return false;
+		return this.get("Repairable") !== undefined;
 	},
 
 	getPopulationBonus: function() {
-		return this.get("Cost/PopulationBonus");
+		return +this.get("Cost/PopulationBonus");
 	},
 
 	armourStrengths: function() {
@@ -235,6 +236,13 @@ m.Template = m.Class({
 		};
 	},
 
+	captureStrength: function() {
+		if (!this.get("Attack/Capture"))
+			return undefined;
+
+		return +this.get("Attack/Capture/Value") || 0;
+	},
+
 	attackTimes: function(type) {
 		if (!this.get("Attack/" + type +""))
 			return undefined;
@@ -269,15 +277,15 @@ m.Template = m.Class({
 		if (!this.get("Attack"))
 			return false;
 		var mcounter = [];
-		for (var i in this.get("Attack"))
+		for (let i in this.get("Attack"))
 		{
 			if (!this.get("Attack/" + i + "/Bonuses"))
 				continue;
-			for (var o in this.get("Attack/" + i + "/Bonuses"))
+			for (let o in this.get("Attack/" + i + "/Bonuses"))
 				if (this.get("Attack/" + i + "/Bonuses/" + o + "/Classes"))
 					mcounter.concat(this.get("Attack/" + i + "/Bonuses/" + o + "/Classes").split(" "));
 		}
-		for (var i in classes)
+		for (let i in classes)
 		{
 			if (mcounter.indexOf(classes[i]) !== -1)
 				return true;
@@ -298,7 +306,7 @@ m.Template = m.Class({
 				var total = this.get("Attack/" + type + "/Bonuses/" + o + "/Classes").split(" ");
 				for (var j in total)
 					if (total[j] === againstClass)
-						return this.get("Attack/" + type + "/Bonuses/" + o + "/Multiplier");
+						return +this.get("Attack/" + type + "/Bonuses/" + o + "/Multiplier");
 			}
 		return 1;
 	},
@@ -326,15 +334,16 @@ m.Template = m.Class({
 		return templates; // TODO: map to Entity?
 	},
 
-	trainableEntities: function() {
+	trainableEntities: function(civ) {
 		if (!this.get("ProductionQueue/Entities/_string"))
 			return undefined;
-		var civ = this.civ();
 		var templates = this.get("ProductionQueue/Entities/_string").replace(/\{civ\}/g, civ).split(/\s+/);
 		return templates;
 	},
 
-	researchableTechs: function() {
+	researchableTechs: function(civ) {
+		if (this.civ() !== civ)     // techs can only be researched in structure from the player civ
+			return undefined;
 		if (!this.get("ProductionQueue/Technologies/_string"))
 			return undefined;
 		var templates = this.get("ProductionQueue/Technologies/_string").split(/\s+/);
@@ -401,6 +410,12 @@ m.Template = m.Class({
 		return this.get("GarrisonHolder/Max");
 	},
 
+	garrisonEjectHealth: function() {
+		if (!this.get("GarrisonHolder"))
+			return undefined;
+		return +this.get("GarrisonHolder/EjectHealth");
+	},
+
 	getDefaultArrow: function() {
 		if (!this.get("BuildingAI"))
 			return undefined;
@@ -460,7 +475,7 @@ m.Template = m.Class({
 	walkSpeed: function() {
 		if (!this.get("UnitMotion") || !this.get("UnitMotion/WalkSpeed"))
 			 return undefined;
-		return this.get("UnitMotion/WalkSpeed");
+		return +this.get("UnitMotion/WalkSpeed");
 	},
 
 	trainingCategory: function() {
@@ -478,7 +493,7 @@ m.Template = m.Class({
 	buildTime: function() {
 		if (!this.get("Cost") || !this.get("Cost/BuildTime"))
 			return undefined;
-		return this.get("Cost/BuildTime");
+		return +this.get("Cost/BuildTime");
 	},
 
 	buildDistance: function() {
@@ -508,22 +523,40 @@ m.Template = m.Class({
 		return (this.get("TerritoryInfluence") !== undefined);
 	},
 
+	hasDefensiveFire: function() {
+		if (!this.get("Attack") || !this.get("Attack/Ranged"))
+			return false;
+		return (this.getDefaultArrow() || this.getArrowMultiplier());
+	},
+
 	territoryInfluenceRadius: function() {
 		if (this.get("TerritoryInfluence") !== undefined)
-			return (this.get("TerritoryInfluence/Radius"));
+			return +this.get("TerritoryInfluence/Radius");
 		else
 			return -1;
 	},
 
 	territoryInfluenceWeight: function() {
 		if (this.get("TerritoryInfluence") !== undefined)
-			return (this.get("TerritoryInfluence/Weight"));
+			return +this.get("TerritoryInfluence/Weight");
 		else
 			return -1;
 	},
 
+	territoryDecayRate: function() {
+		return (this.get("TerritoryDecay") ? +this.get("TerritoryDecay/DecayRate") : 0);
+	},
+
+	defaultRegenRate: function() {
+		return (this.get("Capturable") ? +this.get("Capturable/RegenRate") : 0);
+	},
+
+	garrisonRegenRate: function() {
+		return (this.get("Capturable") ? +this.get("Capturable/GarrisonRegenRate") : 0);
+	},
+
 	visionRange: function() {
-		return this.get("Vision/Range");
+		return +this.get("Vision/Range");
 	}
 });
 
@@ -590,17 +623,16 @@ m.Entity = m.Class({
 		return this._entity.idle;
 	},
 
-	unitAIState: function() { if (this._entity.unitAIState) return this._entity.unitAIState; return undefined; },
-	unitAIOrderData: function() { return this._entity.unitAIOrderData; },
+	unitAIState: function() { return ((this._entity.unitAIState !== undefined) ? this._entity.unitAIState : undefined); },
+	unitAIOrderData: function() { return ((this._entity.unitAIOrderData !== undefined) ? this._entity.unitAIOrderData : undefined); },
 
-	// TODO  understand why we have sometimes rounding problems with maxHitpoints ? making wrongly isHurt=true
-	// problem seems to be with hele civs (i.e. spart)
-	hitpoints: function() { if (this._entity.hitpoints !== undefined) return this._entity.hitpoints; return undefined; },
-	isHurt: function() { return (this.hitpoints() + 1) < this.maxHitpoints(); },
+	hitpoints: function() { return ((this._entity.hitpoints !== undefined) ? this._entity.hitpoints : undefined); },
+	isHurt: function() { return (this.hitpoints() < this.maxHitpoints()); },
 	healthLevel: function() { return (this.hitpoints() / this.maxHitpoints()); },
 	needsHeal: function() { return this.isHurt() && this.isHealable(); },
 	needsRepair: function() { return this.isHurt() && this.isRepairable(); },
-	decaying: function() { if (this._entity.decaying !== undefined) return this._entity.decaying; return undefined; },
+	decaying: function() { return ((this._entity.decaying !== undefined) ? this._entity.decaying : undefined); },
+	capturePoints: function() {return ((this._entity.capturePoints !== undefined) ? this._entity.capturePoints : undefined); },
 
 	/**
 	 * Returns the current training queue state, of the form
@@ -653,31 +685,23 @@ m.Entity = m.Class({
 		return this._entity.owner === player;
 	},
 
-	isFriendly: function(player) {
-		return this.isOwn(player); // TODO: diplomacy
-	},
-
-	isEnemy: function(player) {
-		return !this.isOwn(player); // TODO: diplomacy
-	},
-
 	resourceSupplyAmount: function() {
 		if(this._entity.resourceSupplyAmount === undefined)
 			return undefined;
 		return this._entity.resourceSupplyAmount;
 	},
 
-	resourceSupplyGatherers: function()
+	resourceSupplyNumGatherers: function()
 	{
-		if (this._entity.resourceSupplyGatherers !== undefined)
-			return this._entity.resourceSupplyGatherers;
-		return [];
+		if (this._entity.resourceSupplyNumGatherers !== undefined)
+			return this._entity.resourceSupplyNumGatherers;
+		return undefined;
 	},
 
 	isFull: function()
 	{
-		if (this._entity.resourceSupplyGatherers !== undefined)
-			return (this.maxGatherers() === this._entity.resourceSupplyGatherers.length);
+		if (this._entity.resourceSupplyNumGatherers !== undefined)
+			return (this.maxGatherers() === this._entity.resourceSupplyNumGatherers);
 
 		return undefined;
 	},
@@ -696,60 +720,63 @@ m.Entity = m.Class({
 		if (this.unitAIOrderData().length &&
 			(this.unitAIState().split(".")[1] === "GATHER" || this.unitAIState().split(".")[1] === "RETURNRESOURCE"))
 		{
-			var ress = undefined;
+			let res;
 			// this is an abuse of "_ai" but it works.
-			if (this.unitAIState().split(".")[1] === "GATHER" && this.unitAIOrderData()[0]["target"] !== undefined)
-				ress = this._ai._entities.get(this.unitAIOrderData()[0]["target"]);
-			else if (this.unitAIOrderData()[1] !== undefined && this.unitAIOrderData()[1]["target"] !== undefined)
-				ress = this._ai._entities.get(this.unitAIOrderData()[1]["target"]);
+			if (this.unitAIState().split(".")[1] === "GATHER" && this.unitAIOrderData()[0].target !== undefined)
+				res = this._ai._entities.get(this.unitAIOrderData()[0].target);
+			else if (this.unitAIOrderData()[1] !== undefined && this.unitAIOrderData()[1].target !== undefined)
+				res = this._ai._entities.get(this.unitAIOrderData()[1].target);
+			if (!res)
+				return 0;
+			let type = res.resourceSupplyType();
+			if (!type)
+				return 0;
 
-			if (ress == undefined)
-				return undefined;
-
-			var type = ress.resourceSupplyType();
-			var tstring = type.generic + "." + type.specific;
-
-			if (type.generic == "treasure")
+			if (type.generic === "treasure")
 				return 1000;
 
-			var speed = +this.get("ResourceGatherer/BaseSpeed");
-			speed *= +this.get("ResourceGatherer/Rates/" +tstring);
-
-			if (speed)
-				return speed;
+			let tstring = type.generic + "." + type.specific;
+			let rate = +this.get("ResourceGatherer/BaseSpeed");
+			rate *= +this.get("ResourceGatherer/Rates/" +tstring);
+			if (rate)
+				return rate;
 			return 0;
 		}
 		return undefined;
 	},
 
-	isGarrisonHolder: function() { return this.get("GarrisonHolder") },
+	isBuilder: function() { return this.get("Builder") !== undefined; },
+	isGatherer: function() { return this.get("ResourceGatherer") !== undefined; },
+	canGather: function(type) {
+		if (!this.get("ResourceGatherer") || !this.get("ResourceGatherer/Rates"))
+			return false;
+		for (let r in this.get("ResourceGatherer/Rates"))
+			if (r.split('.')[0] === type)
+				return true;
+		return false;
+	},
 
+	isGarrisonHolder: function() { return this.get("GarrisonHolder") !== undefined; },
 	garrisoned: function() { return this._entity.garrisoned; },
-
 	canGarrisonInside: function() { return this._entity.garrisoned.length < this.garrisonMax(); },
 
-	// TODO: visibility
-
-	move: function(x, z, queued) {
-		queued = queued || false;
+	move: function(x, z, queued = false) {
 		Engine.PostCommand(PlayerID,{"type": "walk", "entities": [this.id()], "x": x, "z": z, "queued": queued });
 		return this;
 	},
 
-	moveToRange: function(x, z, min, max, queued) {
-		queued = queued || false;
+	moveToRange: function(x, z, min, max, queued = false) {
 		Engine.PostCommand(PlayerID,{"type": "walk-to-range", "entities": [this.id()], "x": x, "z": z, "min": min, "max": max, "queued": queued });
 		return this;
 	},
 
-	attackMove: function(x, z, targetClasses, queued) {
-		queued = queued || false;
+	attackMove: function(x, z, targetClasses, queued = false) {
 		Engine.PostCommand(PlayerID,{"type": "attack-walk", "entities": [this.id()], "x": x, "z": z, "targetClasses": targetClasses, "queued": queued });
 		return this;
 	},
 
 	// violent, aggressive, defensive, passive, standground
-	setStance: function(stance,queued){
+	setStance: function(stance, queued = false) {
 		Engine.PostCommand(PlayerID,{"type": "stance", "entities": [this.id()], "name" : stance, "queued": queued });
 		return this;
 	},
@@ -769,18 +796,17 @@ m.Entity = m.Class({
 	unloadAll: function() {
 		if (!this.get("GarrisonHolder"))
 			return undefined;
-		Engine.PostCommand(PlayerID,{"type": "unload-all-own", "garrisonHolders": [this.id()]});
+		Engine.PostCommand(PlayerID,{"type": "unload-all-by-owner", "garrisonHolders": [this.id()]});
 		return this;
 	},
 
-	garrison: function(target, queued) {
-		queued = queued || false;
+	garrison: function(target, queued = false) {
 		Engine.PostCommand(PlayerID,{"type": "garrison", "entities": [this.id()], "target": target.id(),"queued": queued});
 		return this;
 	},
 
-	attack: function(unitId) {
-		Engine.PostCommand(PlayerID,{"type": "attack", "entities": [this.id()], "target": unitId, "queued": false});
+	attack: function(unitId, allowCapture = true, queued = false) {
+		Engine.PostCommand(PlayerID,{"type": "attack", "entities": [this.id()], "target": unitId, "allowCapture": allowCapture, "queued": queued});
 		return this;
 	},
 
@@ -814,20 +840,17 @@ m.Entity = m.Class({
 		return this;
 	},
 
-	gather: function(target, queued) {
-		queued = queued || false;
+	gather: function(target, queued = false) {
 		Engine.PostCommand(PlayerID,{"type": "gather", "entities": [this.id()], "target": target.id(), "queued": queued});
 		return this;
 	},
 
-	repair: function(target, queued) {
-		queued = queued || false;
-		Engine.PostCommand(PlayerID,{"type": "repair", "entities": [this.id()], "target": target.id(), "autocontinue": false, "queued": queued});
+	repair: function(target, autocontinue = false, queued = false) {
+		Engine.PostCommand(PlayerID,{"type": "repair", "entities": [this.id()], "target": target.id(), "autocontinue": autocontinue, "queued": queued});
 		return this;
 	},
 
-	returnResources: function(target, queued) {
-		queued = queued || false;
+	returnResources: function(target, queued = false) {
 		Engine.PostCommand(PlayerID,{"type": "returnresource", "entities": [this.id()], "target": target.id(), "queued": queued});
 		return this;
 	},
@@ -858,9 +881,9 @@ m.Entity = m.Class({
 		return this;
 	},
 
-	train: function(type, count, metadata, promotedTypes)
+	train: function(civ, type, count, metadata, promotedTypes)
 	{
-		var trainable = this.trainableEntities();
+		var trainable = this.trainableEntities(civ);
 		if (!trainable)
 		{
 			error("Called train("+type+", "+count+") on non-training entity "+this);
