@@ -1,11 +1,11 @@
 var API3 = function(m)
 {
 
-m.EntityCollection = function(sharedAI, entities, filters)
+m.EntityCollection = function(sharedAI, entities = new Map(), filters = [])
 {
 	this._ai = sharedAI;
-	this._entities = entities || new Map();
-	this._filters = filters || [];
+	this._entities = entities;
+	this._filters = filters;
 	this.dynamicProp = [];
 	for (var filter of this._filters)
 		if (filter.dynamicProperties.length)
@@ -41,9 +41,9 @@ m.EntityCollection.prototype.Deserialize = function(data, sharedAI)
 		this._filters.push(eval(f));
 
 	if (data.frozen)
-		this.freeze;
+		this.freeze();
 	else
-		this.defreeze;
+		this.defreeze();
 };
 
 // If an entitycollection is frozen, it will never automatically add a unit.
@@ -61,18 +61,12 @@ m.EntityCollection.prototype.defreeze = function()
 
 m.EntityCollection.prototype.toIdArray = function()
 {
-	let ret = [];
-	for (let id of this._entities.keys())
-		ret.push(id);
-	return ret;
+	return [...this._entities.keys()];
 };
 
 m.EntityCollection.prototype.toEntityArray = function()
 {
-	let ret = [];
-	for (let ent of this._entities.values())
-		ret.push(ent);
-	return ret;
+	return [...this._entities.values()];
 };
 
 m.EntityCollection.prototype.values = function()
@@ -87,7 +81,7 @@ m.EntityCollection.prototype.toString = function()
 
 m.EntityCollection.prototype.filter = function(filter, thisp)
 {
-	if (typeof(filter) == "function")
+	if (typeof filter === "function")
 		filter = {"func": filter, "dynamicProperties": []};
 	
 	var ret = new Map();
@@ -110,7 +104,8 @@ m.EntityCollection.prototype.filterNearest = function(targetPos, n)
 			data.push([id, ent, m.SquareVectorDistance(targetPos, ent.position())]);
 
 	// Sort by increasing distance
-	data.sort(function (a, b) { return (a[2] - b[2]); });
+	data.sort((a, b) => a[2] - b[2]);
+
 	if (n === undefined)
 		n = data.length;
 	else
@@ -161,13 +156,7 @@ m.EntityCollection.prototype.moveIndiv = function(x, z, queued)
 {
 	queued = queued || false;
 	for (let id of this._entities.keys())
-	{
-		// The following try {} finally {} block is a workaround for OOS problems in multiplayer games with AI players (check ticket #2000).
-		// It disables JIT compiling of this loop. Don't remove it unless you are sure that the underlying issue has been resolved!
-		// TODO: Check this again after the SpiderMonkey upgrade.
-		try {} finally {}
 		Engine.PostCommand(PlayerID,{"type": "walk", "entities": [id], "x": x, "z": z, "queued": queued});
-	}
 	return this;
 };
 
@@ -185,11 +174,7 @@ m.EntityCollection.prototype.destroy = function()
 
 m.EntityCollection.prototype.attack = function(unit)
 {
-	var unitId;
-	if (typeof(unit) === "Entity")
-		unitId = unit.id();
-	else
-		unitId = unit;
+	var unitId = unit;
 	Engine.PostCommand(PlayerID,{"type": "attack", "entities": this.toIdArray(), "target": unitId, "queued": false});
 	return this;
 };
