@@ -115,15 +115,15 @@ m.Filters = {
 		}, 
 		"dynamicProperties": ['trainingQueue']};
 	},
-	byResearchAvailable: function(){
+	byResearchAvailable: function(civ){
 		return {"func" : function(ent){
-			return ent.researchableTechs() !== undefined;
+			return ent.researchableTechs(civ) !== undefined;
 		},
 		"dynamicProperties": []};
 	},
 	byTargetedEntity: function(targetID){
 		return {"func": function(ent) {
-			return (ent.unitAIOrderData().length && ent.unitAIOrderData()[0]["target"] && ent.unitAIOrderData()[0]["target"] == targetID);
+			return (ent.unitAIOrderData().length && ent.unitAIOrderData()[0].target && ent.unitAIOrderData()[0].target == targetID);
 		},
 		"dynamicProperties": ['unitAIOrderData']};
 	},
@@ -162,7 +162,21 @@ m.Filters = {
 		},
 		"dynamicProperties": []};
 	},
-	
+
+	isBuilt: function(){
+		return {"func": function(ent){
+			return ent.foundationProgress() === undefined;
+		},
+		"dynamicProperties": []};
+	},
+
+	hasDefensiveFire: function(){
+		return {"func": function(ent){
+			return ent.hasDefensiveFire();
+		},
+		"dynamicProperties": []};
+	},
+
 	byDistance: function(startPoint, dist){
 		return {"func": function(ent){
 			if (ent.position() === undefined)
