@@ -9,20 +9,10 @@ var API3 = function(m)
 m.Map = function Map(sharedScript, type, originalMap, actualCopy)
 {
 	// get the correct dimensions according to the map type
-	if (type === "territory" || type === "resource")
-	{
-		var map = sharedScript.territoryMap;
-		this.width = map.width;
-		this.height = map.height;
-		this.cellSize = map.cellSize;
-	}
-	else
-	{
-		var map = sharedScript.passabilityMap;
-		this.width = map.width;
-		this.height = map.height;
-		this.cellSize = map.cellSize;
-	}
+	let map = (type === "territory" || type === "resource") ? sharedScript.territoryMap : sharedScript.passabilityMap;
+	this.width = map.width;
+	this.height = map.height;
+	this.cellSize = map.cellSize;   
 	this.length = this.width * this.height;
 
 	this.maxVal = 255;
@@ -55,16 +45,15 @@ m.Map.prototype.gamePosToMapPos = function(p)
 
 m.Map.prototype.point = function(p)
 {
-	var q = this.gamePosToMapPos(p);
+	let q = this.gamePosToMapPos(p);
 	q[0] = q[0] >= this.width ? this.width-1 : (q[0] < 0 ? 0 : q[0]);
 	q[1] = q[1] >= this.width ? this.width-1 : (q[1] < 0 ? 0 : q[1]);
 	return this.map[q[0] + this.width * q[1]];
 };
 
-m.Map.prototype.addInfluence = function(cx, cy, maxDist, strength, type)
+m.Map.prototype.addInfluence = function(cx, cy, maxDist, strength, type = "linear")
 {
-	strength = strength ? +strength : +maxDist;
-	type = type ? type : 'linear';
+	strength = strength ? strength : maxDist;
 	
 	var x0 = Math.floor(Math.max(0, cx - maxDist));
 	var y0 = Math.floor(Math.max(0, cy - maxDist));
@@ -72,61 +61,51 @@ m.Map.prototype.addInfluence = function(cx, cy, maxDist, strength, type)
 	var y1 = Math.floor(Math.min(this.height-1, cy + maxDist));
 	var maxDist2 = maxDist * maxDist;
 	
-	var str = 0.0;
-	switch (type)
-	{
-		case 'linear':
-			str = +strength / +maxDist;
-		break;
-		case 'quadratic':
-			str = +strength / +maxDist2;
-		break;
-		case 'constant':
-			str = +strength;
-		break;
-	}
-	
 	// code duplicating for speed
 	if (type === 'linear' || type === "linear")
 	{
+		let str = strength / maxDist;	
 		for (let y = y0; y < y1; ++y)
 		{
+			let dy = y - cy;		
 			for (let x = x0; x < x1; ++x)
 			{
 				let dx = x - cx;
-				let dy = y - cy;
 				let r2 = dx*dx + dy*dy;
 				if (r2 < maxDist2)
 				{
-					let quant = str * (maxDist - Math.sqrt(r2));					
-					if (this.map[x + y * this.width] + quant < 0)
-						this.map[x + y * this.width] = 0;
-					else if (this.map[x + y * this.width] + quant > this.maxVal)
-						this.map[x + y * this.width] = this.maxVal;	// avoids overflow.
+					let quant = str * (maxDist - Math.sqrt(r2));
+					let w = x + y * this.width;
+					if (this.map[w] + quant < 0)
+						this.map[w] = 0;
+					else if (this.map[w] + quant > this.maxVal)
+						this.map[w] = this.maxVal;	// avoids overflow.
 					else
-						this.map[x + y * this.width] += quant;
+						this.map[w] += quant;
 				}
 			}
 		}
 	}
 	else if (type === 'quadratic' || type === "quadratic")
 	{
+		let str = strength / maxDist2;
 		for (let y = y0; y < y1; ++y)
 		{
+			let dy = y - cy;		
 			for (let x = x0; x < x1; ++x)
 			{
 				let dx = x - cx;
-				let dy = y - cy;
 				let r2 = dx*dx + dy*dy;
 				if (r2 < maxDist2)
 				{
 					let quant = str * (maxDist2 - r2);
-					if (this.map[x + y * this.width] + quant < 0)
-						this.map[x + y * this.width] = 0;
-					else if (this.map[x + y * this.width] + quant > this.maxVal)
-						this.map[x + y * this.width] = this.maxVal;	// avoids overflow.
+					let w = x + y * this.width;				    
+					if (this.map[w] + quant < 0)
+						this.map[w] = 0;
+					else if (this.map[w] + quant > this.maxVal)
+						this.map[w] = this.maxVal;	// avoids overflow.
 					else
-						this.map[x + y * this.width] += quant;
+						this.map[w] += quant;
 				}
 			}
 		}
@@ -135,19 +114,20 @@ m.Map.prototype.addInfluence = function(cx, cy, maxDist, strength, type)
 	{
 		for (let y = y0; y < y1; ++y)
 		{
+			let dy = y - cy;
 			for (let x = x0; x < x1; ++x)
 			{
 				let dx = x - cx;
-				let dy = y - cy;
 				let r2 = dx*dx + dy*dy;
 				if (r2 < maxDist2)
 				{
-					if (this.map[x + y * this.width] + str < 0)
-						this.map[x + y * this.width] = 0;
-					else if (this.map[x + y * this.width] + str > this.maxVal)
-						this.map[x + y * this.width] = this.maxVal;	// avoids overflow.
+					let w = x + y * this.width;				
+					if (this.map[w] + strength < 0)
+						this.map[w] = 0;
+					else if (this.map[w] + strength > this.maxVal)
+						this.map[w] = this.maxVal;	// avoids overflow.
 					else
-						this.map[x + y * this.width] += str;
+						this.map[w] += strength;
 				}
 			}
 		}
@@ -252,10 +232,8 @@ m.Map.prototype.multiplyInfluence = function(cx, cy, maxDist, strength, type)
 };
 
 // doesn't check for overflow.
-m.Map.prototype.setInfluence = function(cx, cy, maxDist, value)
+m.Map.prototype.setInfluence = function(cx, cy, maxDist, value = 0)
 {
-	value = value ? value : 0;
-	
 	var x0 = Math.max(0, cx - maxDist);
 	var y0 = Math.max(0, cy - maxDist);
 	var x1 = Math.min(this.width, cx + maxDist);
@@ -388,53 +366,115 @@ m.Map.prototype.add = function(map)
 	}
 };
 
-m.Map.prototype.findBestTile = function(radius, obstructionTiles)
+// Find the best non-obstructed tile
+m.Map.prototype.findBestTile = function(radius, obstruction)
 {
-	// Find the best non-obstructed tile
-	let bestIdx = 0;
+	let bestIdx;
 	let bestVal = -1;
-	for (let i = 0; i < this.length; ++i)
+	for (let j = 0; j < this.length; ++j)
 	{
-		let v = this.map[i];
-		if (v > bestVal)
-		{
-			var j = API3.getMaxMapIndex(i, this, obstructionTiles);
-			if (obstructionTiles.map[j] <= radius)
-				continue;
-			bestVal = v;
-			bestIdx = j;
-		}
+		if (this.map[j] <= bestVal)
+			continue;
+		let i = this.getNonObstructedTile(j, radius, obstruction);
+		if (i < 0)
+			continue;
+		bestVal = this.map[j];
+		bestIdx = i;
 	}
 	
 	return [bestIdx, bestVal];
 };
 
-// returns the point with the lowest (but still > radius) point in the immediate vicinity
-m.Map.prototype.findLowestNeighbor = function(x,y,radius)
+// return any non obstructed (small) tile inside the (big) tile i from obstruction map
+m.Map.prototype.getNonObstructedTile = function(i, radius, obstruction)
 {
-	var lowestPt = [0,0];
-	var lowestcoeff = undefined;
-	x = Math.floor(x/this.cellSize);
-	y = Math.floor(y/this.cellSize);
-	for (let xx = x-1; xx <= x+1; ++xx)
+	let ratio = this.cellSize / obstruction.cellSize;
+	let ix = (i % this.width) * ratio;
+	let iy = Math.floor(i / this.width) * ratio;
+	let w = obstruction.width;
+	for (let kx = ix; kx < ix + ratio; ++kx)
 	{
-		for (let yy = y-1; yy <= y+1; ++yy)
+		if (kx < radius || kx >= w - radius)
+			continue;
+		for (let ky = iy; ky < iy + ratio; ++ky)
 		{
-			if (xx < 0 || xx >= this.width || yy < 0 || yy >= this.width)
+			if (ky < radius || ky >= w - radius)
 				continue;
-			if (lowestcoeff && this.map[xx+yy*this.width] > lowestcoeff)
+			if (obstruction.isObstructedTile(kx, ky, radius))
 				continue;
-			lowestcoeff = this.map[xx+yy*this.width];
-			lowestPt = [(xx+0.5)*4, (yy+0.5)*4];
+			return (kx + ky*w);
 		}
 	}
-	return lowestPt;
+	return -1;
 };
 
-m.Map.prototype.dumpIm = function(name, threshold)
+// return true if the area centered on tile kx-ky and with radius is obstructed
+m.Map.prototype.isObstructedTile = function(kx, ky, radius)
+{
+	let w = this.width;
+	if (kx < radius || kx >= w - radius || ky < radius || ky >= w - radius || this.map[kx+ky*w] == 0)
+		return true;
+	for (let dy = 0; dy <= radius; ++dy)
+	{
+		let dxmax = (dy == 0) ? radius : Math.ceil(Math.sqrt(radius*radius - (dy-0.5)*(dy-0.5)));
+		let xp = kx + (ky + dy)*w;
+		let xm = kx + (ky - dy)*w;
+		for (let dx = -dxmax; dx <= dxmax; ++dx)
+			if (this.map[xp + dx] == 0 || this.map[xm + dx] == 0)
+				return true;
+	}
+	return false;
+};
+
+// returns the nearest obstructed point
+// TODO check that the landpassmap index is the same
+m.Map.prototype.findNearestObstructed = function(k, radius)
+{
+	var w = this.width;
+	var ix = k % w;
+	var iy = Math.floor(k / w);
+	var n = (this.cellSize > 8) ? 1 : Math.floor(8 / this.cellSize);
+	for (let i = 1; i <= n; ++i)
+	{
+		let kx = ix - i;
+		let ky = iy + i;
+		for (let j = 1; j <= 8*i; ++j)
+		{
+			if (this.isObstructedTile(kx, ky, radius))
+			{
+				let akx = Math.abs(kx-ix);
+				let aky = Math.abs(ky-iy);
+				if (akx >= aky)
+				{
+					if (kx > ix)
+						--kx;
+					else
+						++kx;
+				}
+				if (aky >= akx)
+				{
+					if (ky > iy)
+						--ky;
+					else
+						++ky;
+				}
+				return (kx + w*ky);
+			}
+			if (j < 2*i+1)
+				++kx;
+			else if (j < 4*i+1)
+				--ky;
+			else if (j < 6*i+1)
+				--kx;
+			else
+				++ky;
+		}
+	}
+	return -1;
+};
+
+m.Map.prototype.dumpIm = function(name = "default.png", threshold = this.maxVal)
 {
-	name = name ? name : "default.png";
-	threshold = threshold ? threshold : this.maxVal;
 	Engine.DumpImage(name, this.map, this.width, this.height, threshold);
 };
 
