@@ -6,57 +6,41 @@ m.SharedScript = function(settings)
 {
 	if (!settings)
 		return;
-	
+
 	this._players = settings.players;
 	this._templates = settings.templates;
 	this._derivedTemplates = {};
 	this._techTemplates = settings.techTemplates;
-		
+
 	this._entityMetadata = {};
-	for (var i in this._players)
+	for (let i in this._players)
 		this._entityMetadata[this._players[i]] = {};
 
-	// always create for 8 + gaia players, since _players isn't aware of the human.
-	this._techModifications = { 0 : {}, 1 : {}, 2 : {}, 3 : {}, 4 : {}, 5 : {}, 6 : {}, 7 : {}, 8 : {} };
-	
 	// array of entity collections
 	this._entityCollections = new Map();
+	this._techModifications = {};
 	// each name is a reference to the actual one.
 	this._entityCollectionsName = new Map();
 	this._entityCollectionsByDynProp = {};
 	this._entityCollectionsUID = 0;
-	
-	// size of the map  
-	// TODO should be set in settings, for the time being recompute it assuming the passability cell size   
-	this.sizeMap = undefined;
-	this.passabilityCell = 4;
 
 	// A few notes about these maps. They're updated by checking for "create" and "destroy" events for all resources
 	// TODO: change the map when the resource amounts change for at least stone and metal mines.
 	this.resourceMaps = {}; // Contains maps showing the density of wood, stone and metal
 	this.CCResourceMaps = {}; // Contains maps showing the density of wood, stone and metal, optimized for CC placement.
 	// Resource maps data.
-	// By how much to divide the resource amount for plotting (ie a tree having 200 wood is "4").
-	this.decreaseFactor = {'wood': 50.0, 'stone': 90.0, 'metal': 90.0, 'food': 40.0};
-}
+	// By how much to divide the resource amount when filling the map (ie a tree having 200 wood is "4").
+	this.decreaseFactor = {"wood": 50.0, "stone": 90.0, "metal": 90.0};
+};
 
-//Return a simple object (using no classes etc) that will be serialized
-//into saved games
-//TODO: that
+//Return a simple object (using no classes etc) that will be serialized into saved games
 m.SharedScript.prototype.Serialize = function()
 {
-	// serializing entities without using the class.
-	var entities = []; 
-	for (let ent of this._entities.values()) 
-		entities.push( ent._entity );
-
 	return {
-		"players" : this._players,
-		"templates" : this._templates,
-		"techTp" : this._techTemplates,
+		"players": this._players,
+		"techTp": this._techTemplates,
 		"techModifications": this._techModifications,
-		"metadata": this._entityMetadata,
-		"entities": entities
+		"metadata": this._entityMetadata
 	};
 };
 
@@ -65,15 +49,10 @@ m.SharedScript.prototype.Serialize = function()
 m.SharedScript.prototype.Deserialize = function(data)
 {
 	this._players = data.players;
-	this._templates = data.templates;
 	this._techTemplates = data.techTp;
 	this._techModifications = data.techModifications;
 	this._entityMetadata = data.metadata;
-	this._derivedTemplates = {};
-
-	this._entities = new Map();
-	for (let ent of data.entities)
-		this._entities.set(ent.id, new m.Entity(this, ent)); 
+	this._derivedTemplates = {}; 
 
 	this.isDeserialized = true;
 };
@@ -111,10 +90,10 @@ m.SharedScript.prototype.GetTemplate = function(name)
 	// If this is a foundation template, construct it automatically
 	if (name.indexOf("foundation|") !== -1)
 	{
-		var base = this.GetTemplate(name.substr(11));
+		let base = this.GetTemplate(name.substr(11));
 		
-		var foundation = {};
-		for (var key in base)
+		let foundation = {};
+		for (let key in base)
 			if (!m.g_FoundationForbiddenComponents[key])
 				foundation[key] = base[key];
 		
@@ -123,10 +102,10 @@ m.SharedScript.prototype.GetTemplate = function(name)
 	}
 	else if (name.indexOf("resource|") !== -1)
 	{
-		var base = this.GetTemplate(name.substr(9));
+		let base = this.GetTemplate(name.substr(9));
 		
-		var resource = {};
-		for (var key in base)
+		let resource = {};
+		for (let key in base)
 			if (!m.g_ResourceForbiddenComponents[key])
 				resource[key] = base[key];
 		
@@ -139,10 +118,14 @@ m.SharedScript.prototype.GetTemplate = function(name)
 };
 
 // Initialize the shared component.
-// We need to now the initial state of the game for this, as we will use it.
+// We need to know the initial state of the game for this, as we will use it.
 // This is called right at the end of the map generation.
 m.SharedScript.prototype.init = function(state, deserialization)
 {
+	if (!deserialization)
+		for (let i = 0; i < state.players.length; ++i)
+			this._techModifications[i] = {};
+
 	this.ApplyTemplatesDelta(state);
 
 	this.passabilityClasses = state.passabilityClasses;
@@ -150,39 +133,37 @@ m.SharedScript.prototype.init = function(state, deserialization)
 	this.playersData = state.players;
 	this.timeElapsed = state.timeElapsed;
 	this.circularMap = state.circularMap;
+	this.mapSize = state.mapSize;
 	this.gameType = state.gameType;
 	this.barterPrices = state.barterPrices;
 
 	this.passabilityMap = state.passabilityMap;
-	this.passabilityMap.cellSize = this.passabilityCell;
-	this.sizeMap = this.passabilityMap.width * this.passabilityMap.cellSize;
+	if (this.mapSize % this.passabilityMap.width != 0)
+		 error("AI shared component inconsistent sizes: map=" + this.mapSize + " while passability=" +  this.passabilityMap.width);
+	this.passabilityMap.cellSize = this.mapSize / this.passabilityMap.width;
 	this.territoryMap = state.territoryMap;
-	this.territoryMap.cellSize = this.sizeMap / this.territoryMap.width;
+	if (this.mapSize % this.territoryMap.width != 0)
+		 error("AI shared component inconsistent sizes: map=" + this.mapSize + " while territory=" +  this.territoryMap.width);
+	this.territoryMap.cellSize = this.mapSize / this.territoryMap.width;
 
 /*
 	var landPassMap = new Uint8Array(this.passabilityMap.data.length);
 	var waterPassMap = new Uint8Array(this.passabilityMap.data.length);
-	var obstructionMap = new Uint8Array(this.passabilityMap.data.length);
-	var obstructionMaskLand = this.passabilityClasses["default"];
-	var obstructionMaskWater = this.passabilityClasses["ship"];
-	var obstructionMask = this.passabilityClasses["pathfinderObstruction"];
+	var obstructionMaskLand = this.passabilityClasses["default-terrain-only"];
+	var obstructionMaskWater = this.passabilityClasses["ship-terrain-only"];
 	for (var i = 0; i < this.passabilityMap.data.length; ++i)
 	{
 		landPassMap[i] = (this.passabilityMap.data[i] & obstructionMaskLand) ? 0 : 255;
 		waterPassMap[i] = (this.passabilityMap.data[i] & obstructionMaskWater) ? 0 : 255;
-		obstructionMap[i] = (this.passabilityMap.data[i] & obstructionMask) ? 0 : 255;
 	}
 	Engine.DumpImage("LandPassMap.png", landPassMap, this.passabilityMap.width, this.passabilityMap.height, 255);
 	Engine.DumpImage("WaterPassMap.png", waterPassMap, this.passabilityMap.width, this.passabilityMap.height, 255);
-	Engine.DumpImage("ObstrctionMap.png", obstructionMap, this.passabilityMap.width, this.passabilityMap.height, 255);
 */
 
-	if (!deserialization)
-	{
-		this._entities = new Map();
-		for (var id in state.entities)
+	this._entities = new Map();
+	if (state.entities)
+		for (let id in state.entities)
 			this._entities.set(+id, new m.Entity(this, state.entities[id]));
-	}
 	// entity collection updated on create/destroy event.
 	this.entities = new m.EntityCollection(this, this._entities);
 	
@@ -199,7 +180,7 @@ m.SharedScript.prototype.init = function(state, deserialization)
 	for (var i in this._players)
 	{
 		this.gameState[this._players[i]] = new m.GameState();
-		this.gameState[this._players[i]].init(this,state,this._players[i]);
+		this.gameState[this._players[i]].init(this,state, this._players[i]);
 	}
 };
 
@@ -227,16 +208,15 @@ m.SharedScript.prototype.onUpdate = function(state)
 	this.barterPrices = state.barterPrices;
 
 	this.passabilityMap = state.passabilityMap;
-	this.passabilityMap.cellSize = this.sizeMap / this.passabilityMap.width;
+	this.passabilityMap.cellSize = this.mapSize / this.passabilityMap.width;
 	this.territoryMap = state.territoryMap;
-	this.territoryMap.cellSize = this.sizeMap / this.territoryMap.width;
+	this.territoryMap.cellSize = this.mapSize / this.territoryMap.width;
 	
 	for (var i in this.gameState)
 		this.gameState[i].update(this,state);
 
-	// TODO: merge those two with "ApplyEntitiesDelta" since after all they do the same.
+	// TODO: merge this with "ApplyEntitiesDelta" since after all they do the same.
 	this.updateResourceMaps(this, this.events);
-	this.terrainAnalyzer.updateMapWithEvents(this);
 	
 	Engine.ProfileStop();
 };
@@ -249,14 +229,8 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 	
 	// by order of updating:
 	// we "Destroy" last because we want to be able to switch Metadata first.
-	var CreateEvents = state.events["Create"];
-	var DestroyEvents = state.events["Destroy"];
-	var RenamingEvents = state.events["EntityRenamed"];
-	var TrainingEvents = state.events["TrainingFinished"];
-	var ConstructionEvents = state.events["ConstructionFinished"];
-	var MetadataEvents = state.events["AIMetadata"];
-	var ownershipChangeEvents = state.events["OwnershipChanged"];
 
+	var CreateEvents = state.events.Create;
 	for (let i = 0; i < CreateEvents.length; ++i)
 	{
 		let evt = CreateEvents[i];
@@ -272,7 +246,7 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 			entCol.updateEnt(entity);
 	}
 
-	for (let evt of RenamingEvents)
+	for (let evt of state.events.EntityRenamed)
 	{	// Switch the metadata: TODO entityCollections are updated only because of the owner change. Should be done properly
 		for (let p in this._players)
 		{
@@ -281,14 +255,14 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 		}
 	}
 
-	for (let evt of TrainingEvents)
+	for (let evt of state.events.TrainingFinished)
 	{	// Apply metadata stored in training queues
-		for each (let entId in evt.entities)
+		for (let entId of evt.entities)
 			for (let key in evt.metadata)
-				this.setMetadata(evt.owner, this._entities.get(entId), key, evt.metadata[key])
+				this.setMetadata(evt.owner, this._entities.get(entId), key, evt.metadata[key]);
 	}
 
-	for (let evt of ConstructionEvents)
+	for (let evt of state.events.ConstructionFinished)
 	{	// we'll move metadata.
 		if (!this._entities.has(evt.entity))
 			continue;
@@ -296,22 +270,23 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 		let newEnt = this._entities.get(evt.newentity);
 		if (this._entityMetadata[ent.owner()] && this._entityMetadata[ent.owner()][evt.entity] !== undefined)
 			for (let key in this._entityMetadata[ent.owner()][evt.entity])
-				this.setMetadata(ent.owner(), newEnt, key, this._entityMetadata[ent.owner()][evt.entity][key])
+				this.setMetadata(ent.owner(), newEnt, key, this._entityMetadata[ent.owner()][evt.entity][key]);
 		foundationFinished[evt.entity] = true;
 	}
 
-	for (let evt of MetadataEvents)
+	for (let evt of state.events.AIMetadata)
 	{
 		if (!this._entities.has(evt.id))
 			continue;	// might happen in some rare cases of foundations getting destroyed, perhaps.
 						// Apply metadata (here for buildings for example)
 		for (let key in evt.metadata)
-			this.setMetadata(evt.owner, this._entities.get(evt.id), key, evt.metadata[key])
+			this.setMetadata(evt.owner, this._entities.get(evt.id), key, evt.metadata[key]);
 	}
 	
-	for (var i = 0; i < DestroyEvents.length; ++i)
+	var DestroyEvents = state.events.Destroy;
+	for (let i = 0; i < DestroyEvents.length; ++i)
 	{
-		var evt = DestroyEvents[i];
+		let evt = DestroyEvents[i];
 		// A small warning: javascript "delete" does not actually delete, it only removes the reference in this object.
 		// the "deleted" object remains in memory, and any older reference to it will still reference it as if it were not "deleted".
 		// Worse, they might prevent it from being garbage collected, thus making it stay alive and consuming ram needlessly.
@@ -320,13 +295,13 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 			continue;// probably should remove the event.
 
 		if (foundationFinished[evt.entity])
-			evt["SuccessfulFoundation"] = true;
+			evt.SuccessfulFoundation = true;
 		
 		// The entity was destroyed but its data may still be useful, so
 		// remember the entity and this AI's metadata concerning it
 		evt.metadata = {};
 		evt.entityObj = this._entities.get(evt.entity);
-		for (var j in this._players)
+		for (let j in this._players)
 			evt.metadata[this._players[j]] = this._entityMetadata[this._players[j]][evt.entity];
 		
 		let entity = this._entities.get(evt.entity);
@@ -335,7 +310,7 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 		this.entities.removeEnt(entity);
 		
 		this._entities.delete(evt.entity);
-		for (var j in this._players)
+		for (let j in this._players)
 			delete this._entityMetadata[this._players[j]][evt.entity];
 	}
 
@@ -352,13 +327,13 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 
 	// apply per-entity aura-related changes.
 	// this supersedes tech-related changes.
-	for (var id in state.changedEntityTemplateInfo)
+	for (let id in state.changedEntityTemplateInfo)
 	{
 		if (!this._entities.has(+id))
 			continue;	// dead, presumably.
-		var changes = state.changedEntityTemplateInfo[id];
+		let changes = state.changedEntityTemplateInfo[id];
 		let entity = this._entities.get(+id);
-		for each (var change in changes)
+		for (let change of changes)
 			entity._auraTemplateModif.set(change.variable, change.value);
 	}
 	Engine.ProfileStop();
@@ -368,15 +343,15 @@ m.SharedScript.prototype.ApplyTemplatesDelta = function(state)
 {
 	Engine.ProfileStart("Shared ApplyTemplatesDelta");
 
-	for (var player in state.changedTemplateInfo)
+	for (let player in state.changedTemplateInfo)
 	{
-		var playerDiff = state.changedTemplateInfo[player];
-		for (var template in playerDiff)
+		let playerDiff = state.changedTemplateInfo[player];
+		for (let template in playerDiff)
 		{
-			var changes = playerDiff[template];
+			let changes = playerDiff[template];
 			if (!this._techModifications[player][template])
 				this._techModifications[player][template] = new Map();
-			for each (var change in changes)
+			for (let change of changes)
 				this._techModifications[player][template].set(change.variable, change.value);
 		}
 	}
@@ -387,7 +362,7 @@ m.SharedScript.prototype.registerUpdatingEntityCollection = function(entCollecti
 {
 	entCollection.setUID(this._entityCollectionsUID);
 	this._entityCollections.set(this._entityCollectionsUID, entCollection);
-	for each (var prop in entCollection.dynamicProperties())
+	for (let prop of entCollection.dynamicProperties())
 	{
 		if (!this._entityCollectionsByDynProp[prop])
 			this._entityCollectionsByDynProp[prop] = new Map();
@@ -403,7 +378,7 @@ m.SharedScript.prototype.removeUpdatingEntityCollection = function(entCollection
 	if (this._entityCollections.has(uid))
 		this._entityCollections.delete(uid);
 
-	for each (let prop in entCollection.dynamicProperties())
+	for (let prop of entCollection.dynamicProperties())
 		if (this._entityCollectionsByDynProp[prop].has(uid))
 			this._entityCollectionsByDynProp[prop].delete(uid);
 };
@@ -415,7 +390,7 @@ m.SharedScript.prototype.updateEntityCollections = function(property, ent)
 
 	for (let entCol of this._entityCollectionsByDynProp[property].values())
 		entCol.updateEnt(ent);
-}
+};
 
 m.SharedScript.prototype.setMetadata = function(player, ent, key, value)
 {
@@ -427,6 +402,7 @@ m.SharedScript.prototype.setMetadata = function(player, ent, key, value)
 	this.updateEntityCollections('metadata', ent);
 	this.updateEntityCollections('metadata.' + key, ent);
 };
+
 m.SharedScript.prototype.getMetadata = function(player, ent, key)
 {
 	var metadata = this._entityMetadata[player][ent.id()];
@@ -435,6 +411,7 @@ m.SharedScript.prototype.getMetadata = function(player, ent, key)
 		return undefined;
 	return metadata[key];
 };
+
 m.SharedScript.prototype.deleteMetadata = function(player, ent, key)
 {
 	var metadata = this._entityMetadata[player][ent.id()];
@@ -443,6 +420,8 @@ m.SharedScript.prototype.deleteMetadata = function(player, ent, key)
 		return true;
 	metadata[key] = undefined;
 	delete metadata[key];
+	this.updateEntityCollections('metadata', ent);    
+	this.updateEntityCollections('metadata.' + key, ent);
 	return true;
 };
 
