@@ -96,9 +96,9 @@ m.PetraBot.prototype.OnUpdate = function(sharedScript)
 	if (this.gameFinished)
 		return;
 
-	for (var i in this.events)
+	for (let i in this.events)
 	{
-		if (i == "AIMetadata" || i == "RangeUpdate")   // not used inside petra
+		if (i == "AIMetadata")   // not used inside petra
 			continue;
 		if(this.savedEvents[i] !== undefined)
 			this.savedEvents[i] = this.savedEvents[i].concat(this.events[i]);
@@ -108,7 +108,7 @@ m.PetraBot.prototype.OnUpdate = function(sharedScript)
 
 	// Run the update every n turns, offset depending on player ID to balance the load
 	this.elapsedTime = this.gameState.getTimeElapsed() / 1000;
-	if ((this.turn + this.player) % 8 == 5)
+	if (!this.playedTurn || (this.turn + this.player) % 8 == 5)
 	{		
 		Engine.ProfileStart("PetraBot bot (player " + this.player +")");
 
@@ -124,7 +124,7 @@ m.PetraBot.prototype.OnUpdate = function(sharedScript)
 
 		this.queueManager.update(this.gameState);
 		
-		for (var i in this.savedEvents)
+		for (let i in this.savedEvents)
 			this.savedEvents[i] = [];
 
 		Engine.ProfileStop();
