@@ -44,7 +44,7 @@ m.Army = function(gameState, ownEntities, foeEntities)
 	this.recalculatePosition(gameState, true);
 
 	return true;
-}
+};
 
 // if not forced, will only recalculate if on a different turn.
 m.Army.prototype.recalculatePosition = function(gameState, force)
@@ -82,16 +82,31 @@ m.Army.prototype.recalculateStrengths = function (gameState)
 	
 	// todo: deal with specifics.
 
-	for (var id of this.foeEntities)
+	for (let id of this.foeEntities)
 		this.evaluateStrength(gameState.getEntityById(id));
-	for (var id of this.ownEntities)
+	for (let id of this.ownEntities)
 		this.evaluateStrength(gameState.getEntityById(id), true);
 };
 
 // adds or remove the strength of the entity either to the enemy or to our units.
 m.Army.prototype.evaluateStrength = function (ent, isOwn, remove)
 {
-	var entStrength = m.getMaxStrength(ent);
+	var entStrength;
+	if (ent.hasClass("Structure"))
+	{
+		if (ent.owner() !== PlayerID)
+			entStrength = (ent.getDefaultArrow() ? 6*ent.getDefaultArrow() : 4);
+		else	// small strength used only when we try to recover capture points
+			entStrength = 2;
+	}
+	else
+		entStrength = m.getMaxStrength(ent);
+
+	// TODO adapt the getMaxStrength function for animals.
+	// For the time being, just increase it for elephants as the returned value is too small.
+	if (ent.hasClass("Animal") && ent.hasClass("Elephant"))
+		entStrength *= 3;
+
 	if (remove)
 		entStrength *= -1;
 
@@ -99,19 +114,18 @@ m.Army.prototype.evaluateStrength = function (ent, isOwn, remove)
 		this.ownStrength += entStrength;
 	else
 		this.foeStrength += entStrength;
-	
-	// todo: deal with specifics.
 };
 
 // add an entity to the enemy army
 // Will return true if the entity was added and false otherwise.
 // won't recalculate our position but will dirty it.
+// force is true at army creation or when merging armies, so in this case we should add it even if far
 m.Army.prototype.addFoe = function (gameState, enemyId, force)
 {
 	if (this.foeEntities.indexOf(enemyId) !== -1)
 		return false;
-	var ent = gameState.getEntityById(enemyId);
-	if (ent === undefined || ent.position() === undefined)
+	let ent = gameState.getEntityById(enemyId);
+	if (!ent || !ent.position())
 		return false;
 	
 	// check distance
@@ -131,24 +145,24 @@ m.Army.prototype.addFoe = function (gameState, enemyId, force)
 // TODO: when there is a technology update, we should probably recompute the strengths, or weird stuffs will happen.
 m.Army.prototype.removeFoe = function (gameState, enemyId, enemyEntity)
 {
-	var idx = this.foeEntities.indexOf(enemyId);
+	let idx = this.foeEntities.indexOf(enemyId);
 	if (idx === -1)
 		return false;
-	var ent = enemyEntity === undefined ? gameState.getEntityById(enemyId) : enemyEntity;
-	if (ent === undefined)
-	{
-		warn("Trying to remove a non-existing enemy entity, crashing for stacktrace");
-		xgzrg();
-	}	
+
 	this.foeEntities.splice(idx, 1);
-	this.evaluateStrength(ent, false, true);
-	ent.setMetadata(PlayerID, "PartOfArmy", undefined);
 	
 	this.assignedAgainst[enemyId] = undefined;
-	for (var to in this.assignedTo)
+	for (let to in this.assignedTo)
 		if (this.assignedTo[to] == enemyId)
 			this.assignedTo[to] = undefined;
-	
+
+	let ent = (enemyEntity ? enemyEntity : gameState.getEntityById(enemyId));
+	if (ent)    // TODO recompute strength when no entities (could happen if capture+destroy)
+	{
+		this.evaluateStrength(ent, false, true);
+		ent.setMetadata(PlayerID, "PartOfArmy", undefined);
+	}
+
 	return true;
 };
 
@@ -159,9 +173,7 @@ m.Army.prototype.addOwn = function (gameState, id, force)
 	if (this.ownEntities.indexOf(id) !== -1)
 		return false;
 	var ent = gameState.getEntityById(id);
-	if (ent === undefined)
-		return false;
-	if(!ent.position() && !force)
+	if (!ent || (!ent.position() && !force))
 		return false;
 
 	this.ownEntities.push(id);
@@ -186,21 +198,8 @@ m.Army.prototype.removeOwn = function (gameState, id, Entity)
 	var idx = this.ownEntities.indexOf(id);
 	if (idx === -1)
 		return false;
-	var ent = Entity === undefined ? gameState.getEntityById(id) : Entity;
-	if (ent === undefined)
-	{
-		warn( id);
-		warn("Trying to remove a non-existing entity, crashing for stacktrace");
-		xgzrg();
-	}
 
 	this.ownEntities.splice(idx, 1);
-	this.evaluateStrength(ent, true, true);
-	ent.setMetadata(PlayerID, "PartOfArmy", undefined);
-	if (ent.getMetadata(PlayerID, "plan") === -2)
-		ent.setMetadata(PlayerID, "plan", -1);
-	else
-		ent.setMetadata(PlayerID, "plan", undefined);
 
 	if (this.assignedTo[id] !== 0)
 	{
@@ -210,31 +209,52 @@ m.Army.prototype.removeOwn = function (gameState, id, Entity)
 	}
 	this.assignedTo[id] = undefined;
 
-	var formerSubrole = ent.getMetadata(PlayerID, "formerSubrole");
-	if (formerSubrole !== undefined)
-		ent.setMetadata(PlayerID, "subrole", formerSubrole);
-	else
-		ent.setMetadata(PlayerID, "subrole", undefined);
-	ent.setMetadata(PlayerID, "formerSubrole", undefined);
-
-// TODO be sure that all units in the transport need the cancelation
-/*	if (!ent.position())	// this unit must still be in a transport plan ... try to cancel it
+	let ent = (Entity ? Entity : gameState.getEntityById(id));
+	if (ent)    // TODO recompute strength when no entities (could happen if capture+destroy)
 	{
-		var planID = ent.getMetadata(PlayerID, "transport");
-		// no plans must mean that the unit was in a ship which was destroyed, so do nothing
-		if (planID)
+		this.evaluateStrength(ent, true, true);
+		ent.setMetadata(PlayerID, "PartOfArmy", undefined);
+		if (ent.getMetadata(PlayerID, "plan") === -2)
+			ent.setMetadata(PlayerID, "plan", -1);
+		else
+			ent.setMetadata(PlayerID, "plan", undefined);
+
+		var formerSubrole = ent.getMetadata(PlayerID, "formerSubrole");
+		if (formerSubrole !== undefined)
+			ent.setMetadata(PlayerID, "subrole", formerSubrole);
+		else
+			ent.setMetadata(PlayerID, "subrole", undefined);
+		ent.setMetadata(PlayerID, "formerSubrole", undefined);
+
+		// TODO be sure that all units in the transport need the cancelation
+/*		if (!ent.position())	// this unit must still be in a transport plan ... try to cancel it
 		{
-			if (gameState.ai.Config.debug > 0)
-				warn("ent from army still in transport plan: plan " + planID + " canceled");
-			var plan = gameState.ai.HQ.navalManager.getPlan(planID);
-			if (plan && !plan.canceled)
-				plan.cancelTransport(gameState);
-		}
-	} */
+			var planID = ent.getMetadata(PlayerID, "transport");
+			// no plans must mean that the unit was in a ship which was destroyed, so do nothing
+			if (planID)
+			{
+				if (gameState.ai.Config.debug > 0)
+					warn("ent from army still in transport plan: plan " + planID + " canceled");
+				var plan = gameState.ai.HQ.navalManager.getPlan(planID);
+				if (plan && !plan.canceled)
+					plan.cancelTransport(gameState);
+			}
+		} */
+	}
 
 	return true;
 };
 
+// Special army set to capture a gaia building.
+// It must only contain one foe (the building to capture) and never be merged
+m.Army.prototype.isCapturing = function (gameState)
+{
+	if (this.foeEntities.length != 1)
+		return false;
+	let ent = gameState.getEntityById(this.foeEntities[0]);
+	return (ent && ent.hasClass("Structure"));
+};    
+
 // this one is "undefined entity" proof because it's called at odd times.
 // Orders a unit to attack an enemy.
 // overridden by specific army classes.
@@ -246,9 +266,9 @@ m.Army.prototype.assignUnit = function (gameState, entID)
 // assumes we already cleared dead units.
 m.Army.prototype.clear = function (gameState)
 {
-	while(this.foeEntities.length > 0)
+	while (this.foeEntities.length > 0)
 		this.removeFoe(gameState,this.foeEntities[0]);
-	while(this.ownEntities.length > 0)
+	while (this.ownEntities.length > 0)
 		this.removeOwn(gameState,this.ownEntities[0]);
 
 	this.assignedAgainst = {};
@@ -264,20 +284,20 @@ m.Army.prototype.clear = function (gameState)
 m.Army.prototype.merge = function (gameState, otherArmy)
 {
 	// copy over all parameters.
-	for (var i in otherArmy.assignedAgainst)
+	for (let i in otherArmy.assignedAgainst)
 	{
 		if (this.assignedAgainst[i] === undefined)
 			this.assignedAgainst[i] = otherArmy.assignedAgainst[i];
 		else
 			this.assignedAgainst[i] = this.assignedAgainst[i].concat(otherArmy.assignedAgainst[i]);
 	}
-	for (var i in otherArmy.assignedTo)
+	for (let i in otherArmy.assignedTo)
 		this.assignedTo[i] = otherArmy.assignedTo[i];
 	
-	for (var id of otherArmy.foeEntities)
-		this.addFoe(gameState, id);
+	for (let id of otherArmy.foeEntities)
+		this.addFoe(gameState, id, true);
 	// TODO: reassign those ?
-	for (var id of otherArmy.ownEntities)
+	for (let id of otherArmy.ownEntities)
 		this.addOwn(gameState, id, true);
 
 	this.recalculatePosition(gameState, true);
@@ -289,72 +309,83 @@ m.Army.prototype.merge = function (gameState, otherArmy)
 // TODO: when there is a technology update, we should probably recompute the strengths, or weird stuffs might happen.
 m.Army.prototype.checkEvents = function (gameState, events)
 {
-	var renameEvents = events["EntityRenamed"];   // take care of promoted and packed units
-	var destroyEvents = events["Destroy"];
-	var convEvents = events["OwnershipChanged"];
-	var garriEvents = events["Garrison"];
-
 	// Warning the metadata is already cloned in shared.js. Futhermore, changes should be done before destroyEvents
 	// otherwise it would remove the old entity from this army list
 	// TODO we should may-be reevaluate the strength
-	for (var msg of renameEvents)
+	for (let evt of events.EntityRenamed)	// take care of promoted and packed units
 	{
-		if (this.foeEntities.indexOf(msg.entity) !== -1)
+		if (this.foeEntities.indexOf(evt.entity) !== -1)
 		{
-			var idx = this.foeEntities.indexOf(msg.entity);
-			this.foeEntities[idx] = msg.newentity;
-			this.assignedAgainst[msg.newentity] = this.assignedAgainst[msg.entity];
-			this.assignedAgainst[msg.entity] = undefined;
-			for (var to in this.assignedTo)
-				if (this.assignedTo[to] == msg.entity)
-					this.assignedTo[to] = msg.newentity;
+			let ent = gameState.getEntityById(evt.newentity);
+			if (ent && ent.templateName().indexOf("resource|") !== -1)  // corpse of animal killed
+				continue;
+			let idx = this.foeEntities.indexOf(evt.entity);
+			this.foeEntities[idx] = evt.newentity;
+			this.assignedAgainst[evt.newentity] = this.assignedAgainst[evt.entity];
+			this.assignedAgainst[evt.entity] = undefined;
+			for (let to in this.assignedTo)
+				if (this.assignedTo[to] == evt.entity)
+					this.assignedTo[to] = evt.newentity;
 		}
-		else if (this.ownEntities.indexOf(msg.entity) !== -1)
+		else if (this.ownEntities.indexOf(evt.entity) !== -1)
 		{
-			var idx = this.ownEntities.indexOf(msg.entity);
-			this.ownEntities[idx] = msg.newentity;
-			this.assignedTo[msg.newentity] = this.assignedTo[msg.entity];
-			this.assignedTo[msg.entity] = undefined;
-			for (var against in this.assignedAgainst)
+			let idx = this.ownEntities.indexOf(evt.entity);
+			this.ownEntities[idx] = evt.newentity;
+			this.assignedTo[evt.newentity] = this.assignedTo[evt.entity];
+			this.assignedTo[evt.entity] = undefined;
+			for (let against in this.assignedAgainst)
 			{
 				if (!this.assignedAgainst[against])
 					continue;
-				if (this.assignedAgainst[against].indexOf(msg.entity) !== -1)
-					this.assignedAgainst[against][this.assignedAgainst[against].indexOf(msg.entity)] = msg.newentity;
+				if (this.assignedAgainst[against].indexOf(evt.entity) !== -1)
+					this.assignedAgainst[against][this.assignedAgainst[against].indexOf(evt.entity)] = evt.newentity;
 			}
 		}
 	}
 
-	for (var msg of destroyEvents)
+	for (let evt of events.Garrison)
+		this.removeFoe(gameState, evt.entity);
+
+	for (let evt of events.OwnershipChanged)	// captured
 	{
-		if (msg.entityObj === undefined)
-			continue;
-		if (msg.entityObj._entity.owner === PlayerID)
-			this.removeOwn(gameState, msg.entity, msg.entityObj);
-		else
-			this.removeFoe(gameState, msg.entity, msg.entityObj);
+		if (!gameState.isPlayerEnemy(evt.to))
+			this.removeFoe(gameState, evt.entity);
+		else if (evt.from === PlayerID)
+			this.removeOwn(gameState, evt.entity);
 	}
 
-	for (var msg of garriEvents)
-		this.removeFoe(gameState, msg.entity);
-
-	for (var msg of convEvents)
+	for (let evt of events.Destroy)
 	{
-		if (msg.to === PlayerID)
-		{
-			// we have converted an enemy, let's assign it as a defender
-			if (this.removeFoe(gameState, msg.entity))
-				this.addOwn(gameState, msg.entity);
-		}
-		else if (msg.from === PlayerID)
-			this.removeOwn(gameState, msg.entity);	// TODO: add allies
+		let entityObj = evt.entityObj || undefined;
+		// we may have capture+destroy, so do not trust owner and check all possibilities
+		this.removeOwn(gameState, evt.entity, entityObj);
+		this.removeFoe(gameState, evt.entity, entityObj);
 	}
 };
 
 // assumes cleaned army.
-// this only checks for breakaways.
 m.Army.prototype.onUpdate = function (gameState)
 {
+	if (this.isCapturing(gameState))
+	{
+		let done = true;
+		let capture = gameState.getEntityById(this.foeEntities[0]).capturePoints();
+		if (capture !== undefined)
+		{
+			for (let j = 0; j < capture.length; ++j)
+			{
+				if (gameState.isPlayerEnemy(j) && capture[j] > 0)
+				{
+					done = false;
+					break;
+				}
+			}
+		}
+		if (done)
+			this.removeFoe(gameState, this.foeEntities[0]);
+		return [];
+	}
+
 	var breakaways = [];
 	// TODO: assign unassigned defenders, cleanup of a few things.
 	// perhaps occasional strength recomputation
