@@ -7,7 +7,7 @@ m.DefenseManager = function(Config)
 	this.Config = Config;
 	this.targetList = [];
 	this.armyMergeSize = this.Config.Defense.armyMergeSize;
-}
+};
 
 m.DefenseManager.prototype.update = function(gameState, events)
 {
@@ -17,7 +17,7 @@ m.DefenseManager.prototype.update = function(gameState, events)
 
 	this.checkEvents(gameState, events);
 
-	this.checkEnemyArmies(gameState, events);
+	this.checkEnemyArmies(gameState);
 	this.checkEnemyUnits(gameState);
 	this.assignDefenders(gameState);
 
@@ -28,7 +28,7 @@ m.DefenseManager.prototype.makeIntoArmy = function(gameState, entityID)
 {
 	// Try to add it to an existing army.
 	for (let army of this.armies)
-		if (army.addFoe(gameState, entityID))
+		if (!army.isCapturing(gameState) && army.addFoe(gameState, entityID))
 			return;	// over
 
 	// Create a new army for it.
@@ -41,7 +41,7 @@ m.DefenseManager.prototype.getArmy = function(partOfArmy)
 	// Find the army corresponding to this ID partOfArmy
 	for (let army of this.armies)
 		if (army.ID === partOfArmy)
-		    return army;
+			return army;
 
 	return undefined;
 };
@@ -52,16 +52,17 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 	if (!entity.position())
 		return false;
 
-	if (this.territoryMap.getOwner(entity.position()) === entity.owner())
+	let territoryOwner = this.territoryMap.getOwner(entity.position());
+	if (territoryOwner !== 0 && !gameState.isPlayerAlly(territoryOwner))
 		return false;
-
-	// check if the entity is trying to build a new base near our buildings, and if yes, add this base in our target list
+	// check if the entity is trying to build a new base near our buildings,
+	// and if yes, add this base in our target list
 	if (entity.unitAIState() && entity.unitAIState() == "INDIVIDUAL.REPAIR.REPAIRING")
 	{
-		var targetId = entity.unitAIOrderData()[0]["target"];
+		let targetId = entity.unitAIOrderData()[0].target;
 		if (this.targetList.indexOf(targetId) !== -1)
 			return true;
-		var target = gameState.getEntityById(targetId);
+		let target = gameState.getEntityById(targetId);
 		if (target && this.territoryMap.getOwner(entity.position()) === PlayerID)
 		{
 			this.targetList.push(targetId);
@@ -69,7 +70,7 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 		}
 		else if (target && target.hasClass("CivCentre"))
 		{
-			var myBuildings = gameState.getOwnStructures();
+			let myBuildings = gameState.getOwnStructures();
 			for (let building of myBuildings.values())
 			{
 				if (API3.SquareVectorDistance(building.position(), entity.position()) > 30000)
@@ -82,13 +83,17 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 
 	if (entity.attackTypes() === undefined || entity.hasClass("Support"))
 		return false;
-
-	for (var i = 0; i < this.targetList.length; ++i)
+	let dist2Min = 6000;
+	// TODO the 30 is to take roughly into account the structure size in following checks. Can be improved
+	if (entity.attackTypes().indexOf("Ranged") !== -1)
+		dist2Min = (entity.attackRange("Ranged").max + 30) * (entity.attackRange("Ranged").max + 30);
+    
+	for (let i = 0; i < this.targetList.length; ++i)
 	{
-		var target = gameState.getEntityById(this.targetList[i]);
+		let target = gameState.getEntityById(this.targetList[i]);
 		if (!target || !target.position())   // the enemy base is either destroyed or built
 			this.targetList.splice(i--, 1);
-		else if (API3.SquareVectorDistance(target.position(), entity.position()) < 6000)
+		else if (API3.SquareVectorDistance(target.position(), entity.position()) < dist2Min)
 			return true;
 	}
 
@@ -101,17 +106,17 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 				continue;
 			if (this.Config.personality.cooperative < 0.6 && cc.foundationProgress() !== undefined)
 				continue;
-			if (API3.SquareVectorDistance(cc.position(), entity.position()) < 6000)
+			if (API3.SquareVectorDistance(cc.position(), entity.position()) < dist2Min)
 				return true;
 		}
 	}
 
-	var myBuildings = gameState.getOwnStructures();
+	let myBuildings = gameState.getOwnStructures();
 	for (let building of myBuildings.values())
 	{
-		if (building.foundationProgress() == 0)
+		if (building.foundationProgress() === 0)
 			continue;
-		if (API3.SquareVectorDistance(building.position(), entity.position()) < 6000)
+		if (API3.SquareVectorDistance(building.position(), entity.position()) < dist2Min)
 			return true;
 	}
 
@@ -123,86 +128,133 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 {
 	var nbPlayers = gameState.sharedScript.playersData.length;
 	var i = gameState.ai.playedTurn % nbPlayers;
-	if (i === PlayerID || gameState.isPlayerAlly(i))
+
+	if (i === PlayerID)
+	{
+		if (!this.armies.length)
+		{
+			// check if we can recover capture points from any of our notdecaying structures		
+			for (let ent of gameState.getOwnStructures().values())
+			{
+				if (ent.decaying())
+					continue;
+				let capture = ent.capturePoints();
+				if (capture === undefined)
+					continue;
+				let lost = 0;
+				for (let j = 0; j < capture.length; ++j)
+					if (gameState.isPlayerEnemy(j))
+						lost += capture[j];
+				if (lost < Math.ceil(0.25 * capture[i]))
+					continue;
+				this.makeIntoArmy(gameState, ent.id());
+				break;
+			}
+		}
+		return;
+	}
+	else if (!gameState.isPlayerEnemy(i))
 		return;
 
-	var self = this;
-
 	// loop through enemy units
-	gameState.getEnemyUnits(i).forEach( function (ent) {
-		// first check: is this unit already part of an army.
+	for (let ent of gameState.getEnemyUnits(i).values())
+	{
 		if (ent.getMetadata(PlayerID, "PartOfArmy") !== undefined)
-			return;
+			continue;
 
-		// TODO do not bother with animals for the time being
+		// keep animals attacking us or our allies
 		if (ent.hasClass("Animal"))
-			return;
+		{
+			if (!ent.unitAIState() || ent.unitAIState().split(".")[1] !== "COMBAT")
+				continue;
+			let orders = ent.unitAIOrderData();
+			if (!orders || !orders.length || !orders[0].target)
+				continue;
+			let target = gameState.getEntityById(orders[0].target);
+			if (!target || !gameState.isPlayerAlly(target.owner()))
+				continue;
+		}
 
 		// TODO what to do for ships ?
 		if (ent.hasClass("Ship") || ent.hasClass("Trader"))
-			return;
+			continue;
 
 		// check if unit is dangerous "a priori"
-		if (self.isDangerous(gameState, ent))
-			self.makeIntoArmy(gameState, ent.id());
-	});
+		if (this.isDangerous(gameState, ent))
+			this.makeIntoArmy(gameState, ent.id());
+	}
+
+	if (i !== 0 || this.armies.length > 1 || gameState.ai.HQ.numActiveBase() === 0)
+		return;
+	// look for possible gaia buildings inside our territory (may happen when enemy resign or after structure decay)
+	// and attack it only if useful (and capturable) or dangereous
+	for (let ent of gameState.getEnemyStructures(i).values())
+	{
+		if (!ent.position() || ent.getMetadata(PlayerID, "PartOfArmy") !== undefined)
+			continue;
+		if (!ent.capturePoints() && !ent.hasDefensiveFire())
+			continue;
+		let owner = this.territoryMap.getOwner(ent.position());
+		if (owner === PlayerID)
+			this.makeIntoArmy(gameState, ent.id());
+	}
 };
 
-m.DefenseManager.prototype.checkEnemyArmies = function(gameState, events)
+m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 {
-	for (var o = 0; o < this.armies.length; ++o)
+	for (let i = 0; i < this.armies.length; ++i)
 	{
-		var army = this.armies[o];
-		army.checkEvents(gameState, events);	// must be called every turn for all armies
-
+		let army = this.armies[i];
 		// this returns a list of IDs: the units that broke away from the army for being too far.
-		var breakaways = army.update(gameState);
+		let breakaways = army.update(gameState);
+		for (let breaker of breakaways)
+			this.makeIntoArmy(gameState, breaker);		// assume dangerosity
 
-		for (let breakers of breakaways)
-			this.makeIntoArmy(gameState, breakers);		// assume dangerosity
-
-		if (army.getState(gameState) === 0)
+		if (army.getState() === 0)
 		{
 			army.clear(gameState);
-			this.armies.splice(o--,1);
+			this.armies.splice(i--,1);
 			continue;
 		}
 	}
-	// Check if we can't merge it with another.
-	for (var o = 0; o < this.armies.length; ++o)
+	// Check if we can't merge it with another
+	for (let i = 0; i < this.armies.length - 1; ++i)
 	{
-		var army = this.armies[o];
-		for (var p = o+1; p < this.armies.length; ++p)
+		let army = this.armies[i];
+		if (army.isCapturing(gameState))
+			continue;
+		for (let j = i+1; j < this.armies.length; ++j)
 		{
-			var otherArmy = this.armies[p];
-			if (API3.SquareVectorDistance(army.foePosition, otherArmy.foePosition) < this.armyMergeSize)
-			{
-				// no need to clear here.
-				army.merge(gameState, otherArmy);
-				this.armies.splice(p--,1);
-			}
+			let otherArmy = this.armies[j];
+			if (otherArmy.isCapturing(gameState) ||
+				API3.SquareVectorDistance(army.foePosition, otherArmy.foePosition) > this.armyMergeSize)
+				continue;
+			// no need to clear here.
+			army.merge(gameState, otherArmy);
+			this.armies.splice(j--,1);
 		}
 	}
 
 	if (gameState.ai.playedTurn % 5 !== 0)
 		return;
 	// Check if any army is no more dangerous (possibly because it has defeated us and destroyed our base)
-	for (var o = 0; o < this.armies.length; ++o)
+	for (let i = 0; i < this.armies.length; ++i)
 	{
-		var army = this.armies[o];
+		let army = this.armies[i];
 		army.recalculatePosition(gameState);
-		var owner = this.territoryMap.getOwner(army.foePosition);
-		if (gameState.isPlayerAlly(owner))
+		let owner = this.territoryMap.getOwner(army.foePosition);
+		if (!gameState.isPlayerEnemy(owner))
 			continue;
 		else if (owner !== 0)   // enemy army back in its territory
 		{
 			army.clear(gameState);
-			this.armies.splice(o--,1);
+			this.armies.splice(i--,1);
 			continue;
 		}
 
-		// army in neutral territory // TODO check smaller distance with all our buildings instead of only ccs with big distance
-		var stillDangerous = false;
+		// army in neutral territory
+		// TODO check smaller distance with all our buildings instead of only ccs with big distance
+		let stillDangerous = false;
 		let bases = gameState.updatingGlobalCollection("allCCs", API3.Filters.byClass("CivCentre"));
 	 	for (let base of bases.values())
 		{
@@ -221,7 +273,7 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState, events)
 			continue;
 
 		army.clear(gameState);
-		this.armies.splice(o--,1);
+		this.armies.splice(i--,1);
 	}
 };
 
@@ -271,111 +323,179 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 		potentialDefenders.push(ent.id());
 	});
 	
-	for (var a = 0; a < armiesNeeding.length; ++a)
-		armiesNeeding[a]["army"].recalculatePosition(gameState);
+	for (let a = 0; a < armiesNeeding.length; ++a)
+		armiesNeeding[a].army.recalculatePosition(gameState);
 
-	for (var i = 0; i < potentialDefenders.length; ++i)
+	for (let i = 0; i < potentialDefenders.length; ++i)
 	{
-		var ent = gameState.getEntityById(potentialDefenders[i]);
+		let ent = gameState.getEntityById(potentialDefenders[i]);
 		if (!ent.position())
 			continue;
-		var aMin = undefined;
-		var distMin = undefined;
-		for (var a = 0; a < armiesNeeding.length; ++a)
+		let aMin;
+		let distMin;
+		for (let a = 0; a < armiesNeeding.length; ++a)
 		{
-			var dist = API3.SquareVectorDistance(ent.position(), armiesNeeding[a]["army"].foePosition);
+			let dist = API3.SquareVectorDistance(ent.position(), armiesNeeding[a].army.foePosition);
 			if (aMin !== undefined && dist > distMin)
 				continue;
 			aMin = a;
 			distMin = dist;
 		}
-		if (aMin === undefined)
-		{
-			for (var a = 0; a < armiesNeeding.length; ++a)
-				API3.warn(" defense/armiesNeeding " + uneval(armiesNeeding[a]["need"]));
-		}
 
-		var str = m.getMaxStrength(ent);
-		armiesNeeding[aMin]["need"] -= str;
-		armiesNeeding[aMin]["army"].addOwn(gameState, potentialDefenders[i]);
-		armiesNeeding[aMin]["army"].assignUnit(gameState, potentialDefenders[i]);
+		// if outside our territory (helping an ally or attacking a cc foundation), keep some troops in backup
+		if (i < 12 && this.territoryMap.getOwner(armiesNeeding[aMin].army.foePosition) !== PlayerID)
+			continue;
 
-		if (armiesNeeding[aMin]["need"] <= 0)
+		armiesNeeding[aMin].need -= m.getMaxStrength(ent);
+		armiesNeeding[aMin].army.addOwn(gameState, potentialDefenders[i]);
+		armiesNeeding[aMin].army.assignUnit(gameState, potentialDefenders[i]);
+
+		if (armiesNeeding[aMin].need <= 0)
 			armiesNeeding.splice(aMin, 1);
-		if (armiesNeeding.length === 0)
+		if (!armiesNeeding.length)
 			break;
 	}
 
-	if (armiesNeeding.length === 0)
+	if (!armiesNeeding.length)
 		return;
 	// If shortage of defenders, produce infantry garrisoned in nearest civil centre
 	var armiesPos = [];
-	for (var a = 0; a < armiesNeeding.length; ++a)
-		armiesPos.push(armiesNeeding[a]["army"].foePosition);
+	for (let a = 0; a < armiesNeeding.length; ++a)
+		armiesPos.push(armiesNeeding[a].army.foePosition);
 	gameState.ai.HQ.trainEmergencyUnits(gameState, armiesPos);
 };
 
+m.DefenseManager.prototype.abortArmy = function(gameState, army)
+{
+	army.clear(gameState);
+	for (let i = 0; i < this.armies.length; ++i)
+	{
+		if (this.armies[i].ID !== army.ID)
+			continue;
+		this.armies.splice(i, 1);
+		break;
+	}
+};
+
 // If our defense structures are attacked, garrison soldiers inside when possible
-// and if a support unit is attacked and has less than 45% health, garrison it inside the nearest cc
+// and if a support unit is attacked and has less than 55% health, garrison it inside the nearest healing structure
 // and if a ranged siege unit (not used for defense) is attacked, garrison it in the nearest fortress
 m.DefenseManager.prototype.checkEvents = function(gameState, events)
 {
-	var attackedEvents = events["Attacked"];
-	for (var evt of attackedEvents)
+	// must be called every turn for all armies
+	for (let army of this.armies)
+		army.checkEvents(gameState, events);
+
+	for (let evt of events.Attacked)
 	{
-		var target = gameState.getEntityById(evt.target);
+		let target = gameState.getEntityById(evt.target);
 		if (!target || !gameState.isEntityOwn(target) || !target.position())
 			continue;
+		// If attacked by one of our allies (he must trying to recover capture points), do not react
+		let attacker = gameState.getEntityById(evt.attacker);
+		if (attacker && gameState.isEntityAlly(attacker))
+			continue;
+
 		if (target.hasClass("Ship"))    // TODO integrate ships later   need to be sure it is accessible
 			continue;
 
-		if (target.hasClass("Support") && target.healthLevel() < 0.45 && !target.getMetadata(PlayerID, "transport")
-			&& target.getMetadata(PlayerID, "plan") !== -2 && target.getMetadata(PlayerID, "plan") !== -3)
+		// If inside a started attack plan, let the plan deal with this unit
+		let plan = target.getMetadata(PlayerID, "plan");
+		if (plan !== undefined && plan >= 0)
+		{
+			let attack = gameState.ai.HQ.attackManager.getPlan(plan);
+			if (attack && attack.state !== "unexecuted")
+				continue;
+		}
+
+		// Signal this attacker to our defense manager, except if we are in enemy territory
+		// TODO treat ship attack
+		if (attacker && attacker.position() && attacker.getMetadata(PlayerID, "PartOfArmy") === undefined &&
+			!attacker.hasClass("Structure") && !attacker.hasClass("Ship"))
+		{
+			let territoryOwner = this.territoryMap.getOwner(attacker.position());
+			if (territoryOwner === 0 || gameState.isPlayerAlly(territoryOwner))
+				this.makeIntoArmy(gameState, attacker.id());
+		}
+
+		if (target.getMetadata(PlayerID, "PartOfArmy") !== undefined)
+		{
+			let army = this.getArmy(target.getMetadata(PlayerID, "PartOfArmy"));
+			if (army.isCapturing(gameState))
+			{
+				let abort = false;
+				// if one of the units trying to capture a structure is attacked,
+				// abort the army so that the unit can defend itself
+				if (army.ownEntities.indexOf(target.id()) !== -1)
+					abort = true;
+				else if (army.foeEntities[0] === target.id() && target.owner() === PlayerID)
+				{
+					// else we may be trying to regain some capture point from one of our structure
+					abort = true;
+					let capture = target.capturePoints();
+					for (let j = 0; j < capture.length; ++j)
+					{
+						if (!gameState.isPlayerEnemy(j) || capture[j] === 0)
+							continue;
+						abort = false;
+						break;
+					}
+				}
+				if (abort)
+					this.abortArmy(gameState, army);
+			}
+			continue;
+		}
+
+		// try to garrison any attacked support unit if low healthlevel
+		if (target.hasClass("Support") && target.healthLevel() < 0.55 &&
+			!target.getMetadata(PlayerID, "transport") && plan !== -2 && plan !== -3)
 		{
 			this.garrisonUnitForHealing(gameState, target);
 			continue;
 		}
 
-		if (target.hasClass("Siege") && !target.hasClass("Melee") && !target.getMetadata(PlayerID, "transport")
-			&& target.getMetadata(PlayerID, "plan") !== -2 && target.getMetadata(PlayerID, "plan") !== -3)
+		// try to garrison any attacked range siege unit 
+		if (target.hasClass("Siege") && !target.hasClass("Melee") &&
+			!target.getMetadata(PlayerID, "transport") && plan !== -2 && plan !== -3)
 		{
-			if (target.getMetadata(PlayerID, "plan") !== undefined && target.getMetadata(PlayerID, "plan") !== -1)
-			{
-				var subrole = target.getMetadata(PlayerID, "subrole");
-				if (subrole && (subrole === "completing" || subrole === "walking" || subrole === "attacking")) 
-					continue;
-			}
 			this.garrisonSiegeUnit(gameState, target);
 			continue;
 		}
 
-		var attacker = gameState.getEntityById(evt.attacker);
 		if (!attacker || !attacker.position())
 			continue;
 
 		if (target.isGarrisonHolder() && target.getArrowMultiplier())
-			this.garrisonRangedUnitsInside(gameState, target, attacker);
+			this.garrisonRangedUnitsInside(gameState, target, {"attacker": attacker});
 	}
 };
 
-m.DefenseManager.prototype.garrisonRangedUnitsInside = function(gameState, target, attacker)
+m.DefenseManager.prototype.garrisonRangedUnitsInside = function(gameState, target, data)
 {
-	if (gameState.ai.HQ.garrisonManager.numberOfGarrisonedUnits(target) >= target.garrisonMax())
+	let minGarrison = data.min ? data.min : target.garrisonMax();
+	let typeGarrison = data.type ? data.type : "protection";
+	if (gameState.ai.HQ.garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
 		return;
-	var attackTypes = target.attackTypes();
-	if (!attackTypes || attackTypes.indexOf("Ranged") === -1)
-		return;
-	var dist = API3.SquareVectorDistance(attacker.position(), target.position());
-	var range = target.attackRange("Ranged").max;
-	if (dist >= range*range)
+	if (target.hitpoints() < target.garrisonEjectHealth() * target.maxHitpoints())
 		return;
+	if (data.attacker)
+	{
+		let attackTypes = target.attackTypes();
+		if (!attackTypes || attackTypes.indexOf("Ranged") === -1)
+			return;
+		let dist = API3.SquareVectorDistance(data.attacker.position(), target.position());
+		let range = target.attackRange("Ranged").max;
+		if (dist >= range*range)
+			return;
+	}
 	var index = gameState.ai.accessibility.getAccessValue(target.position());
 	var garrisonManager = gameState.ai.HQ.garrisonManager;
 	var garrisonArrowClasses = target.getGarrisonArrowClasses();
 	var units = gameState.getOwnUnits().filter(function (ent) { return MatchesClassList(garrisonArrowClasses, ent.classes()); }).filterNearest(target.position());
 	for (let ent of units.values())
 	{
-		if (garrisonManager.numberOfGarrisonedUnits(target) >= target.garrisonMax())
+		if (garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
 			break;
 		if (!ent.position())
 			continue;
@@ -391,7 +511,7 @@ m.DefenseManager.prototype.garrisonRangedUnitsInside = function(gameState, targe
 		}
 		if (gameState.ai.accessibility.getAccessValue(ent.position()) !== index)
 			continue;
-		garrisonManager.garrison(gameState, ent, target, "protection");
+		garrisonManager.garrison(gameState, ent, target, typeGarrison);
 	}
 };
 
@@ -399,7 +519,7 @@ m.DefenseManager.prototype.garrisonRangedUnitsInside = function(gameState, targe
 m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 {
 	let distmin = Math.min();
-	let nearest = undefined;
+	let nearest;
 	let unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	gameState.getAllyStructures().forEach(function(ent) {
@@ -407,6 +527,8 @@ m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 			return;
 		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax())
 			return;
+		if (ent.hitpoints() < ent.garrisonEjectHealth() * ent.maxHitpoints())
+			return;
 		var entAccess = ent.getMetadata(PlayerID, "access");
 		if (!entAccess)
 		{
@@ -415,7 +537,7 @@ m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 		}
 		if (entAccess !== unitAccess)
 			return;
-		var dist = API3.SquareVectorDistance(ent.position(), unit.position());
+		let dist = API3.SquareVectorDistance(ent.position(), unit.position());
 		if (dist > distmin)
 			return;
 		distmin = dist;
@@ -429,7 +551,7 @@ m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 m.DefenseManager.prototype.garrisonUnitForHealing = function(gameState, unit)
 {
 	let distmin = Math.min();
-	let nearest = undefined;
+	let nearest;
 	let unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	gameState.getAllyStructures().forEach(function(ent) {
@@ -439,7 +561,9 @@ m.DefenseManager.prototype.garrisonUnitForHealing = function(gameState, unit)
 			return;
 		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax())
 			return;
-		var entAccess = ent.getMetadata(PlayerID, "access");
+		if (ent.hitpoints() < ent.garrisonEjectHealth() * ent.maxHitpoints())
+			return;
+		let entAccess = ent.getMetadata(PlayerID, "access");
 		if (!entAccess)
 		{
 			entAccess = gameState.ai.accessibility.getAccessValue(ent.position());
@@ -447,7 +571,7 @@ m.DefenseManager.prototype.garrisonUnitForHealing = function(gameState, unit)
 		}
 		if (entAccess !== unitAccess)
 			return;
-		var dist = API3.SquareVectorDistance(ent.position(), unit.position());
+		let dist = API3.SquareVectorDistance(ent.position(), unit.position());
 		if (dist > distmin)
 			return;
 		distmin = dist;
@@ -465,7 +589,7 @@ m.DefenseManager.prototype.Serialize = function()
 	};
 
 	let armies = [];
-	for (var army of this.armies)
+	for (let army of this.armies)
 		armies.push(army.Serialize());
 
 	return { "properties": properties, "armies": armies };
