@@ -11,17 +11,21 @@ m.DiplomacyManager = function(Config)
 {
 	this.Config = Config;
 	this.nextTributeUpdate = -1;
+	this.nextTributeRequest = new Map();
+	this.nextTributeRequest.set("all", 240);
 };
 
 // Check if any allied needs help (tribute) and sent it if we have enough resource
+// or ask for a tribute if we are in need and one ally can help
 m.DiplomacyManager.prototype.tributes = function(gameState)
 {
 	this.nextTributeUpdate = gameState.ai.elapsedTime + 30;
 	var totalResources = gameState.getResources();
 	var availableResources = gameState.ai.queueManager.getAvailableResources(gameState);
+	var mostNeeded;
 	for (let i = 1; i < gameState.sharedScript.playersData.length; ++i)
 	{
-		if (i === PlayerID || !gameState.isPlayerAlly(i))
+		if (i === PlayerID || !gameState.isPlayerAlly(i) || gameState.ai.HQ.attackManager.defeated[i])
 			continue;
 		let allyResources = gameState.sharedScript.playersData[i].resourceCounts;
 		let allyPop = gameState.sharedScript.playersData[i].popCount;
@@ -39,6 +43,27 @@ m.DiplomacyManager.prototype.tributes = function(gameState)
 				tribute[res] = 100;
 				toSend = true;
 			}
+			else if (this.Config.chat && availableResources[res] === 0 && allyResources[res] > totalResources[res] + 600)
+			{
+				if (gameState.ai.elapsedTime < this.nextTributeRequest.get("all"))
+					continue;
+				if (this.nextTributeRequest.has(res) && gameState.ai.elapsedTime < this.nextTributeRequest.get(res))
+					continue;
+				if (!mostNeeded)
+					mostNeeded = gameState.ai.HQ.pickMostNeededResources(gameState);
+				for (let k = 0; k < 2; ++k)
+				{
+					if (mostNeeded[k].type == res && mostNeeded[k].wanted > 0) 
+					{
+						this.nextTributeRequest.set("all", gameState.ai.elapsedTime + 90);
+						this.nextTributeRequest.set(res, gameState.ai.elapsedTime + 240);
+						m.chatRequestTribute(gameState, res);
+						if (this.Config.debug > 1)
+							API3.warn("Tribute on " + res + " requested to player " + i);
+						break;
+					}
+				}
+			}
 		}
 		if (!toSend)
 			continue;
@@ -54,7 +79,7 @@ m.DiplomacyManager.prototype.checkEvents = function (gameState, events)
 {
 	// Increase slowly the cooperative personality trait either when we receive tribute from our allies
 	// or if our allies attack enemies inside our territory
-	for (let evt of events["TributeExchanged"])
+	for (let evt of events.TributeExchanged)
 	{
 		if (evt.to !== PlayerID || !gameState.isPlayerAlly(evt.from))
 			continue;
@@ -69,12 +94,12 @@ m.DiplomacyManager.prototype.checkEvents = function (gameState, events)
 		this.Config.personality.cooperative = Math.min(1, this.Config.personality.cooperative + 0.0002 * tributes);
 	}
 
-	for (let evt of events["Attacked"])
+	for (let evt of events.Attacked)
 	{
 		let target = gameState.getEntityById(evt.target);
-		if (!target || !target.position()
-			|| gameState.ai.HQ.territoryMap.getOwner(target.position()) !== PlayerID
-			|| !gameState.isPlayerEnemy(target.owner()))
+		if (!target || !target.position() ||
+			gameState.ai.HQ.territoryMap.getOwner(target.position()) !== PlayerID ||
+			!gameState.isPlayerEnemy(target.owner()))
 			continue;
 		let attacker = gameState.getEntityById(evt.attacker);
 		if (!attacker || attacker.owner() === PlayerID || !gameState.isPlayerAlly(attacker.owner()))
@@ -94,12 +119,13 @@ m.DiplomacyManager.prototype.update = function(gameState, events)
 
 m.DiplomacyManager.prototype.Serialize = function()
 {
-	return { "nextTributeUpdate": this.nextTributeUpdate };
+	return { "nextTributeUpdate": this.nextTributeUpdate, "nextTributeRequest": this.nextTributeRequest };
 };
 
 m.DiplomacyManager.prototype.Deserialize = function(data)
 {
 	this.nextTributeUpdate = data.nextTributeUpdate;
+	this.nextTributeRequest = data.nextTributeRequest;
 };
 
 return m;
