@@ -67,7 +67,7 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 			availableFishes[sea] = fish.resourceSupplyAmount();
 	}
 
-	for (var i = 0; i < gameState.ai.accessibility.regionSize.length; ++i)
+	for (let i = 0; i < gameState.ai.accessibility.regionSize.length; ++i)
 	{
 		if (!gameState.ai.HQ.navalRegions[i])
 		{
@@ -84,16 +84,16 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 		}
 		else
 		{
-			var collec = this.ships.filter(API3.Filters.byMetadata(PlayerID, "sea", i));
+			let collec = this.ships.filter(API3.Filters.byMetadata(PlayerID, "sea", i));
 			collec.registerUpdates();
 			this.seaShips.push(collec);
 			collec = this.transportShips.filter(API3.Filters.byMetadata(PlayerID, "sea", i));
 			collec.registerUpdates();
 			this.seaTransportShips.push(collec);
-			var collec = this.warShips.filter(API3.Filters.byMetadata(PlayerID, "sea", i));
+			collec = this.warShips.filter(API3.Filters.byMetadata(PlayerID, "sea", i));
 			collec.registerUpdates();
 			this.seaWarShips.push(collec);
-			var collec = this.fishShips.filter(API3.Filters.byMetadata(PlayerID, "sea", i));
+			collec = this.fishShips.filter(API3.Filters.byMetadata(PlayerID, "sea", i));
 			collec.registerUpdates();
 			this.seaFishShips.push(collec);
 			this.wantedTransportShips.push(0);
@@ -107,76 +107,67 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 		}
 	}
 
-	// determination of the possible landing zones
-	var width = gameState.getMap().width;
-	var length = width * gameState.getMap().height;
-	for (var i = 0; i < length; ++i)
-	{
-		var land = gameState.ai.accessibility.landPassMap[i];
-		if (land < 2)
-			continue;
-		var naval = gameState.ai.accessibility.navalPassMap[i];
-		if (naval < 2)
-			continue;
-		if (!this.landingZones[land])
-			this.landingZones[land] = {};
-		if (!this.landingZones[land][naval])
-			this.landingZones[land][naval] = [];
-		this.landingZones[land][naval].push(i);
-	}
-	// and keep only thoses with enough room around when possible
-	for (var land in this.landingZones)
-	{
-		for (var sea in this.landingZones[land])
-		{
-			var nbmax = 0;
-			for (var i = 0; i < this.landingZones[land][sea].length; i++)
-			{
-				var j = this.landingZones[land][sea][i];
-				var nb = 0;
-				if (this.landingZones[land][sea].indexOf(j-1) !== -1)
-					nb++;
-				if (this.landingZones[land][sea].indexOf(j+1) !== -1)
-					nb++;
-				if (this.landingZones[land][sea].indexOf(j+width) !== -1)
-					nb++;
-				if (this.landingZones[land][sea].indexOf(j-width) !== -1)
-					nb++;
-			}
-			if (nb > nbmax)
-				nbmax = nb;
-			var nbcut = Math.min(2, nbmax);
-			for (var i = 0; i < this.landingZones[land][sea].length; i++)
-			{
-				var j = this.landingZones[land][sea][i];
-				var nb = 0;
-				if (this.landingZones[land][sea].indexOf(j-1) !== -1)
-					nb++;
-				if (this.landingZones[land][sea].indexOf(j+1) !== -1)
-					nb++;
-				if (this.landingZones[land][sea].indexOf(j+width) !== -1)
-					nb++;
-				if (this.landingZones[land][sea].indexOf(j-width) !== -1)
-					nb++;
-				if (nb < nbcut)
-					this.landingZones[land][sea].splice(i--, 1);
-			}
-		}
-	}
-
 	// load units and buildings from the config files
-	var civ = gameState.playerData.civ;
+	let civ = gameState.civ();
 	if (civ in this.Config.buildings.naval)
 		this.bNaval = this.Config.buildings.naval[civ];
 	else
 		this.bNaval = this.Config.buildings.naval['default'];
 
-	for (var i in this.bNaval)
+	for (let i in this.bNaval)
 		this.bNaval[i] = gameState.applyCiv(this.bNaval[i]);
 
 	if (deserializing)
 		return;
 
+	// determination of the possible landing zones
+	var width = gameState.getMap().width;
+	var length = width * gameState.getMap().height;
+	for (let i = 0; i < length; ++i)
+	{
+		let land = gameState.ai.accessibility.landPassMap[i];
+		if (land < 2)
+			continue;
+		let naval = gameState.ai.accessibility.navalPassMap[i];
+		if (naval < 2)
+			continue;
+		if (!this.landingZones[land])
+			this.landingZones[land] = {};
+		if (!this.landingZones[land][naval])
+		    this.landingZones[land][naval] = new Set();
+		this.landingZones[land][naval].add(i);
+	}
+	// and keep only thoses with enough room around when possible
+	for (let land in this.landingZones)
+	{
+		for (let sea in this.landingZones[land])
+		{
+			let landing = this.landingZones[land][sea];
+			let nbaround = {};
+			let nbcut = 0;
+			for (let i of landing)
+			{
+				let nb = 0;
+				if (landing.has(i-1))
+					nb++;
+				if (landing.has(i+1))
+					nb++;
+				if (landing.has(i+width))
+					nb++;
+				if (landing.has(i-width))
+					nb++;
+				nbaround[i] = nb;
+				nbcut = Math.max(nb, nbcut);
+			}
+			nbcut = Math.min(2, nbcut);
+			for (let i of landing)
+			{
+				if (nbaround[i] < nbcut)
+					landing.delete(i);
+			}
+		}
+	}
+
 	// Assign our initial docks and ships
 	for (let ship of this.ships.values())
 		this.setShipIndex(gameState, ship);
@@ -184,10 +175,19 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 		this.setDockIndex(gameState, dock);
 };
 
-m.NavalManager.prototype.resetFishingBoats = function(gameState)
+m.NavalManager.prototype.updateFishingBoats = function(sea, num)
 {
-	for (var i = 0; i < gameState.ai.accessibility.regionSize.length; ++i)
-		this.wantedFishShips[i] = 0;
+	if (this.wantedFishShips[sea])
+		this.wantedFishShips[sea] = num;
+};
+
+m.NavalManager.prototype.resetFishingBoats = function(gameState, sea)
+{
+	if (sea !== undefined)
+		this.wantedFishShips[sea] = 0;
+	else
+		for (let i = 0; i < gameState.ai.accessibility.regionSize.length; ++i)
+			this.wantedFishShips[i] = 0;
 };
 
 m.NavalManager.prototype.setShipIndex = function(gameState, ship)
@@ -222,9 +222,9 @@ m.NavalManager.prototype.getDockIndex = function(gameState, dock, onWater)
 		// pre-positioned docks are sometimes not well positionned
 		var dockPos = dock.position();
 		var radius = dock.footprintRadius();
-		for (var i = 0; i < 16; i++)
+		for (let i = 0; i < 16; i++)
 		{
-			var pos = [ dockPos[0] + radius*Math.cos(i*Math.PI/8), dockPos[1] + radius*Math.sin(i*Math.PI/8)];
+			let pos = [ dockPos[0] + radius*Math.cos(i*Math.PI/8), dockPos[1] + radius*Math.sin(i*Math.PI/8)];
 
 			index = gameState.ai.accessibility.getAccessValue(pos, onWater);
 			if (index >= 2)
@@ -236,7 +236,7 @@ m.NavalManager.prototype.getDockIndex = function(gameState, dock, onWater)
 	return index;
 };
 
-// get the list of seas around this region not connected by a dock
+// get the list of seas (or lands) around this region not connected by a dock
 m.NavalManager.prototype.getUnconnectedSeas = function(gameState, region)
 {
 	var seas = gameState.ai.accessibility.regionLinks[region].slice();
@@ -244,7 +244,7 @@ m.NavalManager.prototype.getUnconnectedSeas = function(gameState, region)
 	docks.forEach(function (dock) {
 		if (dock.getMetadata(PlayerID, "access") !== region)
 			return;
-		var i = seas.indexOf(dock.getMetadata(PlayerID, "sea"));
+		let i = seas.indexOf(dock.getMetadata(PlayerID, "sea"));
 		if (i !== -1)
 			seas.splice(i--,1);
 	});
@@ -253,39 +253,35 @@ m.NavalManager.prototype.getUnconnectedSeas = function(gameState, region)
 
 m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 {
-	var evts = events["ConstructionFinished"];
-	// TODO: probably check stuffs like a base destruction.
-	for (var evt of evts)
+	for (let evt of events.ConstructionFinished)
 	{
 		if (!evt || !evt.newentity)
 			continue;
-		var entity = gameState.getEntityById(evt.newentity);
+		let entity = gameState.getEntityById(evt.newentity);
 		if (entity && entity.hasClass("Dock") && entity.isOwn(PlayerID))
 			this.setDockIndex(gameState, entity);
 	}
 
-	var evts = events["TrainingFinished"];
-	for (var evt of evts)
+	for (let evt of events.TrainingFinished)
 	{
 		if (!evt || !evt.entities)
 			continue;
-		for (var entId of evt.entities)
+		for (let entId of evt.entities)
 		{
-			var entity = gameState.getEntityById(entId);
+			let entity = gameState.getEntityById(entId);
 			if (!entity || !entity.hasClass("Ship") || !entity.isOwn(PlayerID))
 				continue;
 			this.setShipIndex(gameState, entity);
 		}
 	}
 
-	var evts = events["Destroy"];
-	for (var evt of evts)
+	for (let evt of events.Destroy)
 	{
 		if (!evt.entityObj || evt.entityObj.owner() !== PlayerID || !evt.metadata || !evt.metadata[PlayerID])
 			continue;
-		if (!evt.entityObj.hasClass("Ship") || !evt.metadata[PlayerID]["transporter"])
+		if (!evt.entityObj.hasClass("Ship") || !evt.metadata[PlayerID].transporter)
 			continue;
-		var plan = this.getPlan(evt.metadata[PlayerID]["transporter"]);
+		var plan = this.getPlan(evt.metadata[PlayerID].transporter);
 		if (!plan)
 			continue;
 
@@ -296,8 +292,8 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 		{
 			// just reset the units onBoard metadata and wait for a new ship to be assigned to this plan
 			plan.units.forEach(function (ent) {
-				if ((ent.getMetadata(PlayerID, "onBoard") === "onBoard" && ent.position())
-					|| ent.getMetadata(PlayerID, "onBoard") === shipId)
+				if ((ent.getMetadata(PlayerID, "onBoard") === "onBoard" && ent.position()) ||
+					ent.getMetadata(PlayerID, "onBoard") === shipId)
 					ent.setMetadata(PlayerID, "onBoard", undefined);
 			});
 			plan.needTransportShips = (plan.transportShips.length === 0);
@@ -309,8 +305,8 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 			plan.units.forEach(function (ent) {
 				if (!ent.position())  // unit from another ship of this plan ... do nothing
 					return;
-				var access = gameState.ai.accessibility.getAccessValue(ent.position());
-				var endPos = ent.getMetadata(PlayerID, "endPos");
+				let access = gameState.ai.accessibility.getAccessValue(ent.position());
+				let endPos = ent.getMetadata(PlayerID, "endPos");
 				ent.setMetadata(PlayerID, "transport", undefined);
 				ent.setMetadata(PlayerID, "onBoard", undefined);
 				ent.setMetadata(PlayerID, "endPos", undefined);
@@ -323,17 +319,24 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 			});
 		}
 	}
+
+	for (let evt of events.OwnershipChanged)	// capture events
+	{
+		if (evt.to === PlayerID)
+		{
+			let ent = gameState.getEntityById(evt.entity);
+			if (ent && ent.hasClass("Dock"))
+				this.setDockIndex(gameState, ent);
+		}
+	}
 };
 
 
 m.NavalManager.prototype.getPlan = function(ID)
 {
-	for (var plan of this.transportPlans)
-	{
-		if (plan.ID !== ID)
-			continue;
-		return plan;
-	}
+	for (let plan of this.transportPlans)
+		if (plan.ID === ID)
+			return plan;
 	return undefined;
 };
 
@@ -354,16 +357,16 @@ m.NavalManager.prototype.requireTransport = function(gameState, entity, startInd
 		return false;
 	}
 
-	for (var plan of this.transportPlans)
+	for (let plan of this.transportPlans)
 	{
 		if (plan.startIndex !== startIndex || plan.endIndex !== endIndex)
-			continue
+			continue;
 		if (plan.state !== "boarding")
-			continue
+			continue;
 		plan.addUnit(entity, endPos);
 		return true;
 	}
-	var plan = new m.TransportPlan(gameState, [entity], startIndex, endIndex, endPos);
+	let plan = new m.TransportPlan(gameState, [entity], startIndex, endIndex, endPos);
 	if (plan.failed)
 	{
 		if (this.Config.debug > 1)
@@ -407,12 +410,11 @@ m.NavalManager.prototype.splitTransport = function(gameState, plan)
  * create a transport from a garrisoned ship to a land location
  * needed at start game when starting with a garrisoned ship
  */
-m.NavalManager.prototype.createTransportIfNeeded = function(gameState, fromPos, toPos)
+m.NavalManager.prototype.createTransportIfNeeded = function(gameState, fromPos, toPos, toAccess)
 {
 	let fromAccess = gameState.ai.accessibility.getAccessValue(fromPos);
 	if (fromAccess !== 1)
 		return;
-	let toAccess = gameState.ai.accessibility.getAccessValue(toPos);
 	if (toAccess < 2)
 		return;
 
@@ -449,16 +451,16 @@ m.NavalManager.prototype.checkLevels = function(gameState, queues)
 	if (queues.ships.length() !== 0)
 		return;
 
-	for (var sea = 0; sea < this.neededTransportShips.length; sea++)
+	for (let sea = 0; sea < this.neededTransportShips.length; sea++)
 		this.neededTransportShips[sea] = 0;
 
-	for (var plan of this.transportPlans)
+	for (let plan of this.transportPlans)
 	{
 		if (!plan.needTransportShips || plan.units.length < 2)
 			continue;
-		var sea = plan.sea;
-		if (gameState.countOwnQueuedEntitiesWithMetadata("sea", sea) > 0
-			|| this.seaTransportShips[sea].length < this.wantedTransportShips[sea])
+		let sea = plan.sea;
+		if (gameState.countOwnQueuedEntitiesWithMetadata("sea", sea) > 0 ||
+			this.seaTransportShips[sea].length < this.wantedTransportShips[sea])
 			continue;
 		++this.neededTransportShips[sea];
 		if (this.wantedTransportShips[sea] === 0 || this.seaTransportShips[sea].length < plan.transportShips.length + 2)
@@ -468,7 +470,7 @@ m.NavalManager.prototype.checkLevels = function(gameState, queues)
 		}
 	}
 
-	for (var sea = 0; sea < this.neededTransportShips.length; sea++)
+	for (let sea = 0; sea < this.neededTransportShips.length; sea++)
 		if (this.neededTransportShips[sea] > 2)
 			++this.wantedTransportShips[sea];
 };
@@ -477,8 +479,8 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 {
 	if (queues.ships.length() > 0)
 		return;
-	if (gameState.countEntitiesByType(gameState.applyCiv("structures/{civ}_dock"), true) +
-		gameState.countEntitiesByType(gameState.applyCiv("structures/{civ}_super_dock"), true) === 0)
+	if (gameState.getOwnEntitiesByClass("Dock", true).filter(API3.Filters.isBuilt()).length +
+		gameState.getOwnEntitiesByClass("Shipyard", true).filter(API3.Filters.isBuilt()).length === 0)
 		return;
 	// check if we have enough transport ships per region.
 	for (var sea = 0; sea < this.seaShips.length; ++sea)
@@ -493,7 +495,7 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 			var template = this.getBestShip(gameState, sea, "transport");
 			if (template)
 			{
-				queues.ships.addItem(new m.TrainingPlan(gameState, template, { "sea": sea }, 1, 1));
+				queues.ships.addPlan(new m.TrainingPlan(gameState, template, { "sea": sea }, 1, 1));
 				continue;
 			}
 		}
@@ -501,10 +503,10 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 
 		if (this.seaFishShips[sea].length < this.wantedFishShips[sea])
 		{
-			var template = this.getBestShip(gameState, sea, "fishing");
+			let template = this.getBestShip(gameState, sea, "fishing");
 			if (template)
 			{
-				queues.ships.addItem(new m.TrainingPlan(gameState, template, { "base": 0, "role": "worker", "sea": sea }, 1, 1));
+				queues.ships.addPlan(new m.TrainingPlan(gameState, template, { "base": 0, "role": "worker", "sea": sea }, 1, 1));
 				continue;
 			}
 		}
@@ -514,7 +516,7 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 // assigns free ships to plans that need some
 m.NavalManager.prototype.assignShipsToPlans = function(gameState)
 {
-	for (var plan of this.transportPlans)
+	for (let plan of this.transportPlans)
 		if (plan.needTransportShips)
 			plan.assignShip(gameState);
 };
@@ -532,7 +534,7 @@ m.NavalManager.prototype.moveApart = function(gameState)
 		{
 			if (ship.isIdle())	// do not stay idle near a dock to not disturb other ships
 			{
-				gameState.getOwnStructures().filter(API3.Filters.byClass("Dock")).forEach(function(dock) {
+				gameState.getAllyStructures().filter(API3.Filters.byClass("Dock")).forEach(function(dock) {
 					if (dock.getMetadata(PlayerID, "sea") !== sea)
 						return;
 					if (API3.SquareVectorDistance(ship.position(), dock.position()) > 2500)
@@ -579,23 +581,25 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 
 	if (gameState.getPopulation() > this.Config.Economy.popForDock)
 	{
-		if (queues.dock.countQueuedUnitsWithClass("NavalMarket") === 0
-			&& gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClass("NavalMarket"), API3.Filters.isFoundation())).length === 0
-			&& gameState.ai.HQ.canBuild(gameState, "structures/{civ}_dock"))
+		if (queues.dock.countQueuedUnitsWithClass("NavalMarket") === 0 &&
+			gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClass("NavalMarket"), API3.Filters.isFoundation())).length === 0 &&
+			gameState.ai.HQ.canBuild(gameState, "structures/{civ}_dock"))
 		{
-			var dockStarted = false;
-			for (var base of gameState.ai.HQ.baseManagers)
+			let dockStarted = false;
+			for (let base of gameState.ai.HQ.baseManagers)
 			{
 				if (dockStarted)
 					break;
 				if (!base.anchor || base.constructing)
 					continue;
-				var remaining = this.getUnconnectedSeas(gameState, base.accessIndex);
-				for (var sea of remaining)
+				let remaining = this.getUnconnectedSeas(gameState, base.accessIndex);
+				for (let sea of remaining)
 				{
 					if (!gameState.ai.HQ.navalRegions[sea])
 						continue;
-					queues.dock.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "land": [base.accessIndex], "sea": sea }));
+					let wantedLand = {};
+					wantedLand[base.accessIndex] = true;
+					queues.dock.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "land": wantedLand, "sea": sea }));
 					dockStarted = true;
 					break;
 				}
@@ -603,32 +607,28 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 		}
 	}
 
-	if (gameState.currentPhase() < 2 || gameState.getPopulation() < this.Config.Economy.popForTown + 15
-		|| queues.militaryBuilding.length() !== 0 || this.bNaval.length === 0)
+	if (gameState.currentPhase() < 2 || gameState.getPopulation() < this.Config.Economy.popForTown + 15 ||
+		queues.militaryBuilding.length() !== 0 || this.bNaval.length === 0)
 		return;
 	var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
 	if (!docks.length)
 		return;
 	var nNaval = 0;
-	for (var naval of this.bNaval)
+	for (let naval of this.bNaval)
 		nNaval += gameState.countEntitiesAndQueuedByType(naval, true);
 
 	if (nNaval === 0 || (nNaval < this.bNaval.length && gameState.getPopulation() > 120))
 	{
-		for (var naval of this.bNaval)
+		for (let naval of this.bNaval)
 		{
 			if (gameState.countEntitiesAndQueuedByType(naval, true) < 1 && gameState.ai.HQ.canBuild(gameState, naval))
 			{
-				var land = [];
-				for (var base of gameState.ai.HQ.baseManagers)
-				{
-					if (!base.anchor)
-						continue;
-					if (land.indexOf(base.accessIndex) === -1)
-						land.push(base.accessIndex);
-				}
-				var sea = docks.toEntityArray()[0].getMetadata(PlayerID, "sea");
-				queues.militaryBuilding.addItem(new m.ConstructionPlan(gameState, naval, { "land": land, "sea": sea }));
+				let wantedLand = {};
+				for (let base of gameState.ai.HQ.baseManagers)
+					if (base.anchor)
+						wantedLand[base.accessIndex] = true;
+				let sea = docks.toEntityArray()[0].getMetadata(PlayerID, "sea");
+				queues.militaryBuilding.addPlan(new m.ConstructionPlan(gameState, naval, { "land": wantedLand, "sea": sea }));
 				break;
 			}
 		}
@@ -638,34 +638,35 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 // goal can be either attack (choose ship with best arrowCount) or transport (choose ship with best capacity)
 m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 {
+	var civ = gameState.civ();
 	var trainableShips = [];
 	gameState.getOwnTrainingFacilities().filter(API3.Filters.byMetadata(PlayerID, "sea", sea)).forEach(function(ent) {
-		var trainables = ent.trainableEntities();
-		for (var trainable of trainables)
+		var trainables = ent.trainableEntities(civ);
+		for (let trainable of trainables)
 		{
 			if (gameState.isDisabledTemplates(trainable))
 				continue;
-			var template = gameState.getTemplate(trainable);
-			if (template.hasClass("Ship") && trainableShips.indexOf(trainable) === -1)
+			let template = gameState.getTemplate(trainable);
+			if (template && template.hasClass("Ship") && trainableShips.indexOf(trainable) === -1)
 				trainableShips.push(trainable);
 		}
 	});
 
 	var best = 0;
-	var bestShip = undefined;
+	var bestShip;
 	var limits = gameState.getEntityLimits();
 	var current = gameState.getEntityCounts();
-	for (var trainable of trainableShips)
+	for (let trainable of trainableShips)
 	{
-		var template = gameState.getTemplate(trainable);
+		let template = gameState.getTemplate(trainable);
 		if (!template.available(gameState))
 			continue;
 
-		var category = template.trainingCategory();
+		let category = template.trainingCategory();
 		if (category && limits[category] && current[category] >= limits[category])
 			continue;
 
-		var arrows = +(template.getDefaultArrow() || 0);
+		let arrows = +(template.getDefaultArrow() || 0);
 		if (goal === "attack")    // choose the maximum default arrows
 		{
 			if (best > arrows)
@@ -674,7 +675,7 @@ m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 		}
 		else if (goal === "transport")   // choose the maximum capacity, with a bonus if arrows or if siege transport
 		{
-			var capacity = +(template.garrisonMax() || 0);
+			let capacity = +(template.garrisonMax() || 0);
 			if (capacity < 2)
 				continue;
 			capacity += 10*arrows;
@@ -685,10 +686,8 @@ m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 			best = capacity;
 		}
 		else if (goal === "fishing")
-		{
 			if (!template.hasClass("FishingBoat"))
 				continue;
-		}
 		bestShip = trainable;
 	}
 	return bestShip;
@@ -701,16 +700,15 @@ m.NavalManager.prototype.update = function(gameState, queues, events)
 	this.checkEvents(gameState, queues, events);
 
 	// close previous transport plans if finished
-	for (var i = 0; i < this.transportPlans.length; ++i)
+	for (let i = 0; i < this.transportPlans.length; ++i)
 	{
-		var remaining = this.transportPlans[i].update(gameState);
-		if (remaining === 0)
-		{
-			if (this.Config.debug > 1)
-				API3.warn("no more units on transport plan " + this.transportPlans[i].ID);
-			this.transportPlans[i].releaseAll();
-			this.transportPlans.splice(i--, 1);
-		}
+		let remaining = this.transportPlans[i].update(gameState);
+		if (remaining)
+			continue;
+		if (this.Config.debug > 1)
+			API3.warn("no more units on transport plan " + this.transportPlans[i].ID);
+		this.transportPlans[i].releaseAll();
+		this.transportPlans.splice(i--, 1);
 	}
 	// assign free ships to plans which need them
 	this.assignShipsToPlans(gameState);
