@@ -25,33 +25,33 @@ m.QueueManager = function(Config, queues)
 	this.Config = Config;
 	this.queues = queues;
 	this.priorities = {};
-	for (var i in Config.priorities)
+	for (let i in Config.priorities)
 		this.priorities[i] = Config.priorities[i];
 	this.accounts = {};
 
 	// the sorting is updated on priority change.
 	this.queueArrays = [];
-	for (var p in this.queues)
+	for (let q in this.queues)
 	{
-		this.accounts[p] = new API3.Resources();
-		this.queueArrays.push([p, this.queues[p]]);
+		this.accounts[q] = new API3.Resources();
+		this.queueArrays.push([q, this.queues[q]]);
 	}
 	var priorities = this.priorities;
-	this.queueArrays.sort(function (a,b) { return (priorities[b[0]] - priorities[a[0]]) });
+	this.queueArrays.sort((a,b) => priorities[b[0]] - priorities[a[0]]);
 };
 
 m.QueueManager.prototype.getAvailableResources = function(gameState)
 {
 	var resources = gameState.getResources();
-	for (var key in this.queues)
+	for (let key in this.queues)
 		resources.subtract(this.accounts[key]);
 	return resources;
 };
 
-m.QueueManager.prototype.getTotalAccountedResources = function(gameState)
+m.QueueManager.prototype.getTotalAccountedResources = function()
 {
 	var resources = new API3.Resources();
-	for (var key in this.queues)
+	for (let key in this.queues)
 		resources.add(this.accounts[key]);
 	return resources;
 };
@@ -63,9 +63,9 @@ m.QueueManager.prototype.currentNeeds = function(gameState)
 	for (let q of this.queueArrays)
 	{
 		let queue = q[1];
-		if (!queue.length() || !queue.queue[0].isGo(gameState))
+		if (!queue.length() || !queue.plans[0].isGo(gameState))
 			continue;
-		var costs = queue.queue[0].getCost();
+		let costs = queue.plans[0].getCost();
 		needed.add(costs);
 	}
 	// get out current resources, not removing accounts.
@@ -80,11 +80,16 @@ m.QueueManager.prototype.currentNeeds = function(gameState)
 // TODO: many things.
 m.QueueManager.prototype.wantedGatherRates = function(gameState)
 {
+	if (gameState.ai.playedTurn == 0)
+	{
+		// default values for first turn when we have not yet set our queues.
+		return { "food": 10, "wood": 10, "stone": 0, "metal": 0 };
+	}
 	// get out current resources, not removing accounts.
 	var current = gameState.getResources();
 	// short queue is the first item of a queue, assumed to be ready in 30s
 	// medium queue is the second item of a queue, assumed to be ready in 60s
-	// long queue contains the is the isGo=false items, assumed to be ready in 300s
+	// long queue contains the isGo=false items, assumed to be ready in 300s
 	var totalShort = { "food": 200, "wood": 200, "stone": 100, "metal": 100 };
 	var totalMedium = { "food": 0, "wood": 0, "stone": 0, "metal": 0 };
 	var totalLong = { "food": 0, "wood": 0, "stone": 0, "metal": 0 };
@@ -92,15 +97,15 @@ m.QueueManager.prototype.wantedGatherRates = function(gameState)
 	//queueArrays because it's faster.
 	for (let q of this.queueArrays)
 	{
-		var queue = q[1];
+		let queue = q[1];
 		if (queue.paused)
 			continue;
 		for (let j = 0; j < queue.length(); ++j)
 		{
 			if (j > 1)
 				break;
-			let cost = queue.queue[j].getCost();
-			if (queue.queue[j].isGo(gameState))
+			let cost = queue.plans[j].getCost();
+			if (queue.plans[j].isGo(gameState))
 			{
 				if (j == 0)
 					total = totalShort;
@@ -111,14 +116,14 @@ m.QueueManager.prototype.wantedGatherRates = function(gameState)
 				total = totalLong;
 			for (let type in total)
 				total[type] += cost[type];
-			if (!queue.queue[j].isGo(gameState))
+			if (!queue.plans[j].isGo(gameState))
 				break;
 		}
 	}
 	// global rates
 	var rates = { "food": 0, "wood": 0, "stone": 0, "metal": 0 };
 	var diff;
-	for (var type in rates)
+	for (let type in rates)
 	{
 		if (current[type] > 0)
 		{
@@ -148,26 +153,26 @@ m.QueueManager.prototype.printQueues = function(gameState)
 			numWorkers++;
 	});
 	API3.warn("---------- QUEUES ------------ with pop " + gameState.getPopulation() + " and workers " + numWorkers);
-	for (var i in this.queues)
+	for (let i in this.queues)
 	{
-		var qStr = "";
-		var q = this.queues[i];
-		if (q.queue.length > 0)
+		let qStr = "";
+		let q = this.queues[i];
+		if (q.length() > 0)
 		{
 			API3.warn(i + ": ( with priority " + this.priorities[i] +" and accounts " + uneval(this.accounts[i]) +")");
 			API3.warn(" while maxAccountWanted(0.6) is " + uneval(q.maxAccountWanted(gameState, 0.6)));
 		}
-		for (var j in q.queue)
+		for (let plan of q.plans)
 		{
-			qStr = "     " + q.queue[j].type + " ";
-			if (q.queue[j].number)
-				qStr += "x" + q.queue[j].number;
-			qStr += "   isGo " + q.queue[j].isGo(gameState);
+			qStr = "     " + plan.type + " ";
+			if (plan.number)
+				qStr += "x" + plan.number;
+			qStr += "   isGo " + plan.isGo(gameState);
 			API3.warn(qStr);
 		}
 	}
 	API3.warn("Accounts");
-	for (var p in this.accounts)
+	for (let p in this.accounts)
 	    API3.warn(p + ": " + uneval(this.accounts[p]));
 	API3.warn("Current Resources: " + uneval(gameState.getResources()));
 	API3.warn("Available Resources: " + uneval(this.getAvailableResources(gameState)));
@@ -177,65 +182,9 @@ m.QueueManager.prototype.printQueues = function(gameState)
 	API3.warn("------------------------------------");
 };
 
-// nice readable HTML version.
-m.QueueManager.prototype.HTMLprintQueues = function(gameState)
-{
-	if (!m.DebugEnabled())
-		return;
-	var strToSend = [];
-	strToSend.push("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\"> <html> <head> <title>Aegis Queue Manager</title> <link rel=\"stylesheet\" href=\"table.css\">  </head> <body> <table> <caption>Aegis Build Order</caption> ");
-	for (var i in this.queues)
-	{
-		strToSend.push("<tr>");
-		
-		var q = this.queues[i];
-		var str = "<th>" + i + "  (" + this.priorities[i] + ")<br><span class=\"ressLevel\">";
-		for (var k of this.accounts[i].types)
-		{
-			str += this.accounts[i][k] + k.substr(0,1).toUpperCase() ;
-			if (k != "metal") str += " / ";
-		}
-		strToSend.push(str + "</span></th>");
-		for (var j in q.queue) {
-			if (q.queue[j].isGo(gameState))
-				strToSend.push("<td>");
-			else
-				strToSend.push("<td class=\"NotGo\">");
-
-			var qStr = "";
-			if (q.queue[j].number)
-				qStr += q.queue[j].number + " ";
-			qStr += q.queue[j].type;
-			qStr += "<br><span class=\"ressLevel\">";
-			var costs = q.queue[j].getCost();
-			for (var k of costs.types)
-			{
-				qStr += costs[k] + k.substr(0,1).toUpperCase() ;
-				if (k != "metal")
-					qStr += " / ";
-			}
-			qStr += "</span></td>";
-			strToSend.push(qStr);
-		}
-		strToSend.push("</tr>");
-	}
-	strToSend.push("</table>");
-	/*strToSend.push("<h3>Accounts</h3>");
-	for (var p in this.accounts)
-	{
-		strToSend.push("<p>" + p + ": " + uneval(this.accounts[p]) + " </p>");
-	}*/
-	strToSend.push("<p>Wanted Gather Rate:" + uneval(this.wantedGatherRates(gameState)) + "</p>");
-	strToSend.push("<p>Current Resources:" + uneval(gameState.getResources()) + "</p>");
-	strToSend.push("<p>Available Resources:" + uneval(this.getAvailableResources(gameState)) + "</p>");
-	strToSend.push("</body></html>");
-	for (var logged of strToSend)
-		log(logged);
-};
-
 m.QueueManager.prototype.clear = function()
 {
-	for (var i in this.queues)
+	for (let i in this.queues)
 		this.queues[i].empty();
 };
 
@@ -245,26 +194,24 @@ m.QueueManager.prototype.clear = function()
 m.QueueManager.prototype.setAccounts = function(gameState, cost, i)
 {
 	var available = this.getAvailableResources(gameState);
-	for (var res of this.accounts[i].types)
+	for (let res of this.accounts[i].types)
 	{
 		if (this.accounts[i][res] >= cost[res])
 			continue;
-		var diff = Math.min(available[res], cost[res] - this.accounts[i][res]);
-		this.accounts[i][res] += diff;
+		this.accounts[i][res] += Math.min(available[res], cost[res] - this.accounts[i][res]);
 	}
 };
 
-
 /**
  * transfer accounts from queue i to queue j
  */
 m.QueueManager.prototype.transferAccounts = function(cost, i, j)
 {
-	for (var res of this.accounts[i].types)
+	for (let res of this.accounts[i].types)
 	{
 		if (this.accounts[j][res] >= cost[res])
 			continue;
-		var diff = Math.min(this.accounts[i][res], cost[res] - this.accounts[j][res]);
+		let diff = Math.min(this.accounts[i][res], cost[res] - this.accounts[j][res]);
 		this.accounts[i][res] -= diff;
 		this.accounts[j][res] += diff;
 	}
@@ -276,14 +223,14 @@ m.QueueManager.prototype.transferAccounts = function(cost, i, j)
 m.QueueManager.prototype.distributeResources = function(gameState)
 {
 	var availableRes = this.getAvailableResources(gameState);
-	for (var res of availableRes.types)
+	for (let res of availableRes.types)
 	{
 		if (availableRes[res] < 0)    // rescale the accounts if we've spent resources already accounted (e.g. by bartering)
 		{
-			var total = gameState.getResources()[res];
-			var scale = total / (total - availableRes[res]);
+			let total = gameState.getResources()[res];
+			let scale = total / (total - availableRes[res]);
 			availableRes[res] = total;
-			for (var j in this.queues)
+			for (let j in this.queues)
 			{
 				this.accounts[j][res] = Math.floor(scale * this.accounts[j][res]);
 				availableRes[res] -= this.accounts[j][res];
@@ -296,9 +243,9 @@ m.QueueManager.prototype.distributeResources = function(gameState)
 			continue;
 		}
 
-		var totalPriority = 0;
-		var tempPrio = {};
-		var maxNeed = {};
+		let totalPriority = 0;
+		let tempPrio = {};
+		let maxNeed = {};
 		// Okay so this is where it gets complicated.
 		// If a queue requires "res" for the next elements (in the queue)
 		// And the account is not high enough for it.
@@ -308,10 +255,10 @@ m.QueueManager.prototype.distributeResources = function(gameState)
 		//	-queues accounts are capped at "resources for the first + 60% of the next"
 		// This avoids getting a high priority queue with many elements hogging all of one resource
 		// uselessly while it awaits for other resources.
-		for (var j in this.queues)
+		for (let j in this.queues)
 		{
 			// returns exactly the correct amount, ie 0 if we're not go.
-			var queueCost = this.queues[j].maxAccountWanted(gameState, 0.6);
+			let queueCost = this.queues[j].maxAccountWanted(gameState, 0.6);
 			if (this.queues[j].length() > 0 && this.accounts[j][res] < queueCost[res] && !this.queues[j].paused)
 			{
 				// adding us to the list of queues that need an update.
@@ -332,12 +279,12 @@ m.QueueManager.prototype.distributeResources = function(gameState)
 		}
 		// Now we allow resources to the accounts. We can at most allow "TempPriority/totalpriority*available"
 		// But we'll sometimes allow less if that would overflow.
-		var available = availableRes[res];
-		var missing = false;
-		for (var j in tempPrio)
+		let available = availableRes[res];
+		let missing = false;
+		for (let j in tempPrio)
 		{
 			// we'll add at much what can be allowed to this queue.
-			var toAdd = Math.floor(availableRes[res] * tempPrio[j]/totalPriority);
+			let toAdd = Math.floor(availableRes[res] * tempPrio[j]/totalPriority);
 			if (toAdd >= maxNeed[j])
 				toAdd = maxNeed[j];
 			else
@@ -348,9 +295,9 @@ m.QueueManager.prototype.distributeResources = function(gameState)
 		}
 		if (missing && available > 0)   // distribute the rest (due to floor) in any queue
 		{
-			for (var j in tempPrio)
+			for (let j in tempPrio)
 			{
-				var toAdd = Math.min(maxNeed[j], available);
+				let toAdd = Math.min(maxNeed[j], available);
 				this.accounts[j][res] += toAdd;
 				available -= toAdd;
 				if (available <= 0)
@@ -367,27 +314,27 @@ m.QueueManager.prototype.switchResource = function(gameState, res)
 	// We have no available resources, see if we can't "compact" them in one queue.
 	// compare queues 2 by 2, and if one with a higher priority could be completed by our amount, give it.
 	// TODO: this isn't perfect compression.
-	for (var j in this.queues)
+	for (let j in this.queues)
 	{
 		if (this.queues[j].length() === 0 || this.queues[j].paused)
 			continue;
 
-		var queue = this.queues[j];
-		var queueCost = queue.maxAccountWanted(gameState, 0);
+		let queue = this.queues[j];
+		let queueCost = queue.maxAccountWanted(gameState, 0);
 		if (this.accounts[j][res] >= queueCost[res])
 			continue;
 
-		for (var i in this.queues)
+		for (let i in this.queues)
 		{
 			if (i === j)
 				continue;
-			var otherQueue = this.queues[i];
+			let otherQueue = this.queues[i];
 			if (this.priorities[i] >= this.priorities[j] || otherQueue.switched !== 0)
 				continue;
 			if (this.accounts[j][res] + this.accounts[i][res] < queueCost[res])
 				continue;
 
-			var diff = queueCost[res] - this.accounts[j][res];
+			let diff = queueCost[res] - this.accounts[j][res];
 			this.accounts[j][res] += diff;
 			this.accounts[i][res] -= diff;
 			++otherQueue.switched;
@@ -428,7 +375,7 @@ m.QueueManager.prototype.update = function(gameState)
 {
 	Engine.ProfileStart("Queue Manager");
 
-	for (var i in this.queues)
+	for (let i in this.queues)
 	{
 		this.queues[i].check(gameState);  // do basic sanity checks on the queue
 		if (this.priorities[i] > 0)
@@ -455,11 +402,7 @@ m.QueueManager.prototype.update = function(gameState)
 // Recovery system: if short of workers after an attack, pause (and reset) some queues to favor worker training
 m.QueueManager.prototype.checkPausedQueues = function(gameState)
 {
-	let numWorkers = 0;
-	gameState.getOwnUnits().forEach (function (ent) {
-		if (ent.getMetadata(PlayerID, "role") == "worker")
-			numWorkers++;
-	});
+	let numWorkers = gameState.getOwnEntitiesByRole("worker", true).length;
 	gameState.getOwnTrainingFacilities().forEach(function(ent) {
 		ent.trainingQueue().forEach(function(item) {
 			if (item.metadata && item.metadata.role && item.metadata.role == "worker")
@@ -467,20 +410,34 @@ m.QueueManager.prototype.checkPausedQueues = function(gameState)
 		});
 	});
 
+	var workersMin = Math.min(Math.max(12, 24 * this.Config.popScaling), this.Config.Economy.popForTown);
 	for (let q in this.queues)
 	{
 		let toBePaused = false;
 		if (gameState.ai.HQ.numActiveBase() === 0)
 			toBePaused = (q !== "dock" && q !== "civilCentre");
-		else if (numWorkers < 8)
+		else if (numWorkers < workersMin / 3)
 			toBePaused = (q !== "citizenSoldier" && q !== "villager" && q !== "emergency");
-		else if (numWorkers < 16)
-			toBePaused = (q === "civilCentre" || q === "economicBuilding"
-				|| q === "militaryBuilding" || q === "defenseBuilding"
-				|| q === "majorTech" || q === "minorTech" || q.indexOf("plan_") !== -1);
-		else if (numWorkers < 24)
-			toBePaused = (q === "civilCentre" || q === "defenseBuilding"
-				|| q == "majorTech" || q.indexOf("_siege") != -1 || q.indexOf("_champ") != -1);
+		else if (numWorkers < workersMin * 2 / 3)
+			toBePaused = (q === "civilCentre" || q === "economicBuilding" ||
+				q === "militaryBuilding" || q === "defenseBuilding" ||
+				q === "majorTech" || q === "minorTech" || q.indexOf("plan_") !== -1);
+		else if (numWorkers < workersMin)
+			toBePaused = (q === "civilCentre" || q === "defenseBuilding" ||
+				q == "majorTech" || q.indexOf("_siege") != -1 || q.indexOf("_champ") != -1);
+
+		if (toBePaused)
+		{
+			if (q === "field" && gameState.ai.HQ.needFarm &&
+				gameState.getOwnStructures().filter(API3.Filters.byClass("Field")).length === 0)
+				toBePaused = false;
+			if (q === "dock" && gameState.ai.HQ.needFish &&
+				gameState.getOwnStructures().filter(API3.Filters.byClass("Dock")).length === 0)
+				toBePaused = false;
+			if (q === "ships" && gameState.ai.HQ.needFish &&
+				gameState.ai.HQ.navalManager.ships.filter(API3.Filters.byClass("FishingBoat")).length === 0)
+				toBePaused = false;
+		}
 
 		let queue = this.queues[q];
 		if (!queue.paused && toBePaused)
@@ -492,23 +449,29 @@ m.QueueManager.prototype.checkPausedQueues = function(gameState)
 			queue.paused = false;
 
 		// And reduce the batch sizes of attack queues
-		if (q.indexOf("plan_") != -1 && numWorkers < 24 && queue.queue[0])
+		if (q.indexOf("plan_") != -1 && numWorkers < workersMin && queue.plans[0])
 		{
-			queue.queue[0].number = 1;
-			if (queue.queue[1])
-				queue.queue[1].number = 1;
+			queue.plans[0].number = 1;
+			if (queue.plans[1])
+				queue.plans[1].number = 1;
 		}
 	}
 };
 
+m.QueueManager.prototype.canAfford = function(queue, cost)
+{
+	if (!this.accounts[queue])
+		return false;
+	return this.accounts[queue].canAfford(cost);
+};
+
 m.QueueManager.prototype.pauseQueue = function(queue, scrapAccounts)
 {
-	if (this.queues[queue])
-	{
-		this.queues[queue].paused = true;
-		if (scrapAccounts)
-			this.accounts[queue].reset();
-	}
+	if (!this.queues[queue])
+		return;
+	this.queues[queue].paused = true;
+	if (scrapAccounts)
+		this.accounts[queue].reset();
 };
 
 m.QueueManager.prototype.unpauseQueue = function(queue)
@@ -519,22 +482,21 @@ m.QueueManager.prototype.unpauseQueue = function(queue)
 
 m.QueueManager.prototype.pauseAll = function(scrapAccounts, but)
 {
-	for (var p in this.queues)
+	for (let q in this.queues)
 	{
-		if (p != but)
-		{
-			if (scrapAccounts)
-				this.accounts[p].reset();
-			this.queues[p].paused = true;
-		}
+		if (q == but)
+			continue;
+		if (scrapAccounts)
+			this.accounts[q].reset();
+		this.queues[q].paused = true;
 	}
 };
 
 m.QueueManager.prototype.unpauseAll = function(but)
 {
-	for (var p in this.queues)
-		if (p != but)
-			this.queues[p].paused = false;
+	for (let q in this.queues)
+		if (q != but)
+			this.queues[q].paused = false;
 };
 
 
@@ -548,10 +510,10 @@ m.QueueManager.prototype.addQueue = function(queueName, priority)
 	this.accounts[queueName] = new API3.Resources();
 
 	this.queueArrays = [];
-	for (var p in this.queues)
-		this.queueArrays.push([p, this.queues[p]]);
+	for (let q in this.queues)
+		this.queueArrays.push([q, this.queues[q]]);
 	var priorities = this.priorities;
-	this.queueArrays.sort(function (a,b) { return (priorities[b[0]] - priorities[a[0]]) });
+	this.queueArrays.sort((a,b) => priorities[b[0]] - priorities[a[0]]);
 };
 
 m.QueueManager.prototype.removeQueue = function(queueName)
@@ -564,10 +526,10 @@ m.QueueManager.prototype.removeQueue = function(queueName)
 	delete this.accounts[queueName];
 		
 	this.queueArrays = [];
-	for (var p in this.queues)
-		this.queueArrays.push([p, this.queues[p]]);
+	for (let q in this.queues)
+		this.queueArrays.push([q, this.queues[q]]);
 	var priorities = this.priorities;
-	this.queueArrays.sort(function (a,b) { return (priorities[b[0]] - priorities[a[0]]) });
+	this.queueArrays.sort((a,b) => priorities[b[0]] - priorities[a[0]]);
 };
 
 m.QueueManager.prototype.getPriority = function(queueName)
@@ -582,20 +544,20 @@ m.QueueManager.prototype.changePriority = function(queueName, newPriority)
 	if (this.queues[queueName] !== undefined)
 		this.priorities[queueName] = newPriority;
 	var priorities = this.priorities;
-	this.queueArrays.sort(function (a,b) { return (priorities[b[0]] - priorities[a[0]]) });
+	this.queueArrays.sort((a,b) => priorities[b[0]] - priorities[a[0]]);
 };
 
 m.QueueManager.prototype.Serialize = function()
 {
 	var accounts = {};
 	var queues = {};
-	for (let p in this.queues)
+	for (let q in this.queues)
 	{
-		queues[p] = this.queues[p].Serialize();
-		accounts[p] = this.accounts[p].Serialize();
+		queues[q] = this.queues[q].Serialize();
+		accounts[q] = this.accounts[q].Serialize();
 		if (this.Config.debug == -100)
-			API3.warn("queueManager serialization: queue " + p + " >>> " + uneval(queues[p])
-				+ " with accounts " + uneval(accounts[p]));
+			API3.warn("queueManager serialization: queue " + q + " >>> " +
+				uneval(queues[q]) + " with accounts " + uneval(accounts[q]));
 	}
 
 	return {
@@ -603,7 +565,7 @@ m.QueueManager.prototype.Serialize = function()
 		"queues": queues,
 		"accounts": accounts
 	};
-}
+};
 
 m.QueueManager.prototype.Deserialize = function(gameState, data)
 {
@@ -613,15 +575,15 @@ m.QueueManager.prototype.Deserialize = function(gameState, data)
 
 	// the sorting is updated on priority change.
 	this.queueArrays = [];
-	for (let p in data.queues)
+	for (let q in data.queues)
 	{
-		this.queues[p] = new m.Queue();
-		this.queues[p].Deserialize(gameState, data.queues[p]);
-		this.accounts[p] = new API3.Resources();
-		this.accounts[p].Deserialize(data.accounts[p]);
-		this.queueArrays.push([p, this.queues[p]]);
+		this.queues[q] = new m.Queue();
+		this.queues[q].Deserialize(gameState, data.queues[q]);
+		this.accounts[q] = new API3.Resources();
+		this.accounts[q].Deserialize(data.accounts[q]);
+		this.queueArrays.push([q, this.queues[q]]);
 	}
-	this.queueArrays.sort(function (a,b) { return (data.priorities[b[0]] - data.priorities[a[0]]) });
+	this.queueArrays.sort((a,b) => data.priorities[b[0]] - data.priorities[a[0]]);
 };
 
 return m;
