@@ -11,7 +11,8 @@ m.TradeManager = function(Config)
 	this.tradeRoute = undefined;
 	this.potentialTradeRoute = undefined;
 	this.routeProspection = false;
-	this.targetNumTraders = Math.round(this.Config.popScaling * this.Config.Economy.targetNumTraders);
+	this.targetNumTraders = this.Config.Economy.targetNumTraders;
+	this.minimalGain = 3;
 	this.warnedAllies = {};
 };
 
@@ -35,7 +36,7 @@ m.TradeManager.prototype.assignTrader = function(ent)
 // TODO take trader ships into account
 m.TradeManager.prototype.trainMoreTraders = function(gameState, queues)
 {
-	if (!this.tradeRoute || queues.trader.countQueuedUnits())
+	if (!this.tradeRoute || queues.trader.hasQueuedUnits())
 		return;
 
 	var numTraders = this.traders.length;
@@ -53,18 +54,20 @@ m.TradeManager.prototype.trainMoreTraders = function(gameState, queues)
 				numLandTraders += item.count;
 		});
 	});
-        if (numTraders >= this.targetNumTraders
-		&& ((!this.tradeRoute.sea && numLandTraders >= Math.floor(this.targetNumTraders/2))
-		|| (this.tradeRoute.sea && numSeaTraders >= Math.floor(this.targetNumTraders/2))))
+        if (numTraders >= this.targetNumTraders &&
+		((!this.tradeRoute.sea && numLandTraders >= Math.floor(this.targetNumTraders/2)) ||
+		  (this.tradeRoute.sea && numSeaTraders >= Math.floor(this.targetNumTraders/2))))
 		return;
 
+	let template;
+	let metadata = { "role": "trader" };
 	if (this.tradeRoute.sea)
 	{
 		// if we have some merchand ships affected to transport, try first to reaffect them
 		// May-be, there were produced at an early stage when no other ship were available
 		// and the naval manager will train now more appropriate ships.
-		var already = false;
-		var shipToSwitch = undefined;
+		let already = false;
+		let shipToSwitch;
 		gameState.ai.HQ.navalManager.seaTransportShips[this.tradeRoute.sea].forEach(function(ship) {
 			if (already || !ship.hasClass("Trader"))
 				return;
@@ -86,16 +89,16 @@ m.TradeManager.prototype.trainMoreTraders = function(gameState, queues)
 			return;
 		}
 
-		var template = gameState.applyCiv("units/{civ}_ship_merchant");
-		var metadata = { "role": "trader", "sea": this.tradeRoute.sea };
+		template = gameState.applyCiv("units/{civ}_ship_merchant");
+		metadata.sea = this.tradeRoute.sea;
 	}
 	else
 	{
-		var template = gameState.applyCiv("units/{civ}_support_trader");
+		template = gameState.applyCiv("units/{civ}_support_trader");
 		if (!this.tradeRoute.source.hasClass("NavalMarket"))
-			var metadata = { "role": "trader", "base": this.tradeRoute.source.getMetadata(PlayerID, "base") };
+			metadata.base = this.tradeRoute.source.getMetadata(PlayerID, "base");
 		else
-			var metadata = { "role": "trader", "base": this.tradeRoute.target.getMetadata(PlayerID, "base") };
+			metadata.base = this.tradeRoute.target.getMetadata(PlayerID, "base");
 	}
 
 	if (!gameState.getTemplate(template))
@@ -104,19 +107,18 @@ m.TradeManager.prototype.trainMoreTraders = function(gameState, queues)
 			API3.warn("Petra error: trying to train " + template + " for civ " + gameState.civ() + " but no template found.");
 		return;
 	}
-	queues.trader.addItem(new m.TrainingPlan(gameState, template, metadata, 1, 1));
+	queues.trader.addPlan(new m.TrainingPlan(gameState, template, metadata, 1, 1));
 };
 
 m.TradeManager.prototype.updateTrader = function(gameState, ent)
 {
 	if (!this.tradeRoute || !ent.isIdle() || !ent.position())
 		return;
+	if (ent.getMetadata(PlayerID, "transport") !== undefined)
+		return;
 
 	Engine.ProfileStart("Trade Manager");
-	if (ent.hasClass("Ship"))
-		var access = ent.getMetadata(PlayerID, "sea");
-	else
-		var access = gameState.ai.accessibility.getAccessValue(ent.position());
+	var access = ent.hasClass("Ship") ? ent.getMetadata(PlayerID, "sea") : gameState.ai.accessibility.getAccessValue(ent.position());
 	var route = this.checkRoutes(gameState, access);
 	if (!route)
 	{
@@ -137,8 +139,11 @@ m.TradeManager.prototype.updateTrader = function(gameState, ent)
 			gameState.ai.HQ.navalManager.requireTransport(gameState, ent, access, route.land, route.source.position());
 		else
 			gameState.ai.HQ.navalManager.requireTransport(gameState, ent, access, route.land, route.target.position());
+		Engine.ProfileStop();
+		return;
 	}
-	else if (nearerSource)
+
+	if (nearerSource)
 		ent.tradeRoute(route.target, route.source);
 	else
 		ent.tradeRoute(route.source, route.target);
@@ -180,7 +185,7 @@ m.TradeManager.prototype.setTradingGoods = function(gameState)
 
 
 	// then add what is needed now
-	var mainNeed = Math.floor(remaining * 70 / 100)
+	var mainNeed = Math.floor(remaining * 70 / 100);
 	var nextNeed = remaining - mainNeed;
 
 	tradingGoods[mostNeeded[0].type] += mainNeed;
@@ -197,7 +202,7 @@ m.TradeManager.prototype.setTradingGoods = function(gameState)
 // only once per turn because the info doesn't update between a turn and fixing isn't worth it.
 m.TradeManager.prototype.performBarter = function(gameState)
 {
-	var barterers = gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClass("BarterMarket"), API3.Filters.not(API3.Filters.isFoundation()))).toEntityArray();
+	var barterers = gameState.getOwnEntitiesByClass("BarterMarket", true).filter(API3.Filters.isBuilt()).toEntityArray();
 	if (barterers.length === 0)
 		return false;
 
@@ -209,27 +214,28 @@ m.TradeManager.prototype.performBarter = function(gameState)
 
 	var prices = gameState.getBarterPrices();
 	// calculates conversion rates
-	var getBarterRate = function (prices,buy,sell) { return Math.round(100 * prices["sell"][sell] / prices["buy"][buy]); };
+	var getBarterRate = function (prices,buy,sell) { return Math.round(100 * prices.sell[sell] / prices.buy[buy]); };
 
 	// loop through each missing resource checking if we could barter and help finishing a queue quickly.
 	for (var buy of needs.types)
 	{
-		if (needs[buy] == 0 || needs[buy] < rates[buy]*30) // check if our rate allows to gather it fast enough
+		if (needs[buy] === 0 || needs[buy] < rates[buy]*30) // check if our rate allows to gather it fast enough
 			continue;
 
 		// pick the best resource to barter.
 		var bestToSell = undefined;
 		var bestRate = 0;
-		for (var sell of needs.types)
+		for (let sell of needs.types)
 		{
 			if (sell === buy)
 				continue;
 			if (needs[sell] > 0 || available[sell] < 500)    // do not sell if we need it or do not have enough buffer
 				continue;
 
+			let barterRateMin;
 			if (sell === "food")
 			{
-				var barterRateMin = 30;
+				barterRateMin = 30;
 				if (available[sell] > 40000)
 					barterRateMin = 0;
 				else if (available[sell] > 15000)
@@ -239,14 +245,14 @@ m.TradeManager.prototype.performBarter = function(gameState)
 			}
 			else 
 			{
-				var barterRateMin = 70;
+				barterRateMin = 70;
 				if (available[sell] > 1000)
 					barterRateMin = 50;
 				if (buy === "food")
 					barterRateMin += 20;
 			}
 
-			var barterRate = getBarterRate(prices, buy, sell);
+			let barterRate = getBarterRate(prices, buy, sell);
 			if (barterRate > bestRate && barterRate > barterRateMin)
 			{
 				bestRate = barterRate;
@@ -257,29 +263,29 @@ m.TradeManager.prototype.performBarter = function(gameState)
 		{
 			barterers[0].barter(buy, bestToSell, 100);
 			if (this.Config.debug > 2)
-				API3.warn("Necessity bartering: sold " + bestToSell +" for " + buy + " >> need sell " + needs[bestToSell]
-					 + " need buy " + needs[buy] + " rate buy " + rates[buy] + " available sell " + available[bestToSell]
-					 + " available buy " + available[buy] + " barterRate " + bestRate);
+				API3.warn("Necessity bartering: sold " + bestToSell +" for " + buy + " >> need sell " + needs[bestToSell] +
+					  " need buy " + needs[buy] + " rate buy " + rates[buy] + " available sell " + available[bestToSell] +
+					  " available buy " + available[buy] + " barterRate " + bestRate);
 			return true;
 		}
 	}
 
 	// now do contingency bartering, selling food to buy finite resources (and annoy our ennemies by increasing prices)
-	if (available["food"] < 1000 || needs["food"] > 0)
+	if (available.food < 1000 || needs.food > 0)
 		return false;
-	var bestToBuy = undefined;
+	var bestToBuy;
 	var bestChoice = 0;
-	for (var buy of needs.types)
+	for (let buy of needs.types)
 	{
 		if (buy === "food")
 			continue;
-		var barterRateMin = 80;
-		if (available[buy] < 5000 && available["food"] > 5000)
+		let barterRateMin = 80;
+		if (available[buy] < 5000 && available.food > 5000)
 			barterRateMin -= (20 - Math.floor(available[buy]/250));
-		var barterRate = getBarterRate(prices, buy, "food");
+		let barterRate = getBarterRate(prices, buy, "food");
 		if (barterRate < barterRateMin)
 			continue;
-		var choice = barterRate / (100 + available[buy]);
+		let choice = barterRate / (100 + available[buy]);
 		if (choice > bestChoice)
 		{
 			bestChoice = choice;
@@ -290,8 +296,8 @@ m.TradeManager.prototype.performBarter = function(gameState)
 	{
 		barterers[0].barter(bestToBuy, "food", 100);
 		if (this.Config.debug > 2)
-			API3.warn("Contingency bartering: sold food for " + bestToBuy + " available sell " + available["food"]
-				 + " available buy " + available[bestToBuy] + " barterRate " + getBarterRate(prices, bestToBuy, "food"));
+			API3.warn("Contingency bartering: sold food for " + bestToBuy + " available sell " + available.food +
+				  " available buy " + available[bestToBuy] + " barterRate " + getBarterRate(prices, bestToBuy, "food"));
 		return true;
 	}
 
@@ -301,8 +307,7 @@ m.TradeManager.prototype.performBarter = function(gameState)
 m.TradeManager.prototype.checkEvents = function(gameState, events)
 {
 	// check if one market from a traderoute is renamed, change the route accordingly
-	let renameEvents = events["EntityRenamed"];
-	for (let evt of renameEvents)
+	for (let evt of events.EntityRenamed)
 	{
 		let ent = gameState.getEntityById(evt.newentity);
 		if (!ent || !ent.hasClass("Market"))
@@ -322,22 +327,41 @@ m.TradeManager.prototype.checkEvents = function(gameState, events)
 		}
 	}
 
-	// if one market is destroyed or built, we may have to look for a better route
-	let destroyEvents = events["Destroy"];
-	for (let evt of destroyEvents)
+	// if one market is destroyed, we should look for a better route
+	for (let evt of events.Destroy)
 	{
 		if (!evt.entityObj)
 			continue;
 		let ent = evt.entityObj;
-		if (!ent || !ent.hasClass("Market") || !gameState.isPlayerAlly(ent.owner()))
+		if (!ent || ent.foundationProgress() !== undefined || !ent.hasClass("Market") || !gameState.isPlayerAlly(ent.owner()))
+			continue;
+		this.routeProspection = true;
+		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_market");
+		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_dock");
+		return true;
+	}
+
+	// same thing if one market is built
+	for (let evt of events.Create)
+	{
+		let ent = gameState.getEntityById(evt.entity);
+		if (!ent || ent.foundationProgress() !== undefined || !ent.hasClass("Market") || !gameState.isPlayerAlly(ent.owner()))
+			continue;
+		this.routeProspection = true;
+		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_market");
+		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_dock");
+		return true;
+	}
+
+
+	// and same thing for captured markets
+	for (let evt of events.OwnershipChanged)
+	{
+		if (!gameState.isPlayerAlly(evt.from) && !gameState.isPlayerAlly(evt.to))
+			continue;
+		let ent = gameState.getEntityById(evt.entity);
+		if (!ent || ent.foundationProgress() !== undefined || !ent.hasClass("Market"))
 			continue;
-		if (this.Config.debug > 1)
-		{
-			if (evt.SuccessfulFoundation)
-				API3.warn("new market build ... checking routes");
-			else
-				API3.warn("one market (or foundation) has been destroyed ... checking routes");
-		}
 		this.routeProspection = true;
 		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_market");
 		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_dock");
@@ -360,7 +384,7 @@ m.TradeManager.prototype.checkRoutes = function(gameState, accessIndex)
 		return false;
 	}
 
-	if (market2.length == 0)
+	if (!market2.length)
 		market2 = market1;
 	var candidate = { "gain": 0 };
 	var potential = { "gain": 0 };
@@ -385,6 +409,8 @@ m.TradeManager.prototype.checkRoutes = function(gameState, accessIndex)
 			if (!land && !sea)
 				continue;
 			var gain = Math.round(API3.SquareVectorDistance(m1.position(), m2.position()) / this.Config.distUnitGain);
+			if (gain < this.minimalGain)
+				continue;
 			if (m1.foundationProgress() === undefined && m2.foundationProgress() === undefined)
 			{
 				if (accessIndex)
@@ -442,7 +468,7 @@ m.TradeManager.prototype.checkRoutes = function(gameState, accessIndex)
 
 	if (this.Config.chat)
 	{
-		var owner = this.tradeRoute.source.owner();
+		let owner = this.tradeRoute.source.owner();
 		if (owner === PlayerID)
 			owner = this.tradeRoute.target.owner();
 		if (owner !== PlayerID && !this.warnedAllies[owner])
@@ -479,31 +505,22 @@ m.TradeManager.prototype.checkTrader = function(gameState, ent)
 		return;
 	}
 
-	if (ent.hasClass("Ship"))
-		var access = ent.getMetadata(PlayerID, "sea");
-	else
-		var access = gameState.ai.accessibility.getAccessValue(ent.position());
+	var access = ent.hasClass("Ship") ? ent.getMetadata(PlayerID, "sea") : gameState.ai.accessibility.getAccessValue(ent.position());
 	var possibleRoute = this.checkRoutes(gameState, access);
-	if (!possibleRoute)
+	// Warning:  presentRoute is from metadata, so contains entity ids
+	if (!possibleRoute ||
+		(possibleRoute.source.id() != presentRoute.source && possibleRoute.source.id() != presentRoute.target) ||
+		(possibleRoute.target.id() != presentRoute.source && possibleRoute.target.id() != presentRoute.target))
 	{
+		// Trader will be assigned in updateTrader
 		ent.stopMoving();
 		ent.setMetadata(PlayerID, "route", undefined);
-		return;
 	}
-
-	// Warning:  presentRoute is from metadata, so contains entity ids
-	if (possibleRoute && (
-		(possibleRoute.source.id() == presentRoute.source && possibleRoute.target.id() == presentRoute.target) ||
-		(possibleRoute.source.id() == presentRoute.target && possibleRoute.target.id() == presentRoute.source)))
-		return;
-
-	ent.stopMoving();
-	ent.setMetadata(PlayerID, "route", undefined);
 };
 
 m.TradeManager.prototype.prospectForNewMarket = function(gameState, queues)
 {
-	if (queues.economicBuilding.countQueuedUnitsWithClass("Market") + queues.dock.countQueuedUnitsWithClass("Market") > 0)
+	if (queues.economicBuilding.hasQueuedUnitsWithClass("Market") || queues.dock.hasQueuedUnitsWithClass("Market"))
 		return;
 	if (!gameState.ai.HQ.canBuild(gameState, "structures/{civ}_market"))
 		return;
@@ -515,7 +532,7 @@ m.TradeManager.prototype.prospectForNewMarket = function(gameState, queues)
 		return;
 	this.checkRoutes(gameState);
 	var marketPos = gameState.ai.HQ.findMarketLocation(gameState, template);
-	if (!marketPos || marketPos[3] == 0)   // marketPos[3] is the expected gain
+	if (!marketPos || marketPos[3] === 0)   // marketPos[3] is the expected gain
 	{	// no position found
 		gameState.ai.HQ.stopBuild(gameState, "structures/{civ}_market");
 		return;
@@ -527,19 +544,25 @@ m.TradeManager.prototype.prospectForNewMarket = function(gameState, queues)
 	if (this.Config.debug > 1)
 	{
 		if (this.potentialTradeRoute)
-			API3.warn("turn " + gameState.ai.playedTurn + "we could have a new route with gain "
-				+ marketPos[3] + " instead of the present " + this.potentialTradeRoute.gain);
+			API3.warn("turn " + gameState.ai.playedTurn + "we could have a new route with gain " +
+				marketPos[3] + " instead of the present " + this.potentialTradeRoute.gain);
 		else
-			API3.warn("turn " + gameState.ai.playedTurn + "we could have a first route with gain "
-				+ marketPos[3]);
+			API3.warn("turn " + gameState.ai.playedTurn + "we could have a first route with gain " +
+				marketPos[3]);
 	}
-	queues.economicBuilding.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_market"));
+
+	if (!this.tradeRoute)
+		gameState.ai.queueManager.changePriority("economicBuilding", 2*this.Config.priorities.economicBuilding);
+	let plan = new m.ConstructionPlan(gameState, "structures/{civ}_market");
+	if (!this.tradeRoute)
+		plan.onStart = function(gameState) { gameState.ai.queueManager.changePriority("economicBuilding", gameState.ai.Config.priorities.economicBuilding); };
+	queues.economicBuilding.addPlan(plan);
 };
 
 m.TradeManager.prototype.isNewMarketWorth = function(expectedGain)
 {
-	if (this.potentialTradeRoute && expectedGain < 2*this.potentialTradeRoute.gain
-		&& expectedGain < this.potentialTradeRoute.gain + 20)
+	if (this.potentialTradeRoute && expectedGain < 2*this.potentialTradeRoute.gain &&
+		expectedGain < this.potentialTradeRoute.gain + 20)
 		return false;
 	return true;
 };
@@ -562,11 +585,11 @@ m.TradeManager.prototype.update = function(gameState, events, queues)
 	if (this.tradeRoute)
 	{
 		this.traders.forEach(function(ent) { self.updateTrader(gameState, ent); });
-		if (gameState.ai.playedTurn % 5 == 0)
+		if (gameState.ai.playedTurn % 5 === 0)
 			this.trainMoreTraders(gameState, queues);
-		if (gameState.ai.playedTurn % 20 == 0 && this.traders.length >= 2)
+		if (gameState.ai.playedTurn % 20 === 0 && this.traders.length >= 2)
 			gameState.ai.HQ.researchManager.researchTradeBonus(gameState, queues);
-		if (gameState.ai.playedTurn % 60 == 0)
+		if (gameState.ai.playedTurn % 60 === 0)
 			this.setTradingGoods(gameState);
 	}
 };
@@ -600,7 +623,7 @@ m.TradeManager.prototype.Serialize = function()
 		"targetNumTraders": this.targetNumTraders,
 		"warnedAllies": this.warnedAllies
 	};
-}
+};
 
 m.TradeManager.prototype.Deserialize = function(gameState, data)
 {
@@ -609,7 +632,7 @@ m.TradeManager.prototype.Deserialize = function(gameState, data)
 	this.routeProspection = data.routeProspection;
 	this.targetNumTraders = data.targetNumTraders;
 	this.warnedAllies = data.warnedAllies;
-}
+};
 
 return m;
 }(PETRA);
