@@ -1,10 +1,3 @@
-/*
-	DESCRIPTION	: Generic utility functions.
-	NOTES	:
-*/
-
-// ====================================================================
-
 function getRandom(randomMin, randomMax)
 {
 	// Returns a random whole number in a min..max range.
@@ -15,8 +8,6 @@ function getRandom(randomMin, randomMax)
 	return Math.round(randomNum);
 }
 
-// ====================================================================
-
 // Get list of XML files in pathname with recursion, excepting those starting with _
 function getXMLFileList(pathname)
 {
@@ -40,9 +31,6 @@ function getXMLFileList(pathname)
 	return result;
 }
 
-// ====================================================================
-
-// Get list of JSON files in pathname
 function getJSONFileList(pathname)
 {
 	var files = Engine.BuildDirEntList(pathname, "*.json", false);
@@ -53,8 +41,6 @@ function getJSONFileList(pathname)
 	return files;
 }
 
-// ====================================================================
-
 // A sorting function for arrays of objects with 'name' properties, ignoring case
 function sortNameIgnoreCase(x, y)
 {
@@ -69,8 +55,6 @@ function sortNameIgnoreCase(x, y)
 		return 0;
 }
 
-// ====================================================================
-
 /**
  * Escape tag start and escape characters, so users cannot use special formatting.
  * Also limit string length to 256 characters (not counting escape characters).
@@ -83,99 +67,37 @@ function escapeText(text)
 	return text.substr(0, 255).replace(/\\/g, "\\\\").replace(/\[/g, "\\[");
 }
 
-// ====================================================================
-
-// Load default player data, for when it's not otherwise specified
-function initPlayerDefaults()
+/**
+ * Returns map description and preview image or placeholder.
+ */
+function getMapDescriptionAndPreview(mapType, mapName)
 {
-	var data = Engine.ReadJSONFile("simulation/data/player_defaults.json");
-	if (!data || !data.PlayerData)
-	{
-		error("Failed to parse player defaults in player_defaults.json (check for valid JSON data)");
-		return [];
-	}
+	var mapData;
+	if (mapType == "random" && mapName == "random")
+		mapData = { "settings": { "Description": translate("A randomly selected map.") } };
+	else if (mapType == "random" && Engine.FileExists(mapName + ".json"))
+		mapData = Engine.ReadJSONFile(mapName + ".json");
+	else if (Engine.FileExists(mapName + ".xml"))
+		mapData = Engine.LoadMapSettings(mapName + ".xml");
 
-	return data.PlayerData;
-}
-
-// ====================================================================
-
-// Load map size data
-function initMapSizes()
-{
-	var sizes = {
-		"shortNames":[],
-		"names":[],
-		"tiles": [],
-		"default": 0
+	return {
+		"description": mapData && mapData.settings && mapData.settings.Description ? translate(mapData.settings.Description) : translate("Sorry, no description available."),
+		"preview": mapData && mapData.settings && mapData.settings.Preview ? mapData.settings.Preview : "nopreview.png"
 	};
-
-	var data = Engine.ReadJSONFile("simulation/data/map_sizes.json");
-	if (!data || !data.Sizes)
-	{
-		error("Failed to parse map sizes in map_sizes.json (check for valid JSON data)");
-		return sizes;
-	}
-
-	translateObjectKeys(data, ["Name", "LongName"]);
-	for (var i = 0; i < data.Sizes.length; ++i)
-	{
-		sizes.shortNames.push(data.Sizes[i].Name);
-		sizes.names.push(data.Sizes[i].LongName);
-		sizes.tiles.push(data.Sizes[i].Tiles);
-
-		if (data.Sizes[i].Default)
-			sizes["default"] = i;
-	}
-
-	return sizes;
 }
 
-// ====================================================================
-
-// Load game speed data
-function initGameSpeeds()
+/**
+ * Sets the mappreview image correctly.
+ * It needs to be cropped as the engine only allows loading square textures.
+ *
+ * @param {string} guiObject
+ * @param {string} filename
+ */
+function setMapPreviewImage(guiObject, filename)
 {
-	var gameSpeeds = {
-		"names": [],
-		"speeds": [],
-		"default": 0
-	};
-
-	var data = Engine.ReadJSONFile("simulation/data/game_speeds.json");
-	if (!data || !data.Speeds)
-	{
-		error("Failed to parse game speeds in game_speeds.json (check for valid JSON data)");
-		return gameSpeeds;
-	}
-
-	translateObjectKeys(data, ["Name"]);
-	for (var i = 0; i < data.Speeds.length; ++i)
-	{
-		gameSpeeds.names.push(data.Speeds[i].Name);
-		gameSpeeds.speeds.push(data.Speeds[i].Speed);
-
-		if (data.Speeds[i].Default)
-			gameSpeeds["default"] = i;
-	}
-
-	return gameSpeeds;
+	Engine.GetGUIObjectByName(guiObject).sprite = "cropped:" + 400/512+ "," + 300/512 + ":session/icons/mappreview/" + filename;
 }
 
-
-// ====================================================================
-
-// Convert integer color values to string (for use in GUI objects)
-function rgbToGuiColor(color)
-{
-	if (color && ("r" in color) && ("g" in color) && ("b" in color))
-		return color.r + " " + color.g + " " + color.b;
-
-	return "0 0 0";
-}
-
-// ====================================================================
-
 /**
  * Convert time in milliseconds to [hh:]mm:ss string representation.
  * @param time Time period in milliseconds (integer)
@@ -190,8 +112,6 @@ function timeToString(time)
 	return Engine.FormatMillisecondsIntoDateString(time, format);
 }
 
-// ====================================================================
-
 function removeDupes(array)
 {
 	// loop backwards to make splice operations cheaper
@@ -203,24 +123,6 @@ function removeDupes(array)
 	}
 }
 
-// ====================================================================
-// "Inside-out" implementation of Fisher-Yates shuffle
-function shuffleArray(source)
-{
-	if (!source.length)
-		return [];
-
-	var result = [source[0]];
-	for (var i = 1; i < source.length; ++i)
-	{
-		var j = Math.floor(Math.random() * i);
-		result[i] = result[j];
-		result[j] = source[i];
-	}
-	return result;
-}
-
-// ====================================================================
 // Filter out conflicting characters and limit the length of a given name.
 // @param name Name to be filtered.
 // @param stripUnicode Whether or not to remove unicode characters.
@@ -257,7 +159,7 @@ function tryAutoComplete(text, autoCompleteList)
 		if (word.toLowerCase().indexOf(lastWord.toLowerCase()) != 0)
 			continue;
 		
-		text = wordSplit.join(" ")
+		text = wordSplit.join(" ");
 		if (text.length > 0)
 			text += " ";
 
@@ -284,3 +186,16 @@ function autoCompleteNick(guiName, playerList)
 	input.caption = newText + text.substring(bufferPosition);
 	input.buffer_position = bufferPosition + (newText.length - textTillBufferPosition.length);
 }
+
+function clearChatMessages()
+{
+	g_ChatMessages.length = 0;
+	Engine.GetGUIObjectByName("chatText").caption = "";
+
+	try {
+		for (let timer of g_ChatTimers)
+			clearTimeout(timer);
+		g_ChatTimers.length = 0;
+	} catch (e) {
+	}
+}
