@@ -7,7 +7,7 @@
 function cancelOnError(msg)
 {
 	// Delete game objects
-	endGame();
+	Engine.EndGame();
 	
 	// Return to pregame
 	Engine.SwitchGuiPage("page_pregame.xml");
@@ -18,12 +18,12 @@ function cancelOnError(msg)
 		Engine.PushGuiPage("page_msgbox.xml", {
 			width: 500,
 			height: 200,
-			message: '[font="serif-bold-18"]' + msg + '[/font]',
-			title: "Loading Aborted",
+			message: '[font="sans-bold-18"]' + msg + '[/font]',
+			title: translate("Loading Aborted"),
 			mode: 2
 		});
 	}
 	
 	// Reset cursor
-	setCursor("arrow-default");
+	Engine.SetCursor("arrow-default");
 }
