@@ -9,7 +9,7 @@ function generateLabel(metadata, engineInfo)
 	var dateString = sprintf(translate("\\[%(date)s]"), { date: dateTimeString });
 	if (engineInfo)
 	{
-		if (!hasSameVersion(metadata, engineInfo))
+		if (!hasSameSavegameVersion(metadata, engineInfo) || !hasSameEngineVersion(metadata, engineInfo))
 			dateString = "[color=\"red\"]" + dateString + "[/color]";
 		else if (!hasSameMods(metadata, engineInfo))
 			dateString = "[color=\"orange\"]" + dateString + "[/color]";
@@ -23,25 +23,38 @@ function generateLabel(metadata, engineInfo)
 /**
  * Check the version compatibility between the saved game to be loaded and the engine
  */
-function hasSameVersion(metadata, engineInfo)
+function hasSameSavegameVersion(metadata, engineInfo)
 {
-	return (metadata.version_major == engineInfo.version_major);
+	return metadata.version_major == engineInfo.version_major;
+}
+
+/**
+ * Check the version compatibility between the saved game to be loaded and the engine
+ */
+function hasSameEngineVersion(metadata, engineInfo)
+{
+	return metadata.engine_version && metadata.engine_version == engineInfo.engine_version;
 }
 
 /**
  * Check the mod compatibility between the saved game to be loaded and the engine
+ *
+ * @param metadata {string[]}
+ * @param engineInfo {string[]}
+ * @returns {boolean}
  */
 function hasSameMods(metadata, engineInfo)
 {
-	if (!metadata.mods)         // only here for backwards compatibility with previous saved games
-		var gameMods = [];
-	else
-		var gameMods = metadata.mods;
-
-	if (gameMods.length != engineInfo.mods.length)
+	if (!metadata.mods || !engineInfo.mods)
 		return false;
-	for (var i = 0; i < gameMods.length; ++i)
-		if (gameMods[i] != engineInfo.mods[i])
-			return false;
-	return true;
+
+	// Ignore the "user" mod which is loaded for releases but not working-copies
+	var modsA = metadata.mods.filter(mod => mod != "user");
+	var modsB = engineInfo.mods.filter(mod => mod != "user");
+
+	if (modsA.length != modsB.length)
+		return false;
+
+	// Mods must be loaded in the same order
+	return modsA.every((mod, index) => mod == modsB[index]);
 }
