@@ -4,6 +4,8 @@ var g_Time = Date.now();
 
 /**
  * Set a timeout to call func() after 'delay' msecs.
+ * func: function to call
+ * delay: delay in ms
  * Returns an id that can be passed to clearTimeout.
  */
 function setTimeout(func, delay)
@@ -13,11 +15,26 @@ function setTimeout(func, delay)
 	return id;
 }
 
+/**
+ * deletes a timer
+ * id: of the timer
+ */
 function clearTimeout(id)
 {
 	delete g_Timers[id];
 }
 
+/**
+* alters an function call
+* id: of the timer
+* func: function to call
+*/
+function setNewTimerFunction(id, func)
+{
+	if (id in g_Timers)
+		g_Timers[id][1] = func;
+}
+
 /**
  * If you want to use timers, then you must call this function regularly
  * (e.g. in a Tick handler)
@@ -45,7 +62,7 @@ function updateTimers()
 			t[1]();
 		} catch (e) {
 			var stack = e.stack.trimRight().replace(/^/mg, '  '); // indent the stack trace
-			error("Error in timer: "+e+"\n"+stack+"\n");
+			error(sprintf("Error in timer: %(error)s", { error: e })+"\n"+stack+"\n");
 		}
 		delete g_Timers[id];
 	}
