@@ -34,20 +34,14 @@ AIProxy.prototype.Init = function()
 {
 	this.changes = null;
 	this.needsFullGet = true;
-	// cache some data across turns
-	this.owner = -1;
-	
 	this.cmpAIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_AIInterface);
-
-	// Let the AIInterface know that we exist and that it should query us
-	this.NotifyChange();
 };
 
 AIProxy.prototype.Serialize = null; // we have no dynamic state to save
 
 AIProxy.prototype.Deserialize = function ()
 {
-	this.cmpAIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_AIInterface);
+	this.Init();
 };
 
 AIProxy.prototype.GetRepresentation = function()
@@ -55,14 +49,9 @@ AIProxy.prototype.GetRepresentation = function()
 	// Return the full representation the first time we're called
 	var ret;
 	if (this.needsFullGet)
-	{
 		ret = this.GetFullRepresentation();
-		this.needsFullGet = false;
-	}
 	else
-	{
 		ret = this.changes;
-	}
 
 	// Initialise changes to null instead of {}, to avoid memory allocations in the
 	// common case where there will be no changes; event handlers should each reset
@@ -74,18 +63,29 @@ AIProxy.prototype.GetRepresentation = function()
 
 AIProxy.prototype.NotifyChange = function()
 {
+	if (this.needsFullGet)
+	{
+		// not yet notified, be sure that the owner is set before doing so
+		// as the Create event is sent only on first ownership changed
+		let cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
+		if (!cmpOwnership || cmpOwnership.GetOwner() < 0)
+			return false;
+	}
+
 	if (!this.changes)
 	{
 		this.changes = {};
 		this.cmpAIInterface.ChangedEntity(this.entity);
 	}
+	return true;
 };
 
 // AI representation-updating event handlers:
 
 AIProxy.prototype.OnPositionChanged = function(msg)
 {
-	this.NotifyChange();
+	if (!this.NotifyChange())
+		return;
 
 	if (msg.inWorld)
 		this.changes.position = [msg.x, msg.z];
@@ -95,104 +95,118 @@ AIProxy.prototype.OnPositionChanged = function(msg)
 
 AIProxy.prototype.OnHealthChanged = function(msg)
 {
-	this.NotifyChange();
-
+	if (!this.NotifyChange())
+		return;
 	this.changes.hitpoints = msg.to;
 };
 
+AIProxy.prototype.OnCapturePointsChanged = function(msg)
+{
+	if (!this.NotifyChange())
+		return;
+	this.changes.capturePoints = msg.capturePoints;
+};
+
 AIProxy.prototype.OnUnitIdleChanged = function(msg)
 {
-	this.NotifyChange();
-
+	if (!this.NotifyChange())
+		return;
 	this.changes.idle = msg.idle;
 };
 
 AIProxy.prototype.OnUnitAIStateChanged = function(msg)
 {
-	this.NotifyChange();
-
+	if (!this.NotifyChange())
+		return;
 	this.changes.unitAIState = msg.to;
 };
 
 AIProxy.prototype.OnUnitAIOrderDataChanged = function(msg)
 {
-	this.NotifyChange();
-
+	if (!this.NotifyChange())
+		return;
 	this.changes.unitAIOrderData = msg.to;
 };
 
 AIProxy.prototype.OnProductionQueueChanged = function(msg)
 {
-	this.NotifyChange();
-
+	if (!this.NotifyChange())
+		return;
 	var cmpProductionQueue = Engine.QueryInterface(this.entity, IID_ProductionQueue);
 	this.changes.trainingQueue = cmpProductionQueue.GetQueue();
 };
 
 AIProxy.prototype.OnGarrisonedUnitsChanged = function(msg)
 {
-	this.NotifyChange();
+	if (!this.NotifyChange())
+		return;
 	
 	var cmpGarrisonHolder = Engine.QueryInterface(this.entity, IID_GarrisonHolder);
 	this.changes.garrisoned = cmpGarrisonHolder.GetEntities();
 
 	// Send a message telling a unit garrisoned or ungarrisoned.
 	// I won't check if the unit is still alive so it'll be up to the AI.
-	var added = msg.added;
-	var removed = msg.removed;
-	for each (var ent in added)
+	for (let ent of msg.added)
 		this.cmpAIInterface.PushEvent("Garrison", {"entity" : ent, "holder": this.entity});
-	for each (var ent in removed)
+	for (let ent of msg.removed)
 		this.cmpAIInterface.PushEvent("UnGarrison", {"entity" : ent, "holder": this.entity});
 };
 
 AIProxy.prototype.OnResourceSupplyChanged = function(msg)
 {
-	this.NotifyChange();
+	if (!this.NotifyChange())
+		return;
 	this.changes.resourceSupplyAmount = msg.to;
 };
 
-AIProxy.prototype.OnResourceSupplyGatherersChanged = function(msg)
+AIProxy.prototype.OnResourceSupplyNumGatherersChanged = function(msg)
 {
-	this.NotifyChange();
-	this.changes.resourceSupplyGatherers = msg.to;
+	if (!this.NotifyChange())
+		return;
+	this.changes.resourceSupplyNumGatherers = msg.to;
 };
 
 AIProxy.prototype.OnResourceCarryingChanged = function(msg)
 {
-	this.NotifyChange();
+	if (!this.NotifyChange())
+		return;
 	this.changes.resourceCarrying = msg.to;
 };
 
 AIProxy.prototype.OnFoundationProgressChanged = function(msg)
 {
-	this.NotifyChange();
+	if (!this.NotifyChange())
+		return;
 	this.changes.foundationProgress = msg.to;
 };
 
 AIProxy.prototype.OnFoundationBuildersChanged = function(msg)
 {
-	this.NotifyChange();
+	if (!this.NotifyChange())
+		return;
 	this.changes.foundationBuilders = msg.to;
 };
 
 AIProxy.prototype.OnTerritoryDecayChanged = function(msg)
 {
-	this.NotifyChange();
+	if (!this.NotifyChange())
+		return;
 	this.changes.decaying = msg.to;
+	this.cmpAIInterface.PushEvent("TerritoryDecayChanged", msg);
 };
 
 // TODO: event handlers for all the other things
 
 AIProxy.prototype.GetFullRepresentation = function()
 {
+	this.needsFullGet = false;
 	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 
 	var ret = {
 		// These properties are constant and won't need to be updated
 		"id": this.entity,
 		"template": cmpTemplateManager.GetCurrentTemplateName(this.entity)
-	}
+	};
 
 	var cmpPosition = Engine.QueryInterface(this.entity, IID_Position);
 	if (cmpPosition)
@@ -222,8 +236,6 @@ AIProxy.prototype.GetFullRepresentation = function()
 	{
 		// Updated by OnOwnershipChanged
 		ret.owner = cmpOwnership.GetOwner();
-		if (!this.owner)
-			this.owner = ret.owner;
 	}
 
 	var cmpUnitAI = Engine.QueryInterface(this.entity, IID_UnitAI);
@@ -256,7 +268,7 @@ AIProxy.prototype.GetFullRepresentation = function()
 	{
 		// Updated by OnResourceSupplyChanged
 		ret.resourceSupplyAmount = cmpResourceSupply.GetCurrentAmount();
-		ret.resourceSupplyGatherers = cmpResourceSupply.GetGatherers();
+		ret.resourceSupplyNumGatherers = cmpResourceSupply.GetNumGatherers();
 	}
 
 	var cmpResourceGatherer = Engine.QueryInterface(this.entity, IID_ResourceGatherer);
@@ -277,6 +289,10 @@ AIProxy.prototype.GetFullRepresentation = function()
 	if (cmpTerritoryDecay)
 		ret.decaying = cmpTerritoryDecay.IsDecaying();
 
+	var cmpCapturable = Engine.QueryInterface(this.entity, IID_Capturable);
+	if (cmpCapturable)
+		ret.capturePoints = cmpCapturable.GetCapturePoints();
+
 	return ret;
 };
 
@@ -296,15 +312,15 @@ AIProxy.prototype.OnOwnershipChanged = function(msg)
 	{
 		this.cmpAIInterface.PushEvent("Create", {"entity" : msg.entity});
 		return;
-	} else if (msg.to === -1)
+	}
+	else if (msg.to === -1)
 	{
 		this.cmpAIInterface.PushEvent("Destroy", {"entity" : msg.entity});
+		this.needsFullGet = true;
 		return;
 	}
 	
-	this.owner = msg.to;
 	this.changes.owner = msg.to;
-	
 	this.cmpAIInterface.PushEvent("OwnershipChanged", msg);
 };
 
@@ -313,15 +329,6 @@ AIProxy.prototype.OnAttacked = function(msg)
 	this.cmpAIInterface.PushEvent("Attacked", msg);
 };
 
-/*
- Deactivated for actually not really being practical for most uses.
- AIProxy.prototype.OnRangeUpdate = function(msg)
-{
-	msg.owner = this.owner;
-	this.cmpAIInterface.PushEvent("RangeUpdate", msg);
-	warn(uneval(msg));
-};*/
-
 AIProxy.prototype.OnConstructionFinished = function(msg)
 {
 	this.cmpAIInterface.PushEvent("ConstructionFinished", msg);
