@@ -1,6 +1,6 @@
 function Auras() {}
 
-var modificationSchema = 
+var modificationSchema =
 	"<element name='Modifications' a:help='Modification list'>" +
 		"<oneOrMore>" +
 			"<element a:help='Name of the value to modify'>" +
@@ -47,6 +47,11 @@ Auras.prototype.Schema =
 						"<text/>" +
 					"</element>" +
 				"</optional>" +
+				"<optional>" +
+					"<element name='OverlayIcon' a:help='Icon to show on the entities affected by this aura'>" +
+						"<text/>" +
+					"</element>" +
+				"</optional>" +
 				"<element name='Affects' a:help='Affected classes'>" +
 					"<text/>" +
 				"</element>" +
@@ -63,38 +68,36 @@ Auras.prototype.Init = function()
 {
 	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 	this.templateName = cmpTemplateManager.GetCurrentTemplateName(this.entity);
-	var auraNames = this.GetAuraNames();
 	this.auras = {};
 	this.affectedPlayers = {};
-	var cmpTechnologyTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager);
 	for (var name in this.template)
 	{
 		this.affectedPlayers[name] = []; // will be calculated on ownership change
-		var aura = {}
+		var aura = {};
 		aura.affects = this.template[name].Affects;
 		if (this.template[name].AffectedPlayers)
 			aura.affectedPlayers = this.template[name].AffectedPlayers.split(/\s+/);
-		aura.modifications = [];
-		for (var value in this.template[name].Modifications)
-		{
-			var mod = {};
-			mod.value = value.replace(/\./g, "/").replace(/\/\//g, ".");
-			if (this.template[name].Modifications[value].Add)
-				mod.add = +this.template[name].Modifications[value].Add;
-			else if (this.template[name].Modifications[value].Multiply)
-				mod.multiply = +this.template[name].Modifications[value].Multiply;
-			aura.modifications.push(mod);
-		}
 		this.auras[name] = aura;
 	}
+	// In case of autogarrisoning, this component can be called before ownership is set.
+	// So it needs to be completely initialised from the start.
+	this.Clean();
+};
+
+Auras.prototype.GetModifierIdentifier = function(name, mod)
+{
+		return this.templateName + "/" + name + "/" + mod.value;
 };
 
 Auras.prototype.GetDescriptions = function()
 {
-	var ret = {}
-	for each (var aura in this.template)
+	var ret = {};
+	for (let name in this.template)
+	{
+		let aura = this.template[name];
 		if (aura.AuraName)
 			ret[aura.AuraName] = aura.AuraDescription || null;
+	}
 	return ret;
 };
 
@@ -103,6 +106,16 @@ Auras.prototype.GetAuraNames = function()
 	return Object.keys(this.template);
 };
 
+Auras.prototype.GetOverlayIcon = function(name)
+{
+	return this.template[name].OverlayIcon || "";
+};
+
+Auras.prototype.GetAffectedEntities = function(name)
+{
+	return this[name].targetUnits;
+};
+
 Auras.prototype.GetRange = function(name)
 {
 	if (!this.IsRangeAura(name))
@@ -129,24 +142,17 @@ Auras.prototype.GetAffectedPlayers = function(name)
 
 Auras.prototype.CalculateAffectedPlayers = function(name)
 {
-	if (this.auras[name].affectedPlayers)
-		var affectedPlayers = this.auras[name].affectedPlayers;
-	else
-		var affectedPlayers = ["Player"];
-
+	var affectedPlayers = this.auras[name].affectedPlayers || ["Player"];
 	this.affectedPlayers[name] = [];
 
-	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
-
+	var cmpPlayer = QueryOwnerInterface(this.entity);
 	if (!cmpPlayer)
 		return;
 
-	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	var numPlayers = cmpPlayerManager.GetNumPlayers();
-
+	var numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
 	for (var i = 0; i < numPlayers; ++i)
 	{
-		for each (var p in affectedPlayers)
+		for (let p of affectedPlayers)
 		{
 			if (p == "Player" ? cmpPlayer.GetPlayerID() == i : cmpPlayer["Is" + p](i))
 			{
@@ -159,17 +165,17 @@ Auras.prototype.CalculateAffectedPlayers = function(name)
 
 Auras.prototype.HasFormationAura = function()
 {
-	return this.GetAuraNames().some(this.IsFormationAura.bind(this));
+	return this.GetAuraNames().some(n => this.IsFormationAura(n));
 };
 
 Auras.prototype.HasGarrisonAura = function()
 {
-	return this.GetAuraNames().some(this.IsGarrisonAura.bind(this));
+	return this.GetAuraNames().some(n => this.IsGarrisonAura(n));
 };
 
 Auras.prototype.HasGarrisonedUnitsAura = function()
 {
-	return this.GetAuraNames().some(this.IsGarrisonedUnitsAura.bind(this));
+	return this.GetAuraNames().some(n => this.IsGarrisonedUnitsAura(n));
 };
 
 Auras.prototype.GetType = function(name)
@@ -210,23 +216,46 @@ Auras.prototype.Clean = function()
 {
 	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	var auraNames = this.GetAuraNames();
+	let targetUnitsClone = {};
 	// remove all bonuses
-	for each (var name in auraNames)
+	for (let name of auraNames)
 	{
+		targetUnitsClone[name] = [];
 		if (!this[name])
 			continue;
 
+		if (this[name].targetUnits)
+			targetUnitsClone[name] = this[name].targetUnits.slice();
+
 		if (this.IsGlobalAura(name))
 			this.RemoveTemplateBonus(name);
 
-		for each(var ent in this[name].targetUnits)
-			this.RemoveBonus(name, ent);
+		this.RemoveBonus(name, this[name].targetUnits);
 
 		if (this[name].rangeQuery)
 			cmpRangeManager.DestroyActiveQuery(this[name].rangeQuery);
 	}
 
-	for each (var name in auraNames)
+	for (let name in this.template)
+	{
+		let modifications = [];
+		for (let value in this.template[name].Modifications)
+		{
+			let mod = {};
+			mod.value = value.replace(/\./g, "/").replace(/\/\//g, ".");
+			let templateModifications = this.template[name].Modifications[value];
+			if (templateModifications.Add)
+				mod.add = ApplyValueModificationsToEntity("Auras/"+name+"/Modifications/"+mod.value+"/Add",
+					+templateModifications.Add, this.entity);
+			else if (templateModifications.Multiply)
+				mod.multiply = ApplyValueModificationsToEntity("Auras/"+name+"/Modifications/"+mod.value+"/Multiply",
+					+templateModifications.Multiply, this.entity);
+			modifications.push(mod);
+		}
+		this.auras[name].modifications = modifications;
+	}
+
+	for (let name of auraNames)
 	{
 		// only calculate the affected players on re-applying the bonuses
 		// this makes sure the template bonuses are removed from the correct players
@@ -240,7 +269,10 @@ Auras.prototype.Clean = function()
 			continue;
 
 		if (!this.IsRangeAura(name))
+		{
+			this.ApplyBonus(name, targetUnitsClone[name]);
 			continue;
+		}
 
 		this[name].rangeQuery = cmpRangeManager.CreateActiveQuery(
 		    this.entity,
@@ -254,10 +286,14 @@ Auras.prototype.Clean = function()
 
 		if (this.IsGlobalAura(name))
 		{
-			// update stats in for all templates 
 			this.ApplyTemplateBonus(name, affectedPlayers);
+
 			// Add self to your own query for consistency with templates.
-			this.OnRangeUpdate({"tag":this[name].rangeQuery, "added":[this.entity], "removed":[]});
+			this.OnRangeUpdate({
+				"tag": this[name].rangeQuery,
+				"added": [this.entity],
+				"removed": []
+			});
 		}
 	}
 };
@@ -265,101 +301,44 @@ Auras.prototype.Clean = function()
 Auras.prototype.GiveMembersWithValidClass = function(auraName, entityList)
 {
 	var match = this.GetClasses(auraName);
-	var r = [];
-	for (var ent of entityList)
-	{
-		var cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
-		if (cmpIdentity && MatchesClassList(cmpIdentity.GetClassesList(), match))
-			r.push(ent);
-	}
-	return r;
-}
+	return entityList.filter(ent => {
+		let cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
+		return cmpIdentity && MatchesClassList(cmpIdentity.GetClassesList(), match);
+	});
+};
 
 Auras.prototype.OnRangeUpdate = function(msg)
 {
-	var auraNames = this.GetAuraNames();
-	for each (var n in auraNames)
+	var auraNames = this.GetAuraNames().filter(n => this[n] && msg.tag == this[n].rangeQuery);
+	for (let name of auraNames)
 	{
-		if (msg.tag == this[n].rangeQuery)
-		{
-			var name = n;
-			break;
-		}
+		this.ApplyBonus(name, msg.added);
+		this.RemoveBonus(name, msg.removed);
 	}
-
-	if (!name)
-		return;
-
-	var targetUnits = this[name].targetUnits;
-	var classes = this.GetClasses(name);
-
-	if (msg.added.length > 0)
-	{
-		var validList = this.GiveMembersWithValidClass(name, msg.added);
-		for each (var e in validList)
-		{
-			targetUnits.push(e);
-			this.ApplyBonus(name, e);
-		}
-	}
-
-	if (msg.removed.length > 0)
-	{
-		for each (var e in msg.removed)
-		{
-			targetUnits.splice(targetUnits.indexOf(e), 1);
-			this.RemoveBonus(name, e);
-		}
-	}
-
 };
 
 Auras.prototype.OnGarrisonedUnitsChanged = function(msg)
 {
-	var auraNames = this.GetAuraNames();
-	for each (var name in auraNames)
+	var auraNames = this.GetAuraNames().filter(n => this.IsGarrisonedUnitsAura(n));
+	for (let name of auraNames)
 	{
-		if (!this.IsGarrisonedUnitsAura(name))
-			continue;
-		for each (var ent in msg.added)
-			this.ApplyBonus(name, ent);
-		for each (var ent in msg.removed)
-			this.RemoveBonus(name, ent);
+		this.ApplyBonus(name, msg.added);
+		this.RemoveBonus(name, msg.removed);
 	}
 };
 
 Auras.prototype.ApplyFormationBonus = function(memberList)
 {
-	var auraNames = this.GetAuraNames();
-	for each (var name in auraNames)
-	{
-		if (!this.IsFormationAura(name))
-			continue;
-
-		var validList = this.GiveMembersWithValidClass(name, memberList);
-		for each (var ent in validList)
-		{
-			this[name].targetUnits.push(ent);
-			this.ApplyBonus(name,ent);
-		}
-	}
+	var auraNames = this.GetAuraNames().filter(n => this.IsFormationAura(n));
+	for (let name of auraNames)
+		this.ApplyBonus(name, memberList);
 };
 
 Auras.prototype.ApplyGarrisonBonus = function(structure)
 {
-	var auraNames = this.GetAuraNames();
-	for each (var name in auraNames)
-	{
-		if (!this.IsGarrisonAura(name))
-			continue;
-
-		var validList = this.GiveMembersWithValidClass(name, [structure]);
-		if (validList.length)
-		{
-			this[name].targetUnits.push(validList[0]);
-			this.ApplyBonus(name,validList[0]);
-		}
-	}
+	var auraNames = this.GetAuraNames().filter(n => this.IsGarrisonAura(n));
+	for (let name of auraNames)
+		this.ApplyBonus(name, [structure]);
 };
 
 Auras.prototype.ApplyTemplateBonus = function(name, players)
@@ -370,38 +349,23 @@ Auras.prototype.ApplyTemplateBonus = function(name, players)
 	var cmpAuraManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_AuraManager);
 	var classes = this.GetClasses(name);
 
-	for each (var mod in modifications)
-		for each (var player in players)
-			cmpAuraManager.ApplyTemplateBonus(mod.value, player, classes, mod, this.templateName + "/" + name + "/" + mod.value);
+	for (let mod of modifications)
+		for (let player of players)
+			cmpAuraManager.ApplyTemplateBonus(mod.value, player, classes, mod, this.GetModifierIdentifier(name, mod));
 };
 
 Auras.prototype.RemoveFormationBonus = function(memberList)
 {
-	var auraNames = this.GetAuraNames();
-	for each (var name in auraNames)
-	{
-		if (!this.IsFormationAura(name))
-			continue;
-
-		for each (var ent in memberList)
-		{
-			this.RemoveBonus(name,ent);
-			this[name].targetUnits.splice(this[name].targetUnits.indexOf(ent), 1);
-		}
-	}
+	var auraNames = this.GetAuraNames().filter(n => this.IsFormationAura(n));
+	for (let name of auraNames)
+		this.RemoveBonus(name, memberList);
 };
 
 Auras.prototype.RemoveGarrisonBonus = function(structure)
 {
-	var auraNames = this.GetAuraNames();
-	for each (var name in auraNames)
-	{
-		if (!this.IsGarrisonAura(name))
-			continue;
-
-		this.RemoveBonus(name,structure);
-		this[name].targetUnits.splice(this[name].targetUnits.indexOf(structure), 1);
-	}
+	var auraNames = this.GetAuraNames().filter(n => this.IsGarrisonAura(n));
+	for (let name of auraNames)
+		this.RemoveBonus(name, [structure]);
 };
 
 Auras.prototype.RemoveTemplateBonus = function(name)
@@ -412,29 +376,61 @@ Auras.prototype.RemoveTemplateBonus = function(name)
 	var modifications = this.GetModifications(name);
 	var cmpAuraManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_AuraManager);
 	var classes = this.GetClasses(name);
-	var players =  this.GetAffectedPlayers(name);
+	var players = this.GetAffectedPlayers(name);
 
-	for each (var mod in modifications)
-		for each (var player in players)
-			cmpAuraManager.RemoveTemplateBonus(mod.value, player, classes, this.templateName + "/" + name + "/" + mod.value);
+	for (let mod of modifications)
+		for (let player of players)
+			cmpAuraManager.RemoveTemplateBonus(mod.value, player, classes, this.GetModifierIdentifier(name, mod));
 };
 
-Auras.prototype.ApplyBonus = function(name, ent)
+Auras.prototype.ApplyBonus = function(name, ents)
 {
-	var modifications = this.GetModifications(name);
-	var cmpAuraManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_AuraManager);
-	for each (var mod in modifications)
-		cmpAuraManager.ApplyBonus(mod.value, ent, mod, this.templateName + "/" + name + "/" + mod.value);
+	var validEnts = this.GiveMembersWithValidClass(name, ents);
+	if (!validEnts.length)
+		return;
 
-};
-
-Auras.prototype.RemoveBonus = function(name, ent)
-{
+	this[name].targetUnits = this[name].targetUnits.concat(validEnts);
 	var modifications = this.GetModifications(name);
 	var cmpAuraManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_AuraManager);
 
-	for each (var mod in modifications)
-		cmpAuraManager.RemoveBonus(mod.value, ent, this.templateName + "/" + name + "/" + mod.value);
+	for (let mod of modifications)
+		cmpAuraManager.ApplyBonus(mod.value, validEnts, mod, this.GetModifierIdentifier(name, mod));
+
+	// update status bars if this has an icon
+	if (!this.GetOverlayIcon(name))
+		return;
+
+	for (let ent of validEnts)
+	{
+		var cmpStatusBars = Engine.QueryInterface(ent, IID_StatusBars);
+		if (cmpStatusBars)
+			cmpStatusBars.AddAuraSource(this.entity, name);
+	}
+};
+
+Auras.prototype.RemoveBonus = function(name, ents)
+{
+	var validEnts = this.GiveMembersWithValidClass(name, ents);
+	if (!validEnts.length)
+		return;
+
+	this[name].targetUnits = this[name].targetUnits.filter(v => validEnts.indexOf(v) == -1);
+	var modifications = this.GetModifications(name);
+	var cmpAuraManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_AuraManager);
+
+	for (let mod of modifications)
+		cmpAuraManager.RemoveBonus(mod.value, validEnts, this.GetModifierIdentifier(name, mod));
+
+	// update status bars if this has an icon
+	if (!this.GetOverlayIcon(name))
+		return;
+
+	for (let ent of validEnts)
+	{
+		var cmpStatusBars = Engine.QueryInterface(ent, IID_StatusBars);
+		if (cmpStatusBars)
+			cmpStatusBars.RemoveAuraSource(this.entity, name);
+	}
 };
 
 Auras.prototype.OnOwnershipChanged = function(msg)
