@@ -5,7 +5,7 @@ Builder.prototype.Schema =
 	"<a:example>" +
 		"<Rate>1.0</Rate>" +
 		"<Entities datatype='tokens'>" +
-			"\n    structures/{civ}_barracks\n    structures/{civ}_civil_centre\n    structures/celt_sb1\n  " +
+			"\n    structures/{civ}_barracks\n    structures/{civ}_civil_centre\n    structures/pers_apadana\n  " +
 		"</Entities>" +
 	"</a:example>" +
 	"<element name='Rate' a:help='Construction speed multiplier (1.0 is normal speed, higher values are faster)'>" +
@@ -37,7 +37,7 @@ Builder.prototype.GetEntitiesList = function()
 		entities = string.split(/\s+/);
 		
 		// Remove disabled entities
-		var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player)
+		var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
 		var disabledEntities = cmpPlayer.GetDisabledTemplates();
 		
 		for (var i = entities.length - 1; i >= 0; --i)
@@ -64,10 +64,10 @@ Builder.prototype.GetRange = function()
  */
 Builder.prototype.PerformBuilding = function(target)
 {
-	var rate = ApplyValueModificationsToEntity("Builder/Rate", +this.template.Rate, this.entity);
+	let rate = ApplyValueModificationsToEntity("Builder/Rate", +this.template.Rate, this.entity);
 
 	// If it's a foundation, then build it
-	var cmpFoundation = Engine.QueryInterface(target, IID_Foundation);
+	let cmpFoundation = Engine.QueryInterface(target, IID_Foundation);
 	if (cmpFoundation)
 	{
 		cmpFoundation.Build(this.entity, rate);
@@ -75,10 +75,10 @@ Builder.prototype.PerformBuilding = function(target)
 	}
 
 	// Otherwise try to repair it
-	var cmpHealth = Engine.QueryInterface(target, IID_Health);
-	if (cmpHealth)
+	let cmpRepairable = Engine.QueryInterface(target, IID_Repairable);
+	if (cmpRepairable)
 	{
-		cmpHealth.Repair(this.entity, rate);
+		cmpRepairable.Repair(this.entity, rate);
 		return;
 	}
 };
