@@ -21,6 +21,11 @@ EndGameManager.prototype.Init = function()
 	this.alliedVictory = true;
 };
 
+EndGameManager.prototype.GetGameType = function()
+{
+	return this.gameType;
+};
+
 EndGameManager.prototype.SetGameType = function(newGameType)
 {
 	this.gameType = newGameType;
@@ -43,9 +48,9 @@ EndGameManager.prototype.MarkPlayerAsWon = function(playerID)
 		if (cmpPlayer.GetState() != "active")
 			continue;
 		if (playerID == cmpPlayer.GetPlayerID() || this.alliedVictory && cmpPlayer.IsMutualAlly(playerID))
-			cmpPlayer.SetState("won")
+			cmpPlayer.SetState("won");
 		else
-			Engine.PostMessage(playerEntityId, MT_PlayerDefeated, { "playerId": i } );
+			Engine.PostMessage(playerEntityId, MT_PlayerDefeated, { "playerId": i, "skip": true } );
 	}
 
 	// Reveal the map to all players
@@ -58,64 +63,36 @@ EndGameManager.prototype.SetAlliedVictory = function(flag)
 	this.alliedVictory = flag;
 };
 
-/*
- * Check players the next turn. Avoids problems in Atlas, with promoting entities etc
- */
-EndGameManager.prototype.CheckConquestCriticalEntities = function()
+EndGameManager.prototype.OnGlobalPlayerDefeated = function(msg)
 {
-	if (this.timeout)
+	if (msg.skip)
 		return;
-	// wait a turn for actually checking the players
-	var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
-	this.timeout = cmpTimer.SetTimeout(this.entity, IID_EndGameManager, "CheckConquestCriticalEntitiesNow", 100, null);	
-};
 
-/*
- * Check players immediately. Might cause problems with converting/promoting entities.
- */
-EndGameManager.prototype.CheckConquestCriticalEntitiesNow = function()
-{
-	if (this.timeout)
-		this.timeout = null;
-	if (this.gameType == "endless")
-		return; 
-
-	// for all other game types, defeat that player
 	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	
-	// Ignore gaia
-	var numPlayers = cmpPlayerManager.GetNumPlayers();
 	var cmpPlayers = [];
-	
+
 	var allies = [];
 	var onlyAlliesLeft = true;
-	// If the player is currently active but needs to be defeated,
-	// mark that player as defeated
-	// cache the cmpPlayer instances of the other players and search the allies
-	for (var i = 1; i < numPlayers; i++)
+
+	var numPlayers = cmpPlayerManager.GetNumPlayers(); 
+
+	for (var i = 1; i < numPlayers; ++i)
 	{
-		// cmpPlayer should always exist for the player ids from 1 to numplayers
-		// so no tests on the existance of cmpPlayer are needed
-		var playerEntityId = cmpPlayerManager.GetPlayerByID(i);
-		cmpPlayers[i] = Engine.QueryInterface(playerEntityId, IID_Player);
-		if (cmpPlayers[i].GetState() != "active") 
+		cmpPlayers[i] = QueryPlayerIDInterface(i);
+		if (cmpPlayers[i].GetState() != "active" || i == msg.playerId) 
 			continue;
-		if (cmpPlayers[i].GetConquestCriticalEntitiesCount() == 0)
-			Engine.PostMessage(playerEntityId, MT_PlayerDefeated, { "playerId": i } );
+
+		if (!allies.length || cmpPlayers[allies[0]].IsMutualAlly(i))
+			allies.push(i);
 		else
-		{
-			if (!allies.length || cmpPlayers[allies[0]].IsMutualAlly(i))
-				allies.push(i);
-			else
-				onlyAlliesLeft = false;
-		}
+			onlyAlliesLeft = false;
 	}
 
 	// check if there are winners, or the game needs to continue
-	if (!allies.length || !onlyAlliesLeft || !(this.alliedVictory || allies.length == 1))
+	if (!allies.length || !onlyAlliesLeft || !this.alliedVictory)
 		return; 
 
-	for each (var p in allies)
+	for (var p of allies)
 		cmpPlayers[p].SetState("won");
 
 	// Reveal the map to all players
@@ -123,4 +100,4 @@ EndGameManager.prototype.CheckConquestCriticalEntitiesNow = function()
 	cmpRangeManager.SetLosRevealAll(-1, true);
 };
 
-Engine.RegisterComponentType(IID_EndGameManager, "EndGameManager", EndGameManager);
+Engine.RegisterSystemComponentType(IID_EndGameManager, "EndGameManager", EndGameManager);
