@@ -15,8 +15,8 @@ Fogging.prototype.Init = function()
 	this.miraged = [];
 	this.seen = [];
 
-	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	for (let player = 0; player < cmpPlayerManager.GetNumPlayers(); ++player)
+	let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
+	for (let player = 0; player < numPlayers; ++player)
 	{
 		this.mirages.push(INVALID_ENTITY);
 		this.miraged.push(false);
@@ -32,8 +32,8 @@ Fogging.prototype.Activate = function()
 	if (mustUpdate)
 	{
 		// Load a mirage for each player who has already seen the entity
-		let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-		for (let player = 0; player < cmpPlayerManager.GetNumPlayers(); ++player)
+		let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
+		for (let player = 0; player < numPlayers; ++player)
 		{
 			if (this.seen[player])
 				this.LoadMirage(player);
@@ -115,24 +115,19 @@ Fogging.prototype.LoadMirage = function(player)
 	// Store valuable information into the mirage component (especially for the GUI)
 	var cmpFoundation = Engine.QueryInterface(this.entity, IID_Foundation);
 	if (cmpFoundation)
-		cmpMirage.CopyFoundation(cmpFoundation.GetBuildPercentage());
+		cmpMirage.CopyFoundation(cmpFoundation);
 
 	var cmpHealth = Engine.QueryInterface(this.entity, IID_Health);
 	if (cmpHealth)
-		cmpMirage.CopyHealth(
-			cmpHealth.GetMaxHitpoints(), 
-			cmpHealth.GetHitpoints(), 
-			cmpHealth.IsRepairable() && (cmpHealth.GetHitpoints() < cmpHealth.GetMaxHitpoints())
-		);
+		cmpMirage.CopyHealth(cmpHealth);
+
+	var cmpCapturable = Engine.QueryInterface(this.entity, IID_Capturable);
+	if (cmpCapturable)
+		cmpMirage.CopyCapturable(cmpCapturable);
 
 	var cmpResourceSupply = Engine.QueryInterface(this.entity, IID_ResourceSupply);
 	if (cmpResourceSupply)
-		cmpMirage.CopyResourceSupply(
-			cmpResourceSupply.GetMaxAmount(), 
-			cmpResourceSupply.GetCurrentAmount(), 
-			cmpResourceSupply.GetType(), 
-			cmpResourceSupply.IsInfinite()
-		);
+		cmpMirage.CopyResourceSupply(cmpResourceSupply);
 
 	// Notify the GUI the entity has been replaced by a mirage, in case it is selected at this moment
 	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
@@ -154,7 +149,7 @@ Fogging.prototype.ForceMiraging = function(player)
 
 Fogging.prototype.IsMiraged = function(player)
 {
-	if (player >= this.mirages.length)
+	if (player < 0 || player >= this.mirages.length)
 		return false;
 
 	return this.miraged[player];
@@ -162,7 +157,7 @@ Fogging.prototype.IsMiraged = function(player)
 
 Fogging.prototype.GetMirage = function(player)
 {
-	if (player >= this.mirages.length)
+	if (player < 0 || player >= this.mirages.length)
 		return INVALID_ENTITY;
 
 	return this.mirages[player];
@@ -170,7 +165,7 @@ Fogging.prototype.GetMirage = function(player)
 
 Fogging.prototype.WasSeen = function(player)
 {
-	if (player >= this.seen.length)
+	if (player < 0 || player >= this.seen.length)
 		return false;
 
 	return this.seen[player];
@@ -178,9 +173,9 @@ Fogging.prototype.WasSeen = function(player)
 
 Fogging.prototype.OnDestroy = function(msg)
 {
+	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	for (var player = 0; player < this.mirages.length; ++player)
 	{
-		var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 		if (cmpRangeManager.GetLosVisibility(this.mirages[player], player) == "hidden")
 		{
 			Engine.DestroyEntity(this.mirages[player]);
@@ -202,7 +197,7 @@ Fogging.prototype.OnOwnershipChanged = function(msg)
 
 Fogging.prototype.OnVisibilityChanged = function(msg)
 {
-	if (msg.player >= this.mirages.length)
+	if (msg.player < 0 || msg.player >= this.mirages.length)
 		return;
 
 	if (msg.newVisibility == VIS_VISIBLE)
