@@ -187,23 +187,6 @@ GarrisonHolder.prototype.GetGarrisonedEntitiesCount = function()
 	return count;
 };
 
-/**
- * Get number of garrisoned units capable of shooting arrows
- * Not necessarily archers
- */
-GarrisonHolder.prototype.GetGarrisonedArcherCount = function(garrisonArrowClasses)
-{
-	var count = 0;
-	for each (var entity in this.entities)
-	{
-		var cmpIdentity = Engine.QueryInterface(entity, IID_Identity);
-		var classes = cmpIdentity.GetClassesList();
-		if (classes.some(function(c){return garrisonArrowClasses.indexOf(c) > -1;}))
-			count++;
-	}
-	return count;
-};
-
 /**
  * Checks if an entity can be allowed to garrison in the building
  * based on its class
@@ -224,8 +207,9 @@ GarrisonHolder.prototype.AllowedToGarrison = function(entity)
  * Garrison a unit inside.
  * Returns true if successful, false if not
  * The timer for AutoHeal is started here
+ * if vgpEntity is given, this visualGarrisonPoint will be used for the entity
  */
-GarrisonHolder.prototype.Garrison = function(entity)
+GarrisonHolder.prototype.Garrison = function(entity, vgpEntity)
 {
 	var cmpPosition = Engine.QueryInterface(entity, IID_Position);
 	if (!cmpPosition)
@@ -234,21 +218,29 @@ GarrisonHolder.prototype.Garrison = function(entity)
 	if (!this.PerformGarrison(entity))
 		return false;
 
-	var visiblyGarrisoned = false;
-	for (var vgp of this.visibleGarrisonPoints)
+	let visibleGarrisonPoint = vgpEntity;
+	if (!visibleGarrisonPoint)
 	{
-		if (vgp.entity)
-			continue;
-		vgp.entity = entity;
-		cmpPosition.SetTurretParent(this.entity, vgp.offset);
-		visiblyGarrisoned = true;
-		var cmpUnitAI = Engine.QueryInterface(entity, IID_UnitAI);
+		for (let vgp of this.visibleGarrisonPoints)
+		{
+			if (vgp.entity)
+				continue;
+			visibleGarrisonPoint = vgp;
+			break;
+		}
+	}
+
+	if (visibleGarrisonPoint)
+	{
+		visibleGarrisonPoint.entity = entity;
+		cmpPosition.SetTurretParent(this.entity, visibleGarrisonPoint.offset);
+		let cmpUnitAI = Engine.QueryInterface(entity, IID_UnitAI);
 		if (cmpUnitAI)
 			cmpUnitAI.SetTurretStance();
-		break;
 	}
-	if (!visiblyGarrisoned)
+	else
 		cmpPosition.MoveOutOfWorld();
+
 	return true;
 };
 
@@ -284,14 +276,14 @@ GarrisonHolder.prototype.PerformGarrison = function(entity)
 
 	var cmpAura = Engine.QueryInterface(entity, IID_Auras);
 	if (cmpAura && cmpAura.HasGarrisonAura())
-		cmpAura.ApplyGarrisonBonus(this.entity);	
+		cmpAura.ApplyGarrisonBonus(this.entity);
 
 	Engine.PostMessage(this.entity, MT_GarrisonedUnitsChanged, { "added" : [entity], "removed": [] });
-	
+
 	var cmpUnitAI = Engine.QueryInterface(entity, IID_UnitAI);
 	if (cmpUnitAI && cmpUnitAI.IsUnderAlert())
 		Engine.PostMessage(cmpUnitAI.GetAlertRaiser(), MT_UnitGarrisonedAfterAlert, {"holder": this.entity, "unit": entity});
-	
+
 	return true;
 };
 
@@ -306,7 +298,7 @@ GarrisonHolder.prototype.Eject = function(entity, forced)
 	// Error: invalid entity ID, usually it's already been ejected
 	if (entityIndex == -1)
 		return false; // Fail
-	
+
 	// Find spawning location
 	var cmpFootprint = Engine.QueryInterface(this.entity, IID_Footprint);
 	var cmpHealth = Engine.QueryInterface(this.entity, IID_Health);
@@ -331,10 +323,10 @@ GarrisonHolder.prototype.Eject = function(entity, forced)
 			return false;
 		}
 	}
-	
+
 	var cmpNewPosition = Engine.QueryInterface(entity, IID_Position);
 	this.entities.splice(entityIndex, 1);
-	
+
 	for (var vgp of this.visibleGarrisonPoints)
 	{
 		if (vgp.entity != entity)
@@ -357,15 +349,15 @@ GarrisonHolder.prototype.Eject = function(entity, forced)
 
 	var cmpAura = Engine.QueryInterface(entity, IID_Auras);
 	if (cmpAura && cmpAura.HasGarrisonAura())
-		cmpAura.RemoveGarrisonBonus(this.entity);	
+		cmpAura.RemoveGarrisonBonus(this.entity);
+
 
-	
 	cmpNewPosition.JumpTo(pos.x, pos.z);
 	cmpNewPosition.SetHeightOffset(0);
 	// TODO: what direction should they face in?
-	
+
 	Engine.PostMessage(this.entity, MT_GarrisonedUnitsChanged, { "added" : [], "removed": [entity] });
-	
+
 	return true;
 };
 
@@ -396,6 +388,8 @@ GarrisonHolder.prototype.OrderWalkToRallyPoint = function(entities)
 /**
  * Ejects units and orders them to move to the Rally Point.
  * Returns true if successful, false if not
+ * If an ejection with a given obstruction radius has failed, we won't try anymore to eject
+ * entities with a bigger obstruction as that is compelled to also fail 
  */
 GarrisonHolder.prototype.PerformEject = function(entities, forced)
 {
@@ -404,8 +398,17 @@ GarrisonHolder.prototype.PerformEject = function(entities, forced)
 
 	var ejectedEntities = [];
 	var success = true;
-	for each (var entity in entities)
+	var failedRadius;
+	for (var entity of entities)
 	{
+		if (failedRadius !== undefined)
+		{
+			var cmpObstruction = Engine.QueryInterface(entity, IID_Obstruction);
+			var radius = cmpObstruction ? cmpObstruction.GetUnitRadius() : 0;
+			if (radius >= failedRadius)
+				continue;
+		}
+
 		if (this.Eject(entity, forced))
 		{
 			var cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
@@ -414,7 +417,16 @@ GarrisonHolder.prototype.PerformEject = function(entities, forced)
 				ejectedEntities.push(entity);
 		}
 		else
+		{
 			success = false;
+			if (failedRadius !== undefined)
+				failedRadius = Math.min(failedRadius, radius);
+			else
+			{
+				var cmpObstruction = Engine.QueryInterface(entity, IID_Obstruction);
+				failedRadius = cmpObstruction ? cmpObstruction.GetUnitRadius() : 0;
+			}
+		}
 	}
 
 	this.OrderWalkToRallyPoint(ejectedEntities);
@@ -475,26 +487,16 @@ GarrisonHolder.prototype.UnloadTemplate = function(extendedTemplate, all, forced
 };
 
 /**
- * Unload all units with same owner as the entity
- * and order them to move to the Rally Point
+ * Unload all units, that belong to certain player
+ * and order all own units to move to the Rally Point
  * Returns true if all successful, false if not
  */
-GarrisonHolder.prototype.UnloadAllOwn = function(forced)
+GarrisonHolder.prototype.UnloadAllByOwner = function(owner, forced)
 {
-	var cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
-	if (!cmpOwnership)
-		return false;
-	var owner = cmpOwnership.GetOwner();
-
-	// Make copy of entity list
-	var entities = [];
-	for each (var entity in this.entities)
-	{
-		var cmpOwnership = Engine.QueryInterface(entity, IID_Ownership);
-		if (cmpOwnership && cmpOwnership.GetOwner() == owner)
-			entities.push(entity);
-	}
-	
+	var entities = this.entities.filter(ent => {
+		var cmpOwnership = Engine.QueryInterface(ent, IID_Ownership);
+		return cmpOwnership && cmpOwnership.GetOwner() == owner;
+	});
 	return this.PerformEject(entities, forced);
 };
 
@@ -527,7 +529,7 @@ GarrisonHolder.prototype.OnHealthChanged = function(msg)
  */
 GarrisonHolder.prototype.HasEnoughHealth = function()
 {
-	var cmpHealth = Engine.QueryInterface(this.entity, IID_Health)
+	var cmpHealth = Engine.QueryInterface(this.entity, IID_Health);
 	var hitpoints = cmpHealth.GetHitpoints();
 	var maxHitpoints = cmpHealth.GetMaxHitpoints();
 	var ejectHitpoints = Math.floor((+this.template.EjectHealth) * maxHitpoints);
@@ -638,8 +640,16 @@ GarrisonHolder.prototype.OnGlobalEntityRenamed = function(msg)
 	var entityIndex = this.entities.indexOf(msg.entity);
 	if (entityIndex != -1)
 	{
+		let vgpRenamed;
+		for (let vgp of this.visibleGarrisonPoints)
+		{
+			if (vgp.entity != msg.entity)
+				continue;
+			vgpRenamed = vgp;
+			break;
+		}
 		this.Eject(msg.entity);
-		this.Garrison(msg.newentity);
+		this.Garrison(msg.newentity, vgpRenamed);
 	}
 
 	if (!this.initGarrison)
@@ -666,12 +676,7 @@ GarrisonHolder.prototype.OnGlobalEntityRenamed = function(msg)
  */
 GarrisonHolder.prototype.OnDiplomacyChanged = function()
 {
-	var entities = []
-	for each (var entity in this.entities)
-	{
-		if (!IsOwnedByMutualAllyOfEntity(this.entity, entity))
-			entities.push(entity);
-	}
+	var entities = this.entities.filter(ent => !IsOwnedByMutualAllyOfEntity(this.entity, ent));
 	this.EjectOrKill(entities);
 };
 
@@ -687,12 +692,13 @@ GarrisonHolder.prototype.EjectOrKill = function(entities)
 	if (cmpPosition.IsInWorld())
 	{
 		var cmpGarrisonHolder = this;
-		var ejectables = entities.filter(function(ent) { return cmpGarrisonHolder.IsEjectable(ent) });
+		var ejectables = entities.filter(function(ent) { return cmpGarrisonHolder.IsEjectable(ent); });
 		if (ejectables.length)
 			this.PerformEject(ejectables, false);
 	}
 
 	// And destroy all remaining entities
+	var killedEntities = [];
 	for each (var entity in entities)
 	{
 		var entityIndex = this.entities.indexOf(entity);
@@ -702,9 +708,11 @@ GarrisonHolder.prototype.EjectOrKill = function(entities)
 		if (cmpHealth)
 			cmpHealth.Kill();
 		this.entities.splice(entityIndex, 1);
+		killedEntities.push(entity);
 	}
 
-	Engine.PostMessage(this.entity, MT_GarrisonedUnitsChanged, { "added" : [], "removed" : entities });
+	if (killedEntities.length > 0)
+		Engine.PostMessage(this.entity, MT_GarrisonedUnitsChanged, { "added" : [], "removed" : killedEntities });
 	this.UpdateGarrisonFlag();
 };
 
@@ -734,9 +742,8 @@ GarrisonHolder.prototype.OnGlobalInitGame = function(msg)
 	for (let ent of this.initGarrison)
 	{
 		let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
-		if (!cmpUnitAI || !cmpUnitAI.CanGarrison(this.entity))
-			continue;
-		this.Garrison(ent);
+		if (cmpUnitAI && cmpUnitAI.CanGarrison(this.entity))
+			this.Garrison(ent);
 	}
 	this.initGarrison = undefined;
 };
