@@ -5,6 +5,7 @@ Health.prototype.Schema =
 	"<a:example>" +
 		"<Max>100</Max>" +
 		"<RegenRate>1.0</RegenRate>" +
+		"<IdleRegenRate>0</IdleRegenRate>" +
 		"<DeathType>corpse</DeathType>" +
 	"</a:example>" +
 	"<element name='Max' a:help='Maximum hitpoints'>" +
@@ -23,6 +24,9 @@ Health.prototype.Schema =
 	"<element name='RegenRate' a:help='Hitpoint regeneration rate per second.'>" +
 		"<data type='decimal'/>" +
 	"</element>" +
+	"<element name='IdleRegenRate' a:help='Hitpoint regeneration rate per second when idle or garrisoned.'>" +
+		"<data type='decimal'/>" +
+	"</element>" +
 	"<element name='DeathType' a:help='Behaviour when the unit dies'>" +
 		"<choice>" +
 			"<value a:help='Disappear instantly'>vanish</value>" +
@@ -30,21 +34,22 @@ Health.prototype.Schema =
 			"<value a:help='Remain in the world with 0 health'>remain</value>" +
 		"</choice>" +
 	"</element>" +
-	"<element name='Unhealable' a:help='Indicates that the entity can not be healed by healer units'>" +
+	"<element name='Undeletable' a:help='Prevent players from deleting this entity.'>" +
 		"<data type='boolean'/>" +
 	"</element>" +
-	"<element name='Repairable' a:help='Indicates that the entity can be repaired by builder units'>" +
+	"<element name='Unhealable' a:help='Indicates that the entity can not be healed by healer units'>" +
 		"<data type='boolean'/>" +
 	"</element>";
 
 Health.prototype.Init = function()
 {
 	// Cache this value so it allows techs to maintain previous health level
-	this.maxHitpoints = +this.template.Max;
+	this.maxHitpoints = Math.round(+this.template.Max);
 	// Default to <Initial>, but use <Max> if it's undefined or zero
 	// (Allowing 0 initial HP would break our death detection code)
 	this.hitpoints = +(this.template.Initial || this.GetMaxHitpoints());
 	this.regenRate = ApplyValueModificationsToEntity("Health/RegenRate", +this.template.RegenRate, this.entity);
+	this.idleRegenRate = ApplyValueModificationsToEntity("Health/IdleRegenRate", +this.template.IdleRegenRate, this.entity);
 	this.CheckRegenTimer();
 };
 
@@ -92,7 +97,7 @@ Health.prototype.SetHitpoints = function(value)
 
 Health.prototype.IsRepairable = function()
 {
-	return (this.template.Repairable == "true");
+	return Engine.QueryInterface(this.entity, IID_Repairable) != null;
 };
 
 Health.prototype.IsUnhealable = function()
@@ -102,6 +107,16 @@ Health.prototype.IsUnhealable = function()
 		|| this.GetHitpoints() >= this.GetMaxHitpoints());
 };
 
+Health.prototype.IsUndeletable = function()
+{
+	return this.template.Undeletable == "true";
+};
+
+Health.prototype.GetIdleRegenRate = function()
+{
+	return this.idleRegenRate;
+};
+
 Health.prototype.GetRegenRate = function()
 {
 	return this.regenRate;
@@ -109,7 +124,14 @@ Health.prototype.GetRegenRate = function()
 
 Health.prototype.ExecuteRegeneration = function()
 {
-	var regen = this.GetRegenRate();
+	let regen = this.GetRegenRate();
+	if (this.GetIdleRegenRate() != 0)
+	{
+		let cmpUnitAI = Engine.QueryInterface(this.entity, IID_UnitAI);
+		if (cmpUnitAI && (cmpUnitAI.IsIdle() || cmpUnitAI.IsGarrisoned() && !cmpUnitAI.IsTurret()))
+			regen += this.GetIdleRegenRate();
+	}
+
 	if (regen > 0)
 		this.Increase(regen);
 	else 
@@ -122,9 +144,9 @@ Health.prototype.ExecuteRegeneration = function()
 Health.prototype.CheckRegenTimer = function()
 {
 	// check if we need a timer
-	if (this.GetRegenRate() == 0 ||
-		this.GetHitpoints() == this.GetMaxHitpoints() && this.GetRegenRate() > 0 ||
-		this.GetHitpoints() == 0)
+	if (this.GetRegenRate() == 0 && this.GetIdleRegenRate() == 0 ||
+	    this.GetHitpoints() == this.GetMaxHitpoints() && this.GetRegenRate() >= 0 && this.GetIdleRegenRate() >= 0 ||
+	    this.GetHitpoints() == 0)
 	{
 		// we don't need a timer, disable if one exists
 		if (this.regenTimer)
@@ -257,8 +279,8 @@ Health.prototype.CreateCorpse = function(leaveResources)
 
 	// Either creates a static local version of the current entity, or a
 	// persistent corpse retaining the ResourceSupply element of the parent.
-	var cmpTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	var templateName = cmpTempMan.GetCurrentTemplateName(this.entity);
+	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+	var templateName = cmpTemplateManager.GetCurrentTemplateName(this.entity);
 	var corpse;
 	if (leaveResources)
 		corpse = Engine.AddEntity("resource|" + templateName);
@@ -315,51 +337,27 @@ Health.prototype.CreateDeathSpawnedEntity = function()
 	return spawnedEntity;
 };
 
-Health.prototype.Repair = function(builderEnt, work)
-{
-	var damage = this.GetMaxHitpoints() - this.GetHitpoints();
-
-	// Do nothing if we're already at full hitpoints
-	if (damage <= 0)
-		return;
-
-	// Calculate the amount of hitpoints that will be added
-	// TODO: what computation should we use?
-	// TODO: should we do some diminishing returns thing? (see Foundation.Build)
-	var amount = Math.min(damage, work);
-
-	// TODO: resource costs?
-
-	// Add hitpoints
-	this.Increase(amount);
-
-	// If we repaired all the damage, send a message to entities
-	// to stop repairing this building
-	if (amount >= damage)
-	{
-		Engine.PostMessage(this.entity, MT_ConstructionFinished,
-			{ "entity": this.entity, "newentity": this.entity });
-	}
-};
-
 Health.prototype.OnValueModification = function(msg)
 {
 	if (msg.component != "Health")
 		return;
 
-	var oldMaxHitpoints = this.GetMaxHitpoints();
-	var newMaxHitpoints = Math.round(ApplyValueModificationsToEntity("Health/Max", +this.template.Max, this.entity));
+	let oldMaxHitpoints = this.GetMaxHitpoints();
+	let newMaxHitpoints = Math.round(ApplyValueModificationsToEntity("Health/Max", +this.template.Max, this.entity));
 	if (oldMaxHitpoints != newMaxHitpoints)
 	{
-		var newHitpoints = Math.round(this.GetHitpoints() * newMaxHitpoints/oldMaxHitpoints);
+		let newHitpoints = Math.round(this.GetHitpoints() * newMaxHitpoints/oldMaxHitpoints);
 		this.maxHitpoints = newMaxHitpoints;
 		this.SetHitpoints(newHitpoints);
 	}
 
-	var oldRegenRate = this.regenRate;
+	let oldRegenRate = this.regenRate;
 	this.regenRate = ApplyValueModificationsToEntity("Health/RegenRate", +this.template.RegenRate, this.entity);
-
-	if (this.regenRate != oldRegenRate)
+	
+	let oldIdleRegenRate = this.idleRegenRate;
+	this.idleRegenRate = ApplyValueModificationsToEntity("Health/IdleRegenRate", +this.template.IdleRegenRate, this.entity);
+	
+	if (this.regenRate != oldRegenRate || this.idleRegenRate != oldIdleRegenRate)
 		this.CheckRegenTimer();
 };
 
