@@ -3,14 +3,24 @@ function Identity() {}
 Identity.prototype.Schema =
 	"<a:help>Specifies various names and values associated with the unit type, typically for GUI display to users.</a:help>" +
 	"<a:example>" +
-		"<Civ>hele</Civ>" +
-		"<GenericName>Infantry Spearman</GenericName>" +
-		"<SpecificName>Hoplite</SpecificName>" +
-		"<Icon>units/hele_infantry_spearman.png</Icon>" +
+		"<Civ>athen</Civ>" +
+		"<GenericName>Athenian Hoplite</GenericName>" +
+		"<SpecificName>Hoplī́tēs Athēnaïkós</SpecificName>" +
+		"<Icon>units/athen_infantry_spearman.png</Icon>" +
 	"</a:example>" +
-	"<element name='Civ' a:help='Civilisation that this unit is primarily associated with, typically a 4-letter code. Choices include: gaia (world objects), cart (Carthaginians), celt (Celts), hele (Hellenes), iber (Iberians), pers (Persians), rome (Romans)'>" +
+	"<element name='Civ' a:help='Civilisation that this unit is primarily associated with, typically a 4-letter code. Choices include: gaia (world objects), athen (Athenians), brit (Britons), cart (Carthaginians), gaul (Gauls), iber (Iberians), mace (Macedonians), pers (Persians), ptol (Ptolemies), rome (Romans), sele (Seleucids), spart (Spartans)'>" +
 		"<text/>" +
 	"</element>" +
+	"<optional>" +
+		"<element name='Lang' a:help='Unit language for voices'>" +
+			"<text/>" +
+		"</element>" +
+	"</optional>" +
+	"<optional>" +
+		"<element name='Gender' a:help='Unit gender for voices. Choices includes male or female.'>" +
+			"<text/>" +
+		"</element>" +
+	"</optional>" +
 	"<element name='GenericName' a:help='Generic English-language name for this class of unit'>" +
 		"<text/>" +
 	"</element>" +
@@ -91,6 +101,14 @@ Identity.prototype.Schema =
 
 Identity.prototype.Init = function()
 {
+	// caching
+	this.classesList = GetIdentityClasses(this.template);
+	this.visibleClassesList = GetVisibleIdentityClasses(this.template);
+};
+
+Identity.prototype.Deserialize = function ()
+{
+	this.Init();
 };
 
 Identity.prototype.Serialize = null; // we have no dynamic state to save
@@ -100,6 +118,16 @@ Identity.prototype.GetCiv = function()
 	return this.template.Civ;
 };
 
+Identity.prototype.GetLang = function()
+{
+	return this.template.Lang || "greek"; // ugly default
+};
+
+Identity.prototype.GetGender = function()
+{
+	return this.template.Gender || "male"; // ugly default
+};
+
 Identity.prototype.GetRank = function()
 {
 	return (this.template.Rank || "");
@@ -107,12 +135,12 @@ Identity.prototype.GetRank = function()
 
 Identity.prototype.GetClassesList = function()
 {
-	return GetIdentityClasses(this.template);
+	return this.classesList;
 };
 
 Identity.prototype.GetVisibleClassesList = function()
 {
-	return GetVisibleIdentityClasses(this.template);
+	return this.visibleClassesList;
 };
 
 Identity.prototype.HasClass = function(name)
