@@ -121,6 +121,7 @@ Pack.prototype.PackProgress = function(data, lateness)
 		this.CancelTimer();
 
 		this.packed = !this.packed;
+		this.packing = false;
 		Engine.PostMessage(this.entity, MT_PackFinished, { packed: this.packed });
 
 		// Done un/packing, copy our parameters to the final entity
@@ -148,6 +149,16 @@ Pack.prototype.PackProgress = function(data, lateness)
 		var cmpNewOwnership = Engine.QueryInterface(newEntity, IID_Ownership);
 		cmpNewOwnership.SetOwner(cmpOwnership.GetOwner());
 
+		// rescale capture points
+		var cmpCapturable = Engine.QueryInterface(this.entity, IID_Capturable);
+		var cmpNewCapturable = Engine.QueryInterface(newEntity, IID_Capturable);
+		if (cmpCapturable && cmpNewCapturable)
+		{
+			let scale = cmpCapturable.GetMaxCapturePoints() / cmpNewCapturable.GetMaxCapturePoints();
+			let newCp = cmpCapturable.GetCapturePoints().map(function (v) { return v / scale; });
+			cmpNewCapturable.SetCapturePoints(newCp);
+		}
+
 		// Maintain current health level
 		var cmpHealth = Engine.QueryInterface(this.entity, IID_Health);
 		var cmpNewHealth = Engine.QueryInterface(newEntity, IID_Health);
