@@ -1,14 +1,19 @@
 function Player() {}
 
 Player.prototype.Schema =
-	"<a:component type='system'/><empty/>";
+	"<element name='SharedLosTech' a:help='Allies will share los when this technology is researched. Leave empty to never share LOS.'>" +
+		"<text/>" +
+	"</element>" +
+	"<element name='SharedDropsitesTech' a:help='Allies will share dropsites when this technology is researched. Leave empty to never share dropsites.'>" +
+		"<text/>" +
+	"</element>";
 
 Player.prototype.Init = function()
 {
 	this.playerID = undefined;
 	this.name = undefined;	// define defaults elsewhere (supporting other languages)
 	this.civ = undefined;
-	this.colour = { "r": 0.0, "g": 0.0, "b": 0.0, "a": 1.0 };
+	this.color = { "r": 0.0, "g": 0.0, "b": 0.0, "a": 1.0 };
 	this.popUsed = 0; // population of units owned or trained by this player
 	this.popBonuses = 0; // sum of population bonuses of player's entities
 	this.maxPop = 300; // maximum population
@@ -19,20 +24,23 @@ Player.prototype.Init = function()
 		"metal": 300,
 		"stone": 300
 	};
-	this.tradingGoods = [                      // goods for next trade-route and its proba in % (the sum of probas must be 100)
+	// goods for next trade-route and its proba in % (the sum of probas must be 100)
+	this.tradingGoods = [
 		{ "goods":  "wood", "proba": 30 },
 		{ "goods": "stone", "proba": 35 },
-		{ "goods": "metal", "proba": 35 } ];
+		{ "goods": "metal", "proba": 35 },
+	];
 	this.team = -1;	// team number of the player, players on the same team will always have ally diplomatic status - also this is useful for team emblems, scoring, etc.
 	this.teamsLocked = false;
 	this.state = "active"; // game state - one of "active", "defeated", "won"
 	this.diplomacy = [];	// array of diplomatic stances for this player with respect to other players (including gaia and self)
-	this.conquestCriticalEntitiesCount = 0; // number of owned units with ConquestCritical class
+	this.sharedDropsites = false;
 	this.formations = [];
 	this.startCam = undefined;
 	this.controlAllUnits = false;
 	this.isAI = false;
 	this.gatherRateMultiplier = 1;
+	this.tradeRateMultiplier = 1;
 	this.cheatsEnabled = false;
 	this.cheatTimeMultiplier = 1;
 	this.heroes = [];
@@ -41,10 +49,10 @@ Player.prototype.Init = function()
 		"wood": markForTranslation("Wood"),
 		"metal": markForTranslation("Metal"),
 		"stone": markForTranslation("Stone"),
-	}
+	};
 	this.disabledTemplates = {};
 	this.disabledTechnologies = {};
-	this.startingTechnologies = {};
+	this.startingTechnologies = [];
 };
 
 Player.prototype.SetPlayerID = function(id)
@@ -69,7 +77,17 @@ Player.prototype.GetName = function()
 
 Player.prototype.SetCiv = function(civcode)
 {
+	var oldCiv = this.civ;
 	this.civ = civcode;
+	// Normally, the civ is only set once
+	// But in Atlas, the map designers can change civs at any time
+	var playerID = this.GetPlayerID();
+	if (oldCiv && playerID && oldCiv != civcode)
+		Engine.BroadcastMessage(MT_CivChanged, {
+			"player": playerID,
+			"from": oldCiv,
+			"to": civcode
+		});
 };
 
 Player.prototype.GetCiv = function()
@@ -77,14 +95,14 @@ Player.prototype.GetCiv = function()
 	return this.civ;
 };
 
-Player.prototype.SetColour = function(r, g, b)
+Player.prototype.SetColor = function(r, g, b)
 {
-	this.colour = { "r": r/255.0, "g": g/255.0, "b": b/255.0, "a": 1.0 };
+	this.color = { "r": r/255.0, "g": g/255.0, "b": b/255.0, "a": 1.0 };
 };
 
-Player.prototype.GetColour = function()
+Player.prototype.GetColor = function()
 {
-	return this.colour;
+	return this.color;
 };
 
 // Try reserving num population slots. Returns 0 on success or number of missing slots otherwise.
@@ -107,6 +125,11 @@ Player.prototype.GetPopulationCount = function()
 	return this.popUsed;
 };
 
+Player.prototype.AddPopulation = function(num)
+{
+	this.popUsed += num;
+};
+
 Player.prototype.SetPopulationBonuses = function(num)
 {
 	this.popBonuses = num;
@@ -129,7 +152,7 @@ Player.prototype.SetMaxPopulation = function(max)
 
 Player.prototype.GetMaxPopulation = function()
 {
-	return Math.round(ApplyValueModificationsToPlayer("Player/MaxPopulation", this.maxPop, this.entity));
+	return Math.round(ApplyValueModificationsToPlayer("Player/MaxPopulation", this.maxPop, this.entity, this.playerID));
 };
 
 Player.prototype.SetGatherRateMultiplier = function(value)
@@ -142,6 +165,16 @@ Player.prototype.GetGatherRateMultiplier = function()
 	return this.gatherRateMultiplier;
 };
 
+Player.prototype.SetTradeRateMultiplier = function(value)
+{
+	this.tradeRateMultiplier = value;
+};
+
+Player.prototype.GetTradeRateMultiplier = function()
+{
+	return this.tradeRateMultiplier;
+};
+
 Player.prototype.GetHeroes = function()
 {
 	return this.heroes;
@@ -186,7 +219,7 @@ Player.prototype.GetResourceCounts = function()
  */
 Player.prototype.AddResource = function(type, amount)
 {
-	this.resourceCount[type] += (+amount);
+	this.resourceCount[type] += +amount;
 };
 
 /**
@@ -196,7 +229,7 @@ Player.prototype.AddResources = function(amounts)
 {
 	for (var type in amounts)
 	{
-		this.resourceCount[type] += (+amounts[type]);
+		this.resourceCount[type] += +amounts[type];
 	}
 };
 
@@ -224,7 +257,7 @@ Player.prototype.SubtractResourcesOrNotify = function(amounts)
 		var i = 0;
 		for (var type in amountsNeeded)
 		{
-			i++;
+			++i;
 			parameters["resourceType"+i] = this.resourceNames[type];
 			parameters["resourceAmount"+i] = amountsNeeded[type];
 		}
@@ -245,7 +278,9 @@ Player.prototype.SubtractResourcesOrNotify = function(amounts)
 		else
 			warn("Localisation: Strings are not localised for more than 4 resources");
 
-		var notification = {
+		// Send as time-notification
+		let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+		cmpGUIInterface.PushNotification({
 			"players": [this.playerID],
 			"message": msg,
 			"parameters": parameters,
@@ -256,9 +291,7 @@ Player.prototype.SubtractResourcesOrNotify = function(amounts)
 				"resourceType3": "withinSentence",
 				"resourceType4": "withinSentence",
 			},
-		};
-		var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGUIInterface.PushNotification(notification);
+		});
 		return false;
 	}
 
@@ -299,7 +332,7 @@ Player.prototype.GetNextTradingGoods = function()
 Player.prototype.GetTradingGoods = function()
 {
 	var tradingGoods = {};
-	for each (var resource in this.tradingGoods)
+	for (let resource of this.tradingGoods)
 		tradingGoods[resource.goods] = resource.proba;
 
 	return tradingGoods;
@@ -331,11 +364,6 @@ Player.prototype.SetState = function(newState)
 	this.state = newState;
 };
 
-Player.prototype.GetConquestCriticalEntitiesCount = function()
-{
-	return this.conquestCriticalEntitiesCount;
-};
-
 Player.prototype.GetTeam = function()
 {
 	return this.team;
@@ -343,27 +371,25 @@ Player.prototype.GetTeam = function()
 
 Player.prototype.SetTeam = function(team)
 {
-	if (!this.teamsLocked)
-	{
-		this.team = team;
+	if (this.teamsLocked)
+		return;
 
-		var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-		if (cmpPlayerManager && this.team != -1)
+	this.team = team;
+
+	// Set all team members as allies
+	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+	if (cmpPlayerManager && this.team != -1)
+		for (let i = 0; i < cmpPlayerManager.GetNumPlayers(); ++i)
 		{
-			// Set all team members as allies
-			for (var i = 0; i < cmpPlayerManager.GetNumPlayers(); ++i)
-			{
-				var cmpPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(i), IID_Player);
-				if (this.team == cmpPlayer.GetTeam())
-				{
-					this.SetAlly(i);
-					cmpPlayer.SetAlly(this.playerID);
-				}
-			}
+			let cmpPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(i), IID_Player);
+			if (this.team != cmpPlayer.GetTeam())
+				continue;
+
+			this.SetAlly(i);
+			cmpPlayer.SetAlly(this.playerID);
 		}
 
-		Engine.BroadcastMessage(MT_DiplomacyChanged, {"player": this.playerID});
-	}
+	Engine.BroadcastMessage(MT_DiplomacyChanged, { "player": this.playerID });
 };
 
 Player.prototype.SetLockTeams = function(value)
@@ -383,9 +409,8 @@ Player.prototype.GetDiplomacy = function()
 
 Player.prototype.SetDiplomacy = function(dipl)
 {
-	// Should we check for teamsLocked here?
 	this.diplomacy = dipl;
-	Engine.BroadcastMessage(MT_DiplomacyChanged, {"player": this.playerID});
+	Engine.BroadcastMessage(MT_DiplomacyChanged, { "player": this.playerID });
 };
 
 Player.prototype.SetDiplomacyIndex = function(idx, value)
@@ -401,54 +426,28 @@ Player.prototype.SetDiplomacyIndex = function(idx, value)
 	if (this.state != "active" || cmpPlayer.state != "active")
 		return;
 
-	// You can have alliances with other players,
-	if (this.teamsLocked)
-	{
-		// but can't stab your team members in the back
-		if (this.team == -1 || this.team != cmpPlayer.GetTeam())
-		{
-			// Break alliance or declare war
-			if (Math.min(this.diplomacy[idx],cmpPlayer.diplomacy[this.playerID]) > value)
-			{
-				this.diplomacy[idx] = value;
-				cmpPlayer.SetDiplomacyIndex(this.playerID, value);
-			}
-			else
-			{
-				this.diplomacy[idx] = value;
-			}
-			Engine.BroadcastMessage(MT_DiplomacyChanged, {"player": this.playerID});
-		}
-	}
-	else
-	{
-		// Break alliance or declare war (worsening of relations is mutual)
-		if (Math.min(this.diplomacy[idx],cmpPlayer.diplomacy[this.playerID]) > value)
-		{
-			// This is duplicated because otherwise we get too much recursion
-			this.diplomacy[idx] = value;
-			cmpPlayer.SetDiplomacyIndex(this.playerID, value);
-		}
-		else
-		{
-			this.diplomacy[idx] = value;
-		}
+	this.diplomacy[idx] = value;
+	Engine.BroadcastMessage(MT_DiplomacyChanged, { "player": this.playerID });
 
-		Engine.BroadcastMessage(MT_DiplomacyChanged, {"player": this.playerID});
-	}
+	// Mutual worsening of relations
+	if (cmpPlayer.diplomacy[this.playerID] > value)
+		cmpPlayer.SetDiplomacyIndex(this.playerID, value);
 };
 
 Player.prototype.UpdateSharedLos = function()
 {
-	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-	if (!cmpRangeManager)
+	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+	let cmpTechnologyManager = Engine.QueryInterface(this.entity, IID_TechnologyManager);
+	if (!cmpRangeManager || !cmpPlayerManager || !cmpTechnologyManager)
 		return;
 
-	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	if (!cmpPlayerManager)
+	let sharedLos = [];
+	if (!cmpTechnologyManager.IsTechnologyResearched(this.template.SharedLosTech))
+	{
+		cmpRangeManager.SetSharedLos(this.playerID, [this.playerID]);
 		return;
-
-	var sharedLos = [];
+	}
 	for (var i = 0; i < cmpPlayerManager.GetNumPlayers(); ++i)
 		if (this.IsMutualAlly(i))
 			sharedLos.push(i);
@@ -478,12 +477,17 @@ Player.prototype.GetStartingCameraRot = function()
 
 Player.prototype.SetStartingCamera = function(pos, rot)
 {
-	this.startCam = {"position": pos, "rotation": rot};
+	this.startCam = { "position": pos, "rotation": rot };
 };
 
 Player.prototype.HasStartingCamera = function()
 {
-	return (this.startCam !== undefined);
+	return this.startCam !== undefined;
+};
+
+Player.prototype.HasSharedDropsites = function()
+{
+	return this.sharedDropsites;
 };
 
 Player.prototype.SetControlAllUnits = function(c)
@@ -524,11 +528,7 @@ Player.prototype.IsAlly = function(id)
  */
 Player.prototype.IsMutualAlly = function(id)
 {
-	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	if (!cmpPlayerManager)
-		return false;
-
-	var cmpPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(id), IID_Player);
+	var cmpPlayer = QueryPlayerIDInterface(id);
 	return this.IsAlly(id) && cmpPlayer && cmpPlayer.IsAlly(this.playerID);
 };
 
@@ -577,7 +577,7 @@ Player.prototype.OnGlobalInitGame = function(msg)
 {
 	let cmpTechnologyManager = Engine.QueryInterface(this.entity, IID_TechnologyManager);
 	if (cmpTechnologyManager)
-		for (let tech in this.startingTechnologies)
+		for (let tech of this.startingTechnologies)
 			cmpTechnologyManager.ResearchTechnology(tech);
 
 	// Replace the "{civ}" code with this civ ID
@@ -599,13 +599,9 @@ Player.prototype.OnGlobalOwnershipChanged = function(msg)
 
 	var cmpIdentity = Engine.QueryInterface(msg.entity, IID_Identity);
 	var cmpCost = Engine.QueryInterface(msg.entity, IID_Cost);
-	var cmpFoundation = Engine.QueryInterface(msg.entity, IID_Foundation);
 
 	if (msg.from == this.playerID)
 	{
-		if (!cmpFoundation && cmpIdentity && cmpIdentity.HasClass("ConquestCritical"))
-			this.conquestCriticalEntitiesCount--;
-
 		if (cmpCost)
 			this.popUsed -= cmpCost.GetPopCost();
 
@@ -619,9 +615,6 @@ Player.prototype.OnGlobalOwnershipChanged = function(msg)
 	}
 	if (msg.to == this.playerID)
 	{
-		if (!cmpFoundation && cmpIdentity && cmpIdentity.HasClass("ConquestCritical"))
-			this.conquestCriticalEntitiesCount++;
-
 		if (cmpCost)
 			this.popUsed += cmpCost.GetPopCost();
 
@@ -634,8 +627,6 @@ Player.prototype.OnPlayerDefeated = function(msg)
 {
 	this.state = "defeated";
 
-	// TODO: Tribute all resources to this player's active allies (if any)
-
 	// Reassign all player's entities to Gaia
 	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	var entities = cmpRangeManager.GetEntitiesByPlayer(this.playerID);
@@ -643,24 +634,37 @@ Player.prototype.OnPlayerDefeated = function(msg)
 	// The ownership change is done in two steps so that entities don't hit idle
 	// (and thus possibly look for "enemies" to attack) before nearby allies get
 	// converted to Gaia as well.
-	for each (var entity in entities)
+	for (var entity of entities)
 	{
 		var cmpOwnership = Engine.QueryInterface(entity, IID_Ownership);
 		cmpOwnership.SetOwnerQuiet(0);
 	}
 
 	// With the real ownership change complete, send OwnershipChanged messages.
-	for each (var entity in entities)
-		Engine.PostMessage(entity, MT_OwnershipChanged, { "entity": entity,
-			"from": this.playerID, "to": 0 });
+	for (var entity of entities)
+		Engine.PostMessage(entity, MT_OwnershipChanged, {
+			"entity": entity,
+			"from": this.playerID,
+			"to": 0
+		});
 
 	// Reveal the map for this player.
 	cmpRangeManager.SetLosRevealAll(this.playerID, true);
 
 	// Send a chat message notifying of the player's defeat.
-	var notification = {"type": "defeat", "players": [this.playerID]};
 	var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGUIInterface.PushNotification(notification);
+	cmpGUIInterface.PushNotification({
+		"type": "defeat",
+		"players": [this.playerID]
+	});
+};
+
+Player.prototype.OnResearchFinished = function(msg)
+{
+	if (msg.tech == this.template.SharedLosTech)
+		this.UpdateSharedLos();
+	else if (msg.tech == this.template.SharedDropsitesTech)
+		this.sharedDropsites = true;
 };
 
 Player.prototype.OnDiplomacyChanged = function()
@@ -690,11 +694,7 @@ Player.prototype.GetCheatTimeMultiplier = function()
 
 Player.prototype.TributeResource = function(player, amounts)
 {
-	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	if (!cmpPlayerManager)
-		return;
-
-	var cmpPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(player), IID_Player);
+	var cmpPlayer = QueryPlayerIDInterface(player);
 	if (!cmpPlayer)
 		return;
 
@@ -706,7 +706,7 @@ Player.prototype.TributeResource = function(player, amounts)
 
 	cmpPlayer.AddResources(amounts);
 
-	var total = Object.keys(amounts).reduce(function (sum, type){ return sum + amounts[type]; }, 0);
+	var total = Object.keys(amounts).reduce((sum, type) => sum + amounts[type], 0);
 	var cmpOurStatisticsTracker = QueryPlayerIDInterface(this.playerID, IID_StatisticsTracker);
 	if (cmpOurStatisticsTracker)
 		cmpOurStatisticsTracker.IncreaseTributesSentCounter(total);
@@ -714,12 +714,20 @@ Player.prototype.TributeResource = function(player, amounts)
 	if (cmpTheirStatisticsTracker)
 		cmpTheirStatisticsTracker.IncreaseTributesReceivedCounter(total);
 
-	var notification = {"type": "tribute", "players": [player], "donator": this.playerID, "amounts": amounts};
 	var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	if (cmpGUIInterface)
-		cmpGUIInterface.PushNotification(notification);
+		cmpGUIInterface.PushNotification({
+			"type": "tribute",
+			"players": [player],
+			"donator": this.playerID,
+			"amounts": amounts
+		});
 
-	Engine.BroadcastMessage(MT_TributeExchanged, {"to": player, "from": this.playerID, "amounts": amounts});
+	Engine.BroadcastMessage(MT_TributeExchanged, {
+		"to": player,
+		"from": this.playerID,
+		"amounts": amounts
+	});
 };
 
 Player.prototype.AddDisabledTemplate = function(template)
@@ -727,23 +735,36 @@ Player.prototype.AddDisabledTemplate = function(template)
 	this.disabledTemplates[template] = true;
 	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, {});
 	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGuiInterface.PushNotification({"type": "resetselectionpannel", "players": [this.GetPlayerID()]});
+	cmpGuiInterface.PushNotification({
+		"type": "resetselectionpannel",
+		"players": [this.GetPlayerID()]
+	});
 };
 
 Player.prototype.RemoveDisabledTemplate = function(template)
 {
 	this.disabledTemplates[template] = false;
 	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, {});
+
 	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGuiInterface.PushNotification({"type": "resetselectionpannel", "players": [this.GetPlayerID()]});
+	cmpGuiInterface.PushNotification({
+		"type": "resetselectionpannel",
+		"players": [this.GetPlayerID()]
+	});
 };
 
 Player.prototype.SetDisabledTemplates = function(templates)
 {
-	this.disabledTemplates = templates;
+	this.disabledTemplates = {};
+	for (let template of templates)
+		this.disabledTemplates[template] = true;
 	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, {});
+
 	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGuiInterface.PushNotification({"type": "resetselectionpannel", "players": [this.GetPlayerID()]});
+	cmpGuiInterface.PushNotification({
+		"type": "resetselectionpannel",
+		"players": [this.GetPlayerID()]
+	});
 };
 
 Player.prototype.GetDisabledTemplates = function()
@@ -765,7 +786,9 @@ Player.prototype.RemoveDisabledTechnology = function(tech)
 
 Player.prototype.SetDisabledTechnologies = function(techs)
 {
-	this.disabledTechnologies = techs;
+	this.disabledTechnologies = {};
+	for (let tech of techs)
+		this.disabledTechnologies[tech] = true;
 	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, {});
 };
 
@@ -776,7 +799,8 @@ Player.prototype.GetDisabledTechnologies = function()
 
 Player.prototype.AddStartingTechnology = function(tech)
 {
-	this.startingTechnologies[tech] = true;
+	if (this.startingTechnologies.indexOf(tech) == -1)
+		this.startingTechnologies.push(tech);
 };
 
 Player.prototype.SetStartingTechnologies = function(techs)
