@@ -30,6 +30,11 @@ Promotion.prototype.GetPromotedTemplateName = function()
 
 Promotion.prototype.Promote = function(promotedTemplateName)
 {
+	// If the unit is dead, don't promote it
+	var cmpCurrentUnitHealth = Engine.QueryInterface(this.entity, IID_Health);
+	if (cmpCurrentUnitHealth.GetHitpoints() == 0)
+		return;
+
 	// Create promoted unit entity
 	var promotedUnitEntity = Engine.AddEntity(promotedTemplateName);
 
@@ -52,7 +57,6 @@ Promotion.prototype.Promote = function(promotedTemplateName)
 	cmpPromotedUnitOwnership.SetOwner(cmpCurrentUnitOwnership.GetOwner());
 
 	// change promoted unit health to the same percent of hitpoints as unit had before promotion
-	var cmpCurrentUnitHealth = Engine.QueryInterface(this.entity, IID_Health);
 	var cmpPromotedUnitHealth = Engine.QueryInterface(promotedUnitEntity, IID_Health);
 	var healthFraction = Math.max(0, Math.min(1, cmpCurrentUnitHealth.GetHitpoints() / cmpCurrentUnitHealth.GetMaxHitpoints()));
 	var promotedUnitHitpoints = Math.round(cmpPromotedUnitHealth.GetMaxHitpoints() * healthFraction);
@@ -72,27 +76,17 @@ Promotion.prototype.Promote = function(promotedTemplateName)
 
 	var cmpCurrentUnitAI = Engine.QueryInterface(this.entity, IID_UnitAI);
 	var cmpPromotedUnitAI = Engine.QueryInterface(promotedUnitEntity, IID_UnitAI);
-	cmpPromotedUnitAI.SetHeldPosition(cmpCurrentUnitAI.GetHeldPosition());
+	var pos = cmpCurrentUnitAI.GetHeldPosition();
+	if (pos)
+		cmpPromotedUnitAI.SetHeldPosition(pos.x, pos.z);
 	if (cmpCurrentUnitAI.GetStanceName())
 		cmpPromotedUnitAI.SwitchToStance(cmpCurrentUnitAI.GetStanceName());
 
 	var orders = cmpCurrentUnitAI.GetOrders();
 	if (cmpCurrentUnitAI.IsGarrisoned())
-	{
-		if (orders.length > 0 && orders[0].type == "Garrison")
-		{
-			// Replace the garrison order by an autogarrison order,
-			// as we are already garrisoned and do not need to do
-			// any further checks (or else we should do them here).
-			orders.shift();
-			cmpPromotedUnitAI.Autogarrison();
-		}
-		else
-			warn("Promoted garrisoned entity with empty order queue.");
-	}
-	else
-		cmpPromotedUnitAI.Cheer();
-
+		cmpPromotedUnitAI.SetGarrisoned();
+	if (cmpCurrentUnitPosition.IsInWorld())	// do not cheer if not visibly garrisoned
+		cmpPromotedUnitAI.Cheer();	
 	cmpPromotedUnitAI.AddOrders(orders);
 
 	var workOrders = cmpCurrentUnitAI.GetWorkOrders();
@@ -156,4 +150,4 @@ Promotion.prototype.OnValueModification = function(msg)
 		this.IncreaseXp(0);
 };
 
-Engine.RegisterComponentType(IID_Promotion, "Promotion", Promotion);
\ No newline at end of file
+Engine.RegisterComponentType(IID_Promotion, "Promotion", Promotion);
