@@ -20,18 +20,24 @@ RallyPoint.prototype.AddPosition = function(x, z)
 RallyPoint.prototype.GetPositions = function()
 {
 	// Update positions for moving target entities
-	// TODO: If we target an enemy unit we should update its position
-	// as long as it is outside of FoW and SoD.
-	
+
+	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+	var cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
+
 	// We must not affect the simulation state here (modifications of the
 	// RallyPointRenderer are allowed though), so copy the state
 	var ret = [];
-	
 	for (var i = 0; i < this.pos.length; i++)
 	{
 		ret.push(this.pos[i]);
+
+		// Update the rallypoint coordinates if the target is alive
+		if (!this.data[i] || !this.data[i].target || !this.TargetIsAlive(this.data[i].target))
+			continue;
 		
-		if (!this.data[i] || !this.data[i].target)
+		// and visible
+		if (cmpRangeManager && cmpOwnership &&
+				cmpRangeManager.GetLosVisibility(this.data[i].target, cmpOwnership.GetOwner()) != "visible")
 			continue;
 
 		// Get the actual position of the target entity
@@ -97,4 +103,17 @@ RallyPoint.prototype.OnGlobalEntityRenamed = function(msg)
 	}
 };
 
+/**
+ * Returns true if the target exists and has non-zero hitpoints.
+ */
+RallyPoint.prototype.TargetIsAlive = function(ent)
+{
+	var cmpFormation = Engine.QueryInterface(ent, IID_Formation);
+	if (cmpFormation)
+		return true;
+
+	var cmpHealth = QueryMiragedInterface(ent, IID_Health);
+	return cmpHealth && cmpHealth.GetHitpoints() != 0;
+};
+
 Engine.RegisterComponentType(IID_RallyPoint, "RallyPoint", RallyPoint);
