@@ -65,6 +65,8 @@ ResourceGatherer.prototype.Init = function()
 
 	// The last exact type gathered, so we can render appropriate props
 	this.lastCarriedType = undefined; // { generic, specific }
+
+	this.RecalculateGatherRatesAndCapacities();
 };
 
 /**
@@ -125,25 +127,28 @@ ResourceGatherer.prototype.GetLastCarriedType = function()
 		return undefined;
 };
 
-ResourceGatherer.prototype.GetBaseSpeed = function()
+// Since this code is very performancecritical and applying technologies quite slow, cache it.
+ResourceGatherer.prototype.RecalculateGatherRatesAndCapacities = function()
 {
 	let cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
 	let multiplier = cmpPlayer ? cmpPlayer.GetGatherRateMultiplier() : 1;
-	return multiplier * ApplyValueModificationsToEntity("ResourceGatherer/BaseSpeed", +this.template.BaseSpeed, this.entity);
+	this.baseSpeed = multiplier * ApplyValueModificationsToEntity("ResourceGatherer/BaseSpeed", +this.template.BaseSpeed, this.entity);
+
+	this.rates = {};
+	for (let r in this.template.Rates)
+	{
+		let rate = ApplyValueModificationsToEntity("ResourceGatherer/Rates/" + r, +this.template.Rates[r], this.entity);
+		this.rates[r] = rate * this.baseSpeed;
+	}
+
+	this.capacities = {};
+	for (let r in this.template.Capacities)
+		this.capacities[r] = ApplyValueModificationsToEntity("ResourceGatherer/Capacities/" + r, +this.template.Capacities[r], this.entity);
 };
 
 ResourceGatherer.prototype.GetGatherRates = function()
 {
-	let ret = {};
-	let baseSpeed = this.GetBaseSpeed();
-
-	for (let r in this.template.Rates)
-	{
-		let rate = ApplyValueModificationsToEntity("ResourceGatherer/Rates/" + r, +this.template.Rates[r], this.entity);
-		ret[r] = rate * baseSpeed;
-	}
-
-	return ret;
+	return this.rates;
 };
 
 ResourceGatherer.prototype.GetGatherRate = function(resourceType)
@@ -151,16 +156,14 @@ ResourceGatherer.prototype.GetGatherRate = function(resourceType)
 	if (!this.template.Rates[resourceType])
 		return 0;
 
-	let baseSpeed = this.GetBaseSpeed();
-	let rate = ApplyValueModificationsToEntity("ResourceGatherer/Rates/" + resourceType, +this.template.Rates[resourceType], this.entity);
-	return rate * baseSpeed;
+	return this.rates[resourceType];
 };
 
 ResourceGatherer.prototype.GetCapacity = function(resourceType)
 {
 	if(!this.template.Capacities[resourceType])
 		return 0;
-	return ApplyValueModificationsToEntity("ResourceGatherer/Capacities/" + resourceType, +this.template.Capacities[resourceType], this.entity);
+	return this.capacities[resourceType];
 };
 
 ResourceGatherer.prototype.GetRange = function()
@@ -235,9 +238,6 @@ ResourceGatherer.prototype.PerformGather = function(target)
 
 	Engine.PostMessage(this.entity, MT_ResourceCarryingChanged, { "to": this.GetCarryingStatus() });
 
-	// Tell the target we're gathering from it
-	Engine.PostMessage(target, MT_ResourceGather,
-		{ "entity": target, "gatherer": this.entity });
 
 	return {
 		"amount": status.amount,
@@ -253,16 +253,12 @@ ResourceGatherer.prototype.PerformGather = function(target)
  */
 ResourceGatherer.prototype.GetTargetGatherRate = function(target)
 {
-	let type;
-	let cmpResourceSupply = Engine.QueryInterface(target, IID_ResourceSupply);
-	let cmpMirage = Engine.QueryInterface(target, IID_Mirage);
-	if (cmpResourceSupply)
-		type = cmpResourceSupply.GetType();
-	else if (cmpMirage && cmpMirage.ResourceSupply())
-		type = cmpMirage.GetType();
-	else
+	let cmpResourceSupply = QueryMiragedInterface(target, IID_ResourceSupply);
+	if (!cmpResourceSupply)
 		return 0;
 
+	let type = cmpResourceSupply.GetType();
+
 	let rate = 0;
 	if (type.specific)
 		rate = this.GetGatherRate(type.generic+"."+type.specific);
@@ -273,7 +269,7 @@ ResourceGatherer.prototype.GetTargetGatherRate = function(target)
 	let cheatMultiplier = cmpPlayer ? cmpPlayer.GetCheatTimeMultiplier() : 1;
 	rate = rate / cheatMultiplier;
 
-	if (cmpMirage)
+	if ("Mirages" in cmpResourceSupply)
 		return rate;
 
 	// Apply diminishing returns with more gatherers, for e.g. infinite farms. For most resources this has no effect. (GetDiminishingReturns will return null.)
@@ -294,11 +290,9 @@ ResourceGatherer.prototype.GetTargetGatherRate = function(target)
 	// The cosine function is an oscillating curve, normally between -1 and 1. Multiplying by 0.5 squishes that down to
 	// between -0.5 and 0.5. Adding 0.5 to that changes the range to 0 to 1. The diminishingReturns constant
 	// adjusts the period of the curve.
-	// Alternatively, just find scythetwirler (who came up with the math here) or alpha123 (who wrote the code) on IRC.
-	let cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
 	let diminishingReturns = cmpResourceSupply.GetDiminishingReturns();
 	if (diminishingReturns)
-		rate = (0.5 * Math.cos((cmpResourceSupply.GetGatherers().length - 1) * Math.PI / diminishingReturns) + 0.5) * rate;
+		rate = (0.5 * Math.cos((cmpResourceSupply.GetNumGatherers() - 1) * Math.PI / diminishingReturns) + 0.5) * rate;
 
 	return rate;
 };
@@ -311,7 +305,14 @@ ResourceGatherer.prototype.GetTargetGatherRate = function(target)
 ResourceGatherer.prototype.CanCarryMore = function(type)
 {
 	let amount = (this.carrying[type] || 0);
-	return (amount < this.GetCapacity(type));
+	return amount < this.GetCapacity(type);
+};
+
+
+ResourceGatherer.prototype.IsCarrying = function(type)
+{
+	let amount = (this.carrying[type] || 0);
+	return amount > 0;
 };
 
 /**
@@ -335,7 +336,7 @@ ResourceGatherer.prototype.IsCarryingAnythingExcept = function(exceptedType)
  */
 ResourceGatherer.prototype.CommitResources = function(types)
 {
-	let cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+	let cmpPlayer = QueryOwnerInterface(this.entity);
 
 	if (cmpPlayer)
 		for each (let type in types)
@@ -360,4 +361,26 @@ ResourceGatherer.prototype.DropResources = function()
 	Engine.PostMessage(this.entity, MT_ResourceCarryingChanged, { "to": this.GetCarryingStatus() });
 };
 
+// Since we cache gather rates, we need to make sure we update them when tech changes.
+// and when our owner change because owners can had different techs.
+ResourceGatherer.prototype.OnValueModification = function(msg)
+{
+	if (msg.component != "ResourceGatherer")
+		return;
+
+	this.RecalculateGatherRatesAndCapacities();
+};
+
+ResourceGatherer.prototype.OnOwnershipChanged = function(msg)
+{
+	if (msg.to === -1)
+		return;
+	this.RecalculateGatherRatesAndCapacities();
+};
+
+ResourceGatherer.prototype.OnGlobalInitGame = function(msg)
+{
+	this.RecalculateGatherRatesAndCapacities();
+};
+
 Engine.RegisterComponentType(IID_ResourceGatherer, "ResourceGatherer", ResourceGatherer);
