@@ -51,6 +51,10 @@ ResourceSupply.prototype.Init = function()
 		this.gatherers.push([]);
 
 	this.infinite = !isFinite(+this.template.Amount);
+
+	[this.type,this.subType] = this.template.Type.split('.');
+	this.cachedType = { "generic" : this.type, "specific" : this.subType };
+
 };
 
 ResourceSupply.prototype.IsInfinite = function()
@@ -78,14 +82,9 @@ ResourceSupply.prototype.GetMaxGatherers = function()
 	return +this.template.MaxGatherers;
 };
 
-ResourceSupply.prototype.GetGatherers = function()
+ResourceSupply.prototype.GetNumGatherers = function()
 {
-	var numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
-	var total = [];
-	for (var playerid = 0; playerid <= numPlayers; playerid++)
-		for (var gatherer = 0; gatherer < this.gatherers[playerid].length; gatherer++)
-			total.push(this.gatherers[playerid][gatherer]);
-	return total;
+	return this.gatherers.reduce((a, b) => a + b.length, 0); 
 };
 
 ResourceSupply.prototype.GetDiminishingReturns = function()
@@ -112,25 +111,23 @@ ResourceSupply.prototype.TakeResources = function(rate)
 	var change = old - this.amount;
 
 	// Remove entities that have been exhausted
-	if (this.amount == 0)
+	if (this.amount === 0)
 		Engine.DestroyEntity(this.entity);
 
 	Engine.PostMessage(this.entity, MT_ResourceSupplyChanged, { "from": old, "to": this.amount });
 
-	return { "amount": change, "exhausted": (this.amount == 0) };
+	return { "amount": change, "exhausted": (this.amount === 0) };
 };
 
 ResourceSupply.prototype.GetType = function()
 {
 	// All resources must have both type and subtype
-
-	var [type, subtype] = this.template.Type.split('.');
-	return { "generic": type, "specific": subtype };
+	return this.cachedType;
 };
 
 ResourceSupply.prototype.IsAvailable = function(player, gathererID)
 {
-	if (this.GetGatherers().length < this.GetMaxGatherers() || this.gatherers[player].indexOf(gathererID) !== -1)
+	if (this.GetNumGatherers() < this.GetMaxGatherers() || this.gatherers[player].indexOf(gathererID) !== -1)
 		return true;
 	return false;
 };
@@ -144,7 +141,7 @@ ResourceSupply.prototype.AddGatherer = function(player, gathererID)
 	{
 		this.gatherers[player].push(gathererID);
 		// broadcast message, mainly useful for the AIs.
-		Engine.PostMessage(this.entity, MT_ResourceSupplyGatherersChanged, { "to": this.GetGatherers() });
+		Engine.PostMessage(this.entity, MT_ResourceSupplyNumGatherersChanged, { "to": this.GetNumGatherers() });
 	}
 	
 	return true;
@@ -166,7 +163,7 @@ ResourceSupply.prototype.RemoveGatherer = function(gathererID, player)
 		{
 			this.gatherers[player].splice(index,1);
 			// broadcast message, mainly useful for the AIs.
-			Engine.PostMessage(this.entity, MT_ResourceSupplyGatherersChanged, { "to": this.GetGatherers() });
+			Engine.PostMessage(this.entity, MT_ResourceSupplyNumGatherersChanged, { "to": this.GetNumGatherers() });
 			return;
 		}
 	}
