@@ -34,7 +34,6 @@ TechnologyManager.prototype.Init = function()
 	                             // e.g. { "Attack/Melee/Hack" : {5: {"origValue": 8, "newValue": 10}, 7: {"origValue": 9, "newValue": 12}, ...}, ...}
 	                             // where 5 and 7 are entity id's
 
-	this.typeCounts = {}; // stores the number of entities of each type 
 	this.classCounts = {}; // stores the number of entities of each Class
 	this.typeCountsByClass = {}; // stores the number of entities of each type for each class i.e.
 	                             // {"someClass": {"unit/spearman": 2, "unit/cav": 5} "someOtherClass":...}
@@ -53,7 +52,7 @@ TechnologyManager.prototype.Init = function()
 TechnologyManager.prototype.OnUpdate = function() 
 {
 	this.UpdateAutoResearch();
-}
+};
 
 
 // This function checks if the requirements of any autoresearch techs are met and if they are it researches them
@@ -71,7 +70,7 @@ TechnologyManager.prototype.UpdateAutoResearch = function()
 			return; // We will have recursively handled any knock-on effects so can just return
 		}
 	}
-}
+};
 
 TechnologyManager.prototype.GetTechnologyTemplate = function(tech)
 {
@@ -187,16 +186,13 @@ TechnologyManager.prototype.CheckTechnologyRequirements = function(reqs)
 
 TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 {
-	// This automatically updates typeCounts, classCounts and typeCountsByClass
+	// This automatically updates classCounts and typeCountsByClass
 	var playerID = (Engine.QueryInterface(this.entity, IID_Player)).GetPlayerID();
 	if (msg.to == playerID)
 	{
 		var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 		var template = cmpTemplateManager.GetCurrentTemplateName(msg.entity);
 
-		this.typeCounts[template] = this.typeCounts[template] || 0;
-		this.typeCounts[template] += 1;
-
 		var cmpIdentity = Engine.QueryInterface(msg.entity, IID_Identity);
 		if (!cmpIdentity)
 			return;
@@ -205,14 +201,14 @@ TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 		// don't use foundations for the class counts but check if techs apply (e.g. health increase)
 		if (!Engine.QueryInterface(msg.entity, IID_Foundation))
 		{
-			for (var i in classes)
+			for (let cls of classes)
 			{
-				this.classCounts[classes[i]] = this.classCounts[classes[i]] || 0;
-				this.classCounts[classes[i]] += 1;
-			
-				this.typeCountsByClass[classes[i]] = this.typeCountsByClass[classes[i]] || {};
-				this.typeCountsByClass[classes[i]][template] = this.typeCountsByClass[classes[i]][template] || 0;
-				this.typeCountsByClass[classes[i]][template] += 1;
+				this.classCounts[cls] = this.classCounts[cls] || 0;
+				this.classCounts[cls] += 1;
+
+				this.typeCountsByClass[cls] = this.typeCountsByClass[cls] || {};
+				this.typeCountsByClass[cls][template] = this.typeCountsByClass[cls][template] || 0;
+				this.typeCountsByClass[cls][template] += 1;
 			}
 		}
 
@@ -227,8 +223,8 @@ TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 				// We only need to find one one tech per component for a match
 				var modifications = this.modifications[name];
 				var component = name.split("/")[0];
-				for (var i in modifications)
- 					if (DoesModificationApply(modifications[i], classes))
+				for (let modif of modifications)
+ 					if (DoesModificationApply(modif, classes))
 					{
 						if (!modifiedComponents[component])
 							modifiedComponents[component] = [];
@@ -245,10 +241,6 @@ TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 	{
 		var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 		var template = cmpTemplateManager.GetCurrentTemplateName(msg.entity);
-		
-		this.typeCounts[template] -= 1;
-		if (this.typeCounts[template] <= 0)
-			delete this.typeCounts[template];
 
 		// don't use foundations for the class counts
 		if (!Engine.QueryInterface(msg.entity, IID_Foundation))
@@ -257,15 +249,15 @@ TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 			if (cmpIdentity)
 			{
 				var classes = cmpIdentity.GetClassesList();
-				for (var i in classes)
+				for (let cls of classes)
 				{
-					this.classCounts[classes[i]] -= 1;
-					if (this.classCounts[classes[i]] <= 0)
-						delete this.classCounts[classes[i]];
+					this.classCounts[cls] -= 1;
+					if (this.classCounts[cls] <= 0)
+						delete this.classCounts[cls];
 
-					this.typeCountsByClass[classes[i]][template] -= 1;
-					if (this.typeCountsByClass[classes[i]][template] <= 0)
-						delete this.typeCountsByClass[classes[i]][template];
+					this.typeCountsByClass[cls][template] -= 1;
+					if (this.typeCountsByClass[cls][template] <= 0)
+						delete this.typeCountsByClass[cls][template];
 				}
 			}
 		}
@@ -295,10 +287,10 @@ TechnologyManager.prototype.ResearchTechnology = function(tech)
 		var affects = [];
 		if (template.affects && template.affects.length > 0)
 		{
-			for (var i in template.affects)
+			for (let affect of template.affects)
 			{
 				// Put the list of classes into an array for convenient access
-				affects.push(template.affects[i].split(/\s+/));
+				affects.push(affect.split(/\s+/));
 			}
 		}
 		else
@@ -337,6 +329,28 @@ TechnologyManager.prototype.ResearchTechnology = function(tech)
 		}
 	}
 
+	if (template.replaces && template.replaces.length > 0)
+	{
+		for (var i of template.replaces) 
+		{
+			if (!i || this.IsTechnologyResearched(i))
+				continue;
+
+			var template = this.GetTechnologyTemplate(i);
+			this.researchedTechs[i] = template;
+
+			// Change the EntityLimit if any
+			let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+			if (cmpPlayer && cmpPlayer.GetPlayerID() !== undefined)
+			{
+				let playerID = cmpPlayer.GetPlayerID();
+				let cmpPlayerEntityLimits = QueryPlayerIDInterface(playerID, IID_EntityLimits);
+				if (cmpPlayerEntityLimits)
+					cmpPlayerEntityLimits.UpdateLimitsFromTech(i);
+			}
+		}
+	}
+
 	this.UpdateAutoResearch();
 
 	var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
@@ -351,10 +365,9 @@ TechnologyManager.prototype.ResearchTechnology = function(tech)
 	if (cmpPlayerEntityLimits)
 		cmpPlayerEntityLimits.UpdateLimitsFromTech(tech);
 
-	// Call the related trigger event 
-	var cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);
-	cmpTrigger.CallEvent("ResearchFinished", {"player": playerID, "tech": tech});
-	
+	// always send research finished message 
+	Engine.PostMessage(this.entity, MT_ResearchFinished, {"player": playerID, "tech": tech});
+
 	for (var component in modifiedComponents)
 	{
 		Engine.PostMessage(SYSTEM_ENTITY, MT_TemplateModification, { "player": playerID, "component": component, "valueNames": modifiedComponents[component]});
