@@ -5,7 +5,7 @@ function TechnologyTemplateManager() {}
 
 TechnologyTemplateManager.prototype.Schema =
 	"<a:component type='system'/><empty/>";
-	
+
 TechnologyTemplateManager.prototype.Init = function()
 {
 	this.allTechs = {};
@@ -20,10 +20,10 @@ TechnologyTemplateManager.prototype.GetTemplate = function(template)
 	if (!this.allTechs[template])
 	{
 		this.allTechs[template] = Engine.ReadJSONFile("technologies/" + template + ".json");
-		if (! this.allTechs[template])
+		if (!this.allTechs[template])
 			error("Failed to load technology \"" + template + "\"");
 	}
-	
+
 	return this.allTechs[template];
 };
 
@@ -32,21 +32,21 @@ TechnologyTemplateManager.prototype.GetAuraTemplate = function(template)
 	if (!this.allAuras[template])
 	{
 		this.allAuras[template] = Engine.ReadJSONFile("auras/" + template + ".json");
-		if (! this.allAuras[template])
+		if (!this.allAuras[template])
 			error("Failed to load aura \"" + template + "\"");
 	}
-	
+
 	return this.allAuras[template];
 };
 
 TechnologyTemplateManager.prototype.ListAllTechs = function()
 {
-    return Engine.FindJSONFiles("technologies", true);
-}
+	return Engine.FindJSONFiles("technologies", true);
+};
 
 TechnologyTemplateManager.prototype.GetAllTechs = function()
 {
 	return this.allTechs;
-}
+};
 
-Engine.RegisterComponentType(IID_TechnologyTemplateManager, "TechnologyTemplateManager", TechnologyTemplateManager);
+Engine.RegisterSystemComponentType(IID_TechnologyTemplateManager, "TechnologyTemplateManager", TechnologyTemplateManager);
