@@ -1,8 +1,11 @@
+Engine.LoadHelperScript("Player.js");
 Engine.LoadComponentScript("interfaces/Attack.js");
 Engine.LoadComponentScript("interfaces/AlertRaiser.js");
 Engine.LoadComponentScript("interfaces/Auras.js");
 Engine.LoadComponentScript("interfaces/Barter.js");
 Engine.LoadComponentScript("interfaces/Builder.js");
+Engine.LoadComponentScript("interfaces/Capturable.js");
+Engine.LoadComponentScript("interfaces/CeasefireManager.js");
 Engine.LoadComponentScript("interfaces/DamageReceiver.js");
 Engine.LoadComponentScript("interfaces/EndGameManager.js");
 Engine.LoadComponentScript("interfaces/EntityLimits.js");
@@ -16,11 +19,12 @@ Engine.LoadComponentScript("interfaces/Pack.js");
 Engine.LoadComponentScript("interfaces/ProductionQueue.js");
 Engine.LoadComponentScript("interfaces/Promotion.js");
 Engine.LoadComponentScript("interfaces/RallyPoint.js");
+Engine.LoadComponentScript("interfaces/Repairable.js");
 Engine.LoadComponentScript("interfaces/ResourceDropsite.js");
 Engine.LoadComponentScript("interfaces/ResourceGatherer.js");
 Engine.LoadComponentScript("interfaces/ResourceSupply.js");
-Engine.LoadComponentScript("interfaces/TechnologyManager.js")
-Engine.LoadComponentScript("interfaces/Trader.js")
+Engine.LoadComponentScript("interfaces/TechnologyManager.js");
+Engine.LoadComponentScript("interfaces/Trader.js");
 Engine.LoadComponentScript("interfaces/Timer.js");
 Engine.LoadComponentScript("interfaces/StatisticsTracker.js");
 Engine.LoadComponentScript("interfaces/UnitAI.js");
@@ -31,10 +35,12 @@ var cmp = ConstructComponent(SYSTEM_ENTITY, "GuiInterface");
 
 
 AddMock(SYSTEM_ENTITY, IID_Barter, {
-	GetPrices: function() { return {
-		"buy": { "food": 150 },
-		"sell": { "food": 25 },
-	}},
+	GetPrices: function() {
+		return {
+			"buy": { "food": 150 },
+			"sell": { "food": 25 }
+		};
+	}
 });
 
 AddMock(SYSTEM_ENTITY, IID_EndGameManager, {
@@ -64,7 +70,8 @@ AddMock(SYSTEM_ENTITY, IID_Timer, {
 AddMock(100, IID_Player, {
 	GetName: function() { return "Player 1"; },
 	GetCiv: function() { return "gaia"; },
-	GetColour: function() { return { r: 1, g: 1, b: 1, a: 1}; },
+	GetColor: function() { return { r: 1, g: 1, b: 1, a: 1}; },
+	CanControlAllUnits: function() { return false; },
 	GetPopulationCount: function() { return 10; },
 	GetPopulationLimit: function() { return 20; },
 	GetMaxPopulation: function() { return 200; },
@@ -76,12 +83,12 @@ AddMock(100, IID_Player, {
 	GetLockTeams: function() { return false; },
 	GetCheatsEnabled: function() { return false; },
 	GetDiplomacy: function() { return [-1, 1]; },
-	GetConquestCriticalEntitiesCount: function() { return 1; },
 	IsAlly: function() { return false; },
 	IsMutualAlly: function() { return false; },
 	IsNeutral: function() { return false; },
 	IsEnemy: function() { return true; },
 	GetDisabledTemplates: function() { return {}; },
+	HasSharedDropsites: function() { return false; },
 });
 
 AddMock(100, IID_EntityLimits, {
@@ -128,8 +135,13 @@ AddMock(100, IID_StatisticsTracker, {
 				"vegetarianFood": 0, 
 			},
 			"treasuresCollected": 0,
+			"lootCollected": 0,
 			"percentMapExplored": 10,
-			"teamPercentMapExplored": 10
+			"teamPercentMapExplored": 10,
+			"percentMapControlled": 10,
+			"teamPercentMapControlled": 10,
+			"peakPercentOfMapControlled": 10,
+			"teamPeakPercentOfMapControlled": 10
 		};
 	},
 	IncreaseTrainedUnitsCounter: function() { return 1; },
@@ -139,8 +151,9 @@ AddMock(100, IID_StatisticsTracker, {
 
 AddMock(101, IID_Player, {
 	GetName: function() { return "Player 2"; },
-	GetCiv: function() { return "celt"; },
-	GetColour: function() { return { r: 1, g: 0, b: 0, a: 1}; },
+	GetCiv: function() { return "mace"; },
+	GetColor: function() { return { r: 1, g: 0, b: 0, a: 1}; },
+	CanControlAllUnits: function() { return true; },
 	GetPopulationCount: function() { return 40; },
 	GetPopulationLimit: function() { return 30; },
 	GetMaxPopulation: function() { return 300; },
@@ -152,12 +165,12 @@ AddMock(101, IID_Player, {
 	GetLockTeams: function() {return false; },
 	GetCheatsEnabled: function() { return false; },
 	GetDiplomacy: function() { return [-1, 1]; },
-	GetConquestCriticalEntitiesCount: function() { return 1; },
 	IsAlly: function() { return true; },
 	IsMutualAlly: function() {return false; },
 	IsNeutral: function() { return false; },
 	IsEnemy: function() { return false; },
 	GetDisabledTemplates: function() { return {}; },
+	HasSharedDropsites: function() { return false; },
 });
 
 AddMock(101, IID_EntityLimits, {
@@ -204,8 +217,13 @@ AddMock(101, IID_StatisticsTracker, {
 				"vegetarianFood": 0, 
 			},
 			"treasuresCollected": 0,
+			"lootCollected": 0,
 			"percentMapExplored": 10,
-			"teamPercentMapExplored": 10
+			"teamPercentMapExplored": 10,
+			"percentMapControlled": 10,
+			"teamPercentMapControlled": 10,
+			"peakPercentOfMapControlled": 10,
+			"teamPeakPercentOfMapControlled": 10
 		};
 	},
 	IncreaseTrainedUnitsCounter: function() { return 1; },
@@ -221,7 +239,8 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 		{
 			name: "Player 1",
 			civ: "gaia",
-			colour: { r:1, g:1, b:1, a:1 },
+			color: { r:1, g:1, b:1, a:1 },
+			controlsAll: false,
 			popCount: 10,
 			popLimit: 20,
 			popMax: 200,
@@ -233,6 +252,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			teamsLocked: false,
 			cheatsEnabled: false,
 			disabledTemplates: {},
+			hasSharedDropsites: false,
 			phase: "village",
 			isAlly: [false, false],
 			isMutualAlly: [false, false],
@@ -259,8 +279,9 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 		},
 		{
 			name: "Player 2",
-			civ: "celt",
-			colour: { r:1, g:0, b:0, a:1 },
+			civ: "mace",
+			color: { r:1, g:0, b:0, a:1 },
+			controlsAll: true,
 			popCount: 40,
 			popLimit: 30,
 			popMax: 300,
@@ -272,6 +293,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			teamsLocked: false,
 			cheatsEnabled: false,
 			disabledTemplates: {},
+			hasSharedDropsites: false,
 			phase: "village",
 			isAlly: [true, true],
 			isMutualAlly: [false, false],
@@ -308,7 +330,8 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 		{
 			name: "Player 1",
 			civ: "gaia",
-			colour: { r:1, g:1, b:1, a:1 },
+			color: { r:1, g:1, b:1, a:1 },
+			controlsAll: false,
 			popCount: 10,
 			popLimit: 20,
 			popMax: 200,
@@ -320,6 +343,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 			teamsLocked: false,
 			cheatsEnabled: false,
 			disabledTemplates: {},
+			hasSharedDropsites: false,
 			phase: "village",
 			isAlly: [false, false],
 			isMutualAlly: [false, false],
@@ -347,14 +371,20 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 					vegetarianFood: 0, 
 				},
 				treasuresCollected: 0,
+				lootCollected: 0,
 				percentMapExplored: 10,
-				teamPercentMapExplored: 10
+				teamPercentMapExplored: 10,
+				percentMapControlled: 10,
+				teamPercentMapControlled: 10,
+				peakPercentOfMapControlled: 10,
+				teamPeakPercentOfMapControlled: 10
 			},
 		},
 		{
 			name: "Player 2",
-			civ: "celt",
-			colour: { r:1, g:0, b:0, a:1 },
+			civ: "mace",
+			color: { r:1, g:0, b:0, a:1 },
+			controlsAll: true,
 			popCount: 40,
 			popLimit: 30,
 			popMax: 300,
@@ -366,6 +396,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 			teamsLocked: false,
 			cheatsEnabled: false,
 			disabledTemplates: {},
+			hasSharedDropsites: false,
 			phase: "village",
 			isAlly: [true, true],
 			isMutualAlly: [false, false],
@@ -393,8 +424,13 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 					vegetarianFood: 0, 
 				},
 				treasuresCollected: 0,
+				lootCollected: 0,
 				percentMapExplored: 10,
-				teamPercentMapExplored: 10
+				teamPercentMapExplored: 10,
+				percentMapControlled: 10,
+				teamPercentMapControlled: 10,
+				peakPercentOfMapControlled: 10,
+				teamPeakPercentOfMapControlled: 10
 			},
 		}
 	],
@@ -416,6 +452,7 @@ AddMock(10, IID_Health, {
 	GetMaxHitpoints: function() { return 60; },
 	IsRepairable: function() { return false; },
 	IsUnhealable: function() { return false; },
+	IsUndeletable: function() { return false; },
 });
 
 AddMock(10, IID_Identity, {
@@ -473,6 +510,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetEntityState(-1, 10), {
 	maxHitpoints: 60,
 	needsRepair: false,
 	needsHeal: true,
+	canDelete: true,
 });
 
 TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedEntityState(-1, 10), {
@@ -486,6 +524,8 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedEntityState(-1, 10), {
 	obstruction: null,
 	turretParent: null,
 	promotion: null,
+	repairRate: null,
+	buildRate: null,
 	resourceDropsite: null,
 	resourceGatherRates: null,
 	resourceSupply: null,
