@@ -3,6 +3,8 @@ Engine.LoadHelperScript("Entity.js");
 Engine.LoadHelperScript("Player.js");
 Engine.LoadComponentScript("interfaces/Attack.js");
 Engine.LoadComponentScript("interfaces/Auras.js");
+Engine.LoadComponentScript("interfaces/BuildingAI.js");
+Engine.LoadComponentScript("interfaces/Capturable.js");
 Engine.LoadComponentScript("interfaces/DamageReceiver.js");
 Engine.LoadComponentScript("interfaces/Formation.js");
 Engine.LoadComponentScript("interfaces/Heal.js");
@@ -86,7 +88,6 @@ function TestFormationExiting(mode)
 		MoveToTargetRange: function(target, min, max) { },
 		StopMoving: function() { },
 		GetPassabilityClassName: function() { return "default"; },
-		SetPassabilityClassName: function() { },
 	});
 
 	AddMock(unit, IID_Vision, {
@@ -95,8 +96,9 @@ function TestFormationExiting(mode)
 
 	AddMock(unit, IID_Attack, {
 		GetRange: function() { return { "max": 10, "min": 0}; },
-		GetBestAttack: function() { return "melee"; },
+		GetFullAttackRange: function() { return { "max": 40, "min": 0}; },
 		GetBestAttackAgainst: function(t) { return "melee"; },
+		GetPreference: function(t) { return 0; },
 		GetTimers: function() { return { "prepare": 500, "repeat": 1000 }; },
 		CanAttack: function(v) { return true; },
 		CompareEntitiesByPreference: function(a, b) { return 0; },
@@ -134,11 +136,9 @@ function TestFormationExiting(mode)
 	});
 
 	AddMock(controller, IID_UnitMotion, {
-		SetUnitRadius: function(r) { },
 		SetSpeed: function(speed) { },
 		MoveToPointRange: function(x, z, minRange, maxRange) { },
 		GetPassabilityClassName: function() { return "default"; },
-		SetPassabilityClassName: function() { },
 	});
 
 	controllerAI.OnCreate();
@@ -238,7 +238,6 @@ function TestMoveIntoFormationWhileAttacking()
 			MoveToTargetRange: function(target, min, max) { },
 			StopMoving: function() { },
 			GetPassabilityClassName: function() { return "default"; },
-			SetPassabilityClassName: function() { },
 		});
 	
 		AddMock(unit + i, IID_Vision, {
@@ -247,7 +246,7 @@ function TestMoveIntoFormationWhileAttacking()
 	
 		AddMock(unit + i, IID_Attack, {
 			GetRange: function() { return {"max":10, "min": 0}; },
-			GetBestAttack: function() { return "melee"; },
+			GetFullAttackRange: function() { return { "max": 40, "min": 0}; },
 			GetBestAttackAgainst: function(t) { return "melee"; },
 			GetTimers: function() { return { "prepare": 500, "repeat": 1000 }; },
 			CanAttack: function(v) { return true; },
@@ -279,12 +278,10 @@ function TestMoveIntoFormationWhileAttacking()
 	});
 
 	AddMock(controller, IID_UnitMotion, {
-		SetUnitRadius: function(r) { },
 		SetSpeed: function(speed) { },
 		MoveToPointRange: function(x, z, minRange, maxRange) { },
 		IsInTargetRange: function(target, min, max) { return true; },
 		GetPassabilityClassName: function() { return "default"; },
-		SetPassabilityClassName: function() { },
 	});
 
 	AddMock(controller, IID_Attack, {
