@@ -5,11 +5,11 @@ var g_ServerName = "";
 var g_IsRejoining = false;
 var g_GameAttributes; // used when rejoining
 var g_PlayerAssignments; // used when rejoining
-var g_userRating; // player rating
+var g_UserRating;
 
 function init(attribs)
 {
-	g_userRating = attribs.rating;
+	g_UserRating = attribs.rating;
 	switch (attribs.multiplayerGameType)
 	{
 	case "join":
@@ -31,13 +31,14 @@ function init(attribs)
 		{
 			Engine.GetGUIObjectByName("hostServerNameWrapper").hidden = false;
 			Engine.GetGUIObjectByName("hostPlayerName").caption = attribs.name;
-			Engine.GetGUIObjectByName("hostServerName").caption = sprintf(translate("%(name)s's game"), { name: attribs.name });
+			Engine.GetGUIObjectByName("hostServerName").caption =
+				sprintf(translate("%(name)s's game"), { "name": attribs.name });
 		}
 		else
 			Engine.GetGUIObjectByName("hostPlayerNameWrapper").hidden = false;
 		break;
 	default:
-		error(sprintf("Unrecognised multiplayer game type: %(gameType)s", { gameType: multiplayerGameType }));
+		error("Unrecognised multiplayer game type: " + attribs.multiplayerGameType);
 		break;
 	}
 }
@@ -46,9 +47,10 @@ function cancelSetup()
 {
 	if (g_IsConnecting)
 		Engine.DisconnectNetworkGame();
-	// Set player lobby presence
+
 	if (Engine.HasXmppClient())
 		Engine.LobbySetPlayerPresence("available");
+
 	Engine.PopGuiPage();
 }
 
@@ -76,7 +78,7 @@ function pollAndHandleNetworkClient()
 		if (!message)
 			break;
 
-		log(sprintf(translate("Net message: %(message)s"), { message: uneval(message) }));
+		log(sprintf(translate("Net message: %(message)s"), { "message": uneval(message) }));
 
 		// If we're rejoining an active game, we don't want to actually display
 		// the game setup screen, so perform similar processing to gamesetup.js
@@ -94,7 +96,7 @@ function pollAndHandleNetworkClient()
 					return;
 
 				default:
-					error(sprintf("Unrecognised netstatus type %(netType)s", { netType: message.status }));
+					error("Unrecognised netstatus type: " + message.status);
 					break;
 				}
 				break;
@@ -108,19 +110,31 @@ function pollAndHandleNetworkClient()
 				break;
 
 			case "start":
+
+				// Copy playernames from initial player assignment to the settings
+				for (let guid in g_PlayerAssignments)
+				{
+					let player = g_PlayerAssignments[guid];
+					if (player.player > 0)	// not observer or GAIA
+						g_GameAttributes.settings.PlayerData[player.player - 1].Name = player.name;
+				}
+
 				Engine.SwitchGuiPage("page_loading.xml", {
 					"attribs": g_GameAttributes,
 					"isNetworked" : true,
+					"isRejoining" : g_IsRejoining,
 					"playerAssignments": g_PlayerAssignments
 				});
 				break;
 
 			case "chat":
-				// Ignore, since we have nowhere to display chat messages
+				break;
+
+			case "netwarn":
 				break;
 
 			default:
-				error(sprintf("Unrecognised net message type %(messageType)s", { messageType: message.type }));
+				error("Unrecognised net message type: " + message.type);
 			}
 		}
 		else
@@ -155,12 +169,16 @@ function pollAndHandleNetworkClient()
 					return;
 
 				default:
-					error(sprintf("Unrecognised netstatus type %(netType)s", { netType: message.status }));
+					error("Unrecognised netstatus type: " + message.status);
 					break;
 				}
 				break;
+
+			case "netwarn":
+				break;
+
 			default:
-				error(sprintf("Unrecognised net message type %(messageType)s", { messageType: message.type }));
+				error("Unrecognised net message type: " + message.type);
 				break;
 			}
 		}
@@ -177,11 +195,12 @@ function startHost(playername, servername)
 {
 	// Save player name
 	Engine.ConfigDB_CreateValue("user", "playername", playername);
-	Engine.ConfigDB_WriteFile("user", "config/user.cfg");
+	Engine.ConfigDB_WriteValueToFile("user", "playername", playername, "config/user.cfg");
+
 	// Disallow identically named games in the multiplayer lobby
 	if (Engine.HasXmppClient())
 	{
-		for each (var g in Engine.GetGameList())
+		for (let g of Engine.GetGameList())
 		{
 			if (g.name === servername)
 			{
@@ -192,8 +211,8 @@ function startHost(playername, servername)
 	}
 	try
 	{
-		if (g_userRating)
-			Engine.StartNetworkHost(playername + " (" + g_userRating + ")");
+		if (g_UserRating)
+			Engine.StartNetworkHost(playername + " (" + g_UserRating + ")");
 		else
 			Engine.StartNetworkHost(playername);
 	}
@@ -201,30 +220,26 @@ function startHost(playername, servername)
 	{
 		cancelSetup();
 		messageBox(400, 200,
-			sprintf("Cannot host game: %(message)s.", { message: e.message }),
+			sprintf("Cannot host game: %(message)s.", { "message": e.message }),
 			"Error", 2);
 		return false;
 	}
 
 	startConnectionStatus("server");
 	g_ServerName = servername;
-	// Set player lobby presence
+
 	if (Engine.HasXmppClient())
 		Engine.LobbySetPlayerPresence("playing");
+
 	return true;
 }
 
 function startJoin(playername, ip)
 {
-	// Save player name and host address
-	Engine.ConfigDB_CreateValue("user", "playername", playername);
-	Engine.ConfigDB_CreateValue("user", "multiplayerserver", ip);
-	Engine.ConfigDB_WriteFile("user", "config/user.cfg");
-
 	try
 	{
-		if (g_userRating)
-			Engine.StartNetworkJoin(playername + " (" + g_userRating + ")", ip);
+		if (g_UserRating)
+			Engine.StartNetworkJoin(playername + " (" + g_UserRating + ")", ip);
 		else
 			Engine.StartNetworkJoin(playername, ip);
 	}
@@ -232,19 +247,29 @@ function startJoin(playername, ip)
 	{
 		cancelSetup();
 		messageBox(400, 200,
-			sprintf("Cannot join game: %(message)s.", { message: e.message }),
+			sprintf("Cannot join game: %(message)s.", { "message": e.message }),
 			"Error", 2);
 		return false;
 	}
 
 	startConnectionStatus("client");
-	// Set player lobby presence
+
 	if (Engine.HasXmppClient())
 		Engine.LobbySetPlayerPresence("playing");
+	else
+	{
+		// Only save the player name and host address if they're valid and we're not in the lobby
+		Engine.ConfigDB_CreateValue("user", "playername", playername);
+		Engine.ConfigDB_WriteValueToFile("user", "playername", playername, "config/user.cfg");
+		Engine.ConfigDB_CreateValue("user", "multiplayerserver", ip);
+		Engine.ConfigDB_WriteValueToFile("user", "multiplayerserver", ip, "config/user.cfg");
+	}
 	return true;
 }
 
 function getDefaultGameName()
 {
-	return sprintf(translate("%(playername)s's game"), { playername: Engine.ConfigDB_GetValue("user", "playername")});
+	return sprintf(translate("%(playername)s's game"), {
+		"playername": Engine.ConfigDB_GetValue("user", "playername")
+	});
 }
