@@ -12,117 +12,99 @@ function Cheat(input)
 
 	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	if (!cmpPlayer.GetCheatsEnabled())
-	{
-		cmpGuiInterface.PushNotification({"type": "chat", "players": [input.player], "message": "Cheats are disbaled in this match"});
 		return;
-	}
 
 	switch(input.action)
 	{
 	case "addresource":
-		// force input.text to be an array
-		input.text = [].concat(input.text);
-		for each (var type in input.text)
-			cmpPlayer.AddResource(type, input.parameter);
-		break;
+		cmpPlayer.AddResource(input.text, input.parameter);
+		return;
 	case "revealmap":
 		var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 		cmpRangeManager.SetLosRevealAll(-1, true);
-		break;
+		return;
 	case "maxpopulation":
 		cmpPlayer.SetPopulationBonuses(500);
-		break;
+		return;
 	case "changemaxpopulation":
 		cmpPlayer.SetMaxPopulation(500);
-		break;
+		return;
 	case "convertunit":
-		for each (var ent in input.selected)
+		for (let ent of input.selected)
 		{
-			var cmpOwnership = Engine.QueryInterface(ent, IID_Ownership);
-			cmpOwnership.SetOwner(cmpPlayer.GetPlayerID());
+			let cmpOwnership = Engine.QueryInterface(ent, IID_Ownership);
+			if (cmpOwnership)
+				cmpOwnership.SetOwner(cmpPlayer.GetPlayerID());
 		}
-		break;
+		return;
 	case "killunits":
-		for each (var ent in input.selected)
+		for (let ent of input.selected)
 		{
-			var cmpHealth = Engine.QueryInterface(ent, IID_Health);
+			let cmpHealth = Engine.QueryInterface(ent, IID_Health);
 			if (cmpHealth)
 				cmpHealth.Kill();
 			else
 				Engine.DestroyEntity(ent);
 		}
-		break;
+		return;
 	case "defeatplayer":
 		var playerEnt = cmpPlayerManager.GetPlayerByID(input.parameter);
 		if (playerEnt == INVALID_ENTITY)
 			return;
-		Engine.PostMessage(playerEnt, MT_PlayerDefeated, { "playerId": input.parameter } );
-		break;
+		Engine.PostMessage(playerEnt, MT_PlayerDefeated, { "playerId": input.parameter });
+		return;
 	case "createunits":
-		if (!input.selected[0])
+		var cmpProductionQueue = input.selected.length && Engine.QueryInterface(input.selected[0], IID_ProductionQueue);
+		if (!cmpProductionQueue)
 		{
-			cmpGuiInterface.PushNotification({"type": "notification", "players": [input.player], "message": "You need to select a building that trains units."});
+			cmpGuiInterface.PushNotification({
+				"type": "text",
+				"players": [input.player],
+				"message": markForTranslation("You need to select a building that trains units."),
+				"translateMessage": true
+			});
 			return;
 		}
 
-		var cmpProductionQueue = Engine.QueryInterface(input.selected[0], IID_ProductionQueue);
-		if (!cmpProductionQueue)
-		{
-			cmpGuiInterface.PushNotification({"type": "notification", "players": [input.player], "message": "You need to select a building that trains units."});
-			return;
-		}
-		for (var i = 0; i < input.parameter; i++)
-			cmpProductionQueue.SpawnUnits (input.templates[i%(input.templates.length)],1, null);
-		break;
+		for (let i = 0; i < Math.min(input.parameter, cmpPlayer.GetMaxPopulation() - cmpPlayer.GetPopulationCount()); ++i)
+			cmpProductionQueue.SpawnUnits(input.templates[i % input.templates.length], 1, null);
+		return;
 	case "fastactions":
 		cmpPlayer.SetCheatTimeMultiplier((cmpPlayer.GetCheatTimeMultiplier() == 1) ? 0.01 : 1);
-		break;
+		return;
 	case "changespeed":
 		cmpPlayer.SetCheatTimeMultiplier(input.parameter);
-		break;
-	case "changephase": 
-		var cmpTechnologyManager = Engine.QueryInterface(playerEnt, IID_TechnologyManager); 
+		return;
+	case "changephase":
+		var cmpTechnologyManager = Engine.QueryInterface(playerEnt, IID_TechnologyManager);
 		if (!cmpTechnologyManager)
 			return;
 
-		// get phase we want to go
-		var phaseToGo;
-		var version = "generic";
-		var special;
+		// store the phase we want in the next input parameter
+		var parameter;
 		if (!cmpTechnologyManager.IsTechnologyResearched("phase_town"))
-		{
-			phaseToGo = "town";
-			special = {"athen":"athen"};
-		}
+			parameter = "phase_town";
 		else if (!cmpTechnologyManager.IsTechnologyResearched("phase_city"))
-		{
-			phaseToGo = "city";
-			special = {"athen":"athen", "celt": Math.round(Math.random()) == 0 ? "britons":"gauls"};
-		}
+			parameter = "phase_city";
 		else
 			return;
 
-		// check, if special civ
-		if (cmpPlayer.civ in special)
-			version = special[cmpPlayer.civ];
-			
-		// rewrite input and call function
-		input.action = "researchTechnology";
-		input.parameter="phase_" + phaseToGo + "_" + version; 
-		Cheat(input);
-		break; 
-	case "researchTechnology": 
-		// check, if name of technology is given
-		if (input.parameter.length == 0)
-		{
-			cmpGuiInterface.PushNotification({"type": "notification", "players": [input.player], "message": "You have to enter the name of a technology or select a building and enter the number of the technology (brainiac number [top|paired].)"});
+		// check if specialised tech exists (like phase_town_athen)
+		var cmpTechnologyTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager);
+		if (cmpTechnologyTemplateManager.ListAllTechs().indexOf(parameter + "_" + cmpPlayer.civ) > -1)
+			parameter += "_" + cmpPlayer.civ;
+
+		Cheat({ "player": input.player, "action": "researchTechnology", "parameter": parameter, "selected": input.selected });
+		return;
+	case "researchTechnology":
+		if (!input.parameter.length)
 			return;
-		}
+
 		var techname = input.parameter;
-		var cmpTechnologyManager = Engine.QueryInterface(playerEnt, IID_TechnologyManager); 
+		var cmpTechnologyManager = Engine.QueryInterface(playerEnt, IID_TechnologyManager);
 		if (!cmpTechnologyManager)
 			return;
-			
+
 		// check, if building is selected
 		if (input.selected[0])
 		{
@@ -143,10 +125,8 @@ function Cheat(input)
 					{
 						var tech = techs[number-1];
 						if (!tech)
-						{
-							cmpGuiInterface.PushNotification({"type": "notification", "players": [input.player], "message": "You have already researched this technology."});
 							return;
-						}
+
 						// get name of tech
 						if (tech.pair)
 							techname = tech[pair];
@@ -154,29 +134,32 @@ function Cheat(input)
 							techname = tech;
 					}
 					else
-					{
-						cmpGuiInterface.PushNotification({"type": "notification", "players": [input.player], "message": "This building only has " + techs.length + " technologies."});
 						return;
-					}
 				}
-			}	
+			}
 		}
 
 		// check, if technology exists
 		var template = cmpTechnologyManager.GetTechnologyTemplate(techname);
 		if (!template)
-		{
-			cmpGuiInterface.PushNotification({"type": "notification", "players": [input.player], "message": "Technology \"" + techname + "\" does not exist"});
 			return;
-		}
 
 		// check, if technology is already researched
 		if (!cmpTechnologyManager.IsTechnologyResearched(techname))
-			cmpTechnologyManager.ResearchTechnology(techname); 
-		break; 
+			cmpTechnologyManager.ResearchTechnology(techname);
+		return;
+	case "metaCheat":
+		for (let resource of ["food", "wood", "metal", "stone"])
+			Cheat({ "player": input.player, "action": "addresource", "text": resource, "parameter": input.parameter });
+		Cheat({ "player": input.player, "action": "maxpopulation" });
+		Cheat({ "player": input.player, "action": "changemaxpopulation" });
+		Cheat({ "player": input.player, "action": "fastactions" });
+		for (let i=0; i<2; ++i)
+			Cheat({ "player": input.player, "action": "changephase", "selected": input.selected });
+		return;
 	default:
 		warn("Cheat '" + input.action + "' is not implemented");
-		break;
+		return;
 	}
 }
 
