@@ -51,7 +51,7 @@ Damage.CauseSplashDamage = function(data)
 			warn("The " + data.shape + " splash damage shape is not implemented!");
 		}
 		// Call CauseDamage which reduces the hitpoints, posts network command, plays sounds....
-		Damage.CauseDamage({"strengths":data.strengths, "target":entity, "attacker":data.attacker, "multiplier":damageMultiplier, "type":data.type + ".Splash"})
+		Damage.CauseDamage({"strengths":data.strengths, "target":entity, "attacker":data.attacker, "multiplier":damageMultiplier, "type":data.type + ".Splash"});
 	}
 };
 
@@ -72,7 +72,7 @@ Damage.CauseDamage = function(data)
 		return;
 
 	// Damage the target
-	var targetState = cmpDamageReceiver.TakeDamage(data.strengths.hack * data.multiplier, data.strengths.pierce * data.multiplier, data.strengths.crush * data.multiplier, data.attacker);
+	var targetState = cmpDamageReceiver.TakeDamage(data.strengths.hack * data.multiplier, data.strengths.pierce * data.multiplier, data.strengths.crush * data.multiplier);
 
 	// If the target was killed run some cleanup
 	if (targetState.killed)
