@@ -15,7 +15,7 @@ function LoadPlayerSettings(settings, newPlayers)
 		settings = {};
 
 	// Get default player data
-	var rawData = Engine.ReadJSONFile("player_defaults.json");
+	var rawData = Engine.ReadJSONFile("settings/player_defaults.json");
 	if (!(rawData && rawData.PlayerData))
 		throw("Player.js: Error reading player_defaults.json");
 
@@ -82,8 +82,8 @@ function LoadPlayerSettings(settings, newPlayers)
 		var cmpPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(i), IID_Player);
 		cmpPlayer.SetName(getSetting(playerData, playerDefaults, i, "Name"));
 		cmpPlayer.SetCiv(getSetting(playerData, playerDefaults, i, "Civ"));
-		var colour = getSetting(playerData, playerDefaults, i, "Colour");
-		cmpPlayer.SetColour(colour.r, colour.g, colour.b);
+		var color = getSetting(playerData, playerDefaults, i, "Color");
+		cmpPlayer.SetColor(color.r, color.g, color.b);
 
 		// Special case for gaia
 		if (i == 0)
@@ -115,8 +115,13 @@ function LoadPlayerSettings(settings, newPlayers)
 		if (getSetting(playerData, playerDefaults, i, "DisabledTechnologies") !== undefined)
 			cmpPlayer.SetDisabledTechnologies(getSetting(playerData, playerDefaults, i, "DisabledTechnologies"));
 
+		let disabledTemplates = []; 
+		if (settings.DisabledTemplates !== undefined)
+			disabledTemplates = settings.DisabledTemplates;
 		if (getSetting(playerData, playerDefaults, i, "DisabledTemplates") !== undefined)
-			cmpPlayer.SetDisabledTemplates(getSetting(playerData, playerDefaults, i, "DisabledTemplates"));
+			disabledTemplates = disabledTemplates.concat(getSetting(playerData, playerDefaults, i, "DisabledTemplates"));
+		if (disabledTemplates.length)
+			cmpPlayer.SetDisabledTemplates(disabledTemplates);
 
 		// If diplomacy explicitly defined, use that; otherwise use teams
 		if (getSetting(playerData, playerDefaults, i, "Diplomacy") !== undefined)
@@ -163,6 +168,10 @@ function LoadPlayerSettings(settings, newPlayers)
 			var cmpPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(i), IID_Player);
 			cmpPlayer.SetLockTeams(true);
 		}
+
+	// Disable the AIIinterface when no AI players are present
+	if (playerData && !playerData.some(function(v) { return v && v.AI ? true : false; }))
+		Engine.QueryInterface(SYSTEM_ENTITY, IID_AIInterface).Disable();
 }
 
 // Get a setting if it exists or return default
@@ -183,7 +192,7 @@ function getSetting(settings, defaults, idx, property)
  * that owns the given entity.
  * iid is typically IID_Player.
  */
-function QueryOwnerInterface(ent, iid)
+function QueryOwnerInterface(ent, iid = IID_Player)
 {
 	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
 
@@ -207,7 +216,7 @@ function QueryOwnerInterface(ent, iid)
  * with the given ID number.
  * iid is typically IID_Player.
  */
-function QueryPlayerIDInterface(id, iid)
+function QueryPlayerIDInterface(id, iid = IID_Player)
 {
 	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
 
@@ -218,6 +227,31 @@ function QueryPlayerIDInterface(id, iid)
 	return Engine.QueryInterface(playerEnt, iid);
 }
 
+/**
+ * Similar to Engine.QueryInterface but first checks if the entity
+ * mirages the interface.
+ */
+function QueryMiragedInterface(ent, iid)
+{
+	var cmp = Engine.QueryInterface(ent, IID_Mirage);
+	if (cmp && !cmp.Mirages(iid))
+		return null;
+	else if (!cmp)
+		cmp = Engine.QueryInterface(ent, iid);
+
+	return cmp;
+}
+
+/**
+ * Similar to Engine.QueryInterface, but checks for all interfaces
+ * implementing a builder list (currently Foundation and Repairable)
+ * TODO Foundation and Repairable could both implement a BuilderList component
+ */
+function QueryBuilderListInterface(ent)
+{
+	return Engine.QueryInterface(ent, IID_Foundation) || Engine.QueryInterface(ent, IID_Repairable);
+}
+
 /**
  * Returns true if the entity 'target' is owned by an ally of
  * the owner of 'entity'.
@@ -375,6 +409,8 @@ function IsOwnedByEnemyOfPlayer(player, target)
 Engine.RegisterGlobal("LoadPlayerSettings", LoadPlayerSettings);
 Engine.RegisterGlobal("QueryOwnerInterface", QueryOwnerInterface);
 Engine.RegisterGlobal("QueryPlayerIDInterface", QueryPlayerIDInterface);
+Engine.RegisterGlobal("QueryMiragedInterface", QueryMiragedInterface);
+Engine.RegisterGlobal("QueryBuilderListInterface", QueryBuilderListInterface);
 Engine.RegisterGlobal("IsOwnedByAllyOfEntity", IsOwnedByAllyOfEntity);
 Engine.RegisterGlobal("IsOwnedByMutualAllyOfEntity", IsOwnedByMutualAllyOfEntity);
 Engine.RegisterGlobal("IsOwnedByPlayer", IsOwnedByPlayer);
