@@ -25,6 +25,16 @@ function LoadMapSettings(settings)
 			cmpRangeManager.SetLosRevealAll(-1, true);
 	}
 
+	if (settings.DisableTreasures)
+	{
+		for (let ent of Engine.GetEntitiesWithInterface(IID_ResourceSupply))
+		{
+			let cmpResourceSupply = Engine.QueryInterface(ent, IID_ResourceSupply);
+			if (cmpResourceSupply.GetType().generic == "treasure")
+				Engine.DestroyEntity(ent);
+		}
+	}
+
 	if (settings.CircularMap)
 	{
 		var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
@@ -51,6 +61,10 @@ function LoadMapSettings(settings)
 				cmpGarrisonHolder.initGarrison = settings.Garrison[holder];
 		}
 	}
+	
+	var cmpCeasefireManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_CeasefireManager);
+	if (settings.Ceasefire)
+		cmpCeasefireManager.StartCeasefire(settings.Ceasefire * 60 * 1000);
 }
 
 Engine.RegisterGlobal("LoadMapSettings", LoadMapSettings);
