@@ -1,5 +1,5 @@
 // This constant used to adjust gain value depending on distance
-const DISTANCE_FACTOR = 1 / 110;
+const DISTANCE_FACTOR = 1 / 115;
 
 // Additional gain (applying to each market) for trading performed between markets of different players, in percents
 const INTERNATIONAL_TRADING_ADDITION = 25;
@@ -11,24 +11,28 @@ function CalculateTraderGain(firstMarket, secondMarket, template, trader)
 
 	var cmpFirstMarketPosition = Engine.QueryInterface(firstMarket, IID_Position);
 	var cmpSecondMarketPosition = Engine.QueryInterface(secondMarket, IID_Position);
-	if (!cmpFirstMarketPosition || !cmpFirstMarketPosition.IsInWorld() || !cmpSecondMarketPosition || !cmpSecondMarketPosition.IsInWorld())
+	if (!cmpFirstMarketPosition || !cmpFirstMarketPosition.IsInWorld() ||
+	    !cmpSecondMarketPosition || !cmpSecondMarketPosition.IsInWorld())
 		return null;
 	var firstMarketPosition = cmpFirstMarketPosition.GetPosition2D();
 	var secondMarketPosition = cmpSecondMarketPosition.GetPosition2D();
 
 	// Calculate ordinary Euclidean distance between markets.
 	// We don't use pathfinder, because ordinary distance looks more fair.
-	var distance = Math.sqrt(Math.pow(firstMarketPosition.x - secondMarketPosition.x, 2) + Math.pow(firstMarketPosition.y - secondMarketPosition.y, 2));
+	var distance = firstMarketPosition.distanceTo(secondMarketPosition);
 	// We calculate gain as square of distance to encourage trading between remote markets
 	gain.traderGain = Math.pow(distance * DISTANCE_FACTOR, 2);
 	if (template && template.GainMultiplier)
-		gain.traderGain *= template.GainMultiplier;
-	gain.traderGain = Math.round(gain.traderGain);
+	{
+		if (trader)
+			gain.traderGain *= ApplyValueModificationsToEntity("Trader/GainMultiplier", +template.GainMultiplier, trader);
+		else	// called from the gui with modifications already applied
+			gain.traderGain *= template.GainMultiplier;
+	}
 	// If trader undefined, the trader owner is supposed to be the same as the first market
-	if (trader)
-		var cmpOwnership = Engine.QueryInterface(trader, IID_Ownership);
-	else
-		var cmpOwnership = Engine.QueryInterface(firstMarket, IID_Ownership);
+	var cmpOwnership = trader ? Engine.QueryInterface(trader, IID_Ownership) : Engine.QueryInterface(firstMarket, IID_Ownership);
+	if (!cmpOwnership)
+		return null;
 	gain.traderOwner = cmpOwnership.GetOwner();
 
 	// If markets belong to different players, add gain from international trading
@@ -36,13 +40,27 @@ function CalculateTraderGain(firstMarket, secondMarket, template, trader)
 	var ownerSecondMarket = Engine.QueryInterface(secondMarket, IID_Ownership).GetOwner();
 	if (ownerFirstMarket != ownerSecondMarket)
 	{
-		var internationalGain1 = ApplyValueModificationsToEntity("Trade/International", INTERNATIONAL_TRADING_ADDITION, firstMarket);
-		gain.market1Gain = Math.round(gain.traderGain * internationalGain1 / 100);
+		gain.market1Gain = gain.traderGain * ApplyValueModificationsToEntity("Trade/International", INTERNATIONAL_TRADING_ADDITION, firstMarket) / 100;
 		gain.market1Owner = ownerFirstMarket;
-		var internationalGain2 = ApplyValueModificationsToEntity("Trade/International", INTERNATIONAL_TRADING_ADDITION, secondMarket);
-		gain.market2Gain = Math.round(gain.traderGain * internationalGain2 / 100);
+		gain.market2Gain = gain.traderGain * ApplyValueModificationsToEntity("Trade/International", INTERNATIONAL_TRADING_ADDITION, secondMarket) / 100;
 		gain.market2Owner = ownerSecondMarket;
+	}
 
+	// Add potential trade multipliers and roundings
+	var cmpPlayer = trader ? QueryOwnerInterface(trader) : QueryOwnerInterface(firstMarket);
+	if (cmpPlayer)
+		gain.traderGain *= cmpPlayer.GetTradeRateMultiplier();
+	gain.traderGain = Math.round(gain.traderGain);
+
+	if (ownerFirstMarket != ownerSecondMarket)
+	{
+		if ((cmpPlayer = QueryOwnerInterface(firstMarket)))
+			gain.market1Gain *= cmpPlayer.GetTradeRateMultiplier();
+		gain.market1Gain = Math.round(gain.market1Gain);
+
+		if ((cmpPlayer = QueryOwnerInterface(secondMarket)))
+			gain.market2Gain *= cmpPlayer.GetTradeRateMultiplier();
+		gain.market2Gain = Math.round(gain.market2Gain);
 	}
 
 	return gain;
