@@ -3,7 +3,7 @@ var g_EncrytedPassword = "";
 var g_PasswordInputIsHidden = false;
 var g_TermsOfServiceRead = false;
 var g_TermsOfUseRead = false;
-var g_hasSystemMessage = false;
+var g_DisplayingSystemMessage = false;
 
 function init()
 {
@@ -16,7 +16,7 @@ function lobbyStop()
 {
 	Engine.GetGUIObjectByName("feedback").caption = "";
 
-	if (g_LobbyIsConnecting == false)
+	if (!g_LobbyIsConnecting)
 		return;
 
 	g_LobbyIsConnecting = false;
@@ -69,21 +69,13 @@ function lobbyStartRegister()
 function onTick()
 {
 	var pageRegisterHidden = Engine.GetGUIObjectByName("pageRegister").hidden;
-	if (pageRegisterHidden)
-	{
-		var username = Engine.GetGUIObjectByName("connectUsername").caption;
-		var password = Engine.GetGUIObjectByName("connectPassword").caption;
-	}
-	else
-	{
-		var username = Engine.GetGUIObjectByName("registerUsername").caption;
-		var password = Engine.GetGUIObjectByName("registerPassword").caption;
-	}
+	var username = Engine.GetGUIObjectByName(pageRegisterHidden ? "connectUsername" : "registerUsername").caption;
+	var password = Engine.GetGUIObjectByName(pageRegisterHidden ? "connectPassword" : "registerPassword").caption;
 	var passwordAgain = Engine.GetGUIObjectByName("registerPasswordAgain").caption;
+
 	var agreeTerms = Engine.GetGUIObjectByName("registerAgreeTerms");
 	var feedback = Engine.GetGUIObjectByName("feedback");
 	var continueButton = Engine.GetGUIObjectByName("continue");
-	var sanitizedName = sanitizePlayerName(username, true, true);
 
 	// Do not change feedback while connecting.
 	if (g_LobbyIsConnecting) {}
@@ -91,6 +83,7 @@ function onTick()
 	else if (!Engine.GetGUIObjectByName("pageWelcome").hidden)
 	{
 		feedback.caption = "";
+		g_DisplayingSystemMessage = false;
 	}
 	// Check that they entered a username.
 	else if (!username)
@@ -99,10 +92,10 @@ function onTick()
 		feedback.caption = translate("Please enter your username");
 	}
 	// Check that they are using a valid username.
-	else if (username != sanitizedName)
+	else if (username != sanitizePlayerName(username, true, true))
 	{
 		continueButton.enabled = false;
-		feedback.caption = translate("Usernames can't contain [, ], unicode, whitespace, or commas");
+		feedback.caption = translate("Usernames can't contain \\[, ], unicode, whitespace, or commas");
 	}
 	// Check that they entered a password.
 	else if (!password)
@@ -113,7 +106,7 @@ function onTick()
 	// Allow them to connect if tests pass up to this point.
 	else if (pageRegisterHidden)
 	{
-		if (!g_hasSystemMessage)
+		if (!g_DisplayingSystemMessage)
 			feedback.caption = "";
 		continueButton.enabled = true;
 	}
@@ -150,61 +143,53 @@ function onTick()
 	// Allow them to register.
 	else
 	{
-		if (!g_hasSystemMessage)
+		if (!g_DisplayingSystemMessage)
 			feedback.caption = "";
 		continueButton.enabled = true;
 	}
 
-	if (!g_LobbyIsConnecting)
-		// The Xmpp Client has not been created
-		return;
-
-	// The XmppClient has been created, we are waiting
-	// to be connected or to receive an error.
-
-	//Receive messages
-	while (true)
+	// Handle queued messages from the XMPP client (if running and if any)
+	var message;
+	while ((message = Engine.LobbyGuiPollMessage()) != undefined)
 	{
-		var message = Engine.LobbyGuiPollMessage();
-		if (!message)
-			break;
+		// TODO: Properly deal with unrecognized messages
+		if (message.type != "system" || !message.level)
+			continue;
 
-		if (message.type == "muc" && message.level == "join")
+		g_LobbyIsConnecting = false;
+
+		switch(message.level) {
+		case "error":
+		case "disconnected":
+		{
+			Engine.GetGUIObjectByName("feedback").caption = message.text;
+			g_DisplayingSystemMessage = true;
+			Engine.StopXmppClient();
+			break;
+		}
+		case "registered":
+			Engine.GetGUIObjectByName("feedback").caption = translate("Registered");
+			g_DisplayingSystemMessage = true;
+			Engine.GetGUIObjectByName("connectUsername").caption = username;
+			Engine.GetGUIObjectByName("connectPassword").caption = password;
+			Engine.StopXmppClient();
+			switchPage("connect");
+			break;
+		case "connected":
 		{
-			// We are connected, switch to the lobby page
 			Engine.PopGuiPage();
-			// Use username as nick.
-			var nick = sanitizePlayerName(username, true, true);
-
-			// Switch to lobby
 			Engine.SwitchGuiPage("page_lobby.xml");
-			// Store nick, login, and password
-			Engine.ConfigDB_CreateValue("user", "playername", nick);
+			Engine.ConfigDB_CreateValue("user", "playername", sanitizePlayerName(username, true, true));
+			Engine.ConfigDB_WriteValueToFile("user", "playername", sanitizePlayerName(username, true, true), "config/user.cfg");
 			Engine.ConfigDB_CreateValue("user", "lobby.login", username);
+			Engine.ConfigDB_WriteValueToFile("user", "lobby.login", username, "config/user.cfg");
 			// We only store the encrypted password, so make sure to re-encrypt it if changed before saving.
 			if (password != g_EncrytedPassword.substring(0, 10))
 				g_EncrytedPassword = Engine.EncryptPassword(password, username);
 			Engine.ConfigDB_CreateValue("user", "lobby.password", g_EncrytedPassword);
-			Engine.ConfigDB_WriteFile("user", "config/user.cfg");
-
-			return;
+			Engine.ConfigDB_WriteValueToFile("user", "lobby.password", g_EncrytedPassword, "config/user.cfg");
+			break;
 		}
-		else if (message.type == "system" && message.text == "registered")
-		{
-			// Great, we are registered. Switch to the connection window.
-			feedback.caption = translate("Registered");
-			Engine.StopXmppClient();
-			g_LobbyIsConnecting = false;
-			Engine.GetGUIObjectByName("connectUsername").caption = username;
-			Engine.GetGUIObjectByName("connectPassword").caption = password;
-			switchPage("connect");
-		}
-		else if(message.type == "system" && (message.level == "error" || message.text == "disconnected"))
-		{
-			g_hasSystemMessage = true;
-			feedback.caption = message.text == "disconnected" ? translate("Disconnected") : message.text;
-			Engine.StopXmppClient();
-			g_LobbyIsConnecting = false;
 		}
 	}
 }
@@ -234,40 +219,40 @@ function switchPage(page)
 	// Then show appropriate page.
 	switch(page)
 	{
-		case "welcome":
-			Engine.GetGUIObjectByName("pageWelcome").hidden = false;
-			break;
-		case "register":
-			var dialog = Engine.GetGUIObjectByName("dialog");
-			var newSize = dialog.size;
-			newSize.bottom += 150;
-			dialog.size = newSize;
-			Engine.GetGUIObjectByName("pageRegister").hidden = false;
-			Engine.GetGUIObjectByName("continue").caption = translate("Register");
-			Engine.GetGUIObjectByName("continue").hidden = false;
-			break;
-		case "connect":
-			Engine.GetGUIObjectByName("pageConnect").hidden = false;
-			Engine.GetGUIObjectByName("continue").caption = translate("Connect");
-			Engine.GetGUIObjectByName("continue").hidden = false;
-			break;
+	case "welcome":
+		Engine.GetGUIObjectByName("pageWelcome").hidden = false;
+		break;
+	case "register":
+		var dialog = Engine.GetGUIObjectByName("dialog");
+		var newSize = dialog.size;
+		newSize.bottom += 150;
+		dialog.size = newSize;
+		Engine.GetGUIObjectByName("pageRegister").hidden = false;
+		Engine.GetGUIObjectByName("continue").caption = translate("Register");
+		Engine.GetGUIObjectByName("continue").hidden = false;
+		break;
+	case "connect":
+		Engine.GetGUIObjectByName("pageConnect").hidden = false;
+		Engine.GetGUIObjectByName("continue").caption = translate("Connect");
+		Engine.GetGUIObjectByName("continue").hidden = false;
+		break;
 	}
 }
 function openTermsOfService()
 {
 	g_TermsOfServiceRead = true;
 	Engine.PushGuiPage("page_manual.xml", {
-				"page":"lobby/Terms_of_Service",
-				"title":translate("Terms of Service"),
-				})
+		"page": "lobby/Terms_of_Service",
+		"title": translate("Terms of Service"),
+	});
 }
 
 function openTermsOfUse()
 {
 	g_TermsOfUseRead = true;
 	Engine.PushGuiPage("page_manual.xml", {
-				"page":"lobby/Terms_of_Use",
-				"title":translate("Terms of Use"),
-				})
+		"page": "lobby/Terms_of_Use",
+		"title": translate("Terms of Use"),
+	});
 }
 
