@@ -316,7 +316,7 @@ RMS.SetProgress(65);
 // create decoration
 var planetm = 1;
 
-if (random_terrain==7)
+if (random_terrain == g_BiomeTropic)
 	planetm = 8;
 
 createDecoration
