@@ -392,7 +392,7 @@ createStragglerTrees(types, avoidClasses(clWater, 1, clForest, 1, clHill, 1, clP
 
 // Set environment
 setSkySet("cirrus");
-setWaterColour(0.447, 0.412, 0.322);			// muddy brown
+setWaterColor(0.447, 0.412, 0.322);			// muddy brown
 setWaterTint(0.447, 0.412, 0.322);				// muddy brown
 setWaterMurkiness(1.0);
 setWaterWaviness(3.0);
