@@ -317,7 +317,7 @@ RMS.SetProgress(65);
 // create decoration
 var planetm = 1;
 
-if (random_terrain==7)
+if (random_terrain == g_BiomeTropic)
 	planetm = 8;
 
 createDecoration
@@ -384,7 +384,8 @@ RMS.SetProgress(85);
 log("Creating straggler trees...");
 var types = [oTree1, oTree2, oTree4, oTree3];	// some variation
 createStragglerTrees(types, [avoidClasses(clForest, 7, clHill, 1, clPlayer, 9, clMetal, 1, clRock, 1), stayClasses(clLand, 7)]);
-setWaterWaviness(10);
+setWaterWaviness(1.0);
+setWaterType("ocean");
 
 // Export map data
 ExportMap();
