@@ -494,7 +494,7 @@ createStragglerTrees(types, avoidClasses(clWater, 1, clForest, 1, clHill, 1, clP
 
 // Set environment
 setSkySet("cirrus");
-setWaterColour(0.114, 0.192, 0.463);
+setWaterColor(0.114, 0.192, 0.463);
 setWaterTint(0.255, 0.361, 0.651);
 setWaterWaviness(3.0);
 setWaterType("ocean");
