@@ -240,7 +240,7 @@ for (var i = 0; i < numIslands; ++i)
 	if (!landAreaLen)
 		break;
 	
-	chosenPoint = landAreas[randInt(0, landAreaLen)];
+	chosenPoint = landAreas[randInt(landAreaLen)];
 	
 	// create big islands
 	placer = new ChainPlacer(floor(scaleByMapSize(4, 8)), floor(scaleByMapSize(8, 14)), floor(scaleByMapSize(25, 60)), 0.07, chosenPoint[0], chosenPoint[1], scaleByMapSize(30,70));
@@ -256,16 +256,16 @@ for (var i = 0; i < numIslands; ++i)
 		avoidClasses(clLand, 3, clPlayer, 3),
 		1, 1
 	);
-	if (newIsland !== undefined)
+	if (newIsland && newIsland.length)
 	{
-		var temp = []
+		var n = 0;
 		for (var j = 0; j < landAreaLen; ++j)
 		{
 			var x = landAreas[j][0], z = landAreas[j][1];
 			if (playerConstraint.allows(x, z) && landConstraint.allows(x, z))
-					temp.push([x, z]);
+				landAreas[n++] = landAreas[j];
 		}
-		landAreas = temp;
+		landAreas.length = n;
 	}
 }
 
@@ -350,13 +350,13 @@ createAreas(
 
 
 // calculate desired number of trees for map (based on size)
-if (random_terrain == 6)
+if (random_terrain == g_BiomeSavanna)
 {
 	var MIN_TREES = 200;
 	var MAX_TREES = 1250;
 	var P_FOREST = 0;
 }
-else if (random_terrain == 7)
+else if (random_terrain == g_BiomeTropic)
 {
 	var MIN_TREES = 1000;
 	var MAX_TREES = 6000;
@@ -379,7 +379,7 @@ var types = [
 	[[tForestFloor1, tMainTerrain, pForest2], [tForestFloor1, pForest2]]
 ];	// some variation
 
-if (random_terrain != 6)
+if (random_terrain != g_BiomeSavanna)
 {
 	var size = numForest / (scaleByMapSize(3,6) * numPlayers);
 	var num = floor(size / types.length);
@@ -404,7 +404,7 @@ RMS.SetProgress(50);
 log("Creating dirt patches...");
 var sizes = [scaleByMapSize(3, 6), scaleByMapSize(5, 10), scaleByMapSize(8, 21)];
 var numb = 1;
-if (random_terrain == 6)
+if (random_terrain == g_BiomeSavanna)
 	numb = 3
 for (var i = 0; i < sizes.length; i++)
 {
@@ -556,10 +556,9 @@ for (var i = 0; i < types.length; ++i)
 }
 
 var planetm = 1;
-if (random_terrain==7)
-{
+if (random_terrain == g_BiomeTropic)
 	planetm = 8;
-}
+
 //create small grass tufts
 log("Creating small grass tufts...");
 group = new SimpleGroup(
