@@ -633,7 +633,7 @@ createObjectGroups(group, 0,
 
 // Adjust environment
 setSkySet("sunny");
-setWaterColour(0.024,0.212,0.024);
+setWaterColor(0.024,0.212,0.024);
 setWaterTint(0.133, 0.725,0.855);
 setWaterWaviness(2.5);
 setWaterType("ocean");
