@@ -511,12 +511,11 @@ createObjectGroups(group, 0,
 
 // Set environment
 setSkySet("cirrus");
-setWaterColour(0.443,0.412,0.322);
+setWaterColor(0.443,0.412,0.322);
 setWaterTint(0.647,0.82,0.949);
-setWaterReflectionTint(0.286,0.58,0.855);
-setWaterWaviness(2.5);
+setWaterWaviness(3.0);
+setWaterType("lake");
 setWaterMurkiness(0.80);
-setWaterReflectionTintStrength(0.25);
 
 // Export map data
 ExportMap();
\ No newline at end of file
