@@ -468,7 +468,7 @@ createObjectGroups(group, 0,
 	scaleByMapSize(16, 262)
 );
 
-setWaterColour(0, 0.227, 0.843);
+setWaterColor(0, 0.227, 0.843);
 setWaterTint(0, 0.545, 0.859);
 setWaterWaviness(1.0);
 setWaterType("clap");
