@@ -469,7 +469,7 @@ createObjectGroups(group, 0,
 
 // Set environment
 setSkySet("cirrus");
-setWaterColour(0.753,0.635,0.345);				// muddy brown
+setWaterColor(0.753,0.635,0.345);				// muddy brown
 setWaterTint(0.161,0.514,0.635);				// clear blue for blueness
 setWaterMurkiness(0.8);
 setWaterWaviness(1.0);
