@@ -1,6 +1,11 @@
-/////////////////////////////////////////////////////////////////////////////////////////////
-//	definitions
-/////////////////////////////////////////////////////////////////////////////////////////////
+const g_BiomeTemperate = 1;
+const g_BiomeSnowy = 2;
+const g_BiomeDesert = 3;
+const g_BiomeAlpine = 4;
+const g_BiomeMediterranean = 5;
+const g_BiomeSavanna = 6;
+const g_BiomeTropic = 7;
+const g_BiomeAutumn = 8;
 
 var biomeID = 1;
 var rbt1 = ["temp_grass_long_b"];
@@ -45,34 +50,44 @@ var rba7 = "actor|props/flora/bush_medit_me.xml";
 var rba8 = "actor|props/flora/bush_medit_sm.xml";
 var rba9 = "actor|flora/trees/oak.xml";
 
-
-/////////////////////////////////////////////////////////////////////////////////////////////
-//	randomizeBiome: randomizes the biomeID and returns the value
-/////////////////////////////////////////////////////////////////////////////////////////////
-
-function randomizeBiome()
+/**
+ * Randomizes environment, optionally excluding some biome IDs.
+ */
+function randomizeBiome(avoid = [])
 {
+	let biomeIndex;
+	do
+		biomeIndex = randInt(1, 8);
+	while (avoid.indexOf(biomeIndex) != -1);
 
-	var random_sky = randInt(1,3)
+	setBiome(biomeIndex);
+
+	return biomeIndex;
+}
+
+function setBiome(biomeIndex)
+{
+	var random_sky = randInt(1,3);
 	if (random_sky==1)
 		setSkySet("cirrus");
 	else if (random_sky ==2)
 		setSkySet("cumulus");
 	else if (random_sky ==3)
 		setSkySet("sunny");
+
 	setSunRotation(randFloat(0, TWO_PI));
 	setSunElevation(randFloat(PI/ 6, PI / 3));
 
-	setUnitsAmbientColour(0.57, 0.58, 0.55);
-	setTerrainAmbientColour(0.447059, 0.509804, 0.54902);
-	
-	biomeID = randInt(1,8);
-	//temperate
-	if (biomeID == 1){
-		
+	setUnitsAmbientColor(0.57, 0.58, 0.55);
+	setTerrainAmbientColor(0.447059, 0.509804, 0.54902);
+
+	biomeID = biomeIndex;
+
+	if (biomeID == g_BiomeTemperate)
+	{
 		// temperate ocean blue, a bit too deep and saturated perhaps but it looks nicer.
 		// this assumes ocean settings, maps that aren't oceans should reset.
-		setWaterColour(0.114, 0.192, 0.463);
+		setWaterColor(0.114, 0.192, 0.463);
 		setWaterTint(0.255, 0.361, 0.651);
 		setWaterWaviness(5.5);
 		setWaterMurkiness(0.83);
@@ -172,13 +187,12 @@ function randomizeBiome()
 		rba8 = "actor|props/flora/bush_medit_sm.xml";
 		rba9 = "actor|flora/trees/oak.xml";
 	}
-	//snowy
-	else if (biomeID == 2)
+	else if (biomeID == g_BiomeSnowy)
 	{
-		setSunColour(0.550, 0.601, 0.644);				// a little darker
+		setSunColor(0.550, 0.601, 0.644);				// a little darker
 		// Water is a semi-deep blue, fairly wavy, fairly murky for an ocean.
 		// this assumes ocean settings, maps that aren't oceans should reset.
-		setWaterColour(0.067, 0.212, 0.361);
+		setWaterColor(0.067, 0.212, 0.361);
 		setWaterTint(0.4, 0.486, 0.765);
 		setWaterWaviness(5.5);
 		setWaterMurkiness(0.83);
@@ -230,13 +244,12 @@ function randomizeBiome()
 		setPPSaturation(0.37);
 		setPPEffect("hdr");
 	}
-	//desert
-	else if (biomeID == 3)
+	else if (biomeID == g_BiomeDesert)
 	{
-		setSunColour(0.733, 0.746, 0.574);	
+		setSunColor(0.733, 0.746, 0.574);	
 
 		// Went for a very clear, slightly blue-ish water in this case, basically no waves.
-		setWaterColour(0, 0.227, 0.843);
+		setWaterColor(0, 0.227, 0.843);
 		setWaterTint(0, 0.545, 0.859);
 		setWaterWaviness(1);
 		setWaterMurkiness(0.22);
@@ -308,12 +321,11 @@ function randomizeBiome()
 		rba8 = "actor|props/flora/bush_desert_dry_a.xml";
 		rba9 = "actor|flora/trees/palm_date.xml";
 	}
-	//alpine
-	else if (biomeID == 4)
+	else if (biomeID == g_BiomeAlpine)
 	{
 		// simulates an alpine lake, fairly deep.
 		// this is not intended for a clear running river, or even an ocean.
-		setWaterColour(0.0, 0.047, 0.286);				// dark majestic blue
+		setWaterColor(0.0, 0.047, 0.286);				// dark majestic blue
 		setWaterTint(0.471, 0.776, 0.863);				// light blue
 		setWaterMurkiness(0.82);
 		setWaterWaviness(2);
@@ -368,11 +380,10 @@ function randomizeBiome()
 		rba8 = "actor|props/flora/bush_desert_a.xml";
 		rba9 = "actor|flora/trees/pine.xml";
 	}
-	//medit
-	else if (biomeID == 5)
+	else if (biomeID == g_BiomeMediterranean)
 	{
 		// Guess what, this is based on the colors of the mediterranean sea.
-		setWaterColour(0.024,0.212,0.024);
+		setWaterColor(0.024,0.212,0.024);
 		setWaterTint(0.133, 0.725,0.855);
 		setWaterWaviness(3);
 		setWaterMurkiness(0.8);
@@ -463,11 +474,10 @@ function randomizeBiome()
 		rba8 = "actor|props/flora/bush_medit_sm.xml";
 		rba9 = "actor|flora/trees/palm_cretan_date.xml";
 	}
-	//savanah
-	else if (biomeID == 6)
+	else if (biomeID == g_BiomeSavanna)
 	{
 		// Using the Malawi as a reference, in parts where it's not too murky from a river nearby.
-		setWaterColour(0.055,0.176,0.431);
+		setWaterColor(0.055,0.176,0.431);
 		setWaterTint(0.227,0.749,0.549);
 		setWaterWaviness(1.5);
 		setWaterMurkiness(0.77);
@@ -504,22 +514,17 @@ function randomizeBiome()
 		rbe5 = "gaia/flora_tree_baobab";
 		rbe6 = "gaia/flora_bush_grapes";
 		rbe7 = "gaia/fauna_chicken";
+
 		var rts = randInt(1,4);
-		if (rts==1){
+		if (rts == 1)
 			rbe8 = "gaia/fauna_wildebeest";
-		}
-		else if (rts==2)
-		{
+		else if (rts == 2)
 			rbe8 = "gaia/fauna_zebra";
-		}
-		else if (rts==3)
-		{
+		else if (rts == 3)
 			rbe8 = "gaia/fauna_giraffe";
-		}
-		else if (rts==4)
-		{
+		else if (rts == 4)
 			rbe8 = "gaia/fauna_elephant_african_bush";
-		}
+
 		rbe9 = "gaia/fauna_fish";
 		rbe10 = "gaia/fauna_gazelle";
 		rbe11 = "gaia/geology_stonemine_desert_quarry";
@@ -537,13 +542,12 @@ function randomizeBiome()
 		rba8 = "actor|props/flora/bush_dry_a.xml";
 		rba9 = "actor|flora/trees/baobab.xml";
 	}
-	//tropic
-	else if (biomeID == 7)
+	else if (biomeID == g_BiomeTropic)
 	{
 		
 		// Bora-Bora ish. Quite transparent, not wavy.
-		// Mostly for shallow maps. Maps where the water level goes deeper should use a much darker Water Colour to simulate deep water holes.
-		setWaterColour(0.584,0.824,0.929);
+		// Mostly for shallow maps. Maps where the water level goes deeper should use a much darker Water Color to simulate deep water holes.
+		setWaterColor(0.584,0.824,0.929);
 		setWaterTint(0.569,0.965,0.945);
 		setWaterWaviness(1.5);
 		setWaterMurkiness(0.35);
@@ -598,13 +602,11 @@ function randomizeBiome()
 		rba8 = "actor|props/flora/plant_tropic_large.xml";
 		rba9 = "actor|flora/trees/tree_tropic.xml";
 	}
-	//autumn
-	else if (biomeID == 8)
+	else if (biomeID == g_BiomeAutumn)
 	{
-			
 		// basically temperate with a reddish twist in the reflection and the tint. Also less wavy.
 		// this assumes ocean settings, maps that aren't oceans should reset.
-		setWaterColour(0.157, 0.149, 0.443);
+		setWaterColor(0.157, 0.149, 0.443);
 		setWaterTint(0.443,0.42,0.824);
 		setWaterWaviness(2.5);
 		setWaterMurkiness(0.83);
@@ -659,13 +661,8 @@ function randomizeBiome()
 		rba8 = "actor|props/flora/bush_desert_dry_a.xml";
 		rba9 = "actor|flora/trees/european_beech_aut.xml";
 	}
-	return biomeID;
 }
 
-/////////////////////////////////////////////////////////////////////////////////////////////
-//	These functions return the values needed for a randomized biome
-/////////////////////////////////////////////////////////////////////////////////////////////
-
 function rBiomeT1()
 {
 	return rbt1;
