@@ -360,8 +360,8 @@ createObjectGroups(group, 0,
 
 // Set environment
 setSkySet("sunny");
-setSunColour(0.746, 0.718, 0.539);	
-setWaterColour(0, 0.227, 0.843);
+setSunColor(0.746, 0.718, 0.539);	
+setWaterColor(0, 0.227, 0.843);
 setWaterTint(0, 0.545, 0.859);
 setWaterWaviness(1.0);
 setWaterType("clap");
