@@ -377,15 +377,15 @@ createObjectGroups(group, 0,
 	scaleByMapSize(100, 1200)
 );
 
-setSunColour(0.87451, 0.847059, 0.647059);
-setWaterColour(0.741176, 0.592157, 0.27451);
+setSunColor(0.87451, 0.847059, 0.647059);
+setWaterColor(0.741176, 0.592157, 0.27451);
 setWaterTint(0.741176, 0.592157, 0.27451);
 setWaterWaviness(2.0);
 setWaterType("clap");
 setWaterMurkiness(0.835938);
 
-setUnitsAmbientColour(0.57, 0.58, 0.55);
-setTerrainAmbientColour(0.447059, 0.509804, 0.54902);
+setUnitsAmbientColor(0.57, 0.58, 0.55);
+setTerrainAmbientColor(0.447059, 0.509804, 0.54902);
 
 setFogFactor(0.25);
 setFogThickness(0.15);
