@@ -1119,7 +1119,7 @@ else if (md == 8) //lakes
 		if (!lakeAreaLen)
 			break;
 		
-		chosenPoint = lakeAreas[randInt(0, lakeAreaLen)];
+		chosenPoint = lakeAreas[randInt(lakeAreaLen)];
 
 		placer = new ChainPlacer(1, floor(scaleByMapSize(4, 8)), floor(scaleByMapSize(40, 180)), 0.7, chosenPoint[0], chosenPoint[1]);
 		terrainPainter = new LayeredPainter(
@@ -1133,19 +1133,18 @@ else if (md == 8) //lakes
 			avoidClasses(clPlayer, 20, clWater, 8),
 			1, 1
 		);
-		
-		if (newLake !== undefined)
+
+		if (newLake && newLake.length)
 		{
-			var temp = []
+			var n = 0;
 			for (var j = 0; j < lakeAreaLen; ++j)
 			{
 				var x = lakeAreas[j][0], z = lakeAreas[j][1];
 				if (playerConstraint.allows(x, z) && waterConstraint.allows(x, z))
-						temp.push([x, z]);
+					lakeAreas[n++] = lakeAreas[j];
 			}
-			lakeAreas = temp;
-		}
-		
+			lakeAreas.length = n;
+		}		
 	}
 }
 //********************************************************************************************************
@@ -1575,13 +1574,13 @@ createAreas(
 
 var multiplier = sqrt(randFloat(0.5,1.2)*randFloat(0.5,1.2));
 // calculate desired number of trees for map (based on size)
-if (random_terrain == 6)
+if (random_terrain == g_BiomeSavanna)
 {
 	var MIN_TREES = floor(200*multiplier);
 	var MAX_TREES = floor(1250*multiplier);
 	var P_FOREST = 0;
 }
-else if (random_terrain == 7)
+else if (random_terrain == g_BiomeTropic)
 {
 	var MIN_TREES = floor(1000*multiplier);
 	var MAX_TREES = floor(6000*multiplier);
@@ -1604,7 +1603,7 @@ var types = [
 	[[tForestFloor1, tMainTerrain, pForest2], [tForestFloor1, pForest2]]
 ];	// some variation
 
-if (random_terrain != 6)
+if (random_terrain != g_BiomeSavanna)
 {
 	var size = numForest / (scaleByMapSize(3,6) * numPlayers);
 	var num = floor(size / types.length);
@@ -1781,10 +1780,9 @@ for (var i = 0; i < types.length; ++i)
 }
 
 var planetm = 1;
-if (random_terrain==7)
-{
+if (random_terrain == g_BiomeTropic)
 	planetm = 8;
-}
+
 //create small grass tufts
 log("Creating small grass tufts...");
 group = new SimpleGroup(
