@@ -1,6 +1,6 @@
 
 var g_Descriptions;
-var savedGameData;
+var g_SavedGameData;
 
 function selectDescription()
 {
@@ -15,11 +15,8 @@ function selectDescription()
 
 function init(data)
 {
-	if (data)
-	{
-		if (data.savedGameData)
-			savedGameData = data.savedGameData;
-	}
+	if (data && data.savedGameData)
+		g_SavedGameData = data.savedGameData;
 
 	var gameSelection = Engine.GetGUIObjectByName("gameSelection");
 	Engine.GetGUIObjectByName("deleteGameButton").enabled = false;
@@ -56,8 +53,8 @@ function saveGame()
 	if (gameSelection.selected != -1)
 	{
 		// Ask for confirmation
-		var btCaptions = [translate("Yes"), translate("No")];
-		var btCode = [function(){ reallySaveGame(name, desc, false); }, null];
+		var btCaptions = [translate("No"), translate("Yes")];
+		var btCode = [null, function(){ reallySaveGame(name, desc, false); }];
 		messageBox(500, 200, sprintf(translate("\"%(label)s\""), { label: gameLabel }) + "\n" + translate("Saved game will be permanently overwritten, are you sure?"), translate("OVERWRITE SAVE"), 0, btCaptions, btCode);
 	}
 	else
@@ -67,9 +64,9 @@ function saveGame()
 function reallySaveGame(name, desc, nameIsPrefix)
 {
 	if (nameIsPrefix)
-		Engine.SaveGamePrefix(name, desc, savedGameData);
+		Engine.SaveGamePrefix(name, desc, g_SavedGameData);
 	else
-		Engine.SaveGame(name, desc, savedGameData);
+		Engine.SaveGame(name, desc, g_SavedGameData);
 
 	closeSave();
 }
@@ -86,8 +83,8 @@ function deleteGame()
 	var gameID = gameSelection.list_data[gameSelection.selected];
 
 	// Ask for confirmation
-	var btCaptions = [translate("Yes"), translate("No")];
-	var btCode = [function(){ reallyDeleteGame(gameID); }, null];
+	var btCaptions = [translate("No"), translate("Yes")];
+	var btCode = [null, function(){ reallyDeleteGame(gameID); }];
 	messageBox(500, 200, sprintf(translate("\"%(label)s\""), { label: gameLabel }) + "\n" + translate("Saved game will be permanently deleted, are you sure?"), translate("DELETE"), 0, btCaptions, btCode);
 }
 
@@ -104,5 +101,5 @@ function reallyDeleteGame(gameID)
 // That's why we have to pass the data to this page even if we don't need it.
 function getSavedGameData()
 {
-	return savedGameData;
+	return g_SavedGameData;
 }
