@@ -58,24 +58,21 @@ function updateCursorAndTooltip()
 	var cursorSet = false;
 	var tooltipSet = false;
 	var informationTooltip = Engine.GetGUIObjectByName("informationTooltip");
-	if (!mouseIsOverObject)
+	if (!mouseIsOverObject && (inputState == INPUT_NORMAL || inputState == INPUT_PRESELECTEDACTION))
 	{
-		var action = determineAction(mouseX, mouseY);
-		if (inputState == INPUT_NORMAL || inputState == INPUT_PRESELECTEDACTION)
+		let action = determineAction(mouseX, mouseY);
+		if (action)
 		{
-			if (action)
+			if (action.cursor)
 			{
-				if (action.cursor)
-				{
-					Engine.SetCursor(action.cursor);
-					cursorSet = true;
-				}
-				if (action.tooltip)
-				{
-					tooltipSet = true;
-					informationTooltip.caption = action.tooltip;
-					informationTooltip.hidden = false;
-				}
+				Engine.SetCursor(action.cursor);
+				cursorSet = true;
+			}
+			if (action.tooltip)
+			{
+				tooltipSet = true;
+				informationTooltip.caption = action.tooltip;
+				informationTooltip.hidden = false;
 			}
 		}
 	}
@@ -84,22 +81,13 @@ function updateCursorAndTooltip()
 		Engine.SetCursor("arrow-default");
 	if (!tooltipSet)
 		informationTooltip.hidden = true;
-	
+
 	var placementTooltip = Engine.GetGUIObjectByName("placementTooltip");
 	if (placementSupport.tooltipMessage)
-	{
-		if (placementSupport.tooltipError)
-			placementTooltip.sprite = "BackgroundErrorTooltip";
-		else
-			placementTooltip.sprite = "BackgroundInformationTooltip";
-		placementTooltip.caption = placementSupport.tooltipMessage;
-		placementTooltip.hidden = false;
-	}
-	else
-	{
-		placementTooltip.caption = "";
-		placementTooltip.hidden = true;
-	}
+		placementTooltip.sprite = placementSupport.tooltipError ? "BackgroundErrorTooltip" : "BackgroundInformationTooltip";
+
+	placementTooltip.caption = placementSupport.tooltipMessage || "";
+	placementTooltip.hidden = !placementSupport.tooltipMessage;
 }
 
 function updateBuildingPlacementPreview()
@@ -130,7 +118,10 @@ function updateBuildingPlacementPreview()
 				{
 					var message = result.message;
 					if (result.translateMessage)
-						message = translate(message);
+						if (result.pluralMessage)
+							message = translatePlural(result.message, result.pluralMessage, result.pluralCount);
+						else
+							message = translate(message);
 					var parameters = result.parameters;
 					if (result.translateParameters)
 						translateObjectKeys(parameters, result.translateParameters);
@@ -143,13 +134,16 @@ function updateBuildingPlacementPreview()
 			{
 				// building can be placed here, and has an attack
 				// show the range advantage in the tooltip
-				var cmd = {x: placementSupport.position.x, 
-				    z: placementSupport.position.z,
-				    range: placementSupport.attack.Ranged.maxRange,
-				    elevationBonus: placementSupport.attack.Ranged.elevationBonus,
+				var cmd = {
+					"x": placementSupport.position.x,
+					"z": placementSupport.position.z,
+					"range": placementSupport.attack.Ranged.maxRange,
+					"elevationBonus": placementSupport.attack.Ranged.elevationBonus,
 				};
-				var averageRange = Engine.GuiInterfaceCall("GetAverageRangeForBuildings",cmd);
-				placementSupport.tooltipMessage = sprintf(translate("Basic range: %(range)s"), { range: Math.round(cmd.range/4) }) + "\n" + sprintf(translate("Average bonus range: %(range)s"), { range: Math.round((averageRange - cmd.range)/4) });
+				var averageRange = Math.round(Engine.GuiInterfaceCall("GetAverageRangeForBuildings", cmd) - cmd.range);
+				var range = Math.round(cmd.range);
+				placementSupport.tooltipMessage = sprintf(translatePlural("Basic range: %(range)s meter", "Basic range: %(range)s meters", range), { "range": range }) + "\n" +
+					sprintf(translatePlural("Average bonus range: %(range)s meter", "Average bonus range: %(range)s meters", averageRange), { "range": averageRange });
 			}
 			return true;
 		}
@@ -159,13 +153,13 @@ function updateBuildingPlacementPreview()
 		if (placementSupport.wallSet && placementSupport.position)
 		{
 			// Fetch an updated list of snapping candidate entities
-			placementSupport.wallSnapEntities = Engine.PickSimilarFriendlyEntities(
+			placementSupport.wallSnapEntities = Engine.PickSimilarPlayerEntities(
 				placementSupport.wallSet.templates.tower,
 				placementSupport.wallSnapEntitiesIncludeOffscreen,
 				true, // require exact template match
 				true  // include foundations
 			);
-			
+
 			return Engine.GuiInterfaceCall("SetWallPlacementPreview", {
 				"wallSet": placementSupport.wallSet,
 				"start": placementSupport.position,
@@ -197,32 +191,24 @@ function getActionInfo(action, target)
 	// If the selection doesn't exist, no action
 	var entState = GetEntityState(selection[0]);
 	if (!entState)
-		return {"possible": false};
+		return { "possible": false };
 
 	if (!target) // TODO move these non-target actions to an object like unit_actions.js
 	{
 		if (action == "set-rallypoint")
 		{
 			var cursor = "";
-			var data = {command: "walk"};
+			var data = { "command": "walk" };
 			if (Engine.HotkeyIsPressed("session.attackmove"))
 			{
-				if (Engine.HotkeyIsPressed("session.attackmoveUnit"))
-					var targetClasses = { "attack": ["Unit"] };
-				else
-					var targetClasses = { "attack": ["Unit", "Structure"] };
 				data.command = "attack-walk";
-				data.targetClasses = targetClasses;
+				data.targetClasses = Engine.HotkeyIsPressed("session.attackmoveUnit") ? { "attack": ["Unit"] } : { "attack": ["Unit", "Structure"] };
 				cursor = "action-attack-move";
 			}
-			return {"possible": true, "data": data, "cursor": cursor};
+			return { "possible": true, "data": data, "cursor": cursor };
 		}
-		else if (action == "move" || action == "attack-move")
-			return {"possible": true};
-		else if (action == "remove-guard")
-			return {"possible": true};
-		else
-			return {"possible": false};
+
+		return { "possible": (action == "move" || action == "attack-move" || action == "remove-guard") };
 	}
 
 	// Look at the first targeted entity
@@ -246,7 +232,7 @@ function getActionInfo(action, target)
 				return r;
 		}
 	}
-	return {"possible": false};
+	return { "possible": false };
 }
 
 /**
@@ -269,10 +255,9 @@ function determineAction(x, y, fromMinimap)
 		return undefined;
 
 	// If the selection isn't friendly units, no action
-	var playerID = Engine.GetPlayerID();
 	var allOwnedByPlayer = selection.every(function(ent) {
 		var entState = GetEntityState(ent);
-		return entState && entState.player == playerID;
+		return entState && entState.player == g_ViewedPlayer;
 	});
 
 	if (!g_DevSettings.controlAll && !allOwnedByPlayer)
@@ -290,7 +275,7 @@ function determineAction(x, y, fromMinimap)
 	// if two actions are possible, the first one is taken
 	// so the most specific should appear first
 	var actions = Object.keys(unitActions).slice();
-	actions.sort(function(a, b) {return unitActions[a].specificness - unitActions[b].specificness;});
+	actions.sort((a, b) => unitActions[a].specificness - unitActions[b].specificness);
 
 	var actionInfo = undefined;
 	if (preSelectedAction != ACTION_NONE)
@@ -304,7 +289,7 @@ function determineAction(x, y, fromMinimap)
 					return r;
 			}
 		}
-		return {"type": "none", "cursor": "", "target": target};
+		return { "type": "none", "cursor": "", "target": target };
 	}
 
 	for (var action of actions)
@@ -326,8 +311,8 @@ function determineAction(x, y, fromMinimap)
 				return r;
 		}
 	}
-		
-	return {"type": "none", "cursor": "", "target": target};
+
+	return { "type": "none", "cursor": "", "target": target };
 }
 
 
@@ -338,8 +323,8 @@ function tryPlaceBuilding(queued)
 	if (placementSupport.mode !== "building")
 	{
 		error(sprintf("[%(functionName)s] Called while in '%(mode)s' placement mode instead of 'building'", {
-			functionName: "tryPlaceBuilding",
-			mode: placementSupport.mode
+			"functionName": "tryPlaceBuilding",
+			"mode": placementSupport.mode
 		}));
 		return false;
 	}
@@ -387,7 +372,7 @@ function tryPlaceWall(queued)
 		}));
 		return false;
 	}
-	
+
 	var wallPlacementInfo = updateBuildingPlacementPreview(); // entities making up the wall (wall segments, towers, ...)
 	if (!(wallPlacementInfo === false || typeof(wallPlacementInfo) === "object"))
 	{
@@ -398,10 +383,10 @@ function tryPlaceWall(queued)
 		}));
 		return false;
 	}
-	
+
 	if (!wallPlacementInfo)
 		return false;
-	
+
 	var selection = g_Selection.toList();
 	var cmd = {
 		"type": "construct-wall",
@@ -414,24 +399,24 @@ function tryPlaceWall(queued)
 		"startSnappedEntity": wallPlacementInfo.startSnappedEnt,
 		"endSnappedEntity": wallPlacementInfo.endSnappedEnt,
 	};
-	
+
 	// make sure that there's at least one non-tower entity getting built, to prevent silly edge cases where the start and end
 	// point are too close together for the algorithm to place a wall segment inbetween, and only the towers are being previewed
 	// (this is somewhat non-ideal and hardcode-ish)
 	var hasWallSegment = false;
-	for (var k in cmd.pieces)
+	for (let piece of cmd.pieces)
 	{
-		if (cmd.pieces[k].template != cmd.wallSet.templates.tower) // TODO: hardcode-ish :(
+		if (piece.template != cmd.wallSet.templates.tower) // TODO: hardcode-ish :(
 		{
 			hasWallSegment = true;
 			break;
 		}
 	}
-	
+
 	if (hasWallSegment)
 	{
 		Engine.PostNetworkCommand(cmd);
-		Engine.GuiInterfaceCall("PlaySound", {"name": "order_repair", "entity": selection[0] });
+		Engine.GuiInterfaceCall("PlaySound", { "name": "order_repair", "entity": selection[0] });
 	}
 
 	return true;
@@ -479,7 +464,7 @@ var unitFilters = {
 		var entState = GetEntityState(entity);
 		if (!entState)
 			return false;
-		return hasClass(entState, "Unit") && entState.unitAI.isIdle;
+		return hasClass(entState, "Unit") && entState.unitAI && entState.unitAI.isIdle;
 	},
 	"isAnything": function (entity) {
 		return true;
@@ -503,7 +488,7 @@ function getPreferredEntities(ents)
 	for (var i = 0; i < filters.length; ++i)
 	{
 		preferredEnts = ents.filter(filters[i]);
-		if (preferredEnts.length > 0) 
+		if (preferredEnts.length > 0)
 			break;
 	}
 	return preferredEnts;
@@ -528,7 +513,7 @@ function handleInputBeforeGui(ev, hoveredObject)
 
 	// Close the menu when interacting with the game world
 	if (!mouseIsOverObject && (ev.type =="mousebuttonup" || ev.type == "mousebuttondown")
-	    && (ev.button == SDL_BUTTON_LEFT || ev.button == SDL_BUTTON_RIGHT))
+		&& (ev.button == SDL_BUTTON_LEFT || ev.button == SDL_BUTTON_RIGHT))
 		closeMenu();
 
 	// State-machine processing:
@@ -549,7 +534,7 @@ function handleInputBeforeGui(ev, hoveredObject)
 		case "mousemotion":
 			var rect = updateBandbox(bandbox, ev, false);
 
-			var ents = Engine.PickFriendlyEntitiesInRect(rect[0], rect[1], rect[2], rect[3], Engine.GetPlayerID());
+			var ents = Engine.PickPlayerEntitiesInRect(rect[0], rect[1], rect[2], rect[3], g_IsObserver ? -1 : Engine.GetPlayerID());
 			var preferredEntities = getPreferredEntities(ents);
 			g_Selection.setHighlightList(preferredEntities);
 
@@ -561,7 +546,7 @@ function handleInputBeforeGui(ev, hoveredObject)
 				var rect = updateBandbox(bandbox, ev, true);
 
 				// Get list of entities limited to preferred entities
-				var ents = getPreferredEntities(Engine.PickFriendlyEntitiesInRect(rect[0], rect[1], rect[2], rect[3], Engine.GetPlayerID()));
+				var ents = getPreferredEntities(Engine.PickPlayerEntitiesInRect(rect[0], rect[1], rect[2], rect[3], g_IsObserver ? -1 : Engine.GetPlayerID()));
 
 				// Remove the bandbox hover highlighting
 				g_Selection.setHighlightList([]);
@@ -645,10 +630,10 @@ function handleInputBeforeGui(ev, hoveredObject)
 			break;
 		}
 		break;
-		
+
 	case INPUT_BUILDING_WALL_CLICK:
 		// User is mid-click in choosing a starting point for building a wall. The build process can still be cancelled at this point
-		// by right-clicking; releasing the left mouse button will 'register' the starting point and commence endpoint choosing mode. 
+		// by right-clicking; releasing the left mouse button will 'register' the starting point and commence endpoint choosing mode.
 		switch (ev.type)
 		{
 		case "mousebuttonup":
@@ -658,14 +643,14 @@ function handleInputBeforeGui(ev, hoveredObject)
 				return true;
 			}
 			break;
-			
+
 		case "mousebuttondown":
 			if (ev.button == SDL_BUTTON_RIGHT)
 			{
 				// Cancel building
 				placementSupport.Reset();
 				updateBuildingPlacementPreview();
-				
+
 				inputState = INPUT_NORMAL;
 				return true;
 			}
@@ -676,36 +661,36 @@ function handleInputBeforeGui(ev, hoveredObject)
 	case INPUT_BUILDING_WALL_PATHING:
 		// User has chosen a starting point for constructing the wall, and is now looking to set the endpoint.
 		// Right-clicking cancels wall building mode, left-clicking sets the endpoint and builds the wall and returns to
-		// normal input mode. Optionally, shift + left-clicking does not return to normal input, and instead allows the 
+		// normal input mode. Optionally, shift + left-clicking does not return to normal input, and instead allows the
 		// user to continue building walls.
 		switch (ev.type)
 		{
 			case "mousemotion":
 				placementSupport.wallEndPosition = Engine.GetTerrainAtScreenPoint(ev.x, ev.y);
-				
+
 				// Update the building placement preview, and by extension, the list of snapping candidate entities for both (!)
 				// the ending point and the starting point to snap to.
-				// 
+				//
 				// TODO: Note that here, we need to fetch all similar entities, including any offscreen ones, to support the case
 				// where the snap entity for the starting point has moved offscreen, or has been deleted/destroyed, or was a
 				// foundation and has been replaced with a completed entity since the user first chose it. Fetching all towers on
 				// the entire map instead of only the current screen might get expensive fast since walls all have a ton of towers
 				// in them. Might be useful to query only for entities within a certain range around the starting point and ending
 				// points.
-				
+
 				placementSupport.wallSnapEntitiesIncludeOffscreen = true;
 				var result = updateBuildingPlacementPreview(); // includes an update of the snap entity candidates
-				
+
 				if (result && result.cost)
 				{
 					placementSupport.tooltipMessage = getEntityCostTooltip(result);
-					var neededResources = Engine.GuiInterfaceCall("GetNeededResources", result.cost);
+					var neededResources = Engine.GuiInterfaceCall("GetNeededResources", { "cost": result.cost });
 					if (neededResources)
 						placementSupport.tooltipMessage += getNeededResourcesTooltip(neededResources);
 				}
-				
+
 				break;
-				
+
 			case "mousebuttondown":
 				if (ev.button == SDL_BUTTON_LEFT)
 				{
@@ -717,7 +702,7 @@ function handleInputBeforeGui(ev, hoveredObject)
 							// continue building, just set a new starting position where we left off
 							placementSupport.position = placementSupport.wallEndPosition;
 							placementSupport.wallEndPosition = undefined;
-							
+
 							inputState = INPUT_BUILDING_WALL_CLICK;
 						}
 						else
@@ -730,7 +715,7 @@ function handleInputBeforeGui(ev, hoveredObject)
 					{
 						placementSupport.tooltipMessage = translate("Cannot build wall here!");
 					}
-					
+
 					updateBuildingPlacementPreview();
 					return true;
 				}
@@ -739,14 +724,14 @@ function handleInputBeforeGui(ev, hoveredObject)
 					// reset to normal input mode
 					placementSupport.Reset();
 					updateBuildingPlacementPreview();
-					
+
 					inputState = INPUT_NORMAL;
 					return true;
 				}
 				break;
 		}
 		break;
-		
+
 	case INPUT_BUILDING_DRAG:
 		switch (ev.type)
 		{
@@ -816,7 +801,7 @@ function handleInputBeforeGui(ev, hoveredObject)
 	case INPUT_MASSTRIBUTING:
 		if (ev.type == "hotkeyup" && ev.hotkey == "session.masstribute")
 		{
-			flushTributing();
+			g_FlushTributing();
 			inputState = INPUT_NORMAL;
 		}
 		break;
@@ -853,7 +838,7 @@ function handleInputAfterGui(ev)
 	{
 		g_ShowAllStatusBars = (ev.type == "hotkeydown");
 		recalculateStatusBarDisplay();
-	} 
+	}
 	else if (ev.hotkey == "session.highlightguarding")
 	{
 		g_ShowGuarding = (ev.type == "hotkeydown");
@@ -922,7 +907,7 @@ function handleInputAfterGui(ev)
 				break;
 		}
 		break;
-		
+
 	case INPUT_PRESELECTEDACTION:
 		switch (ev.type)
 		{
@@ -963,7 +948,7 @@ function handleInputAfterGui(ev)
 			}
 		}
 		break;
-		
+
 	case INPUT_SELECTING:
 		switch (ev.type)
 		{
@@ -1043,7 +1028,7 @@ function handleInputAfterGui(ev)
 					}
 
 					// TODO: Should we handle "control all units" here as well?
-					ents = Engine.PickSimilarFriendlyEntities(templateToMatch, showOffscreen, matchRank, false);
+					ents = Engine.PickSimilarPlayerEntities(templateToMatch, showOffscreen, matchRank, false);
 				}
 				else
 				{
@@ -1082,9 +1067,9 @@ function handleInputAfterGui(ev)
 		switch (ev.type)
 		{
 		case "mousemotion":
-			
+
 			placementSupport.position = Engine.GetTerrainAtScreenPoint(ev.x, ev.y);
-			
+
 			if (placementSupport.mode === "wall")
 			{
 				// Including only the on-screen towers in the next snap candidate list is sufficient here, since the user is
@@ -1095,7 +1080,7 @@ function handleInputAfterGui(ev)
 			else
 			{
 				// cancel if not enough resources
-				if (placementSupport.template && Engine.GuiInterfaceCall("GetNeededResources", GetTemplateData(placementSupport.template).cost))
+				if (placementSupport.template && Engine.GuiInterfaceCall("GetNeededResources", { "cost": GetTemplateData(placementSupport.template).cost }))
 				{
 					placementSupport.Reset();
 					inputState = INPUT_NORMAL;
@@ -1125,9 +1110,7 @@ function handleInputAfterGui(ev)
 				{
 					var validPlacement = updateBuildingPlacementPreview();
 					if (validPlacement !== false)
-					{
 						inputState = INPUT_BUILDING_WALL_CLICK;
-					}
 				}
 				else
 				{
@@ -1216,9 +1199,9 @@ function startBuildingPlacement(buildTemplate, playerState)
 	// TODO: we should clear any highlight selection rings here. If the mouse was over an entity before going onto the GUI
 	// to start building a structure, then the highlight selection rings are kept during the construction of the building.
 	// Gives the impression that somehow the hovered-over entity has something to do with the building you're constructing.
-	
+
 	placementSupport.Reset();
-	
+
 	// find out if we're building a wall, and change the entity appropriately if so
 	var templateData = GetTemplateData(buildTemplate);
 	if (templateData.wallSet)
@@ -1234,9 +1217,9 @@ function startBuildingPlacement(buildTemplate, playerState)
 		inputState = INPUT_BUILDING_PLACEMENT;
 	}
 
-	if (templateData.attack && 
-	    templateData.attack.Ranged && 
-	    templateData.attack.Ranged.maxRange) 
+	if (templateData.attack &&
+		templateData.attack.Ranged &&
+		templateData.attack.Ranged.maxRange)
 	{
 		// add attack information to display a good tooltip
 		placementSupport.attack = templateData.attack;
@@ -1246,13 +1229,13 @@ function startBuildingPlacement(buildTemplate, playerState)
 // Called by GUI when user changes required trading goods
 function selectRequiredGoods(data)
 {
-	Engine.PostNetworkCommand({"type": "select-required-goods", "entities": data.entities, "requiredGoods": data.requiredGoods});
+	Engine.PostNetworkCommand({ "type": "select-required-goods", "entities": data.entities, "requiredGoods": data.requiredGoods} );
 }
 
 // Called by GUI when user clicks exchange resources button
 function exchangeResources(command)
 {
-	Engine.PostNetworkCommand({"type": "barter", "sell": command.sell, "buy": command.buy, "amount": command.amount});
+	Engine.PostNetworkCommand({ "type": "barter", "sell": command.sell, "buy": command.buy, "amount": command.amount });
 }
 
 // Camera jumping: when the user presses a hotkey the current camera location is marked.
@@ -1303,20 +1286,28 @@ function flushTrainingBatch()
 		// Train as many full batches as we can
 		var buildingsCountToTrainFullBatch = Math.floor(batchTrainingEntityAllowedCount / batchTrainingCount);
 		var buildingsToTrainFullBatch = appropriateBuildings.slice(0, buildingsCountToTrainFullBatch);
-		Engine.PostNetworkCommand({"type": "train", "entities": buildingsToTrainFullBatch,
-			"template": batchTrainingType, "count": batchTrainingCount});
+		Engine.PostNetworkCommand({
+			"type": "train",
+			"entities": buildingsToTrainFullBatch,
+			"template": batchTrainingType,
+			"count": batchTrainingCount
+		});
 
 		// Train remainer in one more building
-		var remainderToTrain = batchTrainingEntityAllowedCount % batchTrainingCount;
-		Engine.PostNetworkCommand({"type": "train",
+		Engine.PostNetworkCommand({
+			"type": "train",
 			"entities": [ appropriateBuildings[buildingsCountToTrainFullBatch] ],
-			"template": batchTrainingType, "count": remainderToTrain});
+			"template": batchTrainingType,
+			"count": batchTrainingEntityAllowedCount % batchTrainingCount
+		});
 	}
 	else
-	{
-		Engine.PostNetworkCommand({"type": "train", "entities": appropriateBuildings,
-			"template": batchTrainingType, "count": batchTrainingCount});
-	}
+		Engine.PostNetworkCommand({
+			"type": "train",
+			"entities": appropriateBuildings,
+			"template": batchTrainingType,
+			"count": batchTrainingCount
+		});
 }
 
 function getBuildingsWhichCanTrainEntity(entitiesToCheck, trainEntType)
@@ -1337,15 +1328,17 @@ function getEntityLimitAndCount(playerState, entType)
 		"entLimitChangers": undefined,
 		"canBeAddedCount": undefined
 	};
+	if (!playerState.entityLimits)
+		return r;
 	var template = GetTemplateData(entType);
 	var entCategory = null;
 	if (template.trainingRestrictions)
 		entCategory = template.trainingRestrictions.category;
 	else if (template.buildRestrictions)
 		entCategory = template.buildRestrictions.category;
-	if (entCategory && playerState.entityLimits[entCategory] != null)
+	if (entCategory && playerState.entityLimits[entCategory] !== undefined)
 	{
-		r.entLimit = playerState.entityLimits[entCategory] || Infinity;
+		r.entLimit = playerState.entityLimits[entCategory] || 0;
 		r.entCount = playerState.entityCounts[entCategory] || 0;
 		r.entLimitChangers = playerState.entityLimitChangers[entCategory];
 		r.canBeAddedCount = Math.max(r.entLimit - r.entCount, 0);
@@ -1360,17 +1353,17 @@ function addTrainingByPosition(position)
 	var playerState = simState.players[Engine.GetPlayerID()];
 	var selection = g_Selection.toList();
 
-	if (!selection.length)
+	if (!playerState || !selection.length)
 		return;
-	
+
 	var trainableEnts = getAllTrainableEntitiesFromSelection();
-	
+
 	// Check if the position is valid
-	if (!trainableEnts.length || trainableEnts.length <= position) 
+	if (!trainableEnts.length || trainableEnts.length <= position)
 		return;
-	
+
 	var entToTrain = trainableEnts[position];
-	
+
 	addTrainingToQueue(selection, entToTrain, playerState);
 	return;
 }
@@ -1420,8 +1413,8 @@ function addTrainingToQueue(selection, trainEntType, playerState)
 				else if (limits.canBeAddedCount == undefined ||
 					limits.canBeAddedCount > batchTrainingCount * appropriateBuildings.length)
 				{
-					if (Engine.GuiInterfaceCall("GetNeededResources", multiplyEntityCosts(
-						template, batchTrainingCount + batchIncrementSize)))
+					if (Engine.GuiInterfaceCall("GetNeededResources", { "cost":
+						multiplyEntityCosts(template, batchTrainingCount + batchIncrementSize) }))
 						return;
 
 					batchTrainingCount += batchIncrementSize;
@@ -1438,8 +1431,8 @@ function addTrainingToQueue(selection, trainEntType, playerState)
 		}
 
 		// Don't start a new batch if decrementing or unable to afford it.
-		if (decrement || Engine.GuiInterfaceCall("GetNeededResources",
-			multiplyEntityCosts(template, batchIncrementSize)))
+		if (decrement || Engine.GuiInterfaceCall("GetNeededResources", { "cost":
+			multiplyEntityCosts(template, batchIncrementSize) }))
 			return;
 
 		inputState = INPUT_BATCHTRAINING;
@@ -1455,15 +1448,19 @@ function addTrainingToQueue(selection, trainEntType, playerState)
 		var buildingsForTraining = appropriateBuildings;
 		if (limits.entLimit)
 			buildingsForTraining = buildingsForTraining.slice(0, limits.canBeAddedCount);
-		Engine.PostNetworkCommand({"type": "train", "template": trainEntType,
-			"count": 1, "entities": buildingsForTraining});
+		Engine.PostNetworkCommand({
+			"type": "train",
+			"template": trainEntType,
+			"count": 1,
+			"entities": buildingsForTraining
+		});
 	}
 }
 
 // Called by GUI when user clicks research button
 function addResearchToQueue(entity, researchType)
 {
-	Engine.PostNetworkCommand({"type": "research", "entity": entity, "template": researchType});
+	Engine.PostNetworkCommand({ "type": "research", "entity": entity, "template": researchType });
 }
 
 // Returns the number of units that will be present in a batch if the user clicks
@@ -1509,16 +1506,13 @@ function getTrainingBatchStatus(playerState, entity, trainEntType, selection)
 // Called by GUI when user clicks production queue item
 function removeFromProductionQueue(entity, id)
 {
-	Engine.PostNetworkCommand({"type": "stop-production", "entity": entity, "id": id});
+	Engine.PostNetworkCommand({ "type": "stop-production", "entity": entity, "id": id });
 }
 
 // Called by unit selection buttons
 function changePrimarySelectionGroup(templateName, deselectGroup)
 {
-	if (Engine.HotkeyIsPressed("session.deselectgroup") || deselectGroup)
-		g_Selection.makePrimarySelection(templateName, true);
-	else
-		g_Selection.makePrimarySelection(templateName, false);
+	g_Selection.makePrimarySelection(templateName, Engine.HotkeyIsPressed("session.deselectgroup") || deselectGroup);
 }
 
 // Performs the specified command (delete, town bell, repair, etc.)
@@ -1527,27 +1521,38 @@ function performCommand(entity, commandName)
 	if (!entity)
 		return;
 	var entState = GetExtendedEntityState(entity);
-	var playerID = Engine.GetPlayerID();
 
-	if (!entState.player == playerID && !g_DevSettings.controlAll)
+	if (!controlsPlayer(entState.player))
 		return;
 
 	if (g_EntityCommands[commandName])
 		g_EntityCommands[commandName].execute(entState);
 }
 
+// Performs the specified command for ally unit
+function performAllyCommand(entity, commandName)
+{
+	if (!entity)
+		return;
+	var entState = GetExtendedEntityState(entity);
+
+	var playerState = GetSimState().players[Engine.GetPlayerID()];
+	if (!playerState.isMutualAlly[entState.player] || g_IsObserver)
+		return;
+
+	if (g_AllyEntityCommands[commandName])
+		g_AllyEntityCommands[commandName].execute(entState);
+}
+
 // Performs the specified formation
 function performFormation(entity, formationTemplate)
 {
 	if (entity)
-	{
-		var selection = g_Selection.toList();
 		Engine.PostNetworkCommand({
 			"type": "formation",
-			"entities": selection,
+			"entities": g_Selection.toList(),
 			"name": formationTemplate
 		});
-	}
 }
 
 // Performs the specified group
@@ -1579,10 +1584,10 @@ function performGroup(action, groupId)
 	case "save":
 	case "breakUp":
 		g_Groups.groups[groupId].reset();
-		
+
 		if (action == "save")
 			g_Groups.addEntities(groupId, g_Selection.toList());
-			
+
 		updateGroups();
 		break;
 	}
@@ -1642,7 +1647,7 @@ function transformWallToGate(template)
 	var selection = g_Selection.toList();
 	Engine.PostNetworkCommand({
 		"type": "wall-to-gate",
-		"entities": selection.filter( function(e) { return getWallGateTemplate(e) == template } ),
+		"entities": selection.filter( function(e) { return getWallGateTemplate(e) == template; } ),
 		"template": template,
 	});
 }
@@ -1705,17 +1710,20 @@ function findIdleUnit(classes)
 
 	for (var i = 0; i < classes.length; ++i)
 	{
-		var data = { 
+		var data = {
 			"idleClass": classes[currIdleClass],
 			"prevUnit": lastIdleUnit,
 			"limit": 1,
 			"excludeUnits": []
 		};
+
 		if (append)
 			data.excludeUnits = g_Selection.toList();
 
 		if (selectall)
-			data = { idleClass: classes[currIdleClass] };
+			data = { "idleClass": classes[currIdleClass] };
+
+		data.viewedPlayer = g_ViewedPlayer;
 
 		// Check if we have new valid entity
 		var idleUnits = Engine.GuiInterfaceCall("FindIdleUnits", data);
@@ -1723,7 +1731,7 @@ function findIdleUnit(classes)
 		{
 			lastIdleUnit = idleUnits[0];
 			if (!append && (!selectall || selectall && !matched))
-				g_Selection.reset()
+				g_Selection.reset();
 
 			if (selectall)
 				g_Selection.addList(idleUnits);
@@ -1756,9 +1764,9 @@ function stopUnits(entities)
 function unload(garrisonHolder, entities)
 {
 	if (Engine.HotkeyIsPressed("session.unloadtype"))
-		Engine.PostNetworkCommand({"type": "unload", "entities": entities, "garrisonHolder": garrisonHolder});
+		Engine.PostNetworkCommand({ "type": "unload", "entities": entities, "garrisonHolder": garrisonHolder });
 	else
-		Engine.PostNetworkCommand({"type": "unload", "entities": [entities[0]], "garrisonHolder": garrisonHolder});
+		Engine.PostNetworkCommand({ "type": "unload", "entities": [entities[0]], "garrisonHolder": garrisonHolder });
 }
 
 function unloadTemplate(template)
@@ -1781,10 +1789,9 @@ function unloadTemplate(template)
 
 function unloadSelection()
 {
-	var entities = g_Selection.toList();
 	var parent = 0;
 	var ents = [];
-	for each (var ent in entities)
+	for each (var ent in g_Selection.selected)
 	{
 		var state = GetExtendedEntityState(ent);
 		if (!state || !state.turretParent)
@@ -1795,10 +1802,19 @@ function unloadSelection()
 			ents.push(ent);
 		}
 		else if (state.turretParent == parent)
-			ents.push(ent)
+			ents.push(ent);
 	}
 	if (parent)
-		Engine.PostNetworkCommand({"type": "unload", "entities":ents, "garrisonHolder": parent});
+		Engine.PostNetworkCommand({ "type": "unload", "entities":ents, "garrisonHolder": parent });
+}
+
+function unloadAllByOwner()
+{
+	var garrisonHolders = g_Selection.toList().filter(function(e) {
+		var state = GetEntityState(e);
+		return state && state.garrisonHolder;
+	});
+	Engine.PostNetworkCommand({ "type": "unload-all-by-owner", "garrisonHolders": garrisonHolders });
 }
 
 function unloadAll()
@@ -1811,7 +1827,7 @@ function unloadAll()
 		return false;
 	});
 
-	Engine.PostNetworkCommand({"type": "unload-all", "garrisonHolders": garrisonHolders});
+	Engine.PostNetworkCommand({ "type": "unload-all", "garrisonHolders": garrisonHolders });
 }
 
 function backToWork()
@@ -1821,9 +1837,8 @@ function backToWork()
 		var state = GetEntityState(e);
 		return (state && state.unitAI && state.unitAI.hasWorkOrders);
 	});
-	
-	Engine.PostNetworkCommand({"type": "back-to-work", "entities": workers});
-	
+
+	Engine.PostNetworkCommand({ "type": "back-to-work", "entities": workers });
 }
 
 function removeGuard()
@@ -1833,8 +1848,8 @@ function removeGuard()
 		var state = GetEntityState(e);
 		return (state && state.unitAI && state.unitAI.isGuarding);
 	});
-	
-	Engine.PostNetworkCommand({"type": "remove-guard", "entities": entities});
+
+	Engine.PostNetworkCommand({ "type": "remove-guard", "entities": entities });
 }
 
 function increaseAlertLevel()
@@ -1843,8 +1858,8 @@ function increaseAlertLevel()
 		var state = GetEntityState(e);
 		return (state && state.alertRaiser && state.alertRaiser.canIncreaseLevel);
 	});
-	
-	Engine.PostNetworkCommand({"type": "increase-alert-level", "entities": entities});	
+
+	Engine.PostNetworkCommand({ "type": "increase-alert-level", "entities": entities });
 }
 
 function endOfAlert()
@@ -1853,8 +1868,8 @@ function endOfAlert()
 		var state = GetEntityState(e);
 		return (state && state.alertRaiser && state.alertRaiser.hasRaisedAlert);
 	});
-	
-	Engine.PostNetworkCommand({"type": "alert-end", "entities": entities});
+
+	Engine.PostNetworkCommand({ "type": "alert-end", "entities": entities });
 }
 
 function clearSelection()
@@ -1898,6 +1913,6 @@ function getTrainingQueueItems(selection)
 		}
 		i++;
 	}
-	while (foundNewItems)
+	while (foundNewItems);
 	return queue;
 }
