@@ -4,11 +4,11 @@
  * getItems returns a list of basic items used to fill the panel.
  * This method is obligated. If the items list is empty, the panel
  * won't be rendered.
- * 
- * Then there's a loop over all items provided. In the loop, 
+ *
+ * Then there's a loop over all items provided. In the loop,
  * the item and some other standard data is added to a data object.
  *
- * The standard data is 
+ * The standard data is
  * var data = {
  *   "i":              index
  *   "item":           item coming from the getItems function
@@ -18,27 +18,82 @@
  *   "rowLength":      rowLength
  *   "numberOfItems":  number of items that will be processed
  *   "button":         gui Button object
- *   "affordableMask": gui Unaffordable overlay
  *   "icon":           gui Icon object
  *   "guiSelection":   gui button Selection overlay
  *   "countDisplay":   gui caption space
  * };
  *
- * Then, addData is called, and can be used to abort the processing 
- * of the current item by returning false. 
+ * Then, addData is called, and can be used to abort the processing
+ * of the current item by returning false.
  * It should return true if you want the panel to be filled.
  *
- * addData is used to add data to the data object on top 
+ * addData is used to add data to the data object on top
  * (or instead of) the standard data.
  * addData is not obligated, the function will just continue
  * with the content setters if no addData is present.
- * 
- * After the addData, all functions starting with "set" are called. 
+ *
+ * After the addData, all functions starting with "set" are called.
  * These are used to set various parts of content.
  */
 
+/* cache some formation info */
+var g_availableFormations = new Map();   // available formations per player
+var g_formationsInfo = new Map();
+
 var g_SelectionPanels = {};
 
+// ALERT
+g_SelectionPanels.Alert = {
+	"getMaxNumberOfItems": function()
+	{
+		return 2;
+	},
+	"getItems": function(unitEntState)
+	{
+		if (!unitEntState.alertRaiser)
+			return [];
+		return ["increase", "end"];
+	},
+	"setAction": function(data)
+	{
+		data.button.onPress = function() {
+			if (data.item == "increase")
+				increaseAlertLevel();
+			else if (data.item == "end")
+				endOfAlert();
+		};
+	},
+	"setTooltip": function(data)
+	{
+		if (data.item == "increase")
+		{
+			if (data.unitEntState.alertRaiser.hasRaisedAlert)
+				data.button.tooltip = translate("Increase the alert level to protect more units");
+			else
+				data.button.tooltip = translate("Raise an alert!");
+		}
+		else if (data.item == "end")
+			data.button.tooltip = translate("End of alert.");
+	},
+	"setGraphics": function(data)
+	{
+		if (data.item == "increase")
+		{
+			data.button.hidden = !data.unitEntState.alertRaiser.canIncreaseLevel;
+			if (data.unitEntState.alertRaiser.hasRaisedAlert)
+				data.icon.sprite = "stretched:session/icons/bell_level2.png";
+			else
+				data.icon.sprite = "stretched:session/icons/bell_level1.png";
+		}
+		else if (data.item == "end")
+		{
+			data.button.hidden = !data.unitEntState.alertRaiser.hasRaisedAlert;
+			data.icon.sprite = "stretched:session/icons/bell_level0.png";
+		}
+		data.button.enabled = !data.button.hidden && controlsPlayer(data.unitEntState.player);
+	}
+};
+
 // BARTER
 g_SelectionPanels.Barter = {
 	"getMaxNumberOfItems": function()
@@ -66,7 +121,6 @@ g_SelectionPanels.Barter = {
 			data.amount[a] = Engine.GetGUIObjectByName("unitBarter"+a+"Amount["+data.i+"]");
 		}
 		data.selectionIcon = Engine.GetGUIObjectByName("unitBarterSellSelection["+data.i+"]");
-		data.affordableMask = Engine.GetGUIObjectByName("unitBarterSellUnaffordable["+data.i+"]");
 
 		data.amountToSell = BARTER_RESOURCE_AMOUNT_TO_SELL;
 		if (Engine.HotkeyIsPressed("session.massbarter"))
@@ -77,15 +131,15 @@ g_SelectionPanels.Barter = {
 	"setCountDisplay": function(data)
 	{
 		data.amount.Sell.caption = "-" + data.amountToSell;
-		var sellPrice = data.unitEntState.barterMarket.prices["sell"][g_barterSell];
-		var buyPrice = data.unitEntState.barterMarket.prices["buy"][data.item];
+		var sellPrice = data.unitEntState.barterMarket.prices.sell[g_barterSell];
+		var buyPrice = data.unitEntState.barterMarket.prices.buy[data.item];
 		data.amount.Buy.caption = "+" + Math.round(sellPrice / buyPrice * data.amountToSell);
 	},
 	"setTooltip": function(data)
 	{
 		var resource = getLocalizedResourceName(data.item, "withinSentence");
-		data.button.Buy.tooltip = sprintf(translate("Buy %(resource)s"), {"resource": resource});
-		data.button.Sell.tooltip = sprintf(translate("Sell %(resource)s"), {"resource": resource});
+		data.button.Buy.tooltip = sprintf(translate("Buy %(resource)s"), { "resource": resource });
+		data.button.Sell.tooltip = sprintf(translate("Sell %(resource)s"), { "resource": resource });
 	},
 	"setAction": function(data)
 	{
@@ -99,26 +153,38 @@ g_SelectionPanels.Barter = {
 	},
 	"setGraphics": function(data)
 	{
-		var grayscale = data.isSelected ? "grayscale:" : ""; 
-		data.button.Buy.hidden = data.isSelected;
-		data.button.Sell.hidden = false;
-		for each (var icon in data.icon)
-			icon.sprite = "stretched:"+grayscale+"session/icons/resources/" + data.item + ".png";
+		var grayscale = data.isSelected ? "color: 0 0 0 100:grayscale:" : "";
 
+		// do we have enough of this resource to sell?
 		var neededRes = {};
 		neededRes[data.item] = data.amountToSell;
-		if (Engine.GuiInterfaceCall("GetNeededResources", neededRes))
-			data.affordableMask.hidden = false;
-		else
-			data.affordableMask.hidden = true;
+		var canSellCurrent = Engine.GuiInterfaceCall("GetNeededResources", {
+			"cost": neededRes,
+			"player": data.unitEntState.player
+		}) ? "color:255 0 0 80:" : "";
+
+		// Let's see if we have enough resources to barter.
+		neededRes = {};
+		neededRes[g_barterSell] = data.amountToSell;
+		var canBuyAny = Engine.GuiInterfaceCall("GetNeededResources", {
+			"cost": neededRes,
+			"player": data.unitEntState.player
+		}) ? "color:255 0 0 80:" : "";
+
+		data.icon.Sell.sprite = canSellCurrent + "stretched:"+grayscale+"session/icons/resources/" + data.item + ".png";
+		data.icon.Buy.sprite = canBuyAny + "stretched:"+grayscale+"session/icons/resources/" + data.item + ".png";
+
+		data.button.Buy.hidden = data.isSelected;
+		data.button.Buy.enabled = controlsPlayer(data.unitEntState.player);
+		data.button.Sell.hidden = false;
 		data.selectionIcon.hidden = !data.isSelected;
 	},
 	"setPosition": function(data)
 	{
 		setPanelObjectPosition(data.button.Sell, data.i, data.rowLength);
 		setPanelObjectPosition(data.button.Buy, data.i + data.rowLength, data.rowLength);
-	},
-},
+	}
+};
 
 // COMMAND
 g_SelectionPanels.Command = {
@@ -128,10 +194,68 @@ g_SelectionPanels.Command = {
 	},
 	"getItems": function(unitEntState)
 	{
-		var commands = [];
-		for (var c in g_EntityCommands)
+		let commands = [];
+
+		for (let c in g_EntityCommands)
 		{
 			var info = g_EntityCommands[c].getInfo(unitEntState);
+			if (!info)
+				continue;
+
+			info.name = c;
+			commands.push(info);
+		}
+		return commands;
+	},
+	"setTooltip": function(data)
+	{
+		data.button.tooltip = data.item.tooltip;
+	},
+	"setAction": function(data)
+	{
+		data.button.onPress = function() {
+			if (data.item.callback)
+				data.item.callback(data.item);
+			else
+				performCommand(data.unitEntState.id, data.item.name);
+		};
+	},
+	"setCountDisplay": function(data)
+	{
+		data.countDisplay.caption = data.item.count || "";
+	},
+	"setGraphics": function(data)
+	{
+		data.button.enabled = controlsPlayer(data.unitEntState.player);
+		let grayscale = data.button.enabled ? "" : "grayscale:";
+		data.icon.sprite = "stretched:" + grayscale + "session/icons/" + data.item.icon;
+	},
+	"setPosition": function(data)
+	{
+		var size = data.button.size;
+		// count on square buttons, so size.bottom is the width too
+		var spacer = size.bottom + 1;
+		// relative to the center ( = 50%)
+		size.rleft = size.rright = 50;
+		// offset from the center calculation
+		size.left = (data.i - data.numberOfItems/2) * spacer;
+		size.right = size.left + size.bottom;
+		data.button.size = size;
+	}
+};
+
+//ALLY COMMAND
+g_SelectionPanels.AllyCommand = {
+	"getMaxNumberOfItems": function()
+	{
+		return 2;
+	},
+	"getItems": function(unitEntState)
+	{
+		var commands = [];
+		for (var c in g_AllyEntityCommands)
+		{
+			var info = g_AllyEntityCommands[c].getInfo(unitEntState);
 			if (!info)
 				continue;
 			info.name = c;
@@ -145,15 +269,23 @@ g_SelectionPanels.Command = {
 	},
 	"setAction": function(data)
 	{
-		data.button.onPress = function() { data.item.callback ? data.item.callback(data.item) : performCommand(data.unitEntState.id, data.item.name); };
+		data.button.onPress = function() {
+			if (data.item.callback)
+				data.item.callback(data.item);
+			else
+				performAllyCommand(data.unitEntState.id, data.item.name);
+		};
 	},
+	"conflictsWith": ["Command"],
 	"setCountDisplay": function(data)
 	{
 		data.countDisplay.caption = data.item.count || "";
 	},
 	"setGraphics": function(data)
 	{
-		data.icon.sprite = "stretched:session/icons/" + data.item.icon;
+		data.button.enabled = data.item.count != undefined && data.item.count > 0;
+		let grayscale = data.button.enabled ? "" : "grayscale:";
+		data.icon.sprite = "stretched:" + grayscale + "session/icons/" + data.item.icon;
 	},
 	"setPosition": function(data)
 	{
@@ -166,7 +298,7 @@ g_SelectionPanels.Command = {
 		size.left = (data.i - data.numberOfItems/2) * spacer;
 		size.right = size.left + size.bottom;
 		data.button.size = size;
-	},
+	}
 };
 
 // CONSTRUCTION
@@ -185,13 +317,23 @@ g_SelectionPanels.Construction = {
 		data.template = GetTemplateData(data.entType);
 		if (!data.template) // abort if no template
 			return false;
-		data.technologyEnabled = Engine.GuiInterfaceCall("IsTechnologyResearched", data.template.requiredTechnology);
+
+		data.technologyEnabled = Engine.GuiInterfaceCall("IsTechnologyResearched", {
+			"tech": data.template.requiredTechnology,
+			"player": data.unitEntState.player
+		});
+
 		if (data.template.cost)
-		{
-			var totalCost = multiplyEntityCosts(data.template, 1);
-			data.neededResources = Engine.GuiInterfaceCall("GetNeededResources", totalCost);
-		}
+			data.neededResources = Engine.GuiInterfaceCall("GetNeededResources", {
+				"cost": multiplyEntityCosts(data.template, 1),
+				"player": data.unitEntState.player
+			});
+
 		data.limits = getEntityLimitAndCount(data.playerState, data.entType);
+
+		if (data.template.wallSet)
+			data.template.auras = GetTemplateData(data.template.wallSet.templates.long).auras;
+
 		return true;
 	},
 	"setAction": function(data)
@@ -202,6 +344,7 @@ g_SelectionPanels.Construction = {
 	{
 		var tooltip = getEntityNamesFormatted(data.template);
 		tooltip += getVisibleEntityClassesFormatted(data.template);
+		tooltip += getAurasTooltip(data.template);
 
 		if (data.template.tooltip)
 			tooltip += "\n[font=\"sans-13\"]" + data.template.tooltip + "[/font]";
@@ -210,47 +353,50 @@ g_SelectionPanels.Construction = {
 		tooltip += getPopulationBonusTooltip(data.template);
 
 		tooltip += formatLimitString(data.limits.entLimit, data.limits.entCount, data.limits.entLimitChangers);
+
 		if (!data.technologyEnabled)
-		{
-			var techName = getEntityNames(GetTechnologyData(data.template.requiredTechnology));
-			tooltip += "\n" + sprintf(translate("Requires %(technology)s"), { technology: techName });
-		}
+			tooltip += "\n" + sprintf(translate("Requires %(technology)s"), {
+				"technology": getEntityNames(GetTechnologyData(data.template.requiredTechnology))
+			});
+
 		if (data.neededResources)
 			tooltip += getNeededResourcesTooltip(data.neededResources);
+
 		data.button.tooltip = tooltip;
 		return true;
 	},
 	"setGraphics": function(data)
 	{
-		var grayscale = "";
+		var modifier = "";
 		if (!data.technologyEnabled || data.limits.canBeAddedCount == 0)
 		{
 			data.button.enabled = false;
-			grayscale = "grayscale:";
-			data.affordableMask.hidden = false;
-			data.affordableMask.sprite = "colour: 0 0 0 127";
+			modifier += "color: 0 0 0 127:";
+			modifier += "grayscale:";
 		}
 		else if (data.neededResources)
 		{
 			data.button.enabled = false;
-			data.affordableMask.hidden = false;
-			data.affordableMask.sprite = resourcesToAlphaMask(data.neededResources);
+			modifier += resourcesToAlphaMask(data.neededResources) +":";
 		}
+		else
+			data.button.enabled = controlsPlayer(data.unitEntState.player);
+
 		if (data.template.icon)
-			data.icon.sprite = "stretched:" + grayscale + "session/portraits/" + data.template.icon;
+			data.icon.sprite = modifier + "stretched:session/portraits/" + data.template.icon;
 	},
 	"setPosition": function(data)
 	{
 		var index = data.i + getNumberOfRightPanelButtons();
 		setPanelObjectPosition(data.button, index, data.rowLength);
-	},
+	}
 };
 
 // FORMATION
 g_SelectionPanels.Formation = {
 	"getMaxNumberOfItems": function()
 	{
-		return 16
+		return 16;
 	},
 	"rowLength": 4,
 	"conflictsWith": ["Garrison"],
@@ -258,11 +404,15 @@ g_SelectionPanels.Formation = {
 	{
 		if (!hasClass(unitEntState, "Unit") || hasClass(unitEntState, "Animal"))
 			return [];
-		return Engine.GuiInterfaceCall("GetAvailableFormations");
+		if (!g_availableFormations.has(unitEntState.player))
+			g_availableFormations.set(unitEntState.player, Engine.GuiInterfaceCall("GetAvailableFormations", unitEntState.player));
+		return g_availableFormations.get(unitEntState.player);
 	},
 	"addData": function(data)
 	{
-		data.formationInfo = Engine.GuiInterfaceCall("GetFormationInfoFromTemplate", {"templateName": data.item});
+		if (!g_formationsInfo.has(data.item))
+			g_formationsInfo.set(data.item, Engine.GuiInterfaceCall("GetFormationInfoFromTemplate", { "templateName": data.item }));
+		data.formationInfo = g_formationsInfo.get(data.item);
 		data.formationOk = canMoveSelectionIntoFormation(data.item);
 		data.formationSelected = Engine.GuiInterfaceCall("IsFormationSelected", {
 			"ents": data.selection,
@@ -283,11 +433,11 @@ g_SelectionPanels.Formation = {
 	},
 	"setGraphics": function(data)
 	{
-		data.button.enabled = data.formationOk;
+		data.button.enabled = data.formationOk && controlsPlayer(data.unitEntState.player);
 		var grayscale = data.formationOk ? "" : "grayscale:";
 		data.guiSelection.hidden = !data.formationSelected;
 		data.icon.sprite = "stretched:"+grayscale+"session/icons/"+data.formationInfo.icon;
-	},
+	}
 };
 
 // GARRISON
@@ -306,7 +456,7 @@ g_SelectionPanels.Garrison = {
 		{
 			var state = GetEntityState(ent);
 			if (state.garrisonHolder)
-				groups.add(state.garrisonHolder.entities)
+				groups.add(state.garrisonHolder.entities);
 		}
 		return groups.getEntsGrouped();
 	},
@@ -326,7 +476,7 @@ g_SelectionPanels.Garrison = {
 	},
 	"setTooltip": function(data)
 	{
-		var tooltip = sprintf(translate("Unload %(name)s"), { name: data.name }) + "\n";
+		var tooltip = sprintf(translate("Unload %(name)s"), { "name": data.name }) + "\n";
 		tooltip += translate("Single-click to unload 1. Shift-click to unload all of this type.");
 		data.button.tooltip = tooltip;
 	},
@@ -339,19 +489,16 @@ g_SelectionPanels.Garrison = {
 		var grayscale = "";
 		var ents = data.item.ents;
 		var entplayer = GetEntityState(ents[0]).player;
-		data.button.sprite = "colour: " + rgbToGuiColor(g_Players[entplayer].color);
+		data.button.sprite = "color:" + rgbToGuiColor(g_Players[entplayer].color) +":";
 
-		var player = Engine.GetPlayerID();
-		if(player != data.unitEntState.player && !g_DevSettings.controlAll)
+		if (!controlsPlayer(data.unitEntState.player) && !controlsPlayer(entplayer))
 		{
-			if (entplayer != player)
-			{
-				data.button.enabled = false;
-				grayscale = "grayscale:";
-			}
+			data.button.enabled = false;
+			grayscale = "grayscale:";
 		}
+
 		data.icon.sprite = "stretched:" + grayscale + "session/portraits/" + data.template.icon;
-	},
+	}
 };
 
 // GATE
@@ -366,9 +513,9 @@ g_SelectionPanels.Gate = {
 		var longWallTypes = {};
 		var walls = [];
 		var gates = [];
-		for (var i in selection)
+		for (var ent of selection)
 		{
-			var state = GetEntityState(selection[i]);
+			var state = GetEntityState(ent);
 			if (hasClass(state, "LongWall") && !state.gate && !longWallTypes[state.template])
 			{
 				var gateTemplate = getWallGateTemplate(state.id);
@@ -428,14 +575,16 @@ g_SelectionPanels.Gate = {
 			data.wallCount = data.selection.reduce(function (count, ent) {
 					var state = GetEntityState(ent);
 					if (hasClass(state, "LongWall") && !state.gate)
-						count++;
+						++count;
 					return count;
 				}, 0);
 
 			tooltip += "\n" + getEntityCostTooltip(data.template, data.wallCount);
 
+			data.neededResources = Engine.GuiInterfaceCall("GetNeededResources", {
+				"cost": multiplyEntityCosts(data.template, data.wallCount)
+			});
 
-			data.neededResources = Engine.GuiInterfaceCall("GetNeededResources", multiplyEntityCosts(data.template, data.wallCount));
 			if (data.neededResources)
 				tooltip += getNeededResourcesTooltip(data.neededResources);
 		}
@@ -443,14 +592,14 @@ g_SelectionPanels.Gate = {
 	},
 	"setGraphics": function(data)
 	{
-		data.affordableMask.hidden == data.neededResources ? true : false;
+		data.button.enabled = controlsPlayer(data.unitEntState.player);
 		var gateIcon;
 		if (data.item.gate)
 		{
 			// If already a gate, show locking actions
 			gateIcon = "icons/lock_" + GATE_ACTIONS[data.item.locked ? 0 : 1] + "ed.png";
 			if (data.item.gate.locked === undefined)
-				data.guiSelection.hidden = false
+				data.guiSelection.hidden = false;
 			else
 				data.guiSelection.hidden = data.item.gate.locked != data.item.locked;
 		}
@@ -464,13 +613,13 @@ g_SelectionPanels.Gate = {
 			data.guiSelection.hidden = true;
 		}
 
-		data.icon.sprite = "stretched:session/" + gateIcon;
+		data.icon.sprite = (data.neededResources ? resourcesToAlphaMask(data.neededResources) + ":" : "") + "stretched:session/" + gateIcon;
 	},
 	"setPosition": function(data)
 	{
 		var index = data.i + getNumberOfRightPanelButtons();
 		setPanelObjectPosition(data.button, index, data.rowLength);
-	},
+	}
 };
 
 // PACK
@@ -530,12 +679,14 @@ g_SelectionPanels.Pack = {
 			data.icon.sprite = "stretched:session/icons/unpack.png";
 		else
 			data.icon.sprite = "stretched:session/icons/pack.png";
+
+		data.button.enabled = controlsPlayer(data.unitEntState.player);
 	},
 	"setPosition": function(data)
 	{
 		var index = data.i + getNumberOfRightPanelButtons();
 		setPanelObjectPosition(data.button, index, data.rowLength);
-	},
+	}
 };
 
 // QUEUE
@@ -585,7 +736,10 @@ g_SelectionPanels.Queue = {
 		if (data.item.neededSlots)
 		{
 			tooltip += "\n[color=\"red\"]" + translate("Insufficient population capacity:") + "\n[/color]";
-			tooltip += sprintf(translate("%(population)s %(neededSlots)s"), { population: getCostComponentDisplayName("population"), neededSlots: data.item.neededSlots });
+			tooltip += sprintf(translate("%(population)s %(neededSlots)s"), {
+				"population": getCostComponentDisplayName("population"),
+				"neededSlots": data.item.neededSlots
+			});
 		}
 		data.button.tooltip = tooltip;
 	},
@@ -610,7 +764,9 @@ g_SelectionPanels.Queue = {
 	{
 		if (data.template.icon)
 			data.icon.sprite = "stretched:session/portraits/" + data.template.icon;
-	},
+
+		data.button.enabled = controlsPlayer(data.unitEntState.player);
+	}
 };
 
 // RESEARCH
@@ -628,7 +784,7 @@ g_SelectionPanels.Research = {
 		{
 			var entState = GetEntityState(ent);
 			if (entState.production && entState.production.technologies.length)
-				return entState.production.technologies
+				return entState.production.technologies;
 		}
 		return [];
 	},
@@ -644,7 +800,7 @@ g_SelectionPanels.Research = {
 		data.entType = data.item.pair ? [data.item.top, data.item.bottom] : [data.item];
 		data.template = data.entType.map(GetTechnologyData);
 		// abort if no template found for any of the techs
-		if (!data.template.every(function(v) { return v; }))
+		if (data.template.some(v => !v))
 			return false;
 		// index one row below
 		var shiftedIndex = data.i + data.rowLength;
@@ -652,34 +808,21 @@ g_SelectionPanels.Research = {
 		data.positionsToHide = data.item.pair ? [] : [data.i];
 
 		// add top buttons to the data
-		data.button = data.positions.map(function(p) { 
-			return Engine.GetGUIObjectByName("unitResearchButton["+p+"]"); 
-		});
+		data.button = data.positions.map(p => Engine.GetGUIObjectByName("unitResearchButton["+p+"]"));
+		data.buttonsToHide = data.positionsToHide.map(p => Engine.GetGUIObjectByName("unitResearchButton["+p+"]"));
 
-		data.buttonsToHide = data.positionsToHide.map(function(p) { 
-			return Engine.GetGUIObjectByName("unitResearchButton["+p+"]"); 
-		});
+		data.icon = data.positions.map(p => Engine.GetGUIObjectByName("unitResearchIcon["+p+"]"));
+		data.unchosenIcon = data.positions.map(p => Engine.GetGUIObjectByName("unitResearchUnchosenIcon["+p+"]"));
 
+		data.neededResources = data.template.map(t => Engine.GuiInterfaceCall("GetNeededResources", {
+			"cost": t.cost,
+			"player": data.unitEntState.player
+		}));
 
-		data.affordableMask = data.positions.map(function(p) { 
-			return Engine.GetGUIObjectByName("unitResearchUnaffordable["+p+"]");
-		});
-
-		data.icon = data.positions.map(function(p) { 
-			return Engine.GetGUIObjectByName("unitResearchIcon["+p+"]");
-		});
-
-		data.unchosenIcon = data.positions.map(function(p) { 
-			return Engine.GetGUIObjectByName("unitResearchUnchosenIcon["+p+"]");
-		});
-
-		data.neededResources = data.template.map(function(t) { 
-			return Engine.GuiInterfaceCall("GetNeededResources", t.cost);
-		});
-
-		data.requirementsPassed = data.entType.map(function(e) { 
-			return Engine.GuiInterfaceCall("CheckTechnologyRequirements",e); 
-		});
+		data.requirementsPassed = data.entType.map(e => Engine.GuiInterfaceCall("CheckTechnologyRequirements", {
+			"tech": e,
+			"player": data.unitEntState.player
+		}));
 
 		data.pair = Engine.GetGUIObjectByName("unitResearchPair["+data.i+"]");
 
@@ -701,10 +844,10 @@ g_SelectionPanels.Research = {
 				tooltip += "\n" + template.requirementsTooltip;
 				if (template.classRequirements)
 				{
-					var player = Engine.GetPlayerID();
+					var player = data.unitEntState.player;
 					var current = GetSimState().players[player].classCounts[template.classRequirements.class] || 0;
 					var remaining = template.classRequirements.number - current;
-					tooltip += " " + sprintf(translatePlural("Remaining: %(number)s to build.", "Remaining: %(number)s to build.", remaining), { number: remaining});
+					tooltip += " " + sprintf(translatePlural("Remaining: %(number)s to build.", "Remaining: %(number)s to build.", remaining), { "number": remaining });
 				}
 			}
 			if (data.neededResources[i])
@@ -742,31 +885,27 @@ g_SelectionPanels.Research = {
 	{
 		for (var i in data.entType)
 		{
-			var button = data.button[i];
+			let button = data.button[i];
 			button.hidden = false;
-			var grayscale = "";
+			var modifier = "";
 			if (!data.requirementsPassed[i])
 			{
 				button.enabled = false;
-				grayscale = "grayscale:";
-				data.affordableMask[i].hidden = false;
-				data.affordableMask[i].sprite = "colour: 0 0 0 127";
+				modifier += "color: 0 0 0 127:";
+				modifier += "grayscale:";
 			}
 			else if (data.neededResources[i])
 			{
 				button.enabled = false;
-				data.affordableMask[i].hidden = false;
-				data.affordableMask[i].sprite = resourcesToAlphaMask(data.neededResources[i]);
+				modifier += resourcesToAlphaMask(data.neededResources[i]) + ":";
 			}
 			else
-			{
-				data.affordableMask[i].hidden = true;
-				button.enabled = true; 
-			}
+				button.enabled = controlsPlayer(data.unitEntState.player);
+
 			if (data.template[i].icon)
-				data.icon[i].sprite = "stretched:" + grayscale + "session/portraits/" + data.template[i].icon;
+				data.icon[i].sprite = modifier + "stretched:session/portraits/" + data.template[i].icon;
 		}
-		for (var button of data.buttonsToHide)
+		for (let button of data.buttonsToHide)
 			button.hidden = true;
 		// show the tech connector
 		data.pair.hidden = data.item.pair == null;
@@ -776,7 +915,7 @@ g_SelectionPanels.Research = {
 		for (var i in data.button)
 			setPanelObjectPosition(data.button[i], data.positions[i], data.rowLength);
 		setPanelObjectPosition(data.pair, data.i, data.rowLength);
-	},
+	}
 };
 
 // SELECTION
@@ -800,12 +939,11 @@ g_SelectionPanels.Selection = {
 			return false;
 		data.name = getEntityNames(data.template);
 
-		var ents = g_Selection.groups.getEntsByName(data.item)
-		
+		var ents = g_Selection.groups.getEntsByName(data.item);
 		data.count = ents.length;
-		for (var i in ents)
+		for (var ent of ents)
 		{
-			var state = GetEntityState(ents[i]);
+			var state = GetEntityState(ent);
 
 			if (state.resourceCarrying && state.resourceCarrying.length !== 0)
 			{
@@ -870,7 +1008,7 @@ g_SelectionPanels.Selection = {
 	{
 		if (data.template.icon)
 			data.icon.sprite = "stretched:session/portraits/" + data.template.icon;
-	},
+	}
 };
 
 // STANCE
@@ -899,13 +1037,14 @@ g_SelectionPanels.Stance = {
 	},
 	"setTooltip": function(data)
 	{
-		data.button.tooltip = getStanceDisplayName(data.item);
+		data.button.tooltip = getStanceDisplayName(data.item) + "\n[font=\"sans-13\"]" + getStanceTooltip(data.item) + "[/font]";
 	},
 	"setGraphics": function(data)
 	{
 		data.guiSelection.hidden = !data.stanceSelected;
 		data.icon.sprite = "stretched:session/icons/stances/"+data.item+".png";
-	},
+		data.button.enabled = controlsPlayer(data.unitEntState.player);
+	}
 };
 
 // TRAINING
@@ -924,22 +1063,28 @@ g_SelectionPanels.Training = {
 		data.template = GetTemplateData(data.entType);
 		if (!data.template)
 			return false;
-		data.technologyEnabled = Engine.GuiInterfaceCall("IsTechnologyResearched", data.template.requiredTechnology);
+
+		data.technologyEnabled = Engine.GuiInterfaceCall("IsTechnologyResearched", {
+			"tech": data.template.requiredTechnology,
+			"player": data.unitEntState.player
+		});
 
 		var [buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch] =
 			getTrainingBatchStatus(data.playerState, data.unitEntState.id, data.entType, data.selection);
+
 		data.buildingsCountToTrainFullBatch = buildingsCountToTrainFullBatch;
 		data.fullBatchSize = fullBatchSize;
 		data.remainderBatch = remainderBatch;
+
 		data.trainNum = buildingsCountToTrainFullBatch || 1; // train at least one unit
 		if (Engine.HotkeyIsPressed("session.batchtrain"))
 			data.trainNum = buildingsCountToTrainFullBatch * fullBatchSize + remainderBatch;
 
 		if (data.template.cost)
-		{
-			var totalCosts = multiplyEntityCosts(data.template, data.trainNum);
-			data.neededResources = Engine.GuiInterfaceCall("GetNeededResources", totalCosts);
-		}
+			data.neededResources = Engine.GuiInterfaceCall("GetNeededResources", {
+				"cost": multiplyEntityCosts(data.template, data.trainNum),
+				"player": data.unitEntState.player
+			});
 
 		return true;
 	},
@@ -949,8 +1094,7 @@ g_SelectionPanels.Training = {
 	},
 	"setCountDisplay": function(data)
 	{
-		var count = data.trainNum > 1 ? data.trainNum : "";
-		data.countDisplay.caption = count;
+		data.countDisplay.caption = data.trainNum > 1 ? data.trainNum : "";
 	},
 	"setTooltip": function(data)
 	{
@@ -961,16 +1105,7 @@ g_SelectionPanels.Training = {
 
 		tooltip += getEntityNamesFormatted(data.template);
 		tooltip += getVisibleEntityClassesFormatted(data.template);
-
-		if (data.template.auras)
-		{
-			for (var auraName in data.template.auras)
-			{
-				tooltip += "\n[font=\"sans-bold-13\"]" + translate(auraName) + "[/font]";
-				if (data.template.auras[auraName])
-					tooltip += ": " + translate(data.template.auras[auraName]);
-			}
-		}
+		tooltip += getAurasTooltip(data.template);
 
 		if (data.template.tooltip)
 			tooltip += "\n[font=\"sans-13\"]" + data.template.tooltip + "[/font]";
@@ -995,7 +1130,7 @@ g_SelectionPanels.Training = {
 		if (!data.technologyEnabled)
 		{
 			var techName = getEntityNames(GetTechnologyData(data.template.requiredTechnology));
-			tooltip += "\n" + sprintf(translate("Requires %(technology)s"), { technology: techName });
+			tooltip += "\n" + sprintf(translate("Requires %(technology)s"), { "technology": techName });
 		}
 		if (data.neededResources)
 			tooltip += getNeededResourcesTooltip(data.neededResources);
@@ -1008,7 +1143,7 @@ g_SelectionPanels.Training = {
 	{
 		var index = data.i + getNumberOfRightPanelButtons();
 		setPanelObjectPosition(data.button, index, data.rowLength);
-	},
+	}
 };
 
 
@@ -1023,6 +1158,7 @@ var g_PanelsOrder = [
 	// LEFT PANE
 	"Barter", // must always be visible on markets
 	"Garrison", // more important than Formation, as you want to see the garrisoned units in ships
+	"Alert",
 	"Formation",
 	"Stance", // normal together with formation
 
@@ -1035,6 +1171,7 @@ var g_PanelsOrder = [
 
 	// UNIQUE PANES (importance doesn't matter)
 	"Command",
+	"AllyCommand",
 	"Queue",
 	"Selection",
 ];
