@@ -32,30 +32,41 @@ function canMoveSelectionIntoFormation(formationTemplate)
 
 function getStanceDisplayName(name)
 {
-	var displayName;
-	switch(name)
+	switch (name)
 	{
-		case "violent":
-			displayName = translateWithContext("stance", "Violent");
-			break;
-		case "aggressive":
-			displayName = translateWithContext("stance", "Aggressive");
-			break;
-		case "passive":
-			displayName = translateWithContext("stance", "Passive");
-			break;
-		case "defensive":
-			displayName = translateWithContext("stance", "Defensive");
-			break;
-		case "standground":
-			displayName = translateWithContext("stance", "Standground");
-			break;
-		default:
-			warn(sprintf("Internationalization: Unexpected stance found with code ‘%(stance)s’. This stance must be internationalized.", { stance: name }));
-			displayName = name;
-			break;
+	case "violent":
+		return translateWithContext("stance", "Violent");
+	case "aggressive":
+		return translateWithContext("stance", "Aggressive");
+	case "defensive":
+		return translateWithContext("stance", "Defensive");
+	case "passive":
+		return translateWithContext("stance", "Passive");
+	case "standground":
+		return translateWithContext("stance", "Standground");
+	default:
+		warn("Internationalization: Unexpected stance found: " + name);
+		return name;
+	}
+}
+
+function getStanceTooltip(name)
+{
+	switch (name)
+	{
+	case "violent":
+		return translateWithContext("stance", "Attack nearby opponents, focus on attackers and chase while visible");
+	case "aggressive":
+		return translateWithContext("stance", "Attack nearby opponents");
+	case "defensive":
+		return translateWithContext("stance", "Attack nearby opponents, chase a short distance and return to the original location");
+	case "passive":
+		return translateWithContext("stance", "Flee if attacked");
+	case "standground":
+		return translateWithContext("stance", "Attack opponents in range, but don't move");
+	default:
+		return "";
 	}
-	return displayName;
 }
 
 // ==============================================
@@ -67,15 +78,18 @@ function formatLimitString(trainEntLimit, trainEntCount, trainEntLimitChangers)
 {
 	if (trainEntLimit == undefined)
 		return "";
-	var text = "\n\n" + sprintf(translate("Current Count: %(count)s, Limit: %(limit)s."), { count: trainEntCount, limit: trainEntLimit });
+
+	var text = "\n\n" + sprintf(translate("Current Count: %(count)s, Limit: %(limit)s."), { "count": trainEntCount, "limit": trainEntLimit });
+
 	if (trainEntCount >= trainEntLimit)
 		text = "[color=\"red\"]" + text + "[/color]";
+
 	for (var c in trainEntLimitChangers)
 	{
 		if (trainEntLimitChangers[c] > 0)
-			text += "\n" + sprintf(translate("%(changer)s enlarges the limit with %(change)s."), { changer: translate(c), change: trainEntLimitChangers[c] });
+			text += "\n" + sprintf(translate("%(changer)s enlarges the limit with %(change)s."), { "changer": translate(c), "change": trainEntLimitChangers[c] });
 		else if (trainEntLimitChangers[c] < 0)
-			text += "\n" + sprintf(translate("%(changer)s lessens the limit with %(change)s."), { changer: translate(c), change: (-trainEntLimitChangers[c]) });
+			text += "\n" + sprintf(translate("%(changer)s lessens the limit with %(change)s."), { "changer": translate(c), "change": (-trainEntLimitChangers[c]) });
 	}
 	return text;
 }
@@ -93,22 +107,25 @@ function formatLimitString(trainEntLimit, trainEntCount, trainEntLimitChangers)
 function formatBatchTrainingString(buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch)
 {
 	var totalBatchTrainingCount = buildingsCountToTrainFullBatch * fullBatchSize + remainderBatch;
+
 	// Don't show the batch training tooltip if either units of this type can't be trained at all
 	// or only one unit can be trained
 	if (totalBatchTrainingCount < 2)
 		return "";
+
 	var batchTrainingString = "";
 	var fullBatchesString = "";
 	if (buildingsCountToTrainFullBatch > 0)
 	{
 		if (buildingsCountToTrainFullBatch > 1)
 			fullBatchesString = sprintf(translate("%(buildings)s*%(batchSize)s"), {
-				buildings: buildingsCountToTrainFullBatch,
-				batchSize: fullBatchSize
+				"buildings": buildingsCountToTrainFullBatch,
+				"batchSize": fullBatchSize
 			});
 		else
 			fullBatchesString = fullBatchSize;
 	}
+
 	var remainderBatchString = remainderBatch > 0 ? remainderBatch : "";
 	var batchDetailsString = "";
 	var action = "[font=\"sans-bold-13\"]" + translate("Shift-click") + "[/font]";
@@ -121,22 +138,22 @@ function formatBatchTrainingString(buildingsCountToTrainFullBatch, fullBatchSize
 	{
 		if (remainderBatch > 0)
 			return "\n[font=\"sans-13\"]" + sprintf(translate("%(action)s to train %(number)s (%(fullBatch)s + %(remainderBatch)s)."), {
-				action: action,
-				number: totalBatchTrainingCount,
-				fullBatch: fullBatchesString,
-				remainderBatch: remainderBatch
+				"action": action,
+				"number": totalBatchTrainingCount,
+				"fullBatch": fullBatchesString,
+				"remainderBatch": remainderBatch
 			}) + "[/font]";
 
 		return "\n[font=\"sans-13\"]" + sprintf(translate("%(action)s to train %(number)s (%(fullBatch)s)."), {
-			action: action,
-			number: totalBatchTrainingCount,
-			fullBatch: fullBatchesString
+			"action": action,
+			"number": totalBatchTrainingCount,
+			"fullBatch": fullBatchesString
 		}) + "[/font]";
 	}
 
 	return "\n[font=\"sans-13\"]" + sprintf(translate("%(action)s to train %(number)s."), {
-		action: action,
-		number: totalBatchTrainingCount
+		"action": action,
+		"number": totalBatchTrainingCount
 	}) + "[/font]";
 }
 
