@@ -87,11 +87,11 @@ var unitActions =
 		"specificness": 30,
 	},
 
-	"attack": 
+	"capture": 
 	{
 		"execute": function(target, action, selection, queued)
 		{
-			Engine.PostNetworkCommand({"type": "attack", "entities": selection, "target": action.target, "queued": queued});
+			Engine.PostNetworkCommand({"type": "attack", "entities": selection, "target": action.target, "allowCapture": true, "queued": queued});
 			Engine.GuiInterfaceCall("PlaySound", { "name": "order_attack", "entity": selection[0] });
 			return true;
 		},
@@ -99,15 +99,36 @@ var unitActions =
 		{
 			if (!entState.attack || !targetState.hitpoints)
 				return false;
-			if (playerCheck(entState, targetState, ["Neutral", "Enemy"]))
-				return {"possible": Engine.GuiInterfaceCall("CanAttack", {"entity": entState.id, "target": targetState.id})};
+			return {"possible": Engine.GuiInterfaceCall("CanCapture", {"entity": entState.id, "target": targetState.id})};
+		},
+		"actionCheck": function(target)
+		{
+			if (getActionInfo("capture", target).possible)
+				return {"type": "capture", "cursor": "action-capture", "target": target};
 			return false;
 		},
+		"specificness": 9,
+	},
+
+	"attack":
+	{
+		"execute": function(target, action, selection, queued)
+		{
+			Engine.PostNetworkCommand({"type": "attack", "entities": selection, "target": action.target, "queued": queued, "allowCapture": false});
+			Engine.GuiInterfaceCall("PlaySound", { "name": "order_attack", "entity": selection[0] });
+			return true;
+		},
+		"getActionInfo": function(entState, targetState)
+		{
+			if (!entState.attack || !targetState.hitpoints)
+				return false;
+			return {"possible": Engine.GuiInterfaceCall("CanAttack", {"entity": entState.id, "target": targetState.id})};
+		},
 		"hotkeyActionCheck": function(target)
 		{
 			if (Engine.HotkeyIsPressed("session.attack") && getActionInfo("attack", target).possible)
 				return {"type": "attack", "cursor": "action-attack", "target": target};
-			return false
+			return false;
 		},
 		"actionCheck": function(target)
 		{
@@ -201,7 +222,7 @@ var unitActions =
 				return false;
 			if (getActionInfo("repair", target).possible)
 				return {"type": "repair", "cursor": "action-repair", "target": target};
-			return {"type": "none", "cursor": "action-repair-disabled", "target": undefined};
+			return {"type": "none", "cursor": "action-repair-disabled", "target": null};
 		},
 		"actionCheck": function(target)
 		{
@@ -209,7 +230,7 @@ var unitActions =
 				return {"type": "build", "cursor": "action-repair", "target": target};
 			return false;
 		},
-		"specificness": 4,
+		"specificness": 11,
 	},
 
 	"gather": 
@@ -251,7 +272,13 @@ var unitActions =
 		{
 			if (!targetState.resourceDropsite)
 				return false;
-			if (!playerCheck(entState, targetState, ["Player"]))
+			var playerState = GetSimState().players[entState.player];
+			if (playerState.hasSharedDropsites && targetState.resourceDropsite.shared)
+			{
+				if (!playerCheck(entState, targetState, ["Player", "MutualAlly"]))
+					return false;
+			}
+			else if (!playerCheck(entState, targetState, ["Player"]))
 				return false;
 			if (!entState.resourceCarrying || !entState.resourceCarrying.length)
 				return false;
@@ -274,7 +301,7 @@ var unitActions =
 	{
 		"execute": function(target, action, selection, queued)
 		{
-			Engine.PostNetworkCommand({"type": "setup-trade-route", "entities": selection, "target": action.target, "source": undefined, "route": undefined, "queued": queued});
+			Engine.PostNetworkCommand({"type": "setup-trade-route", "entities": selection, "target": action.target, "source": null, "route": null, "queued": queued});
 			Engine.GuiInterfaceCall("PlaySound", { "name": "order_trade", "entity": selection[0] });
 			return true;
 		},
@@ -303,7 +330,7 @@ var unitActions =
 						gain: getTradingTooltip(tradingDetails.gain)
 					});
 				else
-					tooltip += "\n" + translate("Right-click on another market to set it as a destination trade market.")
+					tooltip += "\n" + translate("Right-click on another market to set it as a destination trade market.");
 				break;
 			case "is second":
 				tooltip = translate("Destination trade market.") + "\n" + sprintf(translate("Gain: %(gain)s"), {
@@ -346,7 +373,7 @@ var unitActions =
 		{
 			if (!hasClass(entState, "Unit") || !targetState.garrisonHolder)
 				return false;
-			if (!playerCheck(entState, targetState, ["Player", "Ally"]))
+			if (!playerCheck(entState, targetState, ["Player", "MutualAlly"]))
 				return false;
 			var tooltip = sprintf(translate("Current garrison: %(garrisoned)s/%(capacity)s"), {
 				garrisoned: targetState.garrisonHolder.garrisonedEntitiesCount,
@@ -369,7 +396,7 @@ var unitActions =
 			var actionInfo =  getActionInfo("garrison", target);
 			if (actionInfo.possible)
 				return {"type": "garrison", "cursor": "action-garrison", "tooltip": actionInfo.tooltip, "target": target};
-			return {"type": "none", "cursor": "action-garrison-disabled", "target": undefined};
+			return {"type": "none", "cursor": "action-garrison-disabled", "target": null};
 		},
 		"hotkeyActionCheck": function(target)
 		{
@@ -407,7 +434,7 @@ var unitActions =
 				return false;
 			if (getActionInfo("guard", target).possible)
 				return {"type": "guard", "cursor": "action-guard", "target": target};
-			return {"type": "none", "cursor": "action-guard-disabled", "target": undefined};
+			return {"type": "none", "cursor": "action-guard-disabled", "target": null};
 		},
 		"hotkeyActionCheck": function(target)
 		{
@@ -479,7 +506,7 @@ var unitActions =
 				cursor = "action-attack-move";
 			}
 
-			if (targetState.garrisonHolder && playerCheck(entState, targetState, ["Player", "Ally"]))
+			if (targetState.garrisonHolder && playerCheck(entState, targetState, ["Player", "MutualAlly"]))
 			{
 				data.command = "garrison";
 				data.target = targetState.id;
@@ -502,12 +529,6 @@ var unitActions =
 				data.resourceType = resourceType;
 				data.resourceTemplate = targetState.template;
 			}
-			else if (targetState.foundation && entState.builder)
-			{
-				data.command = "build";
-				data.target = targetState.id;
-				cursor = "action-build";
-			}
 			else if (hasClass(entState, "Market") && hasClass(targetState, "Market") && entState.id != targetState.id &&
 					(!hasClass(entState, "NavalMarket") || hasClass(targetState, "NavalMarket")) && !playerCheck(entState, targetState, ["Enemy"]))
 			{
@@ -533,20 +554,31 @@ var unitActions =
 						tooltip += "\n" + sprintf(translate("Expected gain: %(gain)s"), { gain: getTradingTooltip(gain) });
 				}
 			}
+			else if (targetState.foundation && playerCheck(entState, targetState, ["Ally"]))
+			{
+				data.command = "build";
+				data.target = targetState.id;
+				cursor = "action-build";
+			}
 			else if (targetState.needsRepair && playerCheck(entState, targetState, ["Ally"]))
 			{
 				data.command = "repair";
 				data.target = targetState.id;
 				cursor = "action-repair";
 			}
+			else if (playerCheck(entState, targetState, ["Enemy"]))
+			{
+				data.target = targetState.id;
+				data.command = "attack";
+				cursor = "action-attack";
+			}
 
 			// Don't allow the rally point to be set on any of the currently selected entities (used for unset)
 			// except if the autorallypoint hotkey is pressed and the target can produce entities
-			var selection = g_Selection.toList();
 			if (!Engine.HotkeyIsPressed("session.autorallypoint") || !targetState.production || !targetState.production.entities.length)
 			{
-				for (var i = 0; i < selection.length; i++)
-					if (targetState.id === selection[i])
+				for each (var ent in g_Selection.selected)
+					if (targetState.id === ent)
 						return false;
 			}
 
@@ -633,7 +665,7 @@ var unitActions =
 
 /**
  * Info and actions for the entity commands
- * Currently displayed in the bottom of the central pane
+ * Currently displayed in the bottom of the central panel
  */
 var g_EntityCommands = 
 {
@@ -643,9 +675,8 @@ var g_EntityCommands =
 		{
 			if (!entState.garrisonHolder)
 				return false;
-			var selection = g_Selection.toList();
 			var count = 0;
-			for (var ent of selection)
+			for each (var ent in g_Selection.selected)
 			{
 				var state = GetEntityState(ent);
 				if (state.garrisonHolder)
@@ -666,12 +697,22 @@ var g_EntityCommands =
 	"delete": {
 		"getInfo": function(entState)
 		{
+			if (!entState.canDelete)
+				return false;
+
 			if (entState.mirage)
 				return {
 					"tooltip": translate("You cannot destroy this entity because it is in the fog-of-war"),
-					"icon": "kill_small.png"
+					"icon": "kill_small_disabled.png"
 				};
 
+			if (entState.capturePoints && entState.capturePoints[entState.player] < entState.maxCapturePoints / 2)
+				return {
+					"tooltip": translate("You cannot destroy this entity as you own less than half the capture points"),
+					"icon": "kill_small_disabled.png"
+				};
+					
+
 			return {
 				"tooltip": translate("Delete"),
 				"icon": "kill_small.png"
@@ -682,6 +723,9 @@ var g_EntityCommands =
 			if (entState.mirage)
 				return;
 
+			if (entState.capturePoints && entState.capturePoints[entState.player] < entState.maxCapturePoints / 2)
+				return;
+
 			var selection = g_Selection.toList();
 			if (selection.length < 1)
 				return;
@@ -834,7 +878,7 @@ var g_EntityCommands =
 		},
 	},
 	// Trading
-	 "select-trading-goods": {
+	"select-trading-goods": {
 		"getInfo": function(entState)
 		{
 			if (!hasClass(entState, "Market"))
@@ -849,43 +893,93 @@ var g_EntityCommands =
 			toggleTrade();
 		},
 	},
-	// Raise alert
-	"increase-alert-level": {
+	// Dropsite sharing
+	"share-dropsite": {
 		"getInfo": function(entState)
 		{
-			if(!entState.alertRaiser || !entState.alertRaiser.canIncreaseLevel)
+			if (!entState.resourceDropsite || !entState.resourceDropsite.sharable)
 				return false;
-
-			if(entState.alertRaiser.hasRaisedAlert)
-				var tooltip = translate("Increase the alert level to protect more units");
-			else
-				var tooltip = translate("Raise an alert!");
+			let playerState = GetSimState().players[entState.player];
+			if (!playerState.isMutualAlly.some((e, i) => e && i != entState.player))
+				return false;
+			if (entState.resourceDropsite.shared)
+				return {
+					"tooltip": translate("Press to prevent allies from using this dropsite"),
+					"icon": "lock_unlocked.png"
+				};
 			return {
-				"tooltip": tooltip,
-				"icon": "bell_level1.png"
+				"tooltip": translate("Press to allow allies to use this dropsite"),
+				"icon": "lock_locked.png"
 			};
 		},
 		"execute": function(entState)
 		{
-			increaseAlertLevel();
+			Engine.PostNetworkCommand({
+				"type": "set-dropsite-sharing",
+				"entities": [entState.id],
+				"shared": !entState.resourceDropsite.shared
+			});
 		},
-	},
-	// End alert
-	"alert-end": {
+	}
+};
+
+var g_AllyEntityCommands =
+{
+	// Unload
+	"unload-all": {
 		"getInfo": function(entState)
 		{
-			if(!entState.alertRaiser || !entState.alertRaiser.hasRaisedAlert)
-				return false
+			if (!entState.garrisonHolder)
+				return false;
+			var count = 0;
+			for each (var ent in g_Selection.selected)
+			{
+				var selectedEntState = GetEntityState(ent);
+				if (selectedEntState.garrisonHolder)
+				{	
+					var player = Engine.GetPlayerID();
+					for (var entity of selectedEntState.garrisonHolder.entities)
+					{
+						var state = GetEntityState(entity);
+						if (state.player == player)
+							count++;
+					}
+				}
+			}
 			return {
-				"tooltip": translate("End of alert."),
-				"icon": "bell_level0.png"
+				"tooltip": translate("Unload All"),
+				"icon": "garrison-out.png",
+				"count": count,
 			};
 		},
 		"execute": function(entState)
 		{
-			endOfAlert();
+			unloadAllByOwner();
 		},
 	},
+	// Dropsite sharing
+	"share-dropsite": {
+		"getInfo": function(entState)
+		{
+			if (Engine.GetPlayerID() == -1 || !GetSimState().players[Engine.GetPlayerID()].hasSharedDropsites)
+				return false;
+			if (!entState.resourceDropsite || !entState.resourceDropsite.sharable)
+				return false;
+			if (entState.resourceDropsite.shared)
+				return {
+					"tooltip": translate("You are allowed to use this dropsite"),
+					"icon": "lock_unlocked.png"
+				};
+			return {
+				"tooltip": translate("The use of this dropsite is prohibited"),
+				"icon": "lock_locked.png"
+			};
+		},
+		"execute": function(entState)
+		{
+			// This command button is always disabled
+		},
+	}
 };
 
 function playerCheck(entState, targetState, validPlayers)
@@ -901,4 +995,4 @@ function playerCheck(entState, targetState, validPlayers)
 			return true;
 	}
 	return false;
-};
+}
