@@ -1,36 +1,24 @@
-//-------------------------------- --------------------------------
-// Utility functions
-//-------------------------------- --------------------------------
-
-//===============================================
-// Player functions
-
 // Get the basic player data
-function getPlayerData(playerAssignments)
+function getPlayerData(playerAssignments, previousData = undefined)
 {
-	var players = [];
+	let players = [];
 
-	var simState = GetSimState();
+	let simState = GetSimState();
 	if (!simState)
 		return players;
 
-	for (var i = 0; i < simState.players.length; i++)
+	for (let i = 0; i < simState.players.length; ++i)
 	{
-		var playerState = simState.players[i];
-
-		var name = playerState.name;
-		var civ = playerState.civ;
-		var color = {
-		    "r": playerState.colour.r*255,
-		    "g": playerState.colour.g*255,
-		    "b": playerState.colour.b*255,
-		    "a": playerState.colour.a*255
-		};
-
-		var player = {
-		    "name": name,
-		    "civ": civ,
-		    "color": color,
+		let playerState = simState.players[i];
+		players.push({
+		    "name": playerState.name,
+		    "civ": playerState.civ,
+		    "color": {
+			    "r": playerState.color.r*255,
+			    "g": playerState.color.g*255,
+			    "b": playerState.color.b*255,
+			    "a": playerState.color.a*255
+			},
 		    "team": playerState.team,
 		    "teamsLocked": playerState.teamsLocked,
 		    "cheatsEnabled": playerState.cheatsEnabled,
@@ -40,59 +28,23 @@ function getPlayerData(playerAssignments)
 		    "isNeutral": playerState.isNeutral,
 		    "isEnemy": playerState.isEnemy,
 		    "guid": undefined, // network guid for players controlled by hosts
-		    "disconnected": false // flag for host-controlled players who have left the game
-		};
-		players.push(player);
+		    "offline": previousData && !!previousData[i].offline
+		});
 	}
 
-	// Overwrite default player names with multiplayer names
 	if (playerAssignments)
-	{
-		for (var playerGuid in playerAssignments)
+		for (let playerGuid in playerAssignments)
 		{
-			var playerAssignment = playerAssignments[playerGuid];
-			if (players[playerAssignment.player])
-			{
-				players[playerAssignment.player].guid = playerGuid;
-				players[playerAssignment.player].name = playerAssignment.name;
-			}
+			let playerAssignment = playerAssignments[playerGuid];
+			if (!players[playerAssignment.player])
+				continue;
+			players[playerAssignment.player].guid = playerGuid;
+			players[playerAssignment.player].name = playerAssignment.name;
 		}
-	}
 
 	return players;
 }
 
-function findGuidForPlayerID(playerAssignments, player)
-{
-	for (var playerGuid in playerAssignments)
-	{
-		var playerAssignment = playerAssignments[playerGuid];
-		if (playerAssignment.player == player)
-			return playerGuid;
-	}
-	return undefined;
-}
-
-// Update player data when a host has connected
-function updatePlayerDataAdd(players, hostGuid, playerAssignment)
-{
-	if (players[playerAssignment.player])
-	{
-		players[playerAssignment.player].guid = hostGuid;
-		players[playerAssignment.player].name = playerAssignment.name;
-		players[playerAssignment.player].offline = false;
-	}
-}
-
-// Update player data when a host has disconnected
-function updatePlayerDataRemove(players, hostGuid)
-{
-	for each (var player in players)
-		if (player.guid == hostGuid)
-			player.offline = true;
-}
-
-
 function hasClass(entState, className)
 {
 	// note: use the functions in globalscripts/Templates.js for more versatile matching
@@ -122,6 +74,12 @@ function getRankIconSprite(entState)
  */
 function getTradingTooltip(gain)
 {
+	if (!gain)
+		return "";
+
+	var playerID = Engine.GetPlayerID();
+	var simState = GetSimState();
+	
 	var gainString = gain.traderGain;
 	if (gain.market1Gain && gain.market1Owner == gain.traderOwner)
 		gainString += translate("+") + gain.market1Gain;
@@ -130,18 +88,18 @@ function getTradingTooltip(gain)
 
 	var tooltip = sprintf(translate("%(gain)s (%(player)s)"), {
 		gain: gainString,
-		player: translate("you")
+		player: (!g_IsNetworked && gain.traderOwner == playerID) ? translate("You") : simState.players[gain.traderOwner].name
 	});
-
+	
 	if (gain.market1Gain && gain.market1Owner != gain.traderOwner)
-		tooltip += translate(", ") + sprintf(translate("%(gain)s (%(player)s)"), {
+		tooltip += translateWithContext("Separation mark in an enumeration", ", ") + sprintf(translate("%(gain)s (%(player)s)"), {
 			gain: gain.market1Gain,
-			player: sprintf(translate("player %(name)s"), { name: gain.market1Owner })
+			player: (!g_IsNetworked && gain.market1Owner == playerID) ? translate("You") : simState.players[gain.market1Owner].name
 		});
 	if (gain.market2Gain && gain.market2Owner != gain.traderOwner)
-		tooltip += translate(", ") + sprintf(translate("%(gain)s (%(player)s)"), {
+		tooltip += translateWithContext("Separation mark in an enumeration", ", ") + sprintf(translate("%(gain)s (%(player)s)"), {
 			gain: gain.market2Gain,
-			player: sprintf(translate("player %(name)s"), { name: gain.market2Owner })
+			player: (!g_IsNetworked && gain.market2Owner == playerID) ? translate("You") : simState.players[gain.market2Owner].name
 		});
 
 	return tooltip;
@@ -159,3 +117,16 @@ function getEntityOrHolder(ent)
 
 	return ent;
 }
+
+/**
+ * Returns a "color:255 0 0 Alpha" string based on how many resources are needed.
+ */
+function resourcesToAlphaMask(neededResources)
+{
+	var totalCost = 0;
+	for (var resource in neededResources)
+		totalCost += +neededResources[resource];
+	var alpha = 50 + Math.round(+totalCost/10.0);
+	alpha = alpha > 125 ? 125 : alpha;
+	return "color:255 0 0 " + alpha;
+}
